@@ -3,9 +3,17 @@
 #ifndef ATTACK_H
 #define ATTACK_H
 
+/* attack.c */
+extern void do_cmd_fire(cmd_code code, cmd_arg args[]);
+extern void textui_cmd_fire_at_nearest(void);
+extern void do_cmd_throw(cmd_code code, cmd_arg args[]);
+extern void textui_cmd_throw(void);
+
+
 extern int breakage_chance(const object_type *o_ptr, bool hit_target);
 extern bool test_hit(int chance, int ac, int vis);
 extern void py_attack(int y, int x);
+int py_attack_hit_chance(const object_type *weapon);
 
 /**
  *
