@@ -52,11 +52,9 @@
  * OPTION: Create and use a hidden directory in the users home directory
  * for storing pref files and character dumps.
  */
-#ifdef SET_UID
-# ifndef PRIVATE_USER_PATH
-#  define PRIVATE_USER_PATH "~/.angband"
-# endif /* PRIVATE_USER_PATH */
-#endif /* SET_UID */
+#if defined(UNIX) && !defined(MACH_O_CARBON) && !defined(PRIVATE_USER_PATH)
+# define PRIVATE_USER_PATH "~/.angband"
+#endif
 
 
 /*
@@ -94,10 +92,10 @@
 /*** Borg ***/
 
 /* Compile in support for the borg. */
-/* #define ALLOW_BORG */
+#define ALLOW_BORG
 
 /* Allow borgs to yield "high scores"? */
-/* #define SCORE_BORGS */
+#define SCORE_BORGS
 
 
 /*
