@@ -17,23 +17,30 @@
  */
 
 #include "angband.h"
-#include "button.h"
+#include "birth.h"
+#include "borg/borg1.h"
 #include "cave.h"
 #include "cmds.h"
+#include "dungeon.h"
 #include "files.h"
 #include "game-event.h"
 #include "generate.h"
+#include "grafmode.h"
 #include "init.h"
-#include "monster/monster.h"
+#include "monster/mon-list.h"
 #include "monster/mon-make.h"
 #include "monster/mon-spell.h"
 #include "monster/mon-util.h"
+#include "monster/monster.h"
 #include "object/tvalsval.h"
+#include "pathfind.h"
 #include "prefs.h"
 #include "savefile.h"
 #include "spells.h"
 #include "target.h"
 
+u16b daycount = 0;
+
 /*
  * Change dungeon level - e.g. by going up stairs or with WoR.
  */
@@ -199,10 +206,9 @@ static void regen_monsters(void)
 	{
 		/* Check the i'th monster */
 		monster_type *m_ptr = cave_monster(cave, i);
-		monster_race *r_ptr = &r_info[m_ptr->r_idx];
 
 		/* Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
 
 		/* Allow regeneration (if needed) */
 		if (m_ptr->hp < m_ptr->maxhp)
@@ -214,7 +220,7 @@ static void regen_monsters(void)
 			if (!frac) frac = 1;
 
 			/* Hack -- Some monsters regenerate quickly */
-			if (rf_has(r_ptr->flags, RF_REGENERATE)) frac *= 2;
+			if (rf_has(m_ptr->race->flags, RF_REGENERATE)) frac *= 2;
 
 			/* Hack -- Regenerate */
 			m_ptr->hp += frac;
@@ -223,7 +229,7 @@ static void regen_monsters(void)
 			if (m_ptr->hp > m_ptr->maxhp) m_ptr->hp = m_ptr->maxhp;
 
 			/* Redraw (later) if needed */
-			if (p_ptr->health_who == i) p_ptr->redraw |= (PR_HEALTH);
+			if (p_ptr->health_who == m_ptr) p_ptr->redraw |= (PR_HEALTH);
 		}
 	}
 }
@@ -540,7 +546,7 @@ static void process_world(struct cave *c)
 	if (one_in_(MAX_M_ALLOC_CHANCE))
 	{
 		/* Make a new monster */
-		(void)pick_and_place_distant_monster(cave, loc(p_ptr->px, p_ptr->py), MAX_SIGHT + 5, FALSE, p_ptr->depth);
+		(void)pick_and_place_distant_monster(cave, loc(p_ptr->px, p_ptr->py), MAX_SIGHT + 5, TRUE, p_ptr->depth);
 	}
 
 	/* Hack -- Check for creature regeneration */
@@ -579,33 +585,22 @@ static void process_world(struct cave *c)
 	/*** Check the Food, and Regenerate ***/
 
 	/* Digest normally */
-	if (p_ptr->food < PY_FOOD_MAX)
+	if (!(turn % 100))
 	{
-		/* Every 100 game turns */
-		if (!(turn % 100))
-		{
-			/* Basic digestion rate based on speed */
-			i = extract_energy[p_ptr->state.speed] * 2;
+		/* Basic digestion rate based on speed */
+		i = extract_energy[p_ptr->state.speed] * 2;
 
-			/* Regeneration takes more food */
-			if (check_state(p_ptr, OF_REGEN, p_ptr->state.flags)) i += 30;
+		/* Regeneration takes more food */
+		if (check_state(p_ptr, OF_REGEN, p_ptr->state.flags)) i += 30;
 
-			/* Slow digestion takes less food */
-			if (check_state(p_ptr, OF_SLOW_DIGEST, p_ptr->state.flags)) i -= 10;
+		/* Slow digestion takes less food */
+		if (check_state(p_ptr, OF_SLOW_DIGEST, p_ptr->state.flags)) i /= 5;
 
-			/* Minimal digestion */
-			if (i < 1) i = 1;
+		/* Minimal digestion */
+		if (i < 1) i = 1;
 
-			/* Digest some food */
-			player_set_food(p_ptr, p_ptr->food - i);
-		}
-	}
-
-	/* Digest quickly when gorged */
-	else
-	{
-		/* Digest a lot of food */
-		player_set_food(p_ptr, p_ptr->food - 100);
+		/* Digest some food */
+		player_set_food(p_ptr, p_ptr->food - i);
 	}
 
 	/* Getting Faint */
@@ -649,7 +644,7 @@ static void process_world(struct cave *c)
 	/* Various things speed up regeneration */
 	if (check_state(p_ptr, OF_REGEN, p_ptr->state.flags))
 		regen_amount *= 2;
-	if (p_ptr->searching || p_ptr->resting)
+	if (p_ptr->searching || player_resting_can_regenerate(p_ptr))
 		regen_amount *= 2;
 
 	/* Some things slow it down */
@@ -675,7 +670,7 @@ static void process_world(struct cave *c)
 	/* Various things speed up regeneration */
 	if (check_state(p_ptr, OF_REGEN, p_ptr->state.flags))
 		regen_amount *= 2;
-	if (p_ptr->searching || p_ptr->resting)
+	if (p_ptr->searching || player_resting_can_regenerate(p_ptr))
 		regen_amount *= 2;
 
 	/* Some things slow it down */
@@ -700,8 +695,7 @@ static void process_world(struct cave *c)
 	o_ptr = &p_ptr->inventory[INVEN_LIGHT];
 
 	/* Burn some fuel in the current light */
-	if (o_ptr->tval == TV_LIGHT)
-	{
+	if (o_ptr->tval == TV_LIGHT) {
 		bitflag f[OF_SIZE];
 		bool burn_fuel = TRUE;
 
@@ -717,35 +711,34 @@ static void process_world(struct cave *c)
 		    burn_fuel = FALSE;
 
 		/* Use some fuel (except on artifacts, or during the day) */
-		if (burn_fuel && o_ptr->timeout > 0)
-		{
+		if (burn_fuel && o_ptr->timeout > 0) {
 			/* Decrease life-span */
 			o_ptr->timeout--;
 
 			/* Hack -- notice interesting fuel steps */
 			if ((o_ptr->timeout < 100) || (!(o_ptr->timeout % 100)))
-			{
 				/* Redraw stuff */
 				p_ptr->redraw |= (PR_EQUIP);
-			}
 
 			/* Hack -- Special treatment when blind */
-			if (p_ptr->timed[TMD_BLIND])
-			{
+			if (p_ptr->timed[TMD_BLIND]) {
 				/* Hack -- save some light for later */
 				if (o_ptr->timeout == 0) o_ptr->timeout++;
-			}
 
 			/* The light is now out */
-			else if (o_ptr->timeout == 0)
-			{
+			} else if (o_ptr->timeout == 0) {
 				disturb(p_ptr, 0, 0);
 				msg("Your light has gone out!");
+
+				/* If it's a torch, now is the time to delete it */
+				if (o_ptr->sval == SV_LIGHT_TORCH) {
+					inven_item_increase(INVEN_LIGHT, -1);
+					inven_item_optimize(INVEN_LIGHT);
+				}
 			}
 
 			/* The light is getting dim */
-			else if ((o_ptr->timeout < 100) && (!(o_ptr->timeout % 10)))
-			{
+			else if ((o_ptr->timeout < 50) && (!(o_ptr->timeout % 20))) {
 				disturb(p_ptr, 0, 0);
 				msg("Your light is growing faint.");
 			}
@@ -761,8 +754,10 @@ static void process_world(struct cave *c)
 	/* Handle experience draining */
 	if (check_state(p_ptr, OF_DRAIN_EXP, p_ptr->state.flags))
 	{
-		if ((p_ptr->exp > 0) && one_in_(10))
-			player_exp_lose(p_ptr, 1, FALSE);
+		if ((p_ptr->exp > 0) && one_in_(10)) {
+			s32b d = damroll(10, 6) + (p_ptr->exp/100) * MON_DRAIN_LIFE;
+			player_exp_lose(p_ptr, d / 10, FALSE);
+		}
 
 		wieldeds_notice_flag(p_ptr, OF_DRAIN_EXP);
 	}
@@ -777,7 +772,7 @@ static void process_world(struct cave *c)
 	/*** Involuntary Movement ***/
 
 	/* Random teleportation */
-	if (check_state(p_ptr, OF_TELEPORT, p_ptr->state.flags) && one_in_(100))
+	if (check_state(p_ptr, OF_TELEPORT, p_ptr->state.flags) && one_in_(50))
 	{
 		wieldeds_notice_flag(p_ptr, OF_TELEPORT);
 		teleport_player(40);
@@ -805,12 +800,50 @@ static void process_world(struct cave *c)
 			else
 			{
 				msgt(MSG_TPLEVEL, "You feel yourself yanked downwards!");
+                
+                /* Force descent to a lower level if allowed */
+                if (OPT(birth_force_descend) && p_ptr->max_depth < MAX_DEPTH - 1
+                  && !is_quest(p_ptr->max_depth)){
+
+                    p_ptr->max_depth = p_ptr->max_depth + 1;
+                }
 
 				/* New depth - back to max depth or 1, whichever is deeper */
 				dungeon_change_level(p_ptr->max_depth < 1 ? 1: p_ptr->max_depth);
 			}
 		}
 	}
+
+	/* Delayed Deep Descent */
+	if (p_ptr->deep_descent) {
+		/* Count down towards recall */
+		p_ptr->deep_descent--;
+
+		/* Activate the recall */
+		if (p_ptr->deep_descent == 0) {
+			int target_depth = p_ptr->max_depth;
+
+			/* Calculate target depth */
+			for (i = 5; i > 0; i--) {
+				if (is_quest(target_depth)) break;
+				if (target_depth >= MAX_DEPTH - 1) break;
+				
+				target_depth++;
+			}
+
+			disturb(p_ptr, 0, 0);
+
+			/* Determine the level */
+			if (target_depth > p_ptr->depth) {
+				msgt(MSG_TPLEVEL, "The floor opens beneath you!");
+				dungeon_change_level(target_depth);
+			} else {
+				/* Otherwise do something disastrous */
+				msgt(MSG_TPLEVEL, "You are thrown back in an explosion!");
+				destroy_area(p_ptr->py, p_ptr->px, 5, TRUE);
+			}		
+		}
+	}
 }
 
 
@@ -827,7 +860,7 @@ static void process_player_aux(void)
 	int i;
 	bool changed = FALSE;
 
-	static int old_monster_race_idx = 0;
+	static monster_race *old_monster_race = 0;
 	static bitflag old_flags[RF_SIZE];
 	static bitflag old_spell_flags[RSF_SIZE];
 
@@ -837,10 +870,10 @@ static void process_player_aux(void)
 	static byte	old_cast_spell = 0;
 
 	/* Tracking a monster */
-	if (p_ptr->monster_race_idx)
+	if (p_ptr->monster_race)
 	{
 		/* Get the monster lore */
-		monster_lore *l_ptr = &l_list[p_ptr->monster_race_idx];
+		monster_lore *l_ptr = get_lore(p_ptr->monster_race);
 
 		for (i = 0; i < MONSTER_BLOW_MAX; i++)
 		{
@@ -853,14 +886,14 @@ static void process_player_aux(void)
 
 		/* Check for change of any kind */
 		if (changed ||
-		    (old_monster_race_idx != p_ptr->monster_race_idx) ||
+		    (old_monster_race != p_ptr->monster_race) ||
 		    !rf_is_equal(old_flags, l_ptr->flags) ||
 		    !rsf_is_equal(old_spell_flags, l_ptr->spell_flags) ||
 		    (old_cast_innate != l_ptr->cast_innate) ||
 		    (old_cast_spell != l_ptr->cast_spell))
 		{
 			/* Memorize old race */
-			old_monster_race_idx = p_ptr->monster_race_idx;
+			old_monster_race = p_ptr->monster_race;
 
 			/* Memorize flags */
 			rf_copy(old_flags, l_ptr->flags);
@@ -881,6 +914,18 @@ static void process_player_aux(void)
 }
 
 
+/*
+ * Place cursor on a monster or the player.
+ */
+static void place_cursor(void) {
+	if (OPT(show_target) && target_sighted()) {
+		s16b col, row;
+		target_get(&col, &row);
+		move_cursor_relative(row, col);
+	}
+}
+
+
 /*
  * Process the player
  *
@@ -905,53 +950,12 @@ static void process_player(void)
 
 	/*** Check for interrupts ***/
 
-	/* Complete resting */
-	if (p_ptr->resting < 0)
-	{
-		/* Basic resting */
-		if (p_ptr->resting == REST_ALL_POINTS)
-		{
-			/* Stop resting */
-			if ((p_ptr->chp == p_ptr->mhp) &&
-			    (p_ptr->csp == p_ptr->msp))
-			{
-				disturb(p_ptr, 0, 0);
-			}
-		}
-
-		/* Complete resting */
-		else if (p_ptr->resting == REST_COMPLETE)
-		{
-			/* Stop resting */
-			if ((p_ptr->chp == p_ptr->mhp) &&
-			    (p_ptr->csp == p_ptr->msp) &&
-			    !p_ptr->timed[TMD_BLIND] && !p_ptr->timed[TMD_CONFUSED] &&
-			    !p_ptr->timed[TMD_POISONED] && !p_ptr->timed[TMD_AFRAID] &&
-			    !p_ptr->timed[TMD_TERROR] &&
-			    !p_ptr->timed[TMD_STUN] && !p_ptr->timed[TMD_CUT] &&
-			    !p_ptr->timed[TMD_SLOW] && !p_ptr->timed[TMD_PARALYZED] &&
-			    !p_ptr->timed[TMD_IMAGE] && !p_ptr->word_recall)
-			{
-				disturb(p_ptr, 0, 0);
-			}
-		}
-		
-		/* Rest until HP or SP are filled */
-		else if (p_ptr->resting == REST_SOME_POINTS)
-		{
-			/* Stop resting */
-			if ((p_ptr->chp == p_ptr->mhp) ||
-			    (p_ptr->csp == p_ptr->msp))
-			{
-				disturb(p_ptr, 0, 0);
-			}
-		}
-	}
+	player_resting_complete_special(p_ptr);
 
 	/* Check for "player abort" */
 	if (p_ptr->running ||
 	    cmd_get_nrepeats() > 0 ||
-	    (p_ptr->resting && !(turn & 0x7F)))
+	    (player_is_resting(p_ptr) && !(turn & 0x7F)))
 	{
 		ui_event e;
 
@@ -984,8 +988,9 @@ static void process_player(void)
 		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
 
-		/* Place the cursor on the player */
-		move_cursor_relative(p_ptr->py, p_ptr->px);
+		/* Place cursor on player/target */
+		place_cursor();
+
 
 		/* Refresh (optional) */
 		Term_fresh();
@@ -1030,23 +1035,9 @@ static void process_player(void)
 		}
 
 		/* Resting */
-		else if (p_ptr->resting)
+		else if (player_is_resting(p_ptr))
 		{
-			/* Timed rest */
-			if (p_ptr->resting > 0)
-			{
-				/* Reduce rest count */
-				p_ptr->resting--;
-
-				/* Redraw the state */
-				p_ptr->redraw |= (PR_STATE);
-			}
-
-			/* Take a turn */
-			p_ptr->energy_use = 100;
-
-			/* Increment the resting counter */
-			p_ptr->resting_turn++;
+			player_resting_step_turn(p_ptr);
 		}
 
 		/* Running */
@@ -1075,8 +1066,8 @@ static void process_player(void)
 			/* Check monster recall */
 			process_player_aux();
 
-			/* Place the cursor on the player */
-			move_cursor_relative(p_ptr->py, p_ptr->px);
+			/* Place cursor on player/target */
+			place_cursor();
 
 			/* Get and process a command */
 			process_command(CMD_GAME, FALSE);
@@ -1107,12 +1098,10 @@ static void process_player(void)
 			/* Shimmer multi-hued monsters */
 			for (i = 1; i < cave_monster_max(cave); i++)
 			{
-				struct monster_race *race;
 				struct monster *mon = cave_monster(cave, i);
-				if (!mon->r_idx)
+				if (!mon->race)
 					continue;
-				race = &r_info[mon->r_idx];
-				if (!rf_has(race->flags, RF_ATTR_MULTI))
+				if (!rf_has(mon->race->flags, RF_ATTR_MULTI))
 					continue;
 				cave_light_spot(cave, mon->fy, mon->fx);
 			}
@@ -1125,7 +1114,7 @@ static void process_player(void)
 				if (mon->mflag & MFLAG_MARK) {
 					if (!(mon->mflag & MFLAG_SHOW)) {
 						mon->mflag &= ~MFLAG_MARK;
-						update_mon(i, FALSE);
+						update_mon(mon, FALSE);
 					}
 				}
 			}
@@ -1149,8 +1138,8 @@ static void process_player(void)
 	if (p_ptr->notice) notice_stuff(p_ptr);
 }
 
-byte flicker = 0;
-byte color_flicker[MAX_COLORS][3] = 
+static byte flicker = 0;
+static byte color_flicker[MAX_COLORS][3] = 
 {
 	{TERM_DARK, TERM_L_DARK, TERM_L_RED},
 	{TERM_WHITE, TERM_L_WHITE, TERM_L_BLUE},
@@ -1203,20 +1192,20 @@ static void do_animation(void)
 	{
 		byte attr;
 		monster_type *m_ptr = cave_monster(cave, i);
-		monster_race *r_ptr = &r_info[m_ptr->r_idx];
 
-		if (!m_ptr || !m_ptr->ml)
+		if (!m_ptr || !m_ptr->race || !m_ptr->ml)
 			continue;
-		else if (rf_has(r_ptr->flags, RF_ATTR_MULTI))
+		else if (rf_has(m_ptr->race->flags, RF_ATTR_MULTI))
 			attr = randint1(BASIC_COLORS - 1);
-		else if (rf_has(r_ptr->flags, RF_ATTR_FLICKER))
-			attr = get_flicker(r_ptr->x_attr);
+		else if (rf_has(m_ptr->race->flags, RF_ATTR_FLICKER))
+			attr = get_flicker(m_ptr->race->x_attr);
 		else
 			continue;
 
 		m_ptr->attr = attr;
 		p_ptr->redraw |= (PR_MAP | PR_MONLIST);
 	}
+
 	flicker++;
 }
 
@@ -1229,7 +1218,7 @@ void idle_update(void)
 {
 	if (!character_dungeon) return;
 
-	if (!OPT(animate_flicker)) return;
+	if (!OPT(animate_flicker) || (use_graphics != GRAPHICS_NONE)) return;
 
 	/* Animate and redraw if necessary */
 	do_animation();
@@ -1262,19 +1251,20 @@ static void dungeon(struct cave *c)
 	p_ptr->leaving = FALSE;
 
 
-	/* Reset the "command" vars */
-	p_ptr->command_arg = 0;
-
-
 	/* Cancel the target */
 	target_set_monster(0);
 
 	/* Cancel the health bar */
-	health_track(p_ptr, 0);
+	health_track(p_ptr, NULL);
 
 	/* Disturb */
 	disturb(p_ptr, 1, 0);
 
+	/*
+	 * Because changing levels doesn't take a turn and PR_MONLIST might not be
+	 * set for a few game turns, manually force an update on level change.
+	 */
+	monster_list_force_subwindow_update();
 
 	/* Track maximum player level */
 	if (p_ptr->max_lev < p_ptr->lev)
@@ -1293,8 +1283,13 @@ static void dungeon(struct cave *c)
 	if (p_ptr->autosave)
 	{
 /* The borg runs so quickly that this is a bad idea. */
-#ifndef ALLOW_BORG 
+#ifndef ALLOW_BORG
 		save_game();
+#endif
+#ifdef ALLOW_BORG
+        if (!borg_active){
+            save_game();
+        }
 #endif
 		p_ptr->autosave = FALSE;
 	}
@@ -1354,16 +1349,6 @@ static void dungeon(struct cave *c)
 	/* Combine / Reorder the pack */
 	p_ptr->notice |= (PN_COMBINE | PN_REORDER | PN_SORT_QUIVER);
 
-	/* Make basic mouse buttons */
-	(void) button_add("[ESC]", ESCAPE);
-	(void) button_add("[Ret]", '\r');
-	(void) button_add("[Spc]", ' ');
-	(void) button_add("[Rpt]", 'n');
-	(void) button_add("[Std]", ',');
-
-	/* Redraw buttons */
-	p_ptr->redraw |= (PR_BUTTONS);
-
 	/* Notice stuff */
 	notice_stuff(p_ptr);
 
@@ -1408,7 +1393,7 @@ static void dungeon(struct cave *c)
 		while ((p_ptr->energy >= 100) && !p_ptr->leaving)
 		{
     		/* Do any necessary animations */
-    		do_animation();
+    		do_animation(); 
 
 			/* process monster with even more energy first */
 			process_monsters(c, (byte)(p_ptr->energy + 1));
@@ -1434,8 +1419,8 @@ static void dungeon(struct cave *c)
 		/* Redraw stuff */
 		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
-		/* Hack -- Highlight the player */
-		move_cursor_relative(p_ptr->py, p_ptr->px);
+		/* Place cursor on player/target */
+		place_cursor();
 
 		/* Handle "leaving" */
 		if (p_ptr->leaving) break;
@@ -1453,8 +1438,8 @@ static void dungeon(struct cave *c)
 		/* Redraw stuff */
 		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
-		/* Hack -- Highlight the player */
-		move_cursor_relative(p_ptr->py, p_ptr->px);
+		/* Place cursor on player/target */
+		place_cursor();
 
 		/* Handle "leaving" */
 		if (p_ptr->leaving) break;
@@ -1472,8 +1457,8 @@ static void dungeon(struct cave *c)
 		/* Redraw stuff */
 		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
-		/* Hack -- Highlight the player */
-		move_cursor_relative(p_ptr->py, p_ptr->px);
+		/* Place cursor on player/target */
+		place_cursor();
 
 		/* Handle "leaving" */
 		if (p_ptr->leaving) break;
@@ -1487,12 +1472,10 @@ static void dungeon(struct cave *c)
 		for (i = cave_monster_max(cave) - 1; i >= 1; i--)
 		{
 			int mspeed;
-
-			/* Access the monster */
+			
+			/* Access the monster (if alive) */
 			m_ptr = cave_monster(cave, i);
-
-			/* Ignore "dead" monsters */
-			if (!m_ptr->r_idx) continue;
+			if (!m_ptr->race) continue;
 
 			/* Calculate the net speed */
 			mspeed = m_ptr->mspeed;
@@ -1517,16 +1500,27 @@ static void dungeon(struct cave *c)
  */
 static void process_some_user_pref_files(void)
 {
+	bool found;
 	char buf[1024];
 
 	/* Process the "user.prf" file */
-	(void)process_pref_file("user.prf", TRUE, TRUE);
+	process_pref_file("user.prf", TRUE, TRUE);
 
 	/* Get the "PLAYER.prf" filename */
-	(void)strnfmt(buf, sizeof(buf), "%s.prf", op_ptr->base_name, TRUE);
+	strnfmt(buf, sizeof(buf), "%s.prf", player_safe_name(p_ptr, TRUE));
 
-	/* Process the "PLAYER.prf" file */
-	(void)process_pref_file(buf, TRUE, TRUE);
+	/* Process the "PLAYER.prf" file, using the character name */
+	found = process_pref_file(buf, TRUE, TRUE);
+
+    /* Try pref file using savefile name if we fail using character name */
+    if (!found) {
+		int filename_index = path_filename_index(savefile);
+		char filename[128];
+
+		my_strcpy(filename, &savefile[filename_index], sizeof(filename));
+		strnfmt(buf, sizeof(buf), "%s.prf", filename);
+		process_pref_file(buf, TRUE, TRUE);
+    }
 }
 
 
@@ -1548,9 +1542,8 @@ static void process_some_user_pref_files(void)
  * character, then a new game will be started.
  *
  * Several platforms (Windows, Macintosh, Amiga) start brand new games
- * with "savefile" and "op_ptr->base_name" both empty, and initialize
- * them later based on the player name.  To prevent weirdness, we must
- * initialize "op_ptr->base_name" to "PLAYER" if it is empty.
+ * with "savefile" empty, and initialize it later based on the player
+ * name.
  *
  * Note that we load the RNG state from savefiles (2.8.0 or later) and
  * so we only initialize it if we were unable to load it.  The loading
@@ -1559,6 +1552,7 @@ static void process_some_user_pref_files(void)
  */
 void play_game(void)
 {
+	u32b window_flag[ANGBAND_TERM_MAX];
 	/* Initialize */
 	bool new_game = init_angband();
 
@@ -1581,6 +1575,18 @@ void play_game(void)
 	/* Hack -- Turn off the cursor */
 	(void)Term_set_cursor(FALSE);
 
+	/* initialize window options that will be overridden by the savefile */
+	memset(window_flag, 0, sizeof(u32b)*ANGBAND_TERM_MAX);
+	if (ANGBAND_TERM_MAX > 1) window_flag[1] = (PW_MESSAGE);
+	if (ANGBAND_TERM_MAX > 2) window_flag[2] = (PW_INVEN);
+	if (ANGBAND_TERM_MAX > 3) window_flag[3] = (PW_MONLIST);
+	if (ANGBAND_TERM_MAX > 4) window_flag[4] = (PW_ITEMLIST);
+	if (ANGBAND_TERM_MAX > 5) window_flag[5] = (PW_MONSTER | PW_OBJECT);
+	if (ANGBAND_TERM_MAX > 6) window_flag[6] = (PW_OVERHEAD);
+	if (ANGBAND_TERM_MAX > 7) window_flag[7] = (PW_PLAYER_2);
+
+	/* Set up the subwindows */
+	subwindows_set_flags(window_flag, ANGBAND_TERM_MAX);
 
 	/*** Try to load the savefile ***/
 
@@ -1607,10 +1613,6 @@ void play_game(void)
 		character_dungeon = FALSE;
 	}
 
-	/* Hack -- Default base_name */
-	if (!op_ptr->base_name[0])
-		my_strcpy(op_ptr->base_name, "PLAYER", sizeof(op_ptr->base_name));
-
 
 	/* Init RNG */
 	if (Rand_quick)
@@ -1618,9 +1620,9 @@ void play_game(void)
 		u32b seed;
 
 		/* Basic seed */
-		seed = (time(NULL));
+		seed = (u32b)(time(NULL));
 
-#ifdef SET_UID
+#ifdef UNIX
 
 		/* Mutate the seed on Unix machines */
 		seed = ((seed >> 3) * (getpid() << 1));
@@ -1643,37 +1645,32 @@ void play_game(void)
 		/* Start in town */
 		p_ptr->depth = 0;
 
-		/* Hack -- seed for flavors */
+		/* Seed for flavors */
 		seed_flavor = randint0(0x10000000);
 
-		/* Hack -- seed for town layout */
+		/* Seed for town layout */
 		seed_town = randint0(0x10000000);
 
-		/* Hack -- seed for random artifacts */
-		seed_randart = randint0(0x10000000);
-
 		/* Roll up a new character. Quickstart is allowed if ht_birth is set */
 		player_birth(p_ptr->ht_birth ? TRUE : FALSE);
-
-		/* Randomize the artifacts if required */
-		if (OPT(birth_randarts) &&
-				(!OPT(birth_keep_randarts) || !p_ptr->randarts)) {
-			do_randart(seed_randart, TRUE);
-			p_ptr->randarts = TRUE;
-		}
 	}
 
+	/* Seed for random artifacts */
+	if (!seed_randart || (new_game && !OPT(birth_keep_randarts)))
+		seed_randart = randint0(0x10000000);
+
+	/* Randomize the artifacts if required */
+	if (OPT(birth_randarts))
+		do_randart(seed_randart, TRUE);
+
 	/* Initialize temporary fields sensibly */
-	p_ptr->object_idx = p_ptr->object_kind_idx = NO_OBJECT;
-	p_ptr->monster_race_idx = 0;
+	p_ptr->object_idx = NO_OBJECT;
+	p_ptr->object_kind = NULL;
+	p_ptr->monster_race = NULL;
 
-	/* Normal machine (process player name) */
-	if (savefile[0])
-		process_player_name(FALSE);
-
-	/* Weird machine (process player name, pick savefile name) */
-	else
-		process_player_name(TRUE);
+	/* Set the savefile name if it's not already set */
+	if (!savefile[0])
+		savefile_set_name(player_safe_name(p_ptr, TRUE));
 
 	/* Stop the player being quite so dead */
 	p_ptr->is_dead = FALSE;
@@ -1754,11 +1751,11 @@ void play_game(void)
 		target_set_monster(0);
 
 		/* Cancel the health bar */
-		health_track(p_ptr, 0);
+		health_track(p_ptr, NULL);
 
 
 		/* Forget the view */
-		forget_view();
+		forget_view(cave);
 
 
 		/* Handle "quit and save" */
@@ -1774,7 +1771,7 @@ void play_game(void)
 			 * remove. Very similar to do_cmd_wiz_cure_all(). */
 			if ((p_ptr->wizard || OPT(cheat_live)) && !get_check("Die? ")) {
 				/* Mark social class, reset age, if needed */
-				if (p_ptr->sc) p_ptr->sc = p_ptr->age = 0;
+				p_ptr->age = 0;
 
 				/* Increase age */
 				p_ptr->age++;
