@@ -15,11 +15,11 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-
 #include "angband.h"
 #include "buildid.h"
 #include "cave.h"
 #include "cmds.h"
+#include "death.h"
 #include "files.h"
 #include "game-cmd.h"
 #include "history.h"
@@ -29,7 +29,70 @@
 #include "savefile.h"
 #include "ui-menu.h"
 
-#define MAX_PANEL 12
+
+/** Panel utilities **/
+
+/* Panel line type */
+struct panel_line {
+	byte attr;
+	const char *label;
+	char value[20];
+};
+
+/* Panel holder type */
+struct panel {
+	size_t len;
+	size_t max;
+	struct panel_line *lines;
+};
+
+/* Allocate some panel lines */
+static struct panel *panel_allocate(int n) {
+	struct panel *p = mem_zalloc(sizeof *p);
+
+	p->len = 0;
+	p->max = n;
+	p->lines = mem_zalloc(p->max * sizeof *p->lines);
+
+	return p;
+}
+
+/* Free up panel lines */
+static void panel_free(struct panel *p) {
+	assert(p);
+	mem_free(p->lines);
+	mem_free(p);
+}
+
+/* Add a new line to the panel */
+static void panel_line(struct panel *p, byte attr, const char *label,
+		const char *fmt, ...) {
+	va_list vp;
+
+	struct panel_line *pl;
+
+	/* Get the next panel line */
+	assert(p);
+	assert(p->len != p->max);
+	pl = &p->lines[p->len++];
+
+	/* Set the basics */
+	pl->attr = attr;
+	pl->label = label;
+
+	/* Set the value */
+	va_start(vp, fmt);
+	vstrnfmt(pl->value, sizeof pl->value, fmt, vp);
+	va_end(vp);
+}
+
+/* Add a spacer line in a panel */
+static void panel_space(struct panel *p) {
+	assert(p);
+	assert(p->len != p->max);
+	p->len++;
+}
+
 
 /*
  * Returns a "rating" of x depending on y, and sets "attr" to the
@@ -131,7 +194,7 @@ static void display_player_equippy(int y, int x)
 	int i;
 
 	byte a;
-	char c;
+	wchar_t c;
 
 	object_type *o_ptr;
 
@@ -408,13 +471,11 @@ static void display_player_sust_info(void)
 	stat_flags[A_WIS] = OF_WIS;
 	stat_flags[A_DEX] = OF_DEX;
 	stat_flags[A_CON] = OF_CON;
-	stat_flags[A_CHR] = OF_CHR;
 	sustain_flags[A_STR] = OF_SUST_STR;
 	sustain_flags[A_INT] = OF_SUST_INT;
 	sustain_flags[A_WIS] = OF_SUST_WIS;
 	sustain_flags[A_DEX] = OF_SUST_DEX;
 	sustain_flags[A_CON] = OF_SUST_CON;
-	sustain_flags[A_CHR] = OF_SUST_CHR;
 
 	/* Header */
 	c_put_str(TERM_WHITE, "abcdefghijkl@", row-1, col);
@@ -527,10 +588,10 @@ static void display_player_sust_info(void)
 
 
 
-static void display_panel(const data_panel *panel, int count, bool left_adj, const region *bounds)
+static void display_panel(const struct panel *p, bool left_adj,
+		const region *bounds)
 {
-	int i;
-	char buffer[50];
+	size_t i;
 	int col = bounds->col;
 	int row = bounds->row;
 	int w = bounds->width;
@@ -538,46 +599,35 @@ static void display_panel(const data_panel *panel, int count, bool left_adj, con
 
 	region_erase(bounds);
 
-	if (left_adj)
-	{
-		for (i = 0; i < count; i++)
-		{
-			int len = panel[i].label ? strlen(panel[i].label) : 0;
+	if (left_adj) {
+		for (i = 0; i < p->len; i++) {
+			struct panel_line *pl = &p->lines[i];
+
+			int len = pl->label ? strlen(pl->label) : 0;
 			if (offset < len) offset = len;
 		}
 		offset += 2;
 	}
 
-	for (i = 0; i < count; i++, row++)
-	{
+	for (i = 0; i < p->len; i++, row++) {
 		int len;
-		if (!panel[i].label) continue;
-		Term_putstr(col, row, strlen(panel[i].label), TERM_WHITE, panel[i].label);
+		struct panel_line *pl = &p->lines[i];
 
-		strnfmt(buffer, sizeof(buffer), panel[i].fmt, panel[i].value[0], panel[i].value[1]);
+		if (!pl->label)
+			continue;
 
-		len = strlen(buffer);
+		Term_putstr(col, row, strlen(pl->label), TERM_WHITE, pl->label);
+
+		len = strlen(pl->value);
 		len = len < w - offset ? len : w - offset - 1;
+
 		if (left_adj)
-			Term_putstr(col+offset, row, len, panel[i].color, buffer);
+			Term_putstr(col+offset, row, len, pl->attr, pl->value);
 		else
-			Term_putstr(col+w-len, row, len, panel[i].color, buffer);
+			Term_putstr(col+w-len, row, len, pl->attr, pl->value);
 	}
 }
 
-
-static const region boundaries [] =
-{
-	/* x   y     width, rows */
-	{ 0,   0,		0,		0 },
-	{ 1,   1,		40,		8 }, /* Name, Class, ... */
-	{ 1,   9,		22,		9 }, /* Cur Exp, Max Exp, ... */
-	{ 26,  9,		17,		9 }, /* AC, melee, ... */
-	{ 48,  9,		24,		8 }, /* skills */
-	{ 21,  2,		18,		5 }, /* Age, ht, wt, ... */
-};
-
-
 static const char *show_title(void)
 {
 	if (p_ptr->wizard)
@@ -625,98 +675,11 @@ static const char *show_speed(void)
 	return buffer;
 }
 
-static const char *show_melee_weapon(const object_type *o_ptr)
-{
-	static char buffer[12];
-	int hit = p_ptr->state.dis_to_h;
-	int dam = p_ptr->state.dis_to_d;
-
-	if (object_attack_plusses_are_visible(o_ptr))
-	{
-		hit += o_ptr->to_h;
-		dam += o_ptr->to_d;
-	}
-
-	strnfmt(buffer, sizeof(buffer), "(%+d,%+d)", hit, dam);
-	return buffer;
-}
-
-static const char *show_missile_weapon(const object_type *o_ptr)
-{
-	static char buffer[12];
-	int hit = p_ptr->state.dis_to_h;
-	int dam = 0;
-
-	if (object_attack_plusses_are_visible(o_ptr))
-	{
-		hit += o_ptr->to_h;
-		dam += o_ptr->to_d;
-	}
-
-	strnfmt(buffer, sizeof(buffer), "(%+d,%+d)", hit, dam);
-	return buffer;
-}
-
 static byte max_color(int val, int max)
 {
 	return val < max ? TERM_YELLOW : TERM_L_GREEN;
 }
 
-
-static const char *show_status(void)
-{
-	int sc = p_ptr->sc;
-	sc /= 10;
-
-	switch (sc)
-	{
-		case 0:
-		case 1:
-			return "Pariah";
-
-		case 2:
-			return "Outcast";
-
-		case 3:
-		case 4:
-			return "Unknown";
-
-		case 5:
-			return "Known";
-
-		case 6:
-		/* Maximum status by birth 75 = 7 */
-		case 7:
-			return "Liked";
-
-		case 8:
-			return "Well-liked";
-
-		case 9:
-		case 10:
-			return "Respected";
-
-		case 11:
-		case 12:
-			return "Role model";
-
-		case 13:
-			return "Feared";
-
-		case 14:
-		case 15:
-			return "Lordly";
-	}
-
-	return format("%d", sc);
-}
-
-/* data_panel array element initializer, for ansi compliance */
-#define P_I(col, lab, format, val1, val2) \
-	{ panel[i].color = col; panel[i].label = lab; panel[i].fmt = format; \
-	 panel[i].value[0] = val1; panel[i].value[1] = val2; \
-	 i++; }
-
 /* colours for table items */
 static const byte colour_table[] =
 {
@@ -725,176 +688,166 @@ static const byte colour_table[] =
 	TERM_L_BLUE
 };
 
-static int get_panel(int oid, data_panel *panel, size_t size)
-{
-	int ret = (s32b) size;
-	switch(oid)
-	{
-  case 1:
-  {
-	int i = 0;
-	assert( size >= (u32b) boundaries[1].page_rows);
-	ret = boundaries[1].page_rows;
-	P_I(TERM_L_BLUE, "Name",	"%y",	s2u(op_ptr->full_name), END  );
-	P_I(TERM_L_BLUE, "Sex",		"%y",	s2u(p_ptr->sex->title), END  );
-	P_I(TERM_L_BLUE, "Race",	"%y",	s2u(p_ptr->race->name), END  );
-	P_I(TERM_L_BLUE, "Class",	"%y",	s2u(p_ptr->class->name), END  );
-	P_I(TERM_L_BLUE, "Title",	"%y",	s2u(show_title()), END  );
-	P_I(TERM_L_BLUE, "HP",	"%y/%y",	i2u(p_ptr->chp), i2u(p_ptr->mhp)  );
-	P_I(TERM_L_BLUE, "SP",	"%y/%y",	i2u(p_ptr->csp), i2u(p_ptr->msp)  );
-	P_I(TERM_L_BLUE, "Level",	"%y",	i2u(p_ptr->lev), END  );
-	assert(i == boundaries[1].page_rows);
-	return ret;
-  }
-  case 2:
-  {
-	int i = 0;
-	assert( ret >= boundaries[2].page_rows);
-	ret = boundaries[2].page_rows;
-	P_I(max_color(p_ptr->lev, p_ptr->max_lev), "Level", "%y", i2u(p_ptr->lev), END  );
-	P_I(max_color(p_ptr->exp, p_ptr->max_exp), "Cur Exp", "%y", i2u(p_ptr->exp), END  );
-	P_I(TERM_L_GREEN, "Max Exp",	"%y",	i2u(p_ptr->max_exp), END  );
-	P_I(TERM_L_GREEN, "Adv Exp",	"%y",	s2u(show_adv_exp()), END  );
-	P_I(TERM_L_GREEN, "MaxDepth",	"%y",	s2u(show_depth()), END  );
-	P_I(TERM_L_GREEN, "Game Turns",	"%y",	i2u(turn), END  );
-	P_I(TERM_L_GREEN, "Standard Turns","%y", i2u(p_ptr->total_energy / 100), END  );
-	P_I(TERM_L_GREEN, "Resting Turns","%y",	i2u(p_ptr->resting_turn), END  );
-	P_I(TERM_L_GREEN, "Gold",		"%y",	i2u(p_ptr->au), END  );
-	assert(i == boundaries[2].page_rows);
-	return ret;
-  }
-  case 3:
-  {
-	int i = 0;
-	assert(ret >= boundaries[3].page_rows);
-	ret = boundaries[3].page_rows;
-	P_I(TERM_L_BLUE, "Armor", "[%y,%+y]",	i2u(p_ptr->state.dis_ac), i2u(p_ptr->state.dis_to_a)  );
-	P_I(TERM_L_BLUE, "Fight", "(%+y,%+y)",	i2u(p_ptr->state.dis_to_h), i2u(p_ptr->state.dis_to_d)  );
-	P_I(TERM_L_BLUE, "Melee", "%y",		s2u(show_melee_weapon(&p_ptr->inventory[INVEN_WIELD])), END  );
-	P_I(TERM_L_BLUE, "Shoot", "%y",		s2u(show_missile_weapon(&p_ptr->inventory[INVEN_BOW])), END  );
-	P_I(TERM_L_BLUE, "Blows", "%y.%y/turn",	i2u(p_ptr->state.num_blows / 100), i2u((p_ptr->state.num_blows / 10) % 10) );
-	P_I(TERM_L_BLUE, "Shots", "%y/turn",	i2u(p_ptr->state.num_shots), END  );
-	P_I(TERM_L_BLUE, "Infra", "%y ft",	i2u(p_ptr->state.see_infra * 10), END  );
-	P_I(TERM_L_BLUE, "Speed", "%y",		s2u(show_speed()), END );
-	P_I(TERM_L_BLUE, "Burden","%.1y lbs",	f2u(p_ptr->total_weight/10.0), END  );
-	assert(i == boundaries[3].page_rows);
-	return ret;
-  }
-  case 4:
-  {
-	static struct {
-		const char *name;
-		int skill;
-		int div;
-	} skills[] =
-	{
-		{ "Saving Throw", SKILL_SAVE, 6 },
-		{ "Stealth", SKILL_STEALTH, 1 },
-		{ "Fighting", SKILL_TO_HIT_MELEE, 12 },
-		{ "Shooting", SKILL_TO_HIT_BOW, 12 },
-		{ "Disarming", SKILL_DISARM, 8 },
-		{ "Magic Device", SKILL_DEVICE, 6 },
-		{ "Perception", SKILL_SEARCH_FREQUENCY, 6 },
-		{ "Searching", SKILL_SEARCH, 6 }
-	};
-	int i;
-	assert(N_ELEMENTS(skills) == boundaries[4].page_rows);
-	ret = N_ELEMENTS(skills);
-	if ((u32b) ret > size) ret = size;
-	for (i = 0; i < ret; i++)
-	{
-		s16b skill = p_ptr->state.skills[skills[i].skill];
-		panel[i].color = TERM_L_BLUE;
-		panel[i].label = skills[i].name;
-		if (skills[i].skill == SKILL_SAVE ||
-				skills[i].skill == SKILL_SEARCH)
-		{
-			if (skill < 0) skill = 0;
-			if (skill > 100) skill = 100;
-			panel[i].fmt = "%y%%";
-			panel[i].value[0] = i2u(skill);
-			panel[i].color = colour_table[skill / 10];
-		}
-		else if (skills[i].skill == SKILL_DEVICE)
-		{
-			panel[i].fmt = "%y";
-			panel[i].value[0] = i2u(skill);
-			panel[i].color = colour_table[skill / 13];
-		}
-		else if (skills[i].skill == SKILL_SEARCH_FREQUENCY)
-		{
-			if (skill <= 0) skill = 1;
-			if (skill >= 50)
-			{
-				panel[i].fmt = "1 in 1";
-				panel[i].color = colour_table[10];
-			}
-			else
-			{
-				/* convert to % chance of searching */
-				skill = 50 - skill;
-				panel[i].fmt = "1 in %y";
-				panel[i].value[0] = i2u(skill);
-				panel[i].color =
-					colour_table[(100 - skill*2) / 10];
-			}
-		}
-		else if (skills[i].skill == SKILL_DISARM)
-		{
-			/* assume disarming a dungeon trap */
-			skill -= 5;
-			if (skill > 100) skill = 100;
-			if (skill < 2) skill = 2;
-			panel[i].fmt = "%y%%";
-			panel[i].value[0] = i2u(skill);
-			panel[i].color = colour_table[skill / 10];
-		}
-		else
-		{
-			panel[i].fmt = "%y";
-			panel[i].value[0] = s2u(likert(skill, skills[i].div, &panel[i].color));
-		}
-	}
-	return ret;
-  }
-  case 5:
-  {
-	int i = 0;
-	assert(ret >= boundaries[5].page_rows);
-	ret = boundaries[5].page_rows;
-	P_I(TERM_L_BLUE, "Age",			"%y",	i2u(p_ptr->age), END );
-	P_I(TERM_L_BLUE, "Height",		"%y",	i2u(p_ptr->ht), END  );
-	P_I(TERM_L_BLUE, "Weight",		"%y",	i2u(p_ptr->wt), END  );
-	P_I(TERM_L_BLUE, "Social",		"%y",	s2u(show_status()), END  );
-	P_I(TERM_L_BLUE, "Maximize",	"%y",	c2u(OPT(birth_maximize) ? 'Y' : 'N'), END);
-#if 0
-	/* Preserve mode deleted */
-	P_I(TERM_L_BLUE, "Preserve",	"%y",	c2u(birth_preserve ? 'Y' : 'N'), END);
-#endif
-	assert(i == boundaries[5].page_rows);
-	return ret;
-  }
- }
-	/* hopefully not reached */
-	return 0;
+
+static struct panel *get_panel_topleft(void) {
+	struct panel *p = panel_allocate(7);
+
+	panel_line(p, TERM_L_BLUE, "Name", "%s", op_ptr->full_name);
+	panel_line(p, TERM_L_BLUE, "Sex", "%s", p_ptr->sex->title);
+	panel_line(p, TERM_L_BLUE, "Race",	"%s", p_ptr->race->name);
+	panel_line(p, TERM_L_BLUE, "Class", "%s", p_ptr->class->name);
+	panel_line(p, TERM_L_BLUE, "Title", "%s", show_title());
+	panel_line(p, TERM_L_BLUE, "HP", "%d/%d", p_ptr->chp, p_ptr->mhp);
+	panel_line(p, TERM_L_BLUE, "SP", "%d/%d", p_ptr->csp, p_ptr->msp);
+
+	return p;
 }
 
+static struct panel *get_panel_midleft(void) {
+	struct panel *p = panel_allocate(9);
+
+	panel_line(p, max_color(p_ptr->lev, p_ptr->max_lev),
+			"Level", "%d", p_ptr->lev);
+	panel_line(p, max_color(p_ptr->exp, p_ptr->max_exp),
+			"Cur Exp", "%d", p_ptr->exp);
+	panel_line(p, TERM_L_GREEN, "Max Exp", "%d", p_ptr->max_exp);
+	panel_line(p, TERM_L_GREEN, "Adv Exp", "%s", show_adv_exp());
+	panel_space(p);
+	panel_line(p, TERM_L_GREEN, "Gold", "%d", p_ptr->au);
+	panel_line(p, TERM_L_GREEN, "Burden", "%.1f lbs",
+			p_ptr->total_weight / 10.0F);
+	panel_line(p, TERM_L_GREEN, "Speed", "%s", show_speed());
+	panel_line(p, TERM_L_GREEN, "Max Depth", "%s", show_depth());
+
+	return p;
+}
+
+static struct panel *get_panel_combat(void) {
+	struct panel *p = panel_allocate(9);
+	struct object *obj;
+	int bth, dam, hit;
+	int melee_dice = 1, melee_sides = 1;
+
+	/* AC */
+	panel_line(p, TERM_L_BLUE, "Armor", "[%d,%+d]",
+			p_ptr->state.dis_ac, p_ptr->state.dis_to_a);
+
+	/* Melee */
+	obj = &p_ptr->inventory[INVEN_WIELD];
+	bth = (p_ptr->state.skills[SKILL_TO_HIT_MELEE] * 10) / BTH_PLUS_ADJ;
+	dam = p_ptr->state.dis_to_d + (object_attack_plusses_are_visible(obj) ? obj->to_d : 0);
+	hit = p_ptr->state.dis_to_h + (object_attack_plusses_are_visible(obj) ? obj->to_h : 0);
+
+	panel_space(p);
+
+	if (obj->kind) {
+		melee_dice = obj->dd;
+		melee_sides = obj->ds;
+	}
+
+	panel_line(p, TERM_L_BLUE, "Melee", "%dd%d,%+d", melee_dice, melee_sides, dam);
+	panel_line(p, TERM_L_BLUE, "To-hit", "%d,%+d", bth / 10, hit);
+	panel_line(p, TERM_L_BLUE, "Blows", "%d.%d/turn",
+			p_ptr->state.num_blows / 100, (p_ptr->state.num_blows / 10 % 10));
+
+	/* Ranged */
+	obj = &p_ptr->inventory[INVEN_BOW];
+	bth = (p_ptr->state.skills[SKILL_TO_HIT_BOW] * 10) / BTH_PLUS_ADJ;
+	hit = p_ptr->state.dis_to_h + (object_attack_plusses_are_visible(obj) ? obj->to_h : 0);
+	dam = object_attack_plusses_are_visible(obj) ? obj->to_d : 0;
+
+	panel_space(p);
+	panel_line(p, TERM_L_BLUE, "Shoot to-dam", "%+d", dam);
+	panel_line(p, TERM_L_BLUE, "To-hit", "%d,%+d", bth / 10, hit);
+	panel_line(p, TERM_L_BLUE, "Shots", "%d/turn", p_ptr->state.num_shots);
+
+	return p;
+}
+
+static struct panel *get_panel_skills(void) {
+	struct panel *p = panel_allocate(7);
+
+	int skill;
+	byte attr;
+	const char *desc;
+
+#define BOUND(x, min, max)		MIN(max, MAX(min, x))
+
+	/* Saving throw */
+	skill = BOUND(p_ptr->state.skills[SKILL_SAVE], 0, 100);
+	panel_line(p, colour_table[skill / 10], "Saving Throw", "%d%%", skill);
+
+	/* Stealth */
+	desc = likert(p_ptr->state.skills[SKILL_STEALTH], 1, &attr);
+	panel_line(p, attr, "Stealth", "%s", desc);
+
+	/* Disarming: -5 because we assume we're disarming a dungeon trap */
+	skill = BOUND(p_ptr->state.skills[SKILL_DISARM] - 5, 2, 100);
+	panel_line(p, colour_table[skill / 10], "Disarming", "%d%%", skill);
+
+	/* Magic devices */
+	skill = p_ptr->state.skills[SKILL_DEVICE];
+	panel_line(p, colour_table[skill / 13], "Magic Devices", "%d", skill);
+
+	/* Search frequency */
+	skill = MAX(p_ptr->state.skills[SKILL_SEARCH_FREQUENCY], 1);
+	if (skill >= 50) {
+		panel_line(p, colour_table[10], "Perception", "1 in 1");
+	} else {
+		/* convert to chance of searching */
+		skill = 50 - skill;
+		panel_line(p, colour_table[(100 - skill*2) / 10],
+				"Perception", "1 in %d", skill);
+	}
+
+	/* Searching ability */
+	skill = BOUND(p_ptr->state.skills[SKILL_SEARCH], 0, 100);
+	panel_line(p, colour_table[skill / 10], "Searching", "%d%%", skill);
+
+	/* Infravision */
+	panel_line(p, TERM_L_GREEN, "Infravision", "%d ft",
+			p_ptr->state.see_infra * 10);
+
+	return p;
+}
+
+static struct panel *get_panel_misc(void) {
+	struct panel *p = panel_allocate(7);
+	byte attr = TERM_L_BLUE;
+
+	panel_line(p, attr, "Age", "%d", p_ptr->age);
+	panel_line(p, attr, "Height", "%d'%d\"", p_ptr->ht / 12, p_ptr->ht % 12);
+	panel_line(p, attr, "Weight", "%dst %dlb", p_ptr->wt / 14, p_ptr->wt % 14);
+	panel_line(p, attr, "Turns used:", "");
+	panel_line(p, attr, "Game", "%d", turn);
+	panel_line(p, attr, "Standard", "%d", p_ptr->total_energy / 100);
+	panel_line(p, attr, "Resting", "%d", p_ptr->resting_turn);
+
+	return p;
+}
+
+/* Panels for main character screen */
+static const struct {
+	region bounds;
+	bool align_left;
+	struct panel *(*panel)(void);
+} panels[] =
+{
+	/*   x  y wid rows */
+	{ {  1, 1, 40, 7 }, TRUE,  get_panel_topleft },	/* Name, Class, ... */
+	{ { 21, 1, 18, 3 }, FALSE, get_panel_misc },	/* Age, ht, wt, ... */
+	{ {  1, 9, 24, 9 }, FALSE, get_panel_midleft },	/* Cur Exp, Max Exp, ... */
+	{ { 29, 9, 19, 9 }, FALSE, get_panel_combat },
+	{ { 52, 9, 20, 8 }, FALSE, get_panel_skills },
+};
+
 void display_player_xtra_info(void)
 {
-	int i;
-	int panels [] = { 1, 2, 3, 4, 5};
-	bool left_adj [] = { 1, 0, 0, 0, 0 };
-	data_panel data[MAX_PANEL];
-
-	for (i = 0; i < (int)N_ELEMENTS(panels); i++)
-	{
-		int oid = panels[i];
-		int rows = get_panel(oid, data, N_ELEMENTS(data));
-
-		/* Hack:  Don't show 'Level' in the name, class ...  panel */
-		if (oid == 1) rows -= 1;
-
-		display_panel(data, rows, left_adj[i], &boundaries[oid]);
+	size_t i;
+	for (i = 0; i < N_ELEMENTS(panels); i++) {
+		struct panel *p = panels[i].panel();
+		display_panel(p, panels[i].align_left, &panels[i].bounds);
+		panel_free(p);
 	}
 
 	/* Indent output by 1 character, and wrap at column 72 */
@@ -933,10 +886,9 @@ void display_player(int mode)
 
 	if (mode)
 	{
-		data_panel data[MAX_PANEL];
-		int rows = get_panel(1, data, N_ELEMENTS(data));
-
-		display_panel(data, rows, 1, &boundaries[1]);
+		struct panel *p = panels[0].panel();
+		display_panel(p, panels[0].align_left, &panels[0].bounds);
+		panel_free(p);
 
 		/* Stat/Sustain flags */
 		display_player_sust_info();
@@ -964,8 +916,8 @@ errr file_character(const char *path, bool full)
 {
 	int i, x, y;
 
-	byte a;
-	char c;
+	int a;
+	wchar_t c;
 
 	ang_file *fp;
 
@@ -973,12 +925,8 @@ errr file_character(const char *path, bool full)
 
 	char o_name[80];
 
-	byte (*old_xchar_hook)(byte c) = Term->xchar_hook;
-
 	char buf[1024];
-
-	/* We use either ascii or system-specific encoding */
- 	int encoding = OPT(xchars_to_file) ? SYSTEM_SPECIFIC : ASCII;
+	char *p;
 
 	/* Unused parameter */
 	(void)full;
@@ -988,9 +936,6 @@ errr file_character(const char *path, bool full)
 	fp = file_open(path, MODE_WRITE, FTYPE_TEXT);
 	if (!fp) return (-1);
 
-	/* Display the requested encoding -- ASCII or system-specific */
- 	if (!OPT(xchars_to_file)) Term->xchar_hook = NULL;
-
 	/* Begin dump */
 	file_putf(fp, "  [%s Character Dump]\n\n", buildid);
 
@@ -1001,6 +946,7 @@ errr file_character(const char *path, bool full)
 	/* Dump part of the screen */
 	for (y = 1; y < 23; y++)
 	{
+		p = buf;
 		/* Dump each row */
 		for (x = 0; x < 79; x++)
 		{
@@ -1008,17 +954,17 @@ errr file_character(const char *path, bool full)
 			(void)(Term_what(x, y, &a, &c));
 
 			/* Dump it */
-			buf[x] = c;
+			p += wctomb(p, c);
 		}
 
 		/* Back up over spaces */
-		while ((x > 0) && (buf[x-1] == ' ')) --x;
+		while ((p > buf) && (p[-1] == ' ')) --p;
 
 		/* Terminate */
-		buf[x] = '\0';
+		*p = '\0';
 
 		/* End the row */
-		x_file_putf(fp, encoding, "%s\n", buf);
+		file_putf(fp, "%s\n", buf);
 	}
 
 	/* Skip a line */
@@ -1030,6 +976,7 @@ errr file_character(const char *path, bool full)
 	/* Dump part of the screen */
 	for (y = 11; y < 20; y++)
 	{
+		p = buf;
 		/* Dump each row */
 		for (x = 0; x < 39; x++)
 		{
@@ -1037,17 +984,17 @@ errr file_character(const char *path, bool full)
 			(void)(Term_what(x, y, &a, &c));
 
 			/* Dump it */
-			buf[x] = c;
+			p += wctomb(p, c);
 		}
 
 		/* Back up over spaces */
-		while ((x > 0) && (buf[x-1] == ' ')) --x;
+		while ((p > buf) && (p[-1] == ' ')) --p;
 
 		/* Terminate */
-		buf[x] = '\0';
+		*p = '\0';
 
 		/* End the row */
-		x_file_putf(fp, encoding, "%s\n", buf);
+		file_putf(fp, "%s\n", buf);
 	}
 
 	/* Skip a line */
@@ -1056,6 +1003,7 @@ errr file_character(const char *path, bool full)
 	/* Dump part of the screen */
 	for (y = 11; y < 20; y++)
 	{
+		p = buf;
 		/* Dump each row */
 		for (x = 0; x < 39; x++)
 		{
@@ -1063,17 +1011,17 @@ errr file_character(const char *path, bool full)
 			(void)(Term_what(x + 40, y, &a, &c));
 
 			/* Dump it */
-			buf[x] = c;
+			p += wctomb(p, c);
 		}
 
 		/* Back up over spaces */
-		while ((x > 0) && (buf[x-1] == ' ')) --x;
+		while ((p > buf) && (p[-1] == ' ')) --p;
 
 		/* Terminate */
-		buf[x] = '\0';
+		*p = '\0';
 
 		/* End the row */
-		x_file_putf(fp, encoding, "%s\n", buf);
+		file_putf(fp, "%s\n", buf);
 	}
 
 	/* Skip some lines */
@@ -1088,9 +1036,9 @@ errr file_character(const char *path, bool full)
 		file_putf(fp, "  [Last Messages]\n\n");
 		while (i-- > 0)
 		{
-			x_file_putf(fp, encoding, "> %s\n", message_str((s16b)i));
+			file_putf(fp, "> %s\n", message_str((s16b)i));
 		}
-		x_file_putf(fp, encoding, "\nKilled by %s.\n\n", p_ptr->died_from);
+		file_putf(fp, "\nKilled by %s.\n\n", p_ptr->died_from);
 	}
 
 
@@ -1106,9 +1054,10 @@ errr file_character(const char *path, bool full)
 		object_desc(o_name, sizeof(o_name), &p_ptr->inventory[i],
 				ODESC_PREFIX | ODESC_FULL);
 
-		x_file_putf(fp, encoding, "%c) %s\n", index_to_label(i), o_name);
-		if (p_ptr->inventory[i].kind)
+		if (p_ptr->inventory[i].kind) {
+			file_putf(fp, "%c) %s\n", index_to_label(i), o_name);
 			object_info_chardump(fp, &p_ptr->inventory[i], 5, 72);
+		}
 	}
 
 	/* Dump the inventory */
@@ -1120,7 +1069,7 @@ errr file_character(const char *path, bool full)
 		object_desc(o_name, sizeof(o_name), &p_ptr->inventory[i],
 					ODESC_PREFIX | ODESC_FULL);
 
-		x_file_putf(fp, encoding, "%c) %s\n", index_to_label(i), o_name);
+		file_putf(fp, "%c) %s\n", index_to_label(i), o_name);
 		object_info_chardump(fp, &p_ptr->inventory[i], 5, 72);
 	}
 	file_putf(fp, "\n\n");
@@ -1137,7 +1086,7 @@ errr file_character(const char *path, bool full)
 		{
 			object_desc(o_name, sizeof(o_name), &st_ptr->stock[i],
 						ODESC_PREFIX | ODESC_FULL);
-			x_file_putf(fp, encoding, "%c) %s\n", I2A(i), o_name);
+			file_putf(fp, "%c) %s\n", I2A(i), o_name);
 
 			object_info_chardump(fp, &st_ptr->stock[i], 5, 72);
 		}
@@ -1154,23 +1103,29 @@ errr file_character(const char *path, bool full)
 	file_putf(fp, "  [Options]\n\n");
 
 	/* Dump options */
-	for (i = OPT_BIRTH; i < OPT_BIRTH + N_OPTS_BIRTH; i++)
-	{
-		if (option_name(i))
-		{
-			file_putf(fp, "%-45s: %s (%s)\n",
-			        option_desc(i),
-			        op_ptr->opt[i] ? "yes" : "no ",
-			        option_name(i));
+	for (i = 0; i < OPT_PAGE_MAX - 1; i++) {
+		int j;
+		const char *title = "";
+		switch (i) {
+			case 0: title = "User interface"; break;
+			case 1: title = "Birth"; break;
 		}
+
+		file_putf(fp, "  [%s]\n\n", title);
+		for (j = 0; j < OPT_PAGE_PER; j++) {
+			int opt = option_page[i][j];
+			if (!option_name(opt)) continue;
+
+			file_putf(fp, "%-45s: %s (%s)\n",
+			        option_desc(opt),
+			        op_ptr->opt[opt] ? "yes" : "no ",
+			        option_name(opt));
+		}
+
+		/* Skip some lines */
+		file_putf(fp, "\n");
 	}
 
-	/* Skip some lines */
-	file_putf(fp, "\n\n");
-
-	/* Return to standard display */
- 	Term->xchar_hook = old_xchar_hook;
-
 	file_close(fp);
 
 
@@ -1255,6 +1210,9 @@ bool show_file(const char *name, const char *what, int line, int mode)
 	char hook[26][32];
 
 	int wid, hgt;
+	
+	/* TRUE if we are inside a RST block that should be skipped */
+	bool skip_lines = FALSE;
 
 
 
@@ -1283,14 +1241,13 @@ bool show_file(const char *name, const char *what, int line, int mode)
 	/* Redirect the name */
 	name = filename;
 
-
 	/* Hack XXX XXX XXX */
 	if (what)
 	{
 		my_strcpy(caption, what, sizeof(caption));
 
 		my_strcpy(path, name, sizeof(path));
-		fff = file_open(path, MODE_READ, -1);
+		fff = file_open(path, MODE_READ, FTYPE_TEXT);
 	}
 
 	/* Look in "help" */
@@ -1299,7 +1256,7 @@ bool show_file(const char *name, const char *what, int line, int mode)
 		strnfmt(caption, sizeof(caption), "Help file '%s'", name);
 
 		path_build(path, sizeof(path), ANGBAND_DIR_HELP, name);
-		fff = file_open(path, MODE_READ, -1);
+		fff = file_open(path, MODE_READ, FTYPE_TEXT);
 	}
 
 	/* Look in "info" */
@@ -1308,7 +1265,7 @@ bool show_file(const char *name, const char *what, int line, int mode)
 		strnfmt(caption, sizeof(caption), "Info file '%s'", name);
 
 		path_build(path, sizeof(path), ANGBAND_DIR_INFO, name);
-		fff = file_open(path, MODE_READ, -1);
+		fff = file_open(path, MODE_READ, FTYPE_TEXT);
 	}
 
 	/* Oops */
@@ -1329,27 +1286,33 @@ bool show_file(const char *name, const char *what, int line, int mode)
 		/* Read a line or stop */
 		if (!file_getl(fff, buf, sizeof(buf))) break;
 
-		/* XXX Parse "menu" items */
-		if (prefix(buf, "***** "))
-		{
-			char b1 = '[', b2 = ']';
+		/* Skip lines if we are inside a RST directive*/
+		if(skip_lines){
+			if(contains_only_spaces(buf))
+				skip_lines=FALSE;
+			continue;
+		}
 
-			/* Notice "menu" requests */
-			if ((buf[6] == b1) && isalpha((unsigned char)buf[7]) &&
-			    (buf[8] == b2) && (buf[9] == ' '))
+		/* Parse a very small subset of RST */
+		/* TODO: should be more flexible */
+		if (prefix(buf, ".. "))
+		{
+			/* parse ".. menu:: [x] filename.txt" (with exact spacing)*/
+			if(prefix(buf+strlen(".. "), "menu:: [") && 
+                           buf[strlen(".. menu:: [x")]==']')
 			{
 				/* This is a menu file */
 				menu = TRUE;
 
 				/* Extract the menu item */
-				k = A2I(buf[7]);
+				k = A2I(buf[strlen(".. menu:: [")]);
 
 				/* Store the menu item (if valid) */
 				if ((k >= 0) && (k < 26))
-					my_strcpy(hook[k], buf + 10, sizeof(hook[0]));
+					my_strcpy(hook[k], buf + strlen(".. menu:: [x] "), sizeof(hook[0]));
 			}
-			/* Notice "tag" requests */
-			else if (buf[6] == '<')
+			/* parse ".. _some_hyperlink_target:" */
+			else if (buf[strlen(".. ")] == '_')
 			{
 				if (tag)
 				{
@@ -1357,7 +1320,7 @@ bool show_file(const char *name, const char *what, int line, int mode)
 					buf[strlen(buf) - 1] = '\0';
 
 					/* Compare with the requested tag */
-					if (streq(buf + 7, tag))
+					if (streq(buf + strlen(".. _"), tag))
 					{
 						/* Remember the tagged line */
 						line = next;
@@ -1365,7 +1328,8 @@ bool show_file(const char *name, const char *what, int line, int mode)
 				}
 			}
 
-			/* Skip this */
+			/* Skip this and enter skip mode*/
+			skip_lines = TRUE;
 			continue;
 		}
 
@@ -1377,7 +1341,6 @@ bool show_file(const char *name, const char *what, int line, int mode)
 	size = next;
 
 
-
 	/* Display the file */
 	while (TRUE)
 	{
@@ -1389,7 +1352,7 @@ bool show_file(const char *name, const char *what, int line, int mode)
 		if (line > (size - (hgt - 4))) line = size - (hgt - 4);
 		if (line < 0) line = 0;
 
-
+		skip_lines = FALSE;
 		/* Re-open the file if needed */
 		if (next > line)
 		{
@@ -1397,7 +1360,7 @@ bool show_file(const char *name, const char *what, int line, int mode)
 			file_close(fff);
 
 			/* Hack -- Re-Open the file */
-			fff = file_open(path, MODE_READ, -1);
+			fff = file_open(path, MODE_READ, FTYPE_TEXT);
 			if (!fff) return (TRUE);
 
 			/* File has been restarted */
@@ -1411,8 +1374,19 @@ bool show_file(const char *name, const char *what, int line, int mode)
 			/* Get a line */
 			if (!file_getl(fff, buf, sizeof(buf))) break;
 
-			/* Skip tags/links */
-			if (prefix(buf, "***** ")) continue;
+			/* Skip lines if we are inside a RST directive*/
+			if(skip_lines){
+				if(contains_only_spaces(buf))
+					skip_lines=FALSE;
+				continue;
+			}
+
+			/* Skip RST directives */
+			if (prefix(buf, ".. "))
+			{
+				skip_lines=TRUE;
+				continue;
+			}
 
 			/* Count the lines */
 			next++;
@@ -1428,8 +1402,25 @@ bool show_file(const char *name, const char *what, int line, int mode)
 			/* Get a line of the file or stop */
 			if (!file_getl(fff, buf, sizeof(buf))) break;
 
-			/* Hack -- skip "special" lines */
-			if (prefix(buf, "***** ")) continue;
+			/* Skip lines if we are inside a RST directive*/
+			if(skip_lines){
+				if(contains_only_spaces(buf))
+					skip_lines=FALSE;
+				continue;
+			}
+
+			/* Skip RST directives */
+			if (prefix(buf, ".. "))
+			{
+				skip_lines=TRUE;
+				continue;
+			}
+
+			/* skip | characters */
+			strskip(buf,'|');
+
+			/* escape backslashes */
+			strescape(buf,'\\');
 
 			/* Count the "real" lines */
 			next++;
@@ -1490,7 +1481,7 @@ bool show_file(const char *name, const char *what, int line, int mode)
 		if (menu)
 		{
 			/* Wait for it */
-			prt("[Press a Number, or ESC to exit.]", hgt - 1, 0);
+			prt("[Press a Letter, or ESC to exit.]", hgt - 1, 0);
 		}
 
 		/* Prompt -- small files */
@@ -1590,8 +1581,7 @@ bool show_file(const char *name, const char *what, int line, int mode)
 			/* down a line */
 			case ARROW_DOWN:
 			case '2':
-			case '\n':
-			case '\r': line++; break;
+			case KC_ENTER: line++; break;
 
 			/* down a page */
 			case KC_PGDOWN:
@@ -1645,78 +1635,6 @@ void do_cmd_help(void)
 }
 
 
-
-/*
- * Process the player name and extract a clean "base name".
- *
- * If "sf" is TRUE, then we initialize "savefile" based on player name.
- *
- * Some platforms (Windows, Macintosh, Amiga) leave the "savefile" empty
- * when a new character is created, and then when the character is done
- * being created, they call this function to choose a new savefile name.
- *
- * This also now handles the turning on and off of the automatic
- * sequential numbering of character names with Roman numerals.  
- */
-void process_player_name(bool sf)
-{
-	int i;
-
-	/* Process the player name */
-	for (i = 0; op_ptr->full_name[i]; i++)
-	{
-		char c = op_ptr->full_name[i];
-
-		/* No control characters */
-		if (iscntrl((unsigned char)c))
-		{
-			/* Illegal characters */
-			quit_fmt("Illegal control char (0x%02X) in player name", c);
-		}
-
-		/* Convert all non-alphanumeric symbols */
-		if (!isalpha((unsigned char)c) && !isdigit((unsigned char)c)) c = '_';
-
-		/* Build "base_name" */
-		op_ptr->base_name[i] = c;
-	}
-
-#if defined(WINDOWS)
-
-	/* Max length */
-	if (i > 8) i = 8;
-
-#endif
-
-	/* Terminate */
-	op_ptr->base_name[i] = '\0';
-
-	/* Require a "base" name */
-	if (!op_ptr->base_name[0])
-	{
-		my_strcpy(op_ptr->base_name, "PLAYER", sizeof(op_ptr->base_name));
-	}
-
-
-	/* Pick savefile name if needed */
-	if (sf)
-	{
-		char temp[128];
-
-#if defined(SET_UID)
-		/* Rename the savefile, using the player_uid and base_name */
-		strnfmt(temp, sizeof(temp), "%d.%s", player_uid, op_ptr->base_name);
-#else
-		/* Rename the savefile, using the base name */
-		strnfmt(temp, sizeof(temp), "%s", op_ptr->base_name);
-#endif
-
-		/* Build the filename */
-		path_build(savefile, sizeof(savefile), ANGBAND_DIR_SAVE, temp);
-	}
-}
-
-
 /*
  * Save the game
  */
@@ -1820,138 +1738,3 @@ void close_game(void)
 	/* Allow suspending now */
 	signals_handle_tstp();
 }
-
-static void write_html_escape_char(ang_file *fp, char c)
-{
-	switch (c)
-	{
-		case '<':
-			file_putf(fp, "&lt;");
-			break;
-		case '>':
-			file_putf(fp, "&gt;");
-			break;
-		case '&':
-			file_putf(fp, "&amp;");
-			break;
-		default:
-			file_putf(fp, "%c", c);
-			break;
-	}
-}
-
-
-/* Take an html screenshot */
-void html_screenshot(const char *name, int mode)
-{
-	int y, x;
-	int wid, hgt;
-
-	byte a = TERM_WHITE;
-	byte oa = TERM_WHITE;
-	char c = ' ';
-
-	const char *new_color_fmt = (mode == 0) ?
-					"<font color=\"#%02X%02X%02X\">"
-				 	: "[COLOR=\"#%02X%02X%02X\"]";
-	const char *change_color_fmt = (mode == 0) ?
-					"</font><font color=\"#%02X%02X%02X\">"
-					: "[/COLOR][COLOR=\"#%02X%02X%02X\"]";
-	const char *close_color_fmt = mode ==  0 ? "</font>" : "[/COLOR]";
-
-	ang_file *fp;
-	char buf[1024];
-
-
-	path_build(buf, sizeof(buf), ANGBAND_DIR_USER, name);
-	fp = file_open(buf, MODE_WRITE, FTYPE_TEXT);
-
-	/* Oops */
-	if (!fp)
-	{
-		plog_fmt("Cannot write the '%s' file!", buf);
-		return;
-	}
-
-	/* Retrieve current screen size */
-	Term_get_size(&wid, &hgt);
-
-	if (mode == 0)
-	{
-		file_putf(fp, "<!DOCTYPE html><html><head>\n");
-		file_putf(fp, "  <meta='generator' content='%s'>\n", buildid);
-		file_putf(fp, "  <title>%s</title>\n", name);
-		file_putf(fp, "</head>\n\n");
-		file_putf(fp, "<body style='color: #fff; background: #000;'>\n");
-		file_putf(fp, "<pre>\n");
-	}
-	else 
-	{
-		file_putf(fp, "[CODE][TT][BC=black][COLOR=white]\n");
-	}
-
-	/* Dump the screen */
-	for (y = 0; y < hgt; y++)
-	{
-		for (x = 0; x < wid; x++)
-		{
-			/* Get the attr/char */
-			(void)(Term_what(x, y, &a, &c));
-
-			/* Color change */
-			if (oa != a && c != ' ')
-			{
-				/* From the default white to another color */
-				if (oa == TERM_WHITE)
-				{
-					file_putf(fp, new_color_fmt,
-					        angband_color_table[a][1],
-					        angband_color_table[a][2],
-					        angband_color_table[a][3]);
-				}
-
-				/* From another color to the default white */
-				else if (a == TERM_WHITE)
-				{
-					file_putf(fp, close_color_fmt);
-				}
-
-				/* Change colors */
-				else
-				{
-					file_putf(fp, change_color_fmt,
-					        angband_color_table[a][1],
-					        angband_color_table[a][2],
-					        angband_color_table[a][3]);
-				}
-
-				/* Remember the last color */
-				oa = a;
-			}
-
-			/* Write the character and escape special HTML characters */
-			if (mode == 0) write_html_escape_char(fp, c);
-			else file_putf(fp, "%c", c);
-		}
-
-		/* End the row */
-		file_putf(fp, "\n");
-	}
-
-	/* Close the last font-color tag if necessary */
-	if (oa != TERM_WHITE) file_putf(fp, close_color_fmt);
-
-	if (mode == 0)
-	{
-		file_putf(fp, "</pre>\n");
-		file_putf(fp, "</body>\n");
-		file_putf(fp, "</html>\n");
-	}
-	else 
-	{
-		file_putf(fp, "[/COLOR][/BC][/TT][/CODE]\n");
-	}
-
-	/* Close it */
-	file_close(fp);
-}
