@@ -24,15 +24,14 @@ EFFECT(CURE_POISON,    FALSE,  1,	"neutralizes poison")
 EFFECT(CURE_BLINDNESS, FALSE,  4,	"cures blindness")
 EFFECT(CURE_PARANOIA,  FALSE,  2,	"removes your fear")
 EFFECT(CURE_CONFUSION, FALSE,  4,	"cures confusion")
-EFFECT(CURE_MIND,      FALSE,  8,	"cures confusion and hallucination, removes fear and grants you temporary resistance to confusion")
-EFFECT(CURE_BODY,      FALSE,  7,	"heals cut damage, and cures stunning, poison and blindness")
-
+EFFECT(CURE_MIND,      FALSE,  8,	"restores 10SP, cures confusion and hallucination, removes fear and grants you temporary resistance to confusion")
+EFFECT(CURE_BODY,      FALSE,  7,	"heals 30 hitpoints, cut damage, and cures stunning, poison and blindness")
 EFFECT(CURE_LIGHT,     FALSE,  3,	"heals 20 hitpoints, some cut damage, makes you a little less confused, and cures blindness")
 EFFECT(CURE_SERIOUS,   FALSE,  6,	"heals 40 hitpoints, cut damage, and cures blindness and confusion")
 EFFECT(CURE_CRITICAL,  FALSE,  9,	"heals 60 hitpoints, cut damage, and cures stunning, poisoning, blindness, and confusion")
-EFFECT(CURE_FULL,      FALSE, 12,	"heals you a really large amount (35% of max HP, minimum 300HP), heals cut damage, and cures stunning, poisoning, blindness, and confusion")
-EFFECT(CURE_FULL2,     FALSE, 18,	"restores 1200 hit points, heals cut damage, and cures stunning, poisoning, blindness, and confusion")
-EFFECT(CURE_NONORLYBIG,FALSE, 21,	"restores 5000 hit points, restores experience and stats, heals cut damage, and cures stunning, poison, blindness, and confusion")
+EFFECT(CURE_FULL,      FALSE, 12,	"heals 35% of max HP (minimum 300HP), cut damage, and cures stunning, poisoning, blindness, and confusion")
+EFFECT(CURE_FULL2,     FALSE, 18,	"heals 1200 hit points, cut damage, and cures stunning, poisoning, blindness, and confusion")
+EFFECT(CURE_NONORLYBIG,FALSE, 21,	"heals 5000 hit points, restores experience and stats, heals cut damage, and cures stunning, poison, blindness, and confusion")
 EFFECT(CURE_TEMP,      FALSE,  9,	"heals cut damage, and cures all stunning, poison, blindness and confusion")
 EFFECT(HEAL1,          FALSE, 13,	"heals 500 hit points")
 EFFECT(HEAL2,          FALSE, 16,	"heals 1000 hit points")
@@ -49,27 +48,23 @@ EFFECT(GAIN_INT,       FALSE, INHIBIT_POWER, "restores and increases your intell
 EFFECT(GAIN_WIS,       FALSE, INHIBIT_POWER, "restores and increases your wisdom")
 EFFECT(GAIN_DEX,       FALSE, INHIBIT_POWER, "restores and increases your dexterity")
 EFFECT(GAIN_CON,       FALSE, INHIBIT_POWER, "restores and increases your constitution")
-EFFECT(GAIN_CHR,       FALSE, INHIBIT_POWER, "restores and increases your charisma")
 EFFECT(GAIN_ALL,       FALSE, INHIBIT_POWER, "restores and increases all your stats")
 EFFECT(BRAWN,          FALSE, 30,	"raises your strength at the expense of a random attribute")
 EFFECT(INTELLECT,      FALSE, 25,	"raises your intelligence at the expense of a random attribute")
 EFFECT(CONTEMPLATION,  FALSE, 25,	"raises your wisdom at the expense of a random attribute")
 EFFECT(TOUGHNESS,      FALSE, 30,	"raises your constitution at the expense of a random attribute")
 EFFECT(NIMBLENESS,     FALSE, 25,	"raises your dexterity at the expense of a random attribute")
-EFFECT(PLEASING,       FALSE,  5,	"raises your charisma at the expense of a random attribute")
 EFFECT(LOSE_STR,       FALSE,  0,	"reduces your strength with damage 5d5")
 EFFECT(LOSE_INT,       FALSE,  0,	"reduces your intelligence with damage 5d5")
 EFFECT(LOSE_WIS,       FALSE,  0,	"reduces your wisdom with damage 5d5")
 EFFECT(LOSE_DEX,       FALSE,  0,	"reduces your dexterity with damage 5d5")
 EFFECT(LOSE_CON,       FALSE,  0,	"reduces your constitution with damage 5d5")
-EFFECT(LOSE_CHR,       FALSE,  0,	"reduces your intelligence with damage 5d5")
 EFFECT(LOSE_CON2,      FALSE,  0,	"reduces your constitution with damage 10d10")
 EFFECT(RESTORE_STR,    FALSE, 10,	"restores your strength")
 EFFECT(RESTORE_INT,    FALSE,  8,	"restores your intelligence")
 EFFECT(RESTORE_WIS,    FALSE,  8,	"restores your wisdom")
 EFFECT(RESTORE_DEX,    FALSE,  9,	"restores your dexterity")
 EFFECT(RESTORE_CON,    FALSE, 10,	"restores your constitution")
-EFFECT(RESTORE_CHR,    FALSE,  4,	"restores your charisma")
 EFFECT(RESTORE_ALL,    FALSE, 15,	"restores all your stats")
 
 EFFECT(RESTORE_ST_LEV, FALSE, 17,	"restores all your stats and your experience points")
@@ -98,14 +93,14 @@ EFFECT(DETECT_INVIS,      FALSE,  6,	"detects invisible creatures nearby")
 EFFECT(DETECT_EVIL,       FALSE,  6,	"detects evil creatures nearby")
 EFFECT(DETECT_ALL,        FALSE, 10,	"detects treasure, traps, doors, stairs, and all creatures nearby")
 
-EFFECT(ENCHANT_TOHIT,  FALSE, 15,	"attempts to magically enhance a weapon's to-hit bonus")
-EFFECT(ENCHANT_TODAM,  FALSE, 20,	"attempts to magically enhance a weapon's to-dam bonus")
-EFFECT(ENCHANT_WEAPON, FALSE, 22,	"attempts to magically enhance a weapon both to-hit and to-dam")
-EFFECT(ENCHANT_ARMOR,  FALSE, 12,	"attempts to magically enhance a piece of armour")
-EFFECT(ENCHANT_ARMOR2, FALSE, 15,	"attempts to magically enhance a piece of armour with high chance of success")
+EFFECT(ENCHANT_TOHIT,  FALSE, 15,	"attempts to magically enhance a weapon's to-hit bonus.  Also gives a chance to break a curse")
+EFFECT(ENCHANT_TODAM,  FALSE, 20,	"attempts to magically enhance a weapon's to-dam bonus.  Also gives a chance to break a curse")
+EFFECT(ENCHANT_WEAPON, FALSE, 22,	"attempts to magically enhance a weapon both to-hit and to-dam.  Also gives a chance to break a curse")
+EFFECT(ENCHANT_ARMOR,  FALSE, 12,	"attempts to magically enhance a piece of armour.  Also gives a chance to break a curse")
+EFFECT(ENCHANT_ARMOR2, FALSE, 15,	"attempts to magically enhance a piece of armour with high chance of success.  Also gives a chance to break a curse")
 EFFECT(RESTORE_ITEM,   FALSE, 10,	"restores an item after disenchantment or damage")
 EFFECT(IDENTIFY,       FALSE,  9,	"reveals to you the extent of an item's magical powers")
-EFFECT(REMOVE_CURSE,   FALSE,  8,	"removes all ordinary curses from all equipped items")
+EFFECT(REMOVE_CURSE,   FALSE, INHIBIT_POWER,	"removes all ordinary curses from all equipped items")
 EFFECT(REMOVE_CURSE2,  FALSE, 20,	"removes all curses from all equipped items")
 EFFECT(LIGHT,          FALSE,  4,	"lights up an area and inflicts 2d8 damage on light-sensitive creatures")
 EFFECT(SUMMON_MON,     FALSE,  0,	"summons monsters at the current dungeon level")
@@ -133,7 +128,7 @@ EFFECT(BLESSING,       FALSE,  6,	"increases your AC and to-hit bonus for 1d12+6
 EFFECT(BLESSING2,      FALSE,  7,	"increases your AC and to-hit bonus for 1d24+12 turns")
 EFFECT(BLESSING3,      FALSE,  8,	"increases your AC and to-hit bonus for 1d48+24 turns")
 EFFECT(RECALL,         FALSE, 15,	"returns you from the dungeon or takes you to the dungeon after a short delay")
-EFFECT(DEEP_DESCENT,   FALSE, 19,	"teleports you two levels down")
+EFFECT(DEEP_DESCENT,   FALSE, 19,	"teleports you up to five dungeon levels lower than the lowest point you have reached so far")
 
 EFFECT(EARTHQUAKES,    FALSE,  5,	"causes an earthquake around you")
 EFFECT(DESTRUCTION2,   FALSE, 12,	"destroys an area around you in the shape of a circle radius 15, and blinds you for 1d10+10 turns")
@@ -184,10 +179,10 @@ EFFECT(ELEC_BOLT,      TRUE,   5,	"creates a lightning bolt (that always beams)
 EFFECT(ELEC_BALL,      TRUE,   9,	"creates a lightning ball with damage 64")
 EFFECT(ELEC_BALL2,     TRUE,  14,	"creates a large lightning ball with damage 250")
 
-EFFECT(DRAIN_LIFE1,    TRUE,   9,	"drains up to 90 hit points of life from a target creature")
-EFFECT(DRAIN_LIFE2,    TRUE,  10,	"drains up to 120 hit points of life from a target creature")
-EFFECT(DRAIN_LIFE3,    TRUE,  11,	"drains up to 150 hit points of life from a target creature")
-EFFECT(DRAIN_LIFE4,    TRUE,  12,	"drains up to 250 hit points of life from a target creature")
+EFFECT(DRAIN_LIFE1,    TRUE,   9,	"drains up to 90 hit points of life from a target creature.  Does not affect undead, demons, golems, elementals or vortices")
+EFFECT(DRAIN_LIFE2,    TRUE,  10,	"drains up to 120 hit points of life from a target creature.  Does not affect undead, demons, golems, elementals or vortices")
+EFFECT(DRAIN_LIFE3,    TRUE,  11,	"drains up to 150 hit points of life from a target creature.  Does not affect undead, demons, golems, elementals or vortices")
+EFFECT(DRAIN_LIFE4,    TRUE,  12,	"drains up to 250 hit points of life from a target creature.  Does not affect undead, demons, golems, elementals or vortices")
 EFFECT(MISSILE,        TRUE,   3,	"fires a magic missile with damage 3d4")
 EFFECT(MANA_BOLT,      TRUE,   7,	"fires a mana bolt with damage 12d8")
 EFFECT(BIZARRE,        TRUE,  20,	"does bizarre things")
@@ -218,7 +213,7 @@ EFFECT(WONDER,         TRUE,   9,	"creates random and unpredictable effects")
 
 EFFECT(WAND_BREATH,    TRUE,  12,	"shoots a large ball of one of the base elements for 120-200 damage")
 EFFECT(STAFF_MAGI,     FALSE, 20,	"restores both intelligence and manapoints to maximum")
-EFFECT(STAFF_HOLY,     FALSE, 12,	"inflicts damage on evil creatures you can see, cures 50 hit points, heals all temporary effects and grants you protection from evil")
+EFFECT(STAFF_HOLY,     FALSE, 12,	"inflicts damage on evil creatures you can see, heals 50 hit points, cures all temporary negative effects and grants you protection from evil")
 EFFECT(DRINK_GOOD,     FALSE,  0,	NULL)
 EFFECT(DRINK_BREATH,   TRUE,   8,	"causes you to breathe either cold or flames for 80 damage")
 EFFECT(DRINK_SALT,     FALSE,  0,	"induces vomiting and paralysis for 4 turns, resulting in severe hunger but also curing poison")
@@ -227,6 +222,10 @@ EFFECT(DRINK_RUIN,     FALSE,  0,	"inflicts 10d10 points of damage and decreases
 EFFECT(DRINK_DETONATE, FALSE,  0,	"inflicts 50d20 points of damage, severe cuts, and stunning")
 EFFECT(FOOD_GOOD,      FALSE,  0,	NULL)
 EFFECT(FOOD_WAYBREAD,  FALSE,  4,	"restores 4d8 hit points and neutralizes poison")
+EFFECT(FOOD_CRUNCH,    FALSE,  0,   NULL)
+EFFECT(FOOD_WHISKY,    FALSE,  0,   NULL)
+EFFECT(FOOD_WINE,      FALSE,  0,   "makes you fearless and foolhardy, but less co-ordinated")
+
 EFFECT(SHROOM_EMERGENCY, FALSE, 7,	"grants temporary resistance to fire and cold, cures 200HP, but also makes you hallucinate wildly")
 EFFECT(SHROOM_TERROR,    FALSE, 5,	"speeds up you temporarily but also makes you mortally afraid")
 EFFECT(SHROOM_STONE,     FALSE, 5,	"turns your skin to stone briefly, which grants an extra 40AC but slows you down")
@@ -238,12 +237,12 @@ EFFECT(RING_FLAMES,    TRUE,  11,	"grants fire resistance for d20+20 turns and c
 EFFECT(RING_ICE,       TRUE,  11,	"grants cold resistance for d20+20 turns and creates a cold ball of damage 75")
 EFFECT(RING_LIGHTNING, TRUE,  11,	"grants electricity resistance for d20+20 turns and creates a lightning ball of damage 85")
 
-EFFECT(DRAGON_BLUE,    TRUE,   18,	"allows you to breathe lightning for 100 damage")
+EFFECT(DRAGON_BLUE,    TRUE,   18,	"allows you to breathe lightning for 150 damage")
 EFFECT(DRAGON_GREEN,   TRUE,   19,	"allows you to breathe poison gas for 150 damage")
 EFFECT(DRAGON_RED,     TRUE,   20,	"allows you to breathe fire for 200 damage")
 EFFECT(DRAGON_MULTIHUED, TRUE, 20,	"allows you to breathe the elements for 250 damage")
-EFFECT(DRAGON_BRONZE,  TRUE,   INHIBIT_POWER,	"allows you to breathe confusion for 120 damage")
-EFFECT(DRAGON_GOLD,    TRUE,   19,	"allows you to breathe sound for 130 damage")
+EFFECT(DRAGON_BRONZE,  TRUE,   INHIBIT_POWER,	"allows you to breathe confusion for 150 damage")
+EFFECT(DRAGON_GOLD,    TRUE,   19,	"allows you to breathe sound for 150 damage")
 EFFECT(DRAGON_CHAOS,   TRUE,   23,	"allows you to breathe chaos or disenchantment for 220 damage")
 EFFECT(DRAGON_LAW,     TRUE,   23,	"allows you to breathe sound/shards for 230 damage")
 EFFECT(DRAGON_BALANCE, TRUE,   24,	"allows you to breathe balance for 250 damage")
