@@ -20,15 +20,12 @@
 #include "angband.h"
 #include "buildid.h"
 #include "cmds.h"
+#include "dungeon.h"
 
 #ifdef USE_GCU
 #include "main.h"
 #include "files.h"
 
-/* locale junk */
-#include "locale.h"
-#include "langinfo.h"
-
 /* Avoid 'struct term' name conflict with <curses.h> (via <term.h>) on AIX */
 #define term System_term
 
@@ -112,7 +109,15 @@ typedef struct term_data {
 } term_data;
 
 /* Max number of windows on screen */
-#define MAX_TERM_DATA 4
+#define MAX_TERM_DATA 6
+
+/* Minimum main term size */
+#define MIN_TERM0_LINES 24
+#define MIN_TERM0_COLS 80
+
+/* Comfortable subterm size */
+#define COMFY_SUBTERM_LINES 5
+#define COMFY_SUBTERM_COLS 40
 
 /* Information about our windows */
 static term_data data[MAX_TERM_DATA];
@@ -120,22 +125,6 @@ static term_data data[MAX_TERM_DATA];
 /* Number of initialized "term" structures */
 static int active = 0;
 
-#define CTRL_ORE 1
-#define CTRL_WALL 2
-#define CTRL_ROCK 3
-
-static char ctrl_char[32] = {
-	'\0', '*', '#', '%', '?', '?', '?', '\'', '+', '?', '?', '+',
-	'+', '+', '+', '+', '~', '-', '-', '-', '_', '+', '+', '+',
-	'+', '|', '?', '?', '?', '?', '?', '.'
-};
-
-static int ctrl_attr[32] = {
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-	0, 0, 0, 0, 0, 0, 0, 0
-};
-
 #ifdef A_COLOR
 
 /*
@@ -157,8 +146,9 @@ static int can_use_color = FALSE;
 static int colortable[BASIC_COLORS];
 
 /* Screen info: use one big Term 0, or other subwindows? */
-static bool use_big_screen = FALSE;
 static bool bold_extended = FALSE;
+static bool ascii_walls = FALSE;
+static int term_count = 4;
 
 /*
  * Background color we should draw with; either BLACK or DEFAULT
@@ -295,7 +285,18 @@ static errr Term_xtra_gcu_alive(int v) {
 	return 0;
 }
 
-const char help_gcu[] = "Text mode, subopts -b(ig screen) -a(scii) -B(old)";
+const char help_gcu[] = "Text mode, subopts\n              -a     Use ASCII walls\n              -b     Big screen (equivalent to -n1)\n              -B     Use brighter bold characters\n              -nN    Use N terminals (up to 6)";
+
+/*
+ * Usage:
+ *
+ * angband -mgcu -- [-a] [-b] [-B] [-nN]
+ *
+ *   -a      Use ASCII walls
+ *   -b      Big screen (equivalent to -n1)
+ *   -B      Use brighter bold characters
+ *   -nN     Use N terminals (up to 6)
+ */
 
 /*
  * Init the "curses" system
@@ -366,55 +367,121 @@ static void Term_nuke_gcu(term *t) {
 	keymap_norm();
 }
 
+/*
+ * Helper function for get_gcu_term_size:
+ * Given inputs, populates size and start (rows and y, or cols and x)
+ * with correct values for a group (column or row) of terms.
+ *   term_group_index: the placement of the group, e.g.
+ *     top row is 0
+ *   term_group_count: the number of groups in this dimension (2 or 3)
+ *   window_size:      the number of grids the window has in this dimension
+ *   min_term0_size:   the minimum main term size in this dimension 
+ *     (80 or 24), also the maximum subterm size
+ *   comfy_subterm_size: in balancing among three groups, we first give the
+ *     main term its minimum, and then allocate evenly between the other
+ *     two subterms until they are both comfy_subterm_size, at which point
+ *     we grow the outer subterm until it reaches min_term0_size. (The
+ *     middle subterm then grows until min_term0_size, and any further
+ *     window space goes to the main term.)
+ */
+static void balance_dimension(int *size, int *start, int term_group_index,
+	int term_group_count, int window_size, int min_term0_size,
+	int comfy_subterm_size) {
+	/* Convenience variable for clarity.
+	 * Note that it is also the number of separator rows/columns */
+	int subterm_group_count = term_group_count - 1;
+
+	if (term_group_index == 0) { 
+		/* main term */
+		*size = MAX(min_term0_size, window_size - subterm_group_count*(min_term0_size + 1)); 
+		*start = 0;
+	} else if (term_group_index == term_group_count - 1) { 
+		/* outer or only subterm */
+		if (window_size <= min_term0_size + subterm_group_count*(comfy_subterm_size + 1)) { 
+			/* Not enough room for min term0 and all subterms comfy.
+			 * Note that we round up here and down for the middle subterm*/
+			*size = (window_size - min_term0_size - subterm_group_count) / subterm_group_count;
+			if (window_size > min_term0_size + subterm_group_count + *size * subterm_group_count)
+				(*size)++;
+		} else {
+			*size = MIN(min_term0_size, window_size - min_term0_size - comfy_subterm_size*(subterm_group_count - 1) - subterm_group_count);
+		}
+		*start = window_size - *size;
+	} else {
+		/* middle subterm */
+		if (window_size <= subterm_group_count*(min_term0_size + 1) + comfy_subterm_size) {
+			/* Outer subterm(s) not yet full-sized, thus at most comfy */
+			*size = MIN(comfy_subterm_size, (window_size - min_term0_size - subterm_group_count) / subterm_group_count);
+		} else {
+			*size = MIN(min_term0_size, window_size - subterm_group_count*(min_term0_size + 1));
+		}
+		*start = 1 + MAX(min_term0_size, window_size - subterm_group_count*(min_term0_size + 1));
+	}
+}
 
 /*
  * For a given term number (i) set the upper left corner (x, y) and the
- * correct dimensions. Terminal layout: 0|2
- *                                      1|3
+ * correct dimensions. Remember to leave one row and column between
+ * subterms.
  */
-void get_gcu_term_size(int i, int *rows, int *cols, int *y, int *x) {
-	if (use_big_screen && i == 0) {
-		*rows = LINES;
-		*cols = COLS;
-		*y = *x = 0;
-	} else if (use_big_screen) {
-		*rows = *cols = *y = *x = 0;
-	} else if (i == 0) {
-		*rows = 24;
-		*cols = 80;
-		*y = *x = 0;
-	} else if (i == 1) {
-		*rows = LINES - 25;
-		*cols = 80;
-		*y = 25;
-		*x = 0;
-	} else if (i == 2) {
-		*rows = 24;
-		*cols = COLS - 81;
-		*y = 0;
-		*x = 81;
-	} else if (i == 3) {
-		*rows = LINES - 25;
-		*cols = COLS - 81;
-		*y = 25;
-		*x = 81;
-	} else {
-		*rows = *cols = *y = *x = 0;
+static void get_gcu_term_size(int i, int *rows, int *cols, int *y, int *x) {
+	bool is_wide = (10 * LINES < 3 * COLS);
+	int term_rows = 1;
+	int term_cols = 1;
+	int term_row_index = 0;
+	int term_col_index = 0;
+
+	assert(i < term_count);
+
+	/* For sufficiently small windows, we can only use one term.
+	 * Each additional row/column of terms requires at least two lines
+	 * for the separators. If everything is as square as possible, 
+	 * the 3rd, 7th, 13th, etc. terms add to the short dimension, while
+	 * the 2nd, 5th, 10th, etc. terms add to the long dimension.
+	 * However, three terms are the special case of 1x3 or 3x1.
+	 */
+	if (is_wide) {
+		while (term_rows*(term_rows + 1) < term_count) term_rows++;
+		while (term_cols*term_cols < term_count) term_cols++;
+		if (term_count == 3) {
+			term_rows = 1;
+			term_cols = 3;
+		}
+		term_col_index = i % term_cols;
+		term_row_index = (int)(i / term_cols);
+	} else { /* !is_wide */ 
+		while (term_rows*term_rows < term_count) term_rows++;
+		while (term_cols*(term_cols + 1) < term_count) term_cols++;
+		if (term_count == 3) {
+			term_rows = 3;
+			term_cols = 1;
+		}
+		term_col_index = (int)(i / term_rows);
+		term_row_index = i % term_rows;
 	}
+
+	if (LINES < MIN_TERM0_LINES + 2 * (term_rows - 1) ||
+		COLS < MIN_TERM0_COLS + 2 * (term_cols - 1)) {
+		term_rows = term_cols = term_count = 1;
+		if (i != 0) {
+			*rows = *cols = *y = *x = 0;
+		}
+		term_col_index = term_row_index = 0;
+	}
+		
+	balance_dimension(cols, x, term_col_index, term_cols, COLS, MIN_TERM0_COLS, COMFY_SUBTERM_COLS);
+	balance_dimension(rows, y, term_row_index, term_rows, LINES, MIN_TERM0_LINES, COMFY_SUBTERM_LINES);
 }
 
 
 /*
  * Query ncurses for new screen size and try to resize the GCU terms.
  */
-void do_gcu_resize(void) {
+static void do_gcu_resize(void) {
 	int i, rows, cols, y, x;
 	term *old_t = Term;
 	
-	for (i = 0; i < MAX_TERM_DATA; i++) {
-		/* If we're using a big screen, we only care about Term-0 */
-		if (use_big_screen && i > 0) break;
-		
+	for (i = 0; i < term_count; i++) {
 		/* Activate the current Term */
 		Term_activate(&data[i].t);
 
@@ -434,7 +501,7 @@ void do_gcu_resize(void) {
  * Process events, with optional wait
  */
 static errr Term_xtra_gcu_event(int v) {
-	int i, j, k;
+	int i, j, k, mods=0;
 
 	if (v) {
 		/* Wait for a keypress; use halfdelay(1) so if the user takes more */
@@ -534,6 +601,12 @@ static errr Term_xtra_gcu_event(int v) {
 		case KEY_UP:    i = ARROW_UP;    break;
 		case KEY_LEFT:  i = ARROW_LEFT;  break;
 		case KEY_RIGHT: i = ARROW_RIGHT; break;
+		case KEY_DC:    i = KC_DELETE; break;
+		case KEY_BACKSPACE: i = KC_BACKSPACE; break;
+		case KEY_ENTER: i = KC_ENTER; mods |= KC_MOD_KEYPAD; break;
+		case 9:         i = KC_TAB; break;
+		case 13:        i = KC_ENTER; break;
+		case 27:        i = ESCAPE; break;
 
 		/* keypad keys */
 		case 0xFC: i = '0'; break;
@@ -545,9 +618,6 @@ static errr Term_xtra_gcu_event(int v) {
 		case 0xC1: i = '7'; break;
 		case 0xF4: i = '9'; break;
 
-		/* try to compensate for inadequate terminfo */
-		case 263: i = '\b'; break;
-
 		default: {
 			if (i < KEY_MIN) break;
 
@@ -565,17 +635,17 @@ static errr Term_xtra_gcu_event(int v) {
 #endif
 
 	/* Enqueue the keypress */
-	Term_keypress(i, 0);
+	Term_keypress(i, mods);
 
 	/* Success */
 	return (0);
 }
 
-int scale_color(int i, int j, int scale) {
+static int scale_color(int i, int j, int scale) {
 	return (angband_color_table[i][j] * (scale - 1) + 127) / 255;
 }
 
-int create_color(int i, int scale) {
+static int create_color(int i, int scale) {
 	int r = scale_color(i, 1, scale);
 	int g = scale_color(i, 2, scale);
 	int b = scale_color(i, 3, scale);
@@ -595,6 +665,31 @@ int create_color(int i, int scale) {
  * React to changes
  */
 static errr Term_xtra_gcu_react(void) {
+	if (ascii_walls) {
+		int i;
+		ascii_walls = FALSE;
+		for (i = 0; i < 4; i++) {
+			// magma as %:D
+			f_info[50].x_char[i] = f_info[52].x_char[i] = 0x23;
+			f_info[50].x_attr[i] = f_info[52].x_attr[i] = 0x01;
+
+			// quartz as %:D
+			f_info[51].x_char[i] = f_info[53].x_char[i] = 0x23;
+			f_info[51].x_attr[i] = f_info[53].x_attr[i] = 0x01;
+
+			// quartz/magma w treasure as *:o
+			f_info[54].x_char[i] = f_info[55].x_char[i] = 0x2A;
+			f_info[54].x_attr[i] = f_info[55].x_attr[i] = 0x03;
+
+			// granite walls as #:D
+			f_info[56].x_char[i] = 0x23;
+			f_info[56].x_attr[i] = 0x01;
+
+			// permanent walls as #:r
+			f_info[60].x_char[i] = 0x23;
+			f_info[60].x_attr[i] = 0x04;
+		}
+	}
 
 #ifdef A_COLOR
 	if (COLORS == 256 || COLORS == 88) {
@@ -613,11 +708,9 @@ static errr Term_xtra_gcu_react(void) {
 
 		for (i = 0; i < BASIC_COLORS; i++) {
 			int fg = create_color(i, scale);
+			int isbold = bold_extended ? A_BRIGHT : A_NORMAL;
 			init_pair(i + 1, fg, bg_color);
-			if (bold_extended)
-				colortable[i] = COLOR_PAIR(i + 1) | A_BRIGHT;
-			else
-				colortable[i] = COLOR_PAIR(i + 1);
+			colortable[i] = COLOR_PAIR(i + 1) | isbold;
 		}
 	}
 #endif
@@ -698,66 +791,40 @@ static errr Term_wipe_gcu(int x, int y, int n) {
 	return 0;
 }
 
-/*
- * Since GCU currently only supports Latin-1 extended chracters, we only
- * install this hook if we're not using UTF-8.
- * Given a position in the ISO Latin-1 character set, return the correct
- * character on this system. Currently
- */
- static byte Term_xchar_gcu(byte c) {
-	return c;
-}
-
-
-/* Hack - replace non-ASCII characters to
- * avoid display glitches in selectors.
- *
- * Note that we do this after the ACS mapping,
- * because the display glitches we are avoiding
- * are in curses itself.
- */
-char filter_char(char c) {
-	if (c < ' ' || c >= 127)
-		return '?';
-	else
-		return c;
-}
-
 
 /*
  * Place some text on the screen using an attribute
  */
-static errr Term_text_gcu(int x, int y, int n, byte a, const char *s) {
+static errr Term_text_gcu(int x, int y, int n, int a, const wchar_t *s) {
 	term_data *td = (term_data *)(Term->data);
 
 #ifdef A_COLOR
-	/* Set the color */
-	if (can_use_color) (void)wattrset(td->win, colortable[a & 255]);
-#endif
+	if (can_use_color) {
+		/* the lower 7 bits of the attribute indicate the fg/bg */
+		int attr = a & 127;
+		int color = colortable[attr];
 
-	/* Move the cursor */
-	wmove(td->win, y, x);
+		/* the high bit of the attribute indicates a reversed fg/bg */
+		bool reversed = a > 127;
 
-	/* Write to screen */
-	while (n--) {
-		unsigned char c = *(s++);
+		/* the following check for A_BRIGHT is to avoid #1813 */
+		int mode;
+		if (reversed && (color & A_BRIGHT))
+			mode = (color & ~A_BRIGHT) | A_BLINK | A_REVERSE;
+		else if (reversed)
+			mode = color | A_REVERSE;
+		else
+			mode = color | A_NORMAL;
 
-		if (c < 32) {
-			wattron(td->win, ctrl_attr[c]);
-			waddch(td->win, filter_char(ctrl_char[c]));
-			wattroff(td->win, ctrl_attr[c]);
-		} else {
-			waddch(td->win, filter_char(c));
-		}
+		wattrset(td->win, mode);
+		mvwaddnwstr(td->win, y, x, s, n);
+		wattrset(td->win, A_NORMAL);
+		return 0;
 	}
-
-#if defined(A_COLOR)
-	/* Unset the color */
-	if (can_use_color) wattrset(td->win, A_NORMAL);
 #endif
 
-	/* Success */
-	return (0);
+	mvwaddnwstr(td->win, y, x, s, n);
+	return 0;
 }
 
 
@@ -786,6 +853,9 @@ static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
 	t->attr_blank = TERM_WHITE;
 	t->char_blank = ' ';
 
+	/* Differentiate between BS/^h, Tab/^i, etc. */
+	t->complex_input = TRUE;
+
 	/* Set some hooks */
 	t->init_hook = Term_init_gcu;
 	t->nuke_hook = Term_nuke_gcu;
@@ -796,13 +866,6 @@ static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
 	t->curs_hook = Term_curs_gcu;
 	t->xtra_hook = Term_xtra_gcu;
 
-	/* only if the locale supports Latin-1 will we enable xchar_hook */
-	if (setlocale(LC_CTYPE, "")) {
-		/* the Latin-1 codeset is ISO-8859-1 */
-		if (strcmp(nl_langinfo(CODESET), "ISO-8859-1") == 0)
-			t->xchar_hook = Term_xchar_gcu;
-	}
-
 	/* Save the data */
 	t->data = td;
 
@@ -814,6 +877,13 @@ static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
 }
 
 static void hook_quit(const char *str) {
+	int i;
+
+	for (i = 0; i < term_count; i++) {
+		if (angband_term[i]) {
+			term_nuke(angband_term[i]);
+		}
+	}
 	endwin();
 }
 
@@ -829,7 +899,6 @@ errr init_gcu(int argc, char **argv) {
 	int i;
 	int rows, cols, y, x;
 	int next_win = 0;
-	bool graphics = TRUE;
 
 	/* Initialize info about terminal capabilities */
 	termtype = getenv("TERM");
@@ -837,29 +906,26 @@ errr init_gcu(int argc, char **argv) {
 
 	/* Parse args */
 	for (i = 1; i < argc; i++) {
-		if (prefix(argv[i], "-b"))
-			use_big_screen = TRUE;
-		else if (prefix(argv[i], "-B"))
+		if (prefix(argv[i], "-b")) {
+			term_count = 1;
+		} else if (prefix(argv[i], "-B")) {
 			bold_extended = TRUE;
-		else if (prefix(argv[i], "-a"))
-			graphics = FALSE;
-		else
+		} else if (prefix(argv[i], "-a")) {
+			ascii_walls = TRUE;
+		} else if (prefix(argv[i], "-n")) {
+			term_count = atoi(&argv[i][2]);
+			if (term_count > MAX_TERM_DATA) term_count = MAX_TERM_DATA;
+			else if (term_count < 1) term_count = 1;
+		} else {
 			plog_fmt("Ignoring option: %s", argv[i]);
-	}
-
-	if (graphics) {
-		ctrl_char[CTRL_WALL] = ' ';
-		ctrl_attr[CTRL_ORE] = A_REVERSE;
-		ctrl_attr[CTRL_WALL] = A_REVERSE;
-		ctrl_attr[CTRL_ROCK] = A_REVERSE;
+		}
 	}
 
 	/* Extract the normal keymap */
 	keymap_norm_prepare();
 
 	/* We do it like this to prevent a link error with curseses that
-	 * lack ESCDELAY.
-	 */
+	 * lack ESCDELAY. */
 	if (!getenv("ESCDELAY"))
 		putenv("ESCDELAY=20");
 
@@ -870,7 +936,7 @@ errr init_gcu(int argc, char **argv) {
 	quit_aux = hook_quit;
 
 	/* Require standard size screen */
-	if (LINES < 24 || COLS < 80)
+	if (LINES < MIN_TERM0_LINES || COLS < MIN_TERM0_COLS) 
 		quit("Angband needs at least an 80x24 'curses' screen");
 
 #ifdef A_COLOR
@@ -919,9 +985,9 @@ errr init_gcu(int argc, char **argv) {
 		colortable[TERM_MUD]         = (COLOR_PAIR(PAIR_YELLOW));
 		colortable[TERM_L_YELLOW]    = (COLOR_PAIR(PAIR_YELLOW | A_BRIGHT));
 		colortable[TERM_MAGENTA]     = (COLOR_PAIR(PAIR_MAGENTA | A_BRIGHT));
-		colortable[TERM_L_TEAL]      = (COLOR_PAIR(PAIR_CYAN | A_BRIGHT));
-		colortable[TERM_L_VIOLET]    = (COLOR_PAIR(PAIR_MAGENTA | A_BRIGHT));
-		colortable[TERM_L_PINK]      = (COLOR_PAIR(PAIR_MAGENTA | A_BRIGHT));
+		colortable[TERM_L_TEAL]      = (COLOR_PAIR(PAIR_CYAN) | A_BRIGHT);
+		colortable[TERM_L_VIOLET]    = (COLOR_PAIR(PAIR_MAGENTA) | A_BRIGHT);
+		colortable[TERM_L_PINK]      = (COLOR_PAIR(PAIR_MAGENTA) | A_BRIGHT);
 		colortable[TERM_MUSTARD]     = (COLOR_PAIR(PAIR_YELLOW));
 		colortable[TERM_BLUE_SLATE]  = (COLOR_PAIR(PAIR_BLUE));
 		colortable[TERM_DEEP_L_BLUE] = (COLOR_PAIR(PAIR_BLUE));
@@ -942,10 +1008,8 @@ errr init_gcu(int argc, char **argv) {
 	/* Extract the game keymap */
 	keymap_game_prepare();
 
-	/*** Now prepare the term(s) ***/
-	for (i = 0; i < MAX_TERM_DATA; i++) {
-		if (use_big_screen && i > 0) break;
-
+	/* Now prepare the term(s) */
+	for (i = 0; i < term_count; i++) {
 		/* Get the terminal dimensions; if the user asked for a big screen
 		 * then we'll put the whole screen in term 0; otherwise we'll divide
 		 * it amongst the available terms */
