@@ -22,6 +22,7 @@
 
 #include "birth.h"
 #include "buildid.h"
+#include "dungeon.h"
 #include "init.h"
 #include "monster/mon-make.h"
 #include "object/pval.h"
@@ -200,8 +201,7 @@ static void generate_player_for_stats()
 	p_ptr->age = 14;
 
 	/* Set social class and (null) history */
-	p_ptr->history = get_history(p_ptr->race->history, &p_ptr->sc);
-	p_ptr->sc_birth = p_ptr->sc;
+	p_ptr->history = get_history(p_ptr->race->history);
 }
 
 static void initialize_character(void)
@@ -241,15 +241,14 @@ static void kill_all_monsters(int level)
 	int i;
 
 	for (i = cave_monster_max(cave) - 1; i >= 1; i--) {
-		const monster_type *m_ptr = cave_monster(cave, i);
-		monster_race *r_ptr = &r_info[m_ptr->r_idx];
+		monster_type *m_ptr = cave_monster(cave, i);
 
-		level_data[level].monsters[m_ptr->r_idx]++;
+		level_data[level].monsters[m_ptr->race->ridx]++;
 
-		monster_death(i, TRUE);
+		monster_death(m_ptr, TRUE);
 
-		if (rf_has(r_ptr->flags, RF_UNIQUE))
-			r_ptr->max_num = 0;
+		if (rf_has(m_ptr->race->flags, RF_UNIQUE))
+			m_ptr->race->max_num = 0;
 	}
 }
 
@@ -288,8 +287,8 @@ static void log_all_objects(int level)
 {
 	int x, y, i;
 
-	for (y = 1; y < DUNGEON_HGT - 1; y++) {
-		for (x = 1; x < DUNGEON_WID - 1; x++) {
+	for (y = 1; y < cave->height - 1; y++) {
+		for (x = 1; x < cave->width - 1; x++) {
 			object_type *o_ptr = get_first_object(y, x);
 
 			if (o_ptr) do {
@@ -803,8 +802,8 @@ static int stats_dump_lists(void)
 
 	struct mon_spell mon_spell_table[] =
 	{
-		#define RSF(a, b, c, d, e, f, g, h, i, j, k, l, m) \
-			{ RSF_##a, b, #a, d, e, f, g, h, i, j, k, l, m },
+		#define RSF(a, b, c, d, e, f, g, h, i, j, k, l, m, n) \
+			{ RSF_##a, b, #a, d, e, f, g, h, i, j, k, l, m, n },
 		#define RV(b, x, y, m) {b, x, y, m}
 		#include "monster/list-mon-spells.h"
 		#undef RV
@@ -1657,6 +1656,8 @@ static errr run_stats(void)
 	err = stats_write_db(run);
 	stats_db_close();
 	if (err) quit_fmt("Problems writing to database!  sqlite3 errno %d.", err);
+
+    mem_free(a_info_save);
 	free_stats_memory();
 	cleanup_angband();
 	if (!quiet) printf("Done!\n");
@@ -1758,7 +1759,7 @@ static errr term_wipe_stats(int x, int y, int n) {
 	return 0;
 }
 
-static errr term_text_stats(int x, int y, int n, byte a, const char *s) {
+static errr term_text_stats(int x, int y, int n, int a, const wchar_t *s) {
 	return 0;
 }
 
