@@ -17,6 +17,7 @@
  */
 #include "angband.h"
 #include "buildid.h"
+#include "dungeon.h"
 
 /*
  * This file helps Angband work with UNIX/X11 computers.
@@ -107,6 +108,7 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #include <X11/keysymdef.h>
+#include <X11/XKBlib.h>
 
 #include "main.h"
 
@@ -196,6 +198,8 @@ typedef struct infofnt infofnt;
  *	- The default Screen for the display
  *	- The virtual root (usually just the root)
  *	- The default colormap (from a macro)
+ *  - The Alt key modifier mask
+ *  - The Super key modifier mask
  *
  *	- The "name" of the display
  *
@@ -222,6 +226,8 @@ struct metadpy
 	Screen *screen;
 	Window root;
 	Colormap cmap;
+	unsigned int alt_mask;
+	unsigned int super_mask;
 
 	char *name;
 
@@ -347,7 +353,7 @@ struct infoclr
  */
 struct infofnt
 {
-	XFontStruct *info;
+	XFontSet	fs;
 
 	const char *name;
 
@@ -393,42 +399,14 @@ struct term_data
 
 /**** Generic Macros ****/
 
-
-
-/* Set current metadpy (Metadpy) to 'M' */
-#define Metadpy_set(M) \
-	Metadpy = M
-
-
-/* Initialize 'M' using Display 'D' */
-#define Metadpy_init_dpy(D) \
-	Metadpy_init_2(D,cNULL)
-
 /* Initialize 'M' using a Display named 'N' */
 #define Metadpy_init_name(N) \
 	Metadpy_init_2((Display*)(NULL),N)
 
-/* Initialize 'M' using the standard Display */
-#define Metadpy_init() \
-	Metadpy_init_name("")
-
-
-/* Init an infowin by giving father as an (info_win*) (or NULL), and data */
-#define Infowin_init_dad(D,X,Y,W,H,B,FG,BG) \
-	Infowin_init_data(((D) ? ((D)->win) : (Window)(None)), \
-	                  X,Y,W,H,B,FG,BG)
-
-
 /* Init a top level infowin by pos,size,bord,Colors */
 #define Infowin_init_top(X,Y,W,H,B,FG,BG) \
 	Infowin_init_data(None,X,Y,W,H,B,FG,BG)
 
-
-/* Request a new standard window by giving Dad infowin and X,Y,W,H */
-#define Infowin_init_std(D,X,Y,W,H,B) \
-	Infowin_init_dad(D,X,Y,W,H,B,Metadpy->fg,Metadpy->bg)
-
-
 /* Set the current Infowin */
 #define Infowin_set(I) \
 	(Infowin = (I))
@@ -442,31 +420,13 @@ struct term_data
 #define Infoclr_init_ppo(F,B,O,M) \
 	Infoclr_init_data(F,B,O,M)
 
-#define Infoclr_init_cco(F,B,O,M) \
-	Infoclr_init_ppo(Infoclr_Pixell(F),Infoclr_Pixell(B),O,M)
-
 #define Infoclr_init_ppn(F,B,O,M) \
 	Infoclr_init_ppo(F,B,Infoclr_Opcode(O),M)
 
-#define Infoclr_init_ccn(F,B,O,M) \
-	Infoclr_init_cco(F,B,Infoclr_Opcode(O),M)
-
-
 /* Set the current infofnt */
 #define Infofnt_set(I) \
 	(Infofnt = (I))
 
-
-/* Errr: Expose Infowin */
-#define Infowin_expose() \
-	(!(Infowin->redraw = 1))
-
-/* Errr: Unxpose Infowin */
-#define Infowin_unexpose() \
-	(Infowin->redraw = 0)
-
-
-
 /**** Generic Globals ****/
 
 
@@ -488,7 +448,7 @@ static infofnt *Infofnt = (infofnt*)(NULL);
 /*
  * Actual color table
  */
-static infoclr *clr[MAX_COLORS];
+static infoclr *clr[MAX_COLORS * BG_MAX];
 
 
 
@@ -593,6 +553,23 @@ static const char *get_default_font(int term_num)
 }
 
 
+/* look up keyboard modifiers using the Xkb extension */
+
+static unsigned int xkb_mask_modifier( XkbDescPtr xkb, const char *name )
+{
+	unsigned int mask=0;
+	
+	if  ( strcmp(name, "Caps Lock") == 0 ) return 2;
+	
+	for(int i = 0; i <= XkbNumVirtualMods; i++ ) {
+		char* modStr = XGetAtomName( xkb->dpy, xkb->names->vmods[i] );
+		if( modStr != NULL && strcmp(name, modStr) == 0 ) {
+			XkbVirtualModsToReal( xkb, 1 << i, &mask );
+			return mask;
+		}
+	}
+	return 0;
+}
 
 /**** Generic code ****/
 
@@ -614,7 +591,8 @@ static const char *get_default_font(int term_num)
 static errr Metadpy_init_2(Display *dpy, const char *name)
 {
 	metadpy *m = Metadpy;
-
+	XkbDescPtr xkb;
+	
 	/*** Open the display if needed ***/
 
 	/* If no Display given, attempt to Create one */
@@ -647,6 +625,16 @@ static errr Metadpy_init_2(Display *dpy, const char *name)
 	m->screen = DefaultScreenOfDisplay(dpy);
 	m->root = RootWindowOfScreen(m->screen);
 
+	/* get the modifier key layout */
+	m->alt_mask = Mod1Mask;
+	m->super_mask = Mod4Mask;
+	xkb = XkbGetKeyboard(dpy, XkbAllComponentsMask, XkbUseCoreKbd);
+	if (xkb != NULL) {
+        m->alt_mask = xkb_mask_modifier( xkb, "Alt" );
+        m->super_mask = xkb_mask_modifier( xkb, "Super" );
+		XkbFreeKeyboard( xkb, 0, True );
+	}
+
 	/* Get the default colormap */
 	m->cmap = DefaultColormapOfScreen(m->screen);
 
@@ -1347,7 +1335,7 @@ static errr Infofnt_nuke(void)
 	if (ifnt->nuke)
 	{
 		/* Free the font */
-		XFreeFont(Metadpy->dpy, ifnt->info);
+		XFreeFontSet(Metadpy->dpy, ifnt->fs);
 	}
 
 	/* Success */
@@ -1358,49 +1346,33 @@ static errr Infofnt_nuke(void)
 /*
  * Prepare a new 'infofnt'
  */
-static errr Infofnt_prepare(XFontStruct *info)
+static errr Infofnt_prepare(XFontSet fs)
 {
 	infofnt *ifnt = Infofnt;
-
-	XCharStruct *cs;
+	int font_count, i;
+	XFontSetExtents *extents;
+	XFontStruct **fonts;
+	char **names;
 
 	/* Assign the struct */
-	ifnt->info = info;
+	ifnt->fs = fs;
+	extents = XExtentsOfFontSet(fs);
 
-	/* Jump into the max bouonds thing */
-	cs = &(info->max_bounds);
+	font_count = XFontsOfFontSet(fs, &fonts, &names);
+	ifnt->asc = 0;
+	for (i = 0; i < font_count; i++, fonts++)
+	   if (ifnt->asc < (*fonts)->ascent) ifnt->asc = (*fonts)->ascent;
 
 	/* Extract default sizing info */
-	ifnt->asc = info->ascent;
-	ifnt->hgt = info->ascent + info->descent;
-	ifnt->wid = cs->width;
-	ifnt->twid = cs->width;
+	ifnt->hgt = extents->max_logical_extent.height;
+	ifnt->wid = extents->max_logical_extent.width;
+	ifnt->twid = extents->max_logical_extent.width;
 
 	/* Success */
 	return (0);
 }
 
 
-#ifndef IGNORE_UNUSED_FUNCTIONS
-
-/*
- * Initialize a new 'infofnt'.
- */
-static errr Infofnt_init_real(XFontStruct *info)
-{
-	/* Wipe the thing */
-	(void)WIPE(Infofnt, infofnt);
-
-	/* No nuking */
-	Infofnt->nuke = 0;
-
-	/* Attempt to prepare it */
-	return (Infofnt_prepare(info));
-}
-
-#endif /* IGNORE_UNUSED_FUNCTIONS */
-
-
 /*
  * Init an infofnt by its Name
  *
@@ -1409,20 +1381,22 @@ static errr Infofnt_init_real(XFontStruct *info)
  */
 static errr Infofnt_init_data(const char *name)
 {
-	XFontStruct *info;
-
+	XFontSet fs;
+	char **missing;
+	int missing_count;
 
 	/*** Load the info Fresh, using the name ***/
 
 	/* If the name is not given, report an error */
 	if (!name) return (-1);
 
-	/* Attempt to load the font */
-	info = XLoadQueryFont(Metadpy->dpy, name);
+
+	fs = XCreateFontSet(Metadpy->dpy, name, &missing, &missing_count, NULL);
 
 	/* The load failed, try to recover */
-	if (!info) return (-1);
-
+	if (!fs) return (-1);
+	if (missing_count)
+		XFreeStringList(missing);
 
 	/*** Init the font ***/
 
@@ -1430,10 +1404,10 @@ static errr Infofnt_init_data(const char *name)
 	(void)WIPE(Infofnt, infofnt);
 
 	/* Attempt to prepare it */
-	if (Infofnt_prepare(info))
+	if (Infofnt_prepare(fs))
 	{
 		/* Free the font */
-		XFreeFont(Metadpy->dpy, info);
+		XFreeFontSet(Metadpy->dpy, fs);
 
 		/* Fail */
 		return (-1);
@@ -1456,7 +1430,7 @@ static errr Infofnt_init_data(const char *name)
 /*
  * Standard Text
  */
-static errr Infofnt_text_std(int x, int y, const char *str, int len)
+static errr Infofnt_text_std(int x, int y, const wchar_t *str, int len)
 {
 	int i;
 	int w, h;
@@ -1469,7 +1443,7 @@ static errr Infofnt_text_std(int x, int y, const char *str, int len)
 	if (!str || !*str) return (-1);
 
 	/* Get the length of the string */
-	if (len < 0) len = strlen(str);
+	if (len < 0) len = wcslen(str);
 
 	/*** Decide where to place the string, vertically ***/
 
@@ -1496,11 +1470,6 @@ static errr Infofnt_text_std(int x, int y, const char *str, int len)
 
 
 	/*** Actually draw 'str' onto the infowin ***/
-
-	/* Be sure the correct font is ready */
-	XSetFont(Metadpy->dpy, Infoclr->gc, Infofnt->info->fid);
-
-
 	y += Infofnt->asc;
 
 
@@ -1513,7 +1482,7 @@ static errr Infofnt_text_std(int x, int y, const char *str, int len)
 		for (i = 0; i < len; ++i)
 		{
 			/* Note that the Infoclr is set up to contain the Infofnt */
-			XDrawImageString(Metadpy->dpy, Infowin->win, Infoclr->gc,
+			XwcDrawImageString(Metadpy->dpy, Infowin->win, Infofnt->fs, Infoclr->gc,
 			                 x + i * td->tile_wid + Infofnt->off, y, str + i, 1);
 		}
 	}
@@ -1522,7 +1491,7 @@ static errr Infofnt_text_std(int x, int y, const char *str, int len)
 	else
 	{
 		/* Note that the Infoclr is set up to contain the Infofnt */
-		XDrawImageString(Metadpy->dpy, Infowin->win, Infoclr->gc,
+		XwcDrawImageString(Metadpy->dpy, Infowin->win, Infofnt->fs, Infoclr->gc,
 		                 x, y, str, len);
 	}
 
@@ -1534,7 +1503,7 @@ static errr Infofnt_text_std(int x, int y, const char *str, int len)
 /*
  * Painting where text would be
  */
-static errr Infofnt_text_non(int x, int y, const char *str, int len)
+static errr Infofnt_text_non(int x, int y, const wchar_t *str, int len)
 {
 	int w, h;
 
@@ -1543,7 +1512,7 @@ static errr Infofnt_text_non(int x, int y, const char *str, int len)
 	/*** Find the width ***/
 
 	/* Negative length is a flag to count the characters in str */
-	if (len < 0) len = strlen(str);
+	if (len < 0) len = wcslen(str);
 
 	/* The total width will be 'len' chars * standard width */
 	w = len * td->tile_wid;
@@ -1610,7 +1579,7 @@ static term_data data[MAX_TERM_DATA];
  * Path to the X11 settings file
  */
 static const char *x11_prefs = "x11-settings.prf";
-char settings[1024];
+static char settings[1024];
 
 
 
@@ -1627,16 +1596,17 @@ static int term_windows_open;
 static void react_keypress(XKeyEvent *ev)
 {
 	int n, ch = 0;
+	metadpy *m = Metadpy;
 
 	KeySym ks;
 
 	char buf[128];
 
-	/* Extract four "modifier flags" */
+	/* Extract "modifier flags" */
 	int mc = (ev->state & ControlMask) ? TRUE : FALSE;
 	int ms = (ev->state & ShiftMask) ? TRUE : FALSE;
-	int mo = (ev->state & Mod1Mask) ? TRUE : FALSE;
-	int mx = (ev->state & Mod2Mask) ? TRUE : FALSE;
+	int mo = (ev->state & m->alt_mask) ? TRUE : FALSE;
+	int mx = (ev->state & m->super_mask) ? TRUE : FALSE;
 	int kp = FALSE;
 
 	byte mods = (mo ? KC_MOD_ALT : 0) | (mx ? KC_MOD_META : 0);
@@ -1684,7 +1654,7 @@ static void react_keypress(XKeyEvent *ev)
 		case XK_KP_Multiply: ch = '*'; kp = TRUE; break;
 		case XK_KP_Subtract: ch = '-'; kp = TRUE; break;
 		case XK_KP_Add: ch = '+'; kp = TRUE; break;
-		case XK_KP_Enter: ch = '\n'; kp = TRUE; break;
+		case XK_KP_Enter: ch = KC_ENTER; kp = TRUE; break;
 		case XK_KP_Equal: ch = '='; kp = TRUE; break;
 
 		case XK_KP_Delete: ch = KC_DELETE; kp = TRUE; break;
@@ -2092,7 +2062,7 @@ static errr Term_wipe_x11(int x, int y, int n)
 	Infoclr_set(clr[TERM_DARK]);
 
 	/* Mega-Hack -- Erase some space */
-	Infofnt_text_non(x, y, "", n);
+	Infofnt_text_non(x, y, L"", n);
 
 	/* Success */
 	return (0);
@@ -2102,7 +2072,7 @@ static errr Term_wipe_x11(int x, int y, int n)
 /*
  * Draw some textual characters.
  */
-static errr Term_text_x11(int x, int y, int n, byte a, const char *s)
+static errr Term_text_x11(int x, int y, int n, int a, const wchar_t *s)
 {
 	/* Draw the text */
 	Infoclr_set(clr[a]);
@@ -2233,7 +2203,7 @@ static errr term_data_init(term_data *td, int i)
 	font = get_default_font(i);
 
 	/* Open the file */
-	fff = file_open(settings, MODE_READ, -1);
+	fff = file_open(settings, MODE_READ, FTYPE_TEXT);
 
 	/* File exists */
 	if (fff)
@@ -2527,6 +2497,9 @@ static errr term_data_init(term_data *td, int i)
 	t->attr_blank = TERM_WHITE;
 	t->char_blank = ' ';
 
+	/* Differentiate between BS/^h, Tab/^i, etc. */
+	t->complex_input = TRUE;
+
 	/* Hooks */
 	t->xtra_hook = Term_xtra_x11;
 	t->curs_hook = Term_curs_x11;
@@ -2652,7 +2625,7 @@ errr init_x11(int argc, char **argv)
 		(void)path_build(settings, sizeof(settings), ANGBAND_DIR_USER, "x11-settings.prf");
 
 		/* Open the file */
-		fff = file_open(settings, MODE_READ, -1);
+		fff = file_open(settings, MODE_READ, FTYPE_TEXT);
 
 		/* File exists */
 		if (fff)
@@ -2701,7 +2674,7 @@ errr init_x11(int argc, char **argv)
 
 
 	/* Prepare normal colors */
-	for (i = 0; i < 256; ++i)
+	for (i = 0; i < MAX_COLORS * BG_MAX; ++i)
 	{
 		Pixell pixel;
 
@@ -2710,26 +2683,57 @@ errr init_x11(int argc, char **argv)
 		Infoclr_set(clr[i]);
 
 		/* Acquire Angband colors */
-		color_table_x11[i][0] = angband_color_table[i][0];
-		color_table_x11[i][1] = angband_color_table[i][1];
-		color_table_x11[i][2] = angband_color_table[i][2];
-		color_table_x11[i][3] = angband_color_table[i][3];
+		color_table_x11[i % MAX_COLORS][0] = angband_color_table[i % MAX_COLORS][0];
+		color_table_x11[i % MAX_COLORS][1] = angband_color_table[i % MAX_COLORS][1];
+		color_table_x11[i % MAX_COLORS][2] = angband_color_table[i % MAX_COLORS][2];
+		color_table_x11[i % MAX_COLORS][3] = angband_color_table[i % MAX_COLORS][3];
 
 		/* Default to monochrome */
 		pixel = ((i == 0) ? Metadpy->bg : Metadpy->fg);
 
-		/* Handle color */
+		/* Handle color 
+		   This block of code has added support for background colours
+                   (from Sil) */
+
 		if (Metadpy->color)
 		{
+			Pixell backpixel;
 			/* Create pixel */
 			pixel = create_pixel(Metadpy->dpy,
-					     color_table_x11[i][1],
-					     color_table_x11[i][2],
-					     color_table_x11[i][3]);
+					     color_table_x11[i % MAX_COLORS][1],
+					     color_table_x11[i % MAX_COLORS][2],
+					     color_table_x11[i % MAX_COLORS][3]);
+			switch (i / MAX_COLORS)
+			{
+				case BG_BLACK:
+					/* Default Background */
+					Infoclr_init_ppn(pixel, Metadpy->bg, "cpy", 0);
+					break;
+				case BG_SAME:
+					/* Background same as foreground*/
+					backpixel = create_pixel(Metadpy->dpy,
+											 color_table_x11[i % MAX_COLORS][1],
+											 color_table_x11[i % MAX_COLORS][2],
+											 color_table_x11[i % MAX_COLORS][3]);
+					Infoclr_init_ppn(pixel, backpixel, "cpy", 0);
+					break;
+				case BG_DARK:
+					/* Highlight Background */
+					backpixel = create_pixel(Metadpy->dpy,
+											 color_table_x11[TERM_SHADE][1],
+											 color_table_x11[TERM_SHADE][2],
+											 color_table_x11[TERM_SHADE][3]);
+					Infoclr_init_ppn(pixel, backpixel, "cpy", 0);
+					break;
+			}
 		}
 
-		/* Initialize the color */
-		Infoclr_init_ppn(pixel, Metadpy->bg, "cpy", 0);
+		/* Handle monochrome */
+		else
+		{
+		    /* Initialize the color */
+		    Infoclr_init_ppn(pixel, Metadpy->bg, "cpy", 0);
+		}
 	}
 
 
