@@ -442,16 +442,6 @@ static errr Term_wipe_xxx(int x, int y, int n)
 	return (0);
 }
 
-/*
- * Given a position in the ISO Latin-1 character set, return
- * the correct character on this system.
- */
- static byte Term_xchar_xxx(byte c)
-{
- 	/* The xxx port uses the Latin-1 standard */
- 	return (c);
-}
-
 
 /*
  * Draw some text on the screen
@@ -485,7 +475,7 @@ static errr Term_wipe_xxx(int x, int y, int n)
  * the "always_text" flag is set, if this flag is not set, all the
  * "black" text will be handled by the "Term_wipe_xxx()" hook.
  */
-static errr Term_text_xxx(int x, int y, int n, byte a, const char *cp)
+static errr Term_text_xxx(int x, int y, int n, int a, const char *cp)
 {
 	term_data *td = (term_data*)(Term->data);
 
@@ -606,7 +596,6 @@ static void term_data_link(int i)
 	t->wipe_hook = Term_wipe_xxx;
 	t->text_hook = Term_text_xxx;
 	t->pict_hook = Term_pict_xxx;
-	t->xchar_hook = Term_xchar_xxx;
 
 	/* Remember where we came from */
 	t->data = td;
