@@ -30,15 +30,14 @@
 /** Variables **/
 
 /** Functions **/
-void cheat_monster_lore(int r_idx, monster_lore *l_ptr);
-void wipe_monster_lore(int r_idx, monster_lore *l_ptr);
-void describe_monster(int r_idx, bool spoilers);
-void roff_top(int r_idx);
-void screen_roff(int r_idx);
-void display_roff(int r_idx);
-void lore_do_probe(int m_idx);
+void cheat_monster_lore(const monster_race *r_ptr, monster_lore *l_ptr);
+void wipe_monster_lore(const monster_race *r_ptr, monster_lore *l_ptr);
+void lore_do_probe(struct monster *m);
 void monster_flags_known(const monster_race *r_ptr, const monster_lore *l_ptr, bitflag flags[RF_SIZE]);
-void lore_treasure(int m_idx, int num_item, int num_gold);
-
+void lore_treasure(struct monster *m_ptr, int num_item, int num_gold);
+void lore_title(textblock *tb, const monster_race *r_ptr);
+void lore_description(textblock *tb, const monster_race *race, const monster_lore *original_lore, bool spoilers);
+void lore_show_interactive(const monster_race *race, const monster_lore *lore);
+void lore_show_subwindow(const monster_race *race, const monster_lore *lore);
 
 #endif /* MONSTER_LORE_H */
