@@ -21,8 +21,6 @@
 #include "object/tvalsval.h"
 #include "object/pval.h"
 
-static size_t obj_desc_name_format(char *buf, size_t max, size_t end,
-		const char *fmt, const char *modstr, bool pluralise);
 
 /**
  * Puts the object base kind's name into buf.
@@ -48,7 +46,7 @@ void object_kind_name(char *buf, size_t max, const object_kind *kind, bool easy_
 	/* If not aware, use flavor */
 	if (!easy_know && !kind->aware && kind->flavor)
 	{
-		if (kind->tval == TV_FOOD && kind->sval > SV_FOOD_MIN_SHROOM)
+		if (kind->tval == TV_FOOD && kind->sval >= SV_FOOD_MIN_SHROOM)
 		{
 			strnfmt(buf, max, "%s Mushroom", kind->flavor->text);
 		}
@@ -64,7 +62,7 @@ void object_kind_name(char *buf, size_t max, const object_kind *kind, bool easy_
 	{
 		char *t;
 
-		if (kind->tval == TV_FOOD && kind->sval > SV_FOOD_MIN_SHROOM)
+		if (kind->tval == TV_FOOD && kind->sval >= SV_FOOD_MIN_SHROOM)
 		{
 			my_strcpy(buf, "Mushroom of ", max);
 			max -= strlen(buf);
@@ -104,27 +102,20 @@ static const char *obj_desc_get_modstr(const object_kind *kind)
 	return "";
 }
 
-static const char *obj_desc_get_basename(const object_type *o_ptr, bool aware)
+static const char *obj_desc_get_basename(const object_type *o_ptr, bool aware, bool terse)
 {
-	bool show_flavor = o_ptr->kind->flavor ? TRUE : FALSE;
-
+	bool show_flavor = !terse && o_ptr->kind->flavor;
 
 	if (o_ptr->ident & IDENT_STORE) show_flavor = FALSE;
 	if (aware && !OPT(show_flavors)) show_flavor = FALSE;
 
-
-
-	/* Known artifacts get special treatment */
-	if (o_ptr->artifact && aware)
+	/* Artifacts are special */
+	if (o_ptr->artifact && (aware || (o_ptr->ident & IDENT_NAME) || terse || !o_ptr->kind->flavor))
 		return o_ptr->kind->name;
 
 	/* Analyze the object */
 	switch (o_ptr->tval)
 	{
-		case TV_SKELETON:
-		case TV_BOTTLE:
-		case TV_JUNK:
-		case TV_SPIKE:
 		case TV_FLASK:
 		case TV_CHEST:
 		case TV_SHOT:
@@ -169,13 +160,19 @@ static const char *obj_desc_get_basename(const object_type *o_ptr, bool aware)
 			return (show_flavor ? "& Scroll~ titled #" : "& Scroll~");
 
 		case TV_MAGIC_BOOK:
-			return "& Book~ of Magic Spells #";
+			if (terse)
+				return "& Book~ #";
+			else
+				return "& Book~ of Magic Spells #";
 
 		case TV_PRAYER_BOOK:
-			return "& Holy Book~ of Prayers #";
+			if (terse)
+				return "& Book~ #";
+			else
+				return "& Holy Book~ of Prayers #";
 
 		case TV_FOOD:
-			if (o_ptr->sval > SV_FOOD_MIN_SHROOM)
+			if (o_ptr->sval >= SV_FOOD_MIN_SHROOM)
 				return (show_flavor ? "& # Mushroom~" : "& Mushroom~");
 			else
 				return o_ptr->kind->name;
@@ -187,7 +184,7 @@ static const char *obj_desc_get_basename(const object_type *o_ptr, bool aware)
 
 static size_t obj_desc_name_prefix(char *buf, size_t max, size_t end,
 		const object_type *o_ptr, bool known, const char *basename,
-		const char *modstr)
+		const char *modstr, bool terse)
 {
 	if (o_ptr->number <= 0)
 		strnfcat(buf, max, &end, "no more ");
@@ -213,10 +210,13 @@ static size_t obj_desc_name_prefix(char *buf, size_t max, size_t end,
 			an = TRUE;
 		}
 
-		if (an)
-			strnfcat(buf, max, &end, "an ");
-		else
-			strnfcat(buf, max, &end, "a ");
+		if (!terse)
+		{
+			if (an)
+				strnfcat(buf, max, &end, "an ");
+			else
+				strnfcat(buf, max, &end, "a ");			
+		}
 	}
 
 	return end;
@@ -235,7 +235,7 @@ static size_t obj_desc_name_prefix(char *buf, size_t max, size_t end,
  * '#' will be replaced with 'modstr' (which may contain the pluralising
  * formats given above).
  */
-static size_t obj_desc_name_format(char *buf, size_t max, size_t end,
+size_t obj_desc_name_format(char *buf, size_t max, size_t end,
 		const char *fmt, const char *modstr, bool pluralise)
 {
 	/* Copy the string */
@@ -314,13 +314,11 @@ static size_t obj_desc_name_format(char *buf, size_t max, size_t end,
  * Format object o_ptr's name into 'buf'.
  */
 static size_t obj_desc_name(char *buf, size_t max, size_t end,
-		const object_type *o_ptr, bool prefix, odesc_detail_t mode,
-		bool spoil)
+		const object_type *o_ptr, bool prefix, int mode, bool spoil, bool terse)
 {
 	bool known = object_is_known(o_ptr) || (o_ptr->ident & IDENT_STORE) || spoil;
 	bool aware = object_flavor_is_aware(o_ptr) || (o_ptr->ident & IDENT_STORE) || spoil;
-
-	const char *basename = obj_desc_get_basename(o_ptr, aware);
+	const char *basename = obj_desc_get_basename(o_ptr, aware, terse);
 	const char *modstr = obj_desc_get_modstr(o_ptr->kind);
 
 	if (aware && !o_ptr->kind->everseen && !spoil)
@@ -328,7 +326,7 @@ static size_t obj_desc_name(char *buf, size_t max, size_t end,
 
 	if (prefix)
 		end = obj_desc_name_prefix(buf, max, end, o_ptr, known,
-				basename, modstr);
+				basename, modstr, terse);
 
 	/* Pluralize if (not forced singular) and
 	 * (not a known/visible artifact) and
@@ -339,7 +337,6 @@ static size_t obj_desc_name(char *buf, size_t max, size_t end,
 			  (object_name_is_visible(o_ptr) || known)) &&
 			(o_ptr->number != 1 || (mode & ODESC_PLURAL)));
 
-
 	/** Append extra names of various kinds **/
 
 	if ((object_name_is_visible(o_ptr) || known) && o_ptr->artifact)
@@ -349,8 +346,12 @@ static size_t obj_desc_name(char *buf, size_t max, size_t end,
 		strnfcat(buf, max, &end, " %s", o_ptr->ego->name);
 
 	else if (aware && !o_ptr->artifact &&
-			(o_ptr->kind->flavor || o_ptr->kind->tval == TV_SCROLL))
-		strnfcat(buf, max, &end, " of %s", o_ptr->kind->name);
+			(o_ptr->kind->flavor || o_ptr->kind->tval == TV_SCROLL)) {
+		if (terse)
+			strnfcat(buf, max, &end, " '%s'", o_ptr->kind->name);
+		else
+			strnfcat(buf, max, &end, " of %s", o_ptr->kind->name);
+	}
 
 	return end;
 }
@@ -373,10 +374,7 @@ static bool obj_desc_show_armor(const object_type *o_ptr)
 		case TV_SOFT_ARMOR:
 		case TV_HARD_ARMOR:
 		case TV_DRAG_ARMOR:
-		{
 			return TRUE;
-			break;
-		}
 	}
 
 	return FALSE;
@@ -394,9 +392,9 @@ static size_t obj_desc_chest(const object_type *o_ptr, char *buf, size_t max, si
 		strnfcat(buf, max, &end, " (empty)");
 
 	/* May be "disarmed" */
-	else if (o_ptr->pval[DEFAULT_PVAL] < 0)
+	else if (!is_locked_chest(o_ptr))
 	{
-		if (chest_traps[0 - o_ptr->pval[DEFAULT_PVAL]])
+		if (chest_trap_type(o_ptr) != 0)
 			strnfcat(buf, max, &end, " (disarmed)");
 		else
 			strnfcat(buf, max, &end, " (unlocked)");
@@ -406,7 +404,7 @@ static size_t obj_desc_chest(const object_type *o_ptr, char *buf, size_t max, si
 	else
 	{
 		/* Describe the traps */
-		switch (chest_traps[o_ptr->pval[DEFAULT_PVAL]])
+		switch (chest_trap_type(o_ptr))
 		{
 			case 0:
 				strnfcat(buf, max, &end, " (Locked)");
@@ -454,8 +452,7 @@ static size_t obj_desc_combat(const object_type *o_ptr, char *buf, size_t max,
 	object_flags(o_ptr, flags);
 	object_flags_known(o_ptr, flags_known);
 
-	if (of_has(flags, OF_SHOW_DICE))
-	{
+	if (of_has(flags, OF_SHOW_DICE)) {
 		/* Only display the real damage dice if the combat stats are known */
 		if (spoil || object_attack_plusses_are_visible(o_ptr))
 			strnfcat(buf, max, &end, " (%dd%d)", o_ptr->dd, o_ptr->ds);
@@ -463,8 +460,7 @@ static size_t obj_desc_combat(const object_type *o_ptr, char *buf, size_t max,
 			strnfcat(buf, max, &end, " (%dd%d)", o_ptr->kind->dd, o_ptr->kind->ds);
 	}
 
-	if (of_has(flags, OF_SHOW_MULT))
-	{
+	if (of_has(flags, OF_SHOW_MULT)) {
 		/* Display shooting power as part of the multiplier */
 		if (of_has(flags, OF_MIGHT) &&
 		    (spoil || object_flag_is_known(o_ptr, OF_MIGHT)))
@@ -474,10 +470,9 @@ static size_t obj_desc_combat(const object_type *o_ptr, char *buf, size_t max,
 	}
 
 	/* Show weapon bonuses */
-	if (spoil || object_attack_plusses_are_visible(o_ptr))
-	{
-		if (of_has(flags, OF_SHOW_MODS) || o_ptr->to_d || o_ptr->to_h)
-		{
+	if (spoil || object_attack_plusses_are_visible(o_ptr)) {
+		if (wield_slot(o_ptr) == INVEN_WIELD || wield_slot(o_ptr) == INVEN_BOW
+				|| obj_is_ammo(o_ptr) || o_ptr->to_d || o_ptr->to_h) {
 			/* Make an exception for body armor with only a to-hit penalty */
 			if (o_ptr->to_h < 0 && o_ptr->to_d == 0 &&
 			    (o_ptr->tval == TV_SOFT_ARMOR ||
@@ -493,17 +488,14 @@ static size_t obj_desc_combat(const object_type *o_ptr, char *buf, size_t max,
 
 
 	/* Show armor bonuses */
-	if (spoil || object_defence_plusses_are_visible(o_ptr))
-	{
+	if (spoil || object_defence_plusses_are_visible(o_ptr)) {
 		if (obj_desc_show_armor(o_ptr))
 			strnfcat(buf, max, &end, " [%d,%+d]", o_ptr->ac, o_ptr->to_a);
 		else if (o_ptr->to_a)
 			strnfcat(buf, max, &end, " [%+d]", o_ptr->to_a);
 	}
 	else if (obj_desc_show_armor(o_ptr))
-	{
 		strnfcat(buf, max, &end, " [%d]", object_was_sensed(o_ptr) ? o_ptr->ac : o_ptr->kind->ac);
-	}
 
 	return end;
 }
@@ -526,6 +518,7 @@ static size_t obj_desc_pval(const object_type *o_ptr, char *buf, size_t max,
 {
 	bitflag f[OF_SIZE], f2[OF_SIZE];
 	int i;
+	bool any = FALSE;
 
 	object_flags(o_ptr, f);
 	create_mask(f2, FALSE, OFT_PVAL, OFT_STAT, OFT_MAX);
@@ -535,9 +528,10 @@ static size_t obj_desc_pval(const object_type *o_ptr, char *buf, size_t max,
 	strnfcat(buf, max, &end, " <");
 	for (i = 0; i < o_ptr->num_pvals; i++) {
 		if (spoil || object_this_pval_is_visible(o_ptr, i)) {
-			if (i > 0)
+			if (any)
 				strnfcat(buf, max, &end, ", ");
 			strnfcat(buf, max, &end, "%+d", o_ptr->pval[i]);
+			any = TRUE;
 		}
 	}
 
@@ -677,11 +671,11 @@ static size_t obj_desc_aware(const object_type *o_ptr, char *buf, size_t max,
  *
  * \returns The number of bytes used of the buffer.
  */
-size_t object_desc(char *buf, size_t max, const object_type *o_ptr,
-				   odesc_detail_t mode)
+size_t object_desc(char *buf, size_t max, const object_type *o_ptr, int mode)
 {
 	bool prefix = mode & ODESC_PREFIX;
-	bool spoil = (mode & ODESC_SPOIL);
+	bool spoil = mode & ODESC_SPOIL;
+	bool terse = mode & ODESC_TERSE;
 	bool known;
 
 	size_t end = 0, i = 0;
@@ -699,6 +693,12 @@ size_t object_desc(char *buf, size_t max, const object_type *o_ptr,
 
 	/*** Some things get really simple descriptions ***/
 
+	if (o_ptr->marked == MARK_AWARE) {
+		if (prefix)
+			return strnfmt(buf, max, "an unknown item");
+		return strnfmt(buf, max, "unknown item");
+	}
+
 	if (o_ptr->tval == TV_GOLD)
 		return strnfmt(buf, max, "%d gold pieces worth of %s%s",
 				o_ptr->pval[DEFAULT_PVAL], o_ptr->kind->name,
@@ -707,7 +707,7 @@ size_t object_desc(char *buf, size_t max, const object_type *o_ptr,
 	/** Construct the name **/
 
 	/* Copy the base name to the buffer */
-	end = obj_desc_name(buf, max, end, o_ptr, prefix, mode, spoil);
+	end = obj_desc_name(buf, max, end, o_ptr, prefix, mode, spoil, terse);
 
 	if (mode & ODESC_COMBAT)
 	{
@@ -730,9 +730,7 @@ size_t object_desc(char *buf, size_t max, const object_type *o_ptr,
 		end = obj_desc_charges(o_ptr, buf, max, end);
 
 		if (mode & ODESC_STORE)
-		{
 			end = obj_desc_aware(o_ptr, buf, max, end);
-		}
 		else
 			end = obj_desc_inscrip(o_ptr, buf, max, end);
 	}
