@@ -86,7 +86,6 @@ static const flag_type pval_flags[] =
 	{ OF_WIS,     "wisdom" },
 	{ OF_DEX,     "dexterity" },
 	{ OF_CON,     "constitution" },
-	{ OF_CHR,     "charisma" },
 	{ OF_STEALTH, "stealth" },
 	{ OF_INFRA,   "infravision" },
 	{ OF_TUNNEL,  "tunneling" },
@@ -145,6 +144,14 @@ static const flag_type ignore_flags[] =
 	{ OF_IGNORE_COLD, "cold" },
 };
 
+static const flag_type hates_flags[] =
+{
+	{ OF_HATES_ACID, "acid" },
+	{ OF_HATES_ELEC, "electricity" },
+	{ OF_HATES_FIRE, "fire" },
+	{ OF_HATES_COLD, "cold" },
+};
+
 static const flag_type sustain_flags[] =
 {
 	{ OF_SUST_STR, "strength" },
@@ -152,7 +159,6 @@ static const flag_type sustain_flags[] =
 	{ OF_SUST_WIS, "wisdom" },
 	{ OF_SUST_DEX, "dexterity" },
 	{ OF_SUST_CON, "constitution" },
-	{ OF_SUST_CHR, "charisma" },
 };
 
 static const flag_type misc_flags[] =
@@ -202,7 +208,7 @@ static bool describe_stats(textblock *tb, const object_type *o_ptr,
 		bitflag flags[MAX_PVALS][OF_SIZE], oinfo_detail_t mode)
 {
 	const char *descs[N_ELEMENTS(pval_flags)];
-	size_t count, i;
+	size_t count = 0, i;
 	bool full = mode & OINFO_FULL;
 	bool dummy = mode & OINFO_DUMMY;
 	bool search = FALSE;
@@ -303,7 +309,7 @@ static bool describe_immune(textblock *tb, const bitflag flags[OF_SIZE])
 
 
 /*
- * Describe 'ignores' of an object.
+ * Describe IGNORE_ flags of an object.
  */
 static bool describe_ignores(textblock *tb, const bitflag flags[OF_SIZE])
 {
@@ -320,6 +326,24 @@ static bool describe_ignores(textblock *tb, const bitflag flags[OF_SIZE])
 	return TRUE;
 }
 
+/*
+ * Describe HATES_ flags of an object.
+ */
+static bool describe_hates(textblock *tb, const bitflag flags[OF_SIZE])
+{
+	const char *descs[N_ELEMENTS(hates_flags)];
+	size_t count = info_collect(tb, hates_flags, N_ELEMENTS(hates_flags),
+			flags, descs);
+
+	if (!count)
+		return FALSE;
+
+	textblock_append(tb, "Can be destroyed by ");
+	info_out_list(tb, descs, count);
+
+	return TRUE;
+}
+
 
 /*
  * Describe stat sustains.
@@ -489,83 +513,144 @@ static void calculate_missile_crits(player_state *state, int weight,
 	*div  = 100;
 }
 
-/*
- * Describe combat advantages.
- */
-static bool describe_combat(textblock *tb, const object_type *o_ptr,
-		oinfo_detail_t mode)
-{
-	bool full = mode & OINFO_FULL;
 
-	const char *desc[SL_MAX] = { 0 };
-	int i;
-	int mult[SL_MAX];
-	int cnt, dam, total_dam, plus = 0;
-	int xtra_postcrit = 0, xtra_precrit = 0;
-	int crit_mult, crit_div, crit_add;
+/*
+ * Describe blows.
+ */
+static bool describe_blows(textblock *tb, const object_type *o_ptr,
+		player_state state, bitflag f[OF_SIZE])
+{
 	int str_plus, dex_plus, old_blows = 0, new_blows, extra_blows;
 	int str_faster = -1, str_done = -1;
+	int dex_plus_bound;
+	int str_plus_bound;
+	int i;
+
+	bitflag tmp_f[OF_SIZE];
+
+	dex_plus_bound = STAT_RANGE - state.stat_ind[A_DEX];
+	str_plus_bound = STAT_RANGE - state.stat_ind[A_STR];
+
+	/* Write to the text block */
+	textblock_append_c(tb, TERM_L_GREEN, "%d.%d ",
+			state.num_blows / 100, (state.num_blows / 10) % 10);
+	textblock_append(tb, "blow%s/round.\n",
+			(state.num_blows > 100) ? "s" : "");
+
+	/* Check to see if extra STR or DEX would yield extra blows */
+	old_blows = state.num_blows;
+	extra_blows = 0;
+
+	/* First we need to look for extra blows on other items, as
+	 * state does not track these */
+	for (i = INVEN_BOW; i < INVEN_TOTAL; i++)
+	{
+		if (!p_ptr->inventory[i].kind)
+			continue;
+
+		object_flags_known(&p_ptr->inventory[i], tmp_f);
+
+		if (of_has(tmp_f, OF_BLOWS))
+			extra_blows += p_ptr->inventory[i].pval[which_pval(&p_ptr->inventory[i], OF_BLOWS)];
+	}
+
+	/* Then we add blows from the weapon being examined */
+	if (of_has(f, OF_BLOWS))
+		extra_blows += o_ptr->pval[which_pval(o_ptr, OF_BLOWS)];
+
+	/* Then we check for extra "real" blows */
+	for (dex_plus = 0; dex_plus < dex_plus_bound; dex_plus++)
+	{
+		for (str_plus = 0; str_plus < str_plus_bound; str_plus++)
+        {
+			state.stat_ind[A_STR] += str_plus;
+			state.stat_ind[A_DEX] += dex_plus;
+			new_blows = calc_blows(o_ptr, &state, extra_blows);
+
+			/* Test to make sure that this extra blow is a
+			 * new str/dex combination, not a repeat
+			 */
+			if ((new_blows - new_blows % 10) > (old_blows - old_blows % 10) &&
+				(str_plus < str_done ||
+				str_done == -1))
+			{
+				textblock_append(tb, "With +%d STR and +%d DEX you would get %d.%d blows\n",
+					str_plus, dex_plus, (new_blows / 100),
+					(new_blows / 10) % 10);
+				state.stat_ind[A_STR] -= str_plus;
+				state.stat_ind[A_DEX] -= dex_plus;
+				str_done = str_plus;
+				break;
+			}
+
+			/* If the combination doesn't increment
+			 * the displayed blows number, it might still
+			 * take a little less energy
+			 */
+			if (new_blows > old_blows &&
+				(str_plus < str_faster ||
+				str_faster == -1) &&
+				(str_plus < str_done ||
+				str_done == -1))
+			{
+				textblock_append(tb, "With +%d STR and +%d DEX you would attack a bit faster\n",
+					str_plus, dex_plus);
+				state.stat_ind[A_STR] -= str_plus;
+				state.stat_ind[A_DEX] -= dex_plus;
+				str_faster = str_plus;
+				continue;
+			}
+
+			state.stat_ind[A_STR] -= str_plus;
+			state.stat_ind[A_DEX] -= dex_plus;
+		}
+	}
+
+	return TRUE;
+}
+
+
+/*
+ * Describe damage.
+ */
+static bool describe_damage(textblock *tb, const object_type *o_ptr,
+		player_state state, bitflag f[OF_SIZE], oinfo_detail_t mode)
+{
+	const char *desc[SL_MAX] = { 0 };
+	size_t i, cnt;
+	int mult[SL_MAX];
+	int dice, sides, dam, total_dam, plus = 0;
+	int xtra_postcrit = 0, xtra_precrit = 0;
+	int crit_mult, crit_div, crit_add;
+	int old_blows = 0;
 	object_type *bow = &p_ptr->inventory[INVEN_BOW];
 
-	bitflag f[OF_SIZE], tmp_f[OF_SIZE], mask[OF_SIZE];
+	bitflag tmp_f[OF_SIZE], mask[OF_SIZE];
 
 	bool weapon = (wield_slot(o_ptr) == INVEN_WIELD);
 	bool ammo   = (p_ptr->state.ammo_tval == o_ptr->tval) &&
 	              (bow->kind);
 	int multiplier = 1;
+	bool full = mode & OINFO_FULL;
 
 	/* Create the "all slays" mask */
 	create_mask(mask, FALSE, OFT_SLAY, OFT_KILL, OFT_BRAND, OFT_MAX);
 
-	/* Abort if we've nothing to say */
-	if (mode & OINFO_DUMMY) return FALSE;
-
-	if (!weapon && !ammo)
-	{
-		/* Potions can have special text */
-		if (o_ptr->tval != TV_POTION ||
-				o_ptr->dd == 0 || o_ptr->ds == 0 ||
-				!object_flavor_is_aware(o_ptr))
-			return FALSE;
-
-		textblock_append(tb, "It can be thrown at creatures with damaging effect.\n");
-		return TRUE;
-	}
-
-	if (full) {
-		object_flags(o_ptr, f);
+	/* Use displayed dice if real dice not known */
+	if (full || object_attack_plusses_are_visible(o_ptr)) {
+		dice = o_ptr->dd;
+		sides = o_ptr->ds;
 	} else {
-		object_flags_known(o_ptr, f);
+		dice = o_ptr->kind->dd;
+		sides = o_ptr->kind->ds;
 	}
 
-	textblock_append_c(tb, TERM_L_WHITE, "Combat info:\n");
-
-	if (weapon)
-	{
-		/*
-		 * Get the player's hypothetical state, were they to be
-		 * wielding this item.
-		 */
-		player_state state;
-		int dex_plus_bound;
-		int str_plus_bound;
-
-		object_type inven[INVEN_TOTAL];
-
-		memcpy(inven, p_ptr->inventory, INVEN_TOTAL * sizeof(object_type));
-		inven[INVEN_WIELD] = *o_ptr;
-
-		if (full) object_know_all_flags(&inven[INVEN_WIELD]);
-
-		calc_bonuses(inven, &state, TRUE);
-		dex_plus_bound = STAT_RANGE - state.stat_ind[A_DEX];
-		str_plus_bound = STAT_RANGE - state.stat_ind[A_STR];
-
-		dam = ((o_ptr->ds + 1) * o_ptr->dd * 5);
+	/* Calculate damage */
+	dam = ((sides + 1) * dice * 5);
 
+	if (weapon)	{
 		xtra_postcrit = state.dis_to_d * 10;
-		if (object_attack_plusses_are_visible(o_ptr))
-		{
+		if (object_attack_plusses_are_visible(o_ptr) || full) {
 			xtra_precrit += o_ptr->to_d * 10;
 			plus += o_ptr->to_h;
 		}
@@ -573,97 +658,15 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 		calculate_melee_crits(&state, o_ptr->weight, plus,
 				&crit_mult, &crit_add, &crit_div);
 
-		/* Warn about heavy weapons */
-		if (adj_str_hold[state.stat_ind[A_STR]] < o_ptr->weight / 10)
-			textblock_append_c(tb, TERM_L_RED, "You are too weak to use this weapon.\n");
-
-		textblock_append_c(tb, TERM_L_GREEN, "%d.%d ",
-				state.num_blows / 100, (state.num_blows / 10) % 10);
-		textblock_append(tb, "blow%s/round.\n",
-				(state.num_blows > 100) ? "s" : "");
-
-		/* Check to see if extra STR or DEX would yield extra blows */
 		old_blows = state.num_blows;
-		extra_blows = 0;
-
-		/* First we need to look for extra blows on other items, as
-		 * state does not track these */
-		for (i = INVEN_BOW; i < INVEN_TOTAL; i++)
-		{
-			if (!p_ptr->inventory[i].kind)
-				continue;
-			object_flags_known(&p_ptr->inventory[i], tmp_f);
-
-			if (of_has(tmp_f, OF_BLOWS))
-				extra_blows += p_ptr->inventory[i].pval[which_pval(&p_ptr->inventory[i], OF_BLOWS)];
-		}
-
-		/* Then we add blows from the weapon being examined */
-		if (of_has(f, OF_BLOWS))
-			extra_blows += o_ptr->pval[which_pval(o_ptr, OF_BLOWS)];
-
-		/* Then we check for extra "real" blows */
-		for (dex_plus = 0; dex_plus < dex_plus_bound; dex_plus++)
-		{
-			for (str_plus = 0; str_plus < str_plus_bound; str_plus++)
-	        {
-				state.stat_ind[A_STR] += str_plus;
-				state.stat_ind[A_DEX] += dex_plus;
-				new_blows = calc_blows(o_ptr, &state, extra_blows);
-
-				/* Test to make sure that this extra blow is a
-				 * new str/dex combination, not a repeat
-				 */
-				if ((new_blows - new_blows % 10) > (old_blows - old_blows % 10) &&
-					(str_plus < str_done ||
-					str_done == -1))
-				{
-					textblock_append(tb, "With +%d STR and +%d DEX you would get %d.%d blows\n",
-						str_plus, dex_plus, (new_blows / 100),
-						(new_blows / 10) % 10);
-					state.stat_ind[A_STR] -= str_plus;
-					state.stat_ind[A_DEX] -= dex_plus;
-					str_done = str_plus;
-					break;
-				}
-
-				/* If the combination doesn't increment
-				 * the displayed blows number, it might still
-				 * take a little less energy
-				 */
-				if (new_blows > old_blows &&
-					(str_plus < str_faster ||
-					str_faster == -1) &&
-					(str_plus < str_done ||
-					str_done == -1))
-				{
-					textblock_append(tb, "With +%d STR and +%d DEX you would attack a bit faster\n",
-						str_plus, dex_plus);
-					state.stat_ind[A_STR] -= str_plus;
-					state.stat_ind[A_DEX] -= dex_plus;
-					str_faster = str_plus;
-					continue;
-				}
-
-				state.stat_ind[A_STR] -= str_plus;
-				state.stat_ind[A_DEX] -= dex_plus;
-			}
-		}
-	}
-	else
-	{
-		int tdis = 6 + 2 * p_ptr->state.ammo_mult;
-
-		if (object_attack_plusses_are_visible(o_ptr))
+	} else { /* Ammo */
+		if (object_attack_plusses_are_visible(o_ptr) || full)
 			plus += o_ptr->to_h;
 
 		calculate_missile_crits(&p_ptr->state, o_ptr->weight, plus,
 				&crit_mult, &crit_add, &crit_div);
 
-		/* Calculate damage */
-		dam = ((o_ptr->ds + 1) * o_ptr->dd * 5);
-
-		if (object_attack_plusses_are_visible(o_ptr))
+		if (object_attack_plusses_are_visible(o_ptr) || full)
 			dam += (o_ptr->to_d * 10);
 		if (object_attack_plusses_are_visible(bow))
 			dam += (bow->to_d * 10);
@@ -673,31 +676,27 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 		 * ammo (fully known) does not leak information about launcher */
 		object_flags_known(bow, tmp_f);
 		of_union(f, tmp_f);
-
-		textblock_append(tb, "Hits targets up to ");
-		textblock_append_c(tb, TERM_L_GREEN, format("%d", tdis * 10));
-		textblock_append(tb, " feet away.\n");
 	}
 
 	/* Collect slays */
 	/* Melee weapons get slays and brands from other items now */
-	if (weapon)
-	{
-		bool nonweap = FALSE;
+	if (weapon)	{
+		bool nonweap_slay = FALSE;
 
-		for (i = INVEN_LEFT; i < INVEN_TOTAL; i++)
-		{
+		for (i = INVEN_LEFT; i < INVEN_TOTAL; i++) {
 			if (!p_ptr->inventory[i].kind)
 				continue;
+
 			object_flags_known(&p_ptr->inventory[i], tmp_f);
 
-			of_inter(tmp_f, mask); /* strip out non-slays */
+			/* Strip out non-slays */
+			of_inter(tmp_f, mask);
 
 			if (of_union(f, tmp_f))
-				nonweap = TRUE;
+				nonweap_slay = TRUE;
 		}
 
-		if (nonweap)
+		if (nonweap_slay)
 			textblock_append(tb, "This weapon may benefit from one or more off-weapon brands or slays.\n");
 	}
 
@@ -705,19 +704,21 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 
 	if (ammo) multiplier = p_ptr->state.ammo_mult;
 
+	/* Output damage for creatures effected by the brands or slays */
 	cnt = list_slays(f, mask, desc, NULL, mult, TRUE);
-	for (i = 0; i < cnt; i++)
-	{
-		int melee_adj_mult = ammo ? 0 : 1; /* ammo mult adds fully, melee mult is times 1, so adds 1 less */
+	for (i = 0; i < cnt; i++) {
+		/* ammo mult adds fully, melee mult is times 1, so adds 1 less */
+		int melee_adj_mult = ammo ? 0 : 1;
+
 		/* Include bonus damage and slay in stated average */
 		total_dam = dam * (multiplier + mult[i] - melee_adj_mult) + xtra_precrit;
 		total_dam = (total_dam * crit_mult + crit_add) / crit_div;
 		total_dam += xtra_postcrit;
-		if (weapon) 
+
+		if (weapon)
 			total_dam = (total_dam * old_blows) / 100;
 		else
 			total_dam *= p_ptr->state.num_shots;
-		
 
 		if (total_dam <= 0)
 			textblock_append_c(tb, TERM_L_RED, "%d", 0);
@@ -727,7 +728,6 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 		else
 			textblock_append_c(tb, TERM_L_GREEN, "%d", total_dam / 10);
 
-
 		textblock_append(tb, " vs. %s, ", desc[i]);
 	}
 
@@ -737,6 +737,8 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 	total_dam = dam * multiplier + xtra_precrit;
 	total_dam = (total_dam * crit_mult + crit_add) / crit_div;
 	total_dam += xtra_postcrit;
+
+	/* Normal damage, not considering brands or slays */
 	if (weapon)
 		total_dam = (total_dam * old_blows) / 100;
 	else
@@ -753,22 +755,96 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 	if (cnt) textblock_append(tb, " vs. others");
 	textblock_append(tb, ".\n");
 
+	return TRUE;
+}
+
+
+/*
+ * Describe combat advantages.
+ */
+static bool describe_combat(textblock *tb, const object_type *o_ptr,
+		oinfo_detail_t mode)
+{
+	bool full = mode & OINFO_FULL;
+
+	object_type *bow = &p_ptr->inventory[INVEN_BOW];
+
+	bitflag f[OF_SIZE];
+
+	bool weapon = (wield_slot(o_ptr) == INVEN_WIELD);
+	bool ammo   = (p_ptr->state.ammo_tval == o_ptr->tval) &&
+	              (bow->kind);
+
+	/* The player's hypothetical state, were they to wield this item */
+	player_state state;
+
+	/* Abort if we've nothing to say */
+	if (mode & OINFO_DUMMY) return FALSE;
+
+	if (!weapon && !ammo) {
+		/* Potions can have special text */
+		if (o_ptr->tval != TV_POTION ||
+				o_ptr->dd == 0 || o_ptr->ds == 0 ||
+				!object_flavor_is_aware(o_ptr))
+			return FALSE;
+
+		textblock_append(tb, "It can be thrown at creatures with damaging effect.\n");
+		return TRUE;
+	}
+
+	if (full)
+		object_flags(o_ptr, f);
+	else
+		object_flags_known(o_ptr, f);
+
+	textblock_append_c(tb, TERM_L_WHITE, "Combat info:\n");
+
+	if (weapon)	{
+		object_type inven[INVEN_TOTAL];
+
+		memcpy(inven, p_ptr->inventory, INVEN_TOTAL * sizeof(object_type));
+		inven[INVEN_WIELD] = *o_ptr;
+
+		if (full) object_know_all_flags(&inven[INVEN_WIELD]);
+
+		/* Calculate the player's hypothetical state */
+		calc_bonuses(inven, &state, TRUE);
+
+		/* Warn about heavy weapons */
+		if (adj_str_hold[state.stat_ind[A_STR]] < o_ptr->weight / 10)
+			textblock_append_c(tb, TERM_L_RED, "You are too weak to use this weapon.\n");
+
+		/* Describe blows */
+		describe_blows(tb, o_ptr, state, f);
+	} else { /* Ammo */
+		/* Range of the weapon */
+		int tdis = 6 + 2 * p_ptr->state.ammo_mult;
+
+		/* Output the range */
+		textblock_append(tb, "Hits targets up to ");
+		textblock_append_c(tb, TERM_L_GREEN, format("%d", tdis * 10));
+		textblock_append(tb, " feet away.\n");
+	}
+
+	/* Describe damage */
+	describe_damage(tb, o_ptr, state, f, mode);
+
 	/* Note the impact flag */
 	if (of_has(f, OF_IMPACT))
 		textblock_append(tb, "Sometimes creates earthquakes on impact.\n");
 
 	/* Add breakage chance */
-	if (ammo)
-	{
+	if (ammo) {
 		int chance = breakage_chance(o_ptr, TRUE);
 		textblock_append_c(tb, TERM_L_GREEN, "%d%%", chance);
 		textblock_append(tb, " chance of breaking upon contact.\n");
 	}
 
-	/* You always have something to say... */
+	/* Something has been said */
 	return TRUE;
 }
 
+
 /*
  * Describe objects that can be used for digging.
  */
@@ -888,36 +964,38 @@ static bool describe_food(textblock *tb, const object_type *o_ptr,
  * Describe things that look like lights.
  */
 static bool describe_light(textblock *tb, const object_type *o_ptr,
-		const bitflag flags[OF_SIZE], bool terse)
+		const bitflag flags[OF_SIZE], oinfo_detail_t mode)
 {
 	int rad = 0;
 
 	bool artifact = o_ptr->artifact ? TRUE : FALSE;
 	bool no_fuel = of_has(flags, OF_NO_FUEL) ? TRUE : FALSE;
 	bool is_light = (o_ptr->tval == TV_LIGHT) ? TRUE : FALSE;
+	bool terse = mode & OINFO_TERSE;
 
-	if (o_ptr->tval != TV_LIGHT && !of_has(flags, OF_LIGHT))
+	if (!is_light && !of_has(flags, OF_LIGHT))
+		return FALSE;
+
+	/* Prevent unidentified objects (especially artifact lights) from showing bad radius
+	 and refueling info, but allow it to appear in ego knowledge and spoilers */
+	if (!object_is_known(o_ptr) && !(mode & (OINFO_EGO | OINFO_FULL | OINFO_DUMMY)))
 		return FALSE;
 
 	/* Work out radius */
-	if (artifact && is_light)
-		rad = 3;
-	else if (is_light)
-		rad = 2;
 	if (of_has(flags, OF_LIGHT))
-		rad++;
+		rad = o_ptr->pval[which_pval(o_ptr, OF_LIGHT)];
 
 	/* Describe here */
 	textblock_append(tb, "Radius ");
 	textblock_append_c(tb, TERM_L_GREEN, format("%d", rad));
 	if (no_fuel && !artifact)
 		textblock_append(tb, " light.  No fuel required.");
-	else if (is_light && o_ptr->sval == SV_LIGHT_TORCH)
-		textblock_append(tb, " light, reduced when running out of fuel.");
+/*	else if (is_light && o_ptr->sval == SV_LIGHT_TORCH)
+		textblock_append(tb, " light, reduced when running out of fuel."); */
 	else
 		textblock_append(tb, " light.");
 
-	if (!terse && is_light && !no_fuel)
+	if (!terse && is_light && !no_fuel && o_ptr->sval != SV_LIGHT_TORCH)
 	{
 		const char *name = (o_ptr->sval == SV_LIGHT_TORCH) ? "torches" : "lanterns";
 		int turns = (o_ptr->sval == SV_LIGHT_TORCH) ? FUEL_TORCH : FUEL_LAMP;
@@ -994,7 +1072,7 @@ static bool describe_effect(textblock *tb, const object_type *o_ptr, bool full,
 	else if (o_ptr->tval == TV_FOOD)
 		textblock_append(tb, "When eaten, it ");
 	else if (o_ptr->tval == TV_POTION)
-		textblock_append(tb, "When drunk, it ");
+		textblock_append(tb, "When quaffed, it ");
 	else if (o_ptr->tval == TV_SCROLL)
 	    textblock_append(tb, "When read, it ");
 	else
@@ -1056,10 +1134,14 @@ static bool describe_effect(textblock *tb, const object_type *o_ptr, bool full,
 }
 
 
-static bool describe_origin(textblock *tb, const object_type *o_ptr)
+static bool describe_origin(textblock *tb, const object_type *o_ptr, bool terse)
 {
 	char origin_text[80];
 
+	/* Only give this info in chardumps if wieldable */
+	if (terse && !obj_can_wear(o_ptr))
+		return FALSE;
+
 	if (o_ptr->origin_depth)
 		strnfmt(origin_text, sizeof(origin_text), "%d feet (level %d)",
 		        o_ptr->origin_depth * 50, o_ptr->origin_depth);
@@ -1233,10 +1315,10 @@ static bool describe_ego(textblock *tb, const struct ego_item *ego)
 /*
  * Output object information
  */
-static textblock *object_info_out(const object_type *o_ptr, oinfo_detail_t mode)
+static textblock *object_info_out(const object_type *o_ptr, int mode)
 {
 	bitflag flags[OF_SIZE];
-	bitflag pval_flags[MAX_PVALS][OF_SIZE];
+	bitflag pv_flags[MAX_PVALS][OF_SIZE];
 	bool something = FALSE;
 	bool known = object_is_known(o_ptr);
 
@@ -1247,16 +1329,26 @@ static textblock *object_info_out(const object_type *o_ptr, oinfo_detail_t mode)
 
 	textblock *tb = textblock_new();
 
+	/* Unaware objects get simple descriptions */
+	if (!full && (o_ptr->marked == MARK_AWARE)){
+		textblock_append(tb, "\n\nYou do not know what this is.\n");
+		return tb;
+	}
+	
 	/* Grab the object flags */
 	if (full) {
 		object_flags(o_ptr, flags);
-		object_pval_flags(o_ptr, pval_flags);
+		object_pval_flags(o_ptr, pv_flags);
 	} else {
 		object_flags_known(o_ptr, flags);
-		object_pval_flags_known(o_ptr, pval_flags);
+		object_pval_flags_known(o_ptr, pv_flags);
+
+		/* Don't include base flags when terse */
+		if (terse)
+			of_diff(flags, o_ptr->kind->base->flags);
 	}
 
-	if (subjective) describe_origin(tb, o_ptr);
+	if (subjective) describe_origin(tb, o_ptr, terse);
 	if (!terse) describe_flavor_text(tb, o_ptr, ego);
 
 	if (!full && !known)
@@ -1266,32 +1358,33 @@ static textblock *object_info_out(const object_type *o_ptr, oinfo_detail_t mode)
 	}
 
 	if (describe_curses(tb, o_ptr, flags)) something = TRUE;
-	if (describe_stats(tb, o_ptr, pval_flags, mode)) something = TRUE;
+	if (describe_stats(tb, o_ptr, pv_flags, mode)) something = TRUE;
 	if (describe_slays(tb, flags, o_ptr->tval)) something = TRUE;
 	if (describe_immune(tb, flags)) something = TRUE;
 	if (describe_ignores(tb, flags)) something = TRUE;
+	dedup_hates_flags(flags);
+	if (describe_hates(tb, flags)) something = TRUE;
 	if (describe_sustains(tb, flags)) something = TRUE;
 	if (describe_misc_magic(tb, flags)) something = TRUE;
+	if (describe_light(tb, o_ptr, flags, mode)) something = TRUE;
 	if (ego && describe_ego(tb, o_ptr->ego)) something = TRUE;
 	if (something) textblock_append(tb, "\n");
 
-	if (!ego && describe_effect(tb, o_ptr, full, terse, subjective))
-	{
+	if (!ego && describe_effect(tb, o_ptr, full, terse, subjective)) {
 		something = TRUE;
 		textblock_append(tb, "\n");
 	}
 
-	if (subjective && describe_combat(tb, o_ptr, mode))
-	{
+	if (subjective && describe_combat(tb, o_ptr, mode)) {
 		something = TRUE;
 		textblock_append(tb, "\n");
 	}
 
 	if (!terse && describe_food(tb, o_ptr, subjective, full)) something = TRUE;
-	if (describe_light(tb, o_ptr, flags, terse)) something = TRUE;
 	if (!terse && subjective && describe_digger(tb, o_ptr, mode)) something = TRUE;
 
-	if (!something)
+	/* Hack? Don't append anything in terse (for chararacter dump), since that seems to cause extra linebreaks */
+	if (!something && !terse)
 		textblock_append(tb, "\n\nThis item does not seem to possess any special abilities.");
 
 	return tb;
