@@ -17,9 +17,11 @@
  */
 
 #include "angband.h"
-#include "button.h"
 #include "tvalsval.h"
 #include "cmds.h"
+#include "game-cmd.h"
+#include "keymap.h"
+
 
 /*
  * Display a list of objects.  Each object may be prefixed with a label.
@@ -38,11 +40,13 @@ static void show_obj_list(int num_obj, int num_head, char labels[50][80],
 	char o_name[50][80];
 	char tmp_val[80];
 	
-	bool in_term;
-	
-	in_term = (mode & OLIST_WINDOW) ? TRUE : FALSE;
+	bool in_term = (mode & OLIST_WINDOW) ? TRUE : FALSE;
+	bool terse = FALSE;
 
 	if (in_term) max_len = 40;
+	if (in_term && Term->wid < 40) mode &= ~(OLIST_WEIGHT);
+
+	if (Term->wid < 50) terse = TRUE;
 
 	/* Calculate name offset and max name length */
 	for (i = 0; i < num_obj; i++)
@@ -58,7 +62,8 @@ static void show_obj_list(int num_obj, int num_head, char labels[50][80],
 				strnfmt(o_name[i], sizeof(o_name[i]), "(nothing)");
 		}
 		else
-			object_desc(o_name[i], sizeof(o_name[i]), o_ptr, ODESC_PREFIX | ODESC_FULL);
+			object_desc(o_name[i], sizeof(o_name[i]), o_ptr, ODESC_PREFIX | ODESC_FULL |
+				(terse ? ODESC_TERSE : 0));
 
 		/* Max length of label + object name */
 		max_len = MAX(max_len, strlen(labels[i]) + strlen(o_name[i]));
@@ -140,7 +145,7 @@ static void show_obj_list(int num_obj, int num_head, char labels[50][80],
 			ex_offset_ctr += 9;
 		}
 
-		if (mode & OLIST_FAIL)
+		if (mode & OLIST_FAIL && obj_can_fail(o_ptr))
 		{
 			int fail = (9 + get_use_device_chance(o_ptr)) / 10;
 			if (object_effect_is_known(o_ptr))
@@ -175,7 +180,7 @@ static void show_obj_list(int num_obj, int num_head, char labels[50][80],
 			if (j == p_ptr->quiver_slots - 1 && p_ptr->quiver_remainder > 0)
 				count = p_ptr->quiver_remainder;
 			else
-				count = 99;
+				count = MAX_STACK_SIZE-1;
 
 			/* Clear the line */
 			prt("", row + i, MAX(col - 2, 0));
@@ -212,7 +217,7 @@ static void show_obj_list(int num_obj, int num_head, char labels[50][80],
  * off to show_obj_list() for display.  Mode flags documented in
  * object.h
  */
-void show_inven(olist_detail_t mode)
+void show_inven(int mode)
 {
 	int i, last_slot = -1;
 	int diff = weight_remaining();
@@ -254,7 +259,7 @@ void show_inven(olist_detail_t mode)
 		if (item_tester_okay(o_ptr))
 			strnfmt(labels[num_obj], sizeof(labels[num_obj]), "%c) ", index_to_label(i));
 
-		/* Unacceptable items are still displayed in term windows */
+		/* Unacceptable items are still sometimes shown */
 		else if (in_term)
 			my_strcpy(labels[num_obj], "   ", sizeof(labels[num_obj]));
 
@@ -280,19 +285,20 @@ void show_inven(olist_detail_t mode)
  * off to show_obj_list() for display.  Mode flags documented in
  * object.h
  */
-void show_equip(olist_detail_t mode)
+void show_equip(int mode)
 {
 	int i, last_slot = 0;
 
 	object_type *o_ptr;
 
-   int num_obj = 0;
-   char labels[50][80];
-   object_type *objects[50];
+	int num_obj = 0;
+	char labels[50][80];
+	object_type *objects[50];
 
 	char tmp_val[80];
 
-   bool in_term = (mode & OLIST_WINDOW) ? TRUE : FALSE;
+	bool in_term = (mode & OLIST_WINDOW) ? TRUE : FALSE;
+	bool show_empty = (mode & OLIST_SEMPTY) ? TRUE : FALSE;
 
 	/* Find the last equipment slot to display */
 	for (i = INVEN_WIELD; i < ALL_INVEN_TOTAL; i++)
@@ -334,8 +340,8 @@ void show_equip(olist_detail_t mode)
 		if (item_tester_okay(o_ptr))
 			strnfmt(labels[num_obj], sizeof(labels[num_obj]), "%c) ", index_to_label(i));
 
-		/* Unacceptable items are still displayed in term windows */
-		else if (in_term)
+		/* Unacceptable items are still sometimes shown */
+		else if ((!o_ptr->kind && show_empty) || in_term)
 			my_strcpy(labels[num_obj], "   ", sizeof(labels[num_obj]));
 
 		/* Unacceptable items are skipped in the main window */
@@ -360,15 +366,15 @@ void show_equip(olist_detail_t mode)
  * off to show_obj_list() for display.  Mode flags documented in
  * object.h
  */
-void show_floor(const int *floor_list, int floor_num, olist_detail_t mode)
+void show_floor(const int *floor_list, int floor_num, int mode)
 {
 	int i;
 
 	object_type *o_ptr;
 
-   int num_obj = 0;
-   char labels[50][80];
-   object_type *objects[50];
+	int num_obj = 0;
+	char labels[50][80];
+	object_type *objects[50];
 
 	if (floor_num > MAX_FLOOR_STACK) floor_num = MAX_FLOOR_STACK;
 
@@ -434,11 +440,11 @@ bool verify_item(const char *prompt, int item)
 
 
 /*
- * Hack -- allow user to "prevent" certain choices.
+ * Hack -- prevent certain choices depending on the inscriptions on the item.
  *
  * The item can be negative to mean "item on floor".
  */
-static bool get_item_allow(int item, unsigned char ch, bool is_harmless)
+bool get_item_allow(int item, unsigned char ch, cmd_code cmd, bool is_harmless)
 {
 	object_type *o_ptr;
 	char verify_inscrip[] = "!*";
@@ -451,19 +457,35 @@ static bool get_item_allow(int item, unsigned char ch, bool is_harmless)
 	else
 		o_ptr = object_byid(0 - item);
 
-	/* Check for a "prevention" inscription */
+	/* Hack - Only shift the command key if it actually needs to be shifted. */
+	if (ch < 0x20)
+		ch = UN_KTRL(ch);
+
+	/* The inscription to look for */
 	verify_inscrip[1] = ch;
 
-	/* Find both sets of inscriptions, add together, and prompt that number of times */
+	/* Look for the inscription */
 	n = check_for_inscrip(o_ptr, verify_inscrip);
 
+	/* Also look for for the inscription '!*' */
 	if (!is_harmless)
 		n += check_for_inscrip(o_ptr, "!*");
 
-	while (n--)
-	{
-		if (!verify_item("Really try", item))
-			return (FALSE);
+	/* Choose string for the prompt */
+	if (n) {
+		char prompt[1024];
+
+		const char *verb = cmd_get_verb(cmd);
+		if (!verb)
+			verb = "do that with";
+
+		strnfmt(prompt, sizeof(prompt), "Really %s", verb);
+
+		/* Promt for confirmation n times */
+		while (n--) {
+			if (!verify_item(prompt, item))
+				return (FALSE);
+		}
 	}
 
 	/* Allow it */
@@ -485,6 +507,7 @@ static int get_tag(int *cp, char tag, cmd_code cmd, bool quiver_tags)
 {
 	int i;
 	const char *s;
+	int mode = OPT(rogue_like_commands) ? KEYMAP_MODE_ROGUE : KEYMAP_MODE_ORIG;
 
 	/* (f)ire is handled differently from all others, due to the quiver */
 	if (quiver_tags)
@@ -515,6 +538,8 @@ static int get_tag(int *cp, char tag, cmd_code cmd, bool quiver_tags)
 		/* Process all tags */
 		while (s)
 		{
+			unsigned char cmdkey;
+
 			/* Check the normal tags */
 			if (s[1] == tag)
 			{
@@ -525,8 +550,14 @@ static int get_tag(int *cp, char tag, cmd_code cmd, bool quiver_tags)
 				return (TRUE);
 			}
 
+			cmdkey = cmd_lookup_key(cmd, mode);
+
+			/* Hack - Only shift the command key if it actually needs to be shifted. */
+			if (cmdkey < 0x20)
+				cmdkey = UN_KTRL(cmdkey);
+
 			/* Check the special tags */
-			if ((cmd_lookup(s[1]) == cmd) && (s[2] == tag))
+			if ((s[1] == cmdkey) && (s[2] == tag))
 			{
 				/* Save the actual inventory ID */
 				*cp = i;
@@ -598,9 +629,11 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 {
 	int py = p_ptr->py;
 	int px = p_ptr->px;
-	unsigned char cmdkey = cmd_lookup_key(cmd);
+	unsigned char cmdkey = cmd_lookup_key(cmd,
+			OPT(rogue_like_commands) ? KEYMAP_MODE_ROGUE : KEYMAP_MODE_ORIG);
 
-	struct keypress which;
+	//struct keypress which;
+	ui_event press;
 
 	int j, k;
 
@@ -615,11 +648,10 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 	bool use_inven = ((mode & USE_INVEN) ? TRUE : FALSE);
 	bool use_equip = ((mode & USE_EQUIP) ? TRUE : FALSE);
 	bool use_floor = ((mode & USE_FLOOR) ? TRUE : FALSE);
-	bool use_quiver = ((mode & QUIVER_TAGS) ? TRUE : FALSE);
 	bool is_harmless = ((mode & IS_HARMLESS) ? TRUE : FALSE);
 	bool quiver_tags = ((mode & QUIVER_TAGS) ? TRUE : FALSE);
 
-	olist_detail_t olist_mode = 0;
+	int olist_mode = 0;
 
 	bool allow_inven = FALSE;
 	bool allow_equip = FALSE;
@@ -635,14 +667,21 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 
 	bool show_list = TRUE;
 
+	/* Hack - Only shift the command key if it actually needs to be shifted. */
+	if (cmdkey < 0x20)
+		cmdkey = UN_KTRL(cmdkey);
 
 	/* Object list display modes */
 	if (mode & SHOW_FAIL)
-		olist_mode |= (OLIST_FAIL);
+		olist_mode |= OLIST_FAIL;
 	else
-		olist_mode |= (OLIST_WEIGHT);
+		olist_mode |= OLIST_WEIGHT;
+
 	if (mode & SHOW_PRICES)
-		olist_mode |= (OLIST_PRICE);
+		olist_mode |= OLIST_PRICE;
+
+	if (mode & SHOW_EMPTY)
+		olist_mode |= OLIST_SEMPTY;
 
 	/* Paranoia XXX XXX XXX */
 	message_flush();
@@ -686,7 +725,7 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 
 
 	/* Scan all non-gold objects in the grid */
-	floor_num = scan_floor(floor_list, N_ELEMENTS(floor_list), py, px, 0x03);
+	floor_num = scan_floor(floor_list, N_ELEMENTS(floor_list), py, px, 0x0B);
 
 	/* Full floor */
 	f1 = 0;
@@ -715,23 +754,27 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 	else
 	{
 		/* Hack -- Start on equipment if requested */
-		if ((p_ptr->command_wrk == USE_EQUIP) && use_equip)
+		if ((p_ptr->command_wrk == USE_EQUIP) && allow_equip)
 			p_ptr->command_wrk = USE_EQUIP;
+		else if ((p_ptr->command_wrk == USE_INVEN) && allow_inven)
+			p_ptr->command_wrk = USE_INVEN;
+		else if ((p_ptr->command_wrk == USE_FLOOR) && allow_floor)
+			p_ptr->command_wrk = USE_FLOOR;
 
 		/* If we are using the quiver then start on equipment */
-		else if (use_quiver)
+		else if (quiver_tags && allow_equip)
 			p_ptr->command_wrk = USE_EQUIP;
 
 		/* Use inventory if allowed */
-		else if (use_inven)
+		else if (use_inven && allow_inven)
 			p_ptr->command_wrk = USE_INVEN;
 
 		/* Use equipment if allowed */
-		else if (use_equip)
+		else if (use_equip && allow_equip)
 			p_ptr->command_wrk = USE_EQUIP;
 
 		/* Use floor if allowed */
-		else if (use_floor)
+		else if (use_floor && allow_floor)
 			p_ptr->command_wrk = USE_FLOOR;
 
 		/* Hack -- Use (empty) inventory */
@@ -787,8 +830,15 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 		/* Viewing inventory */
 		if (p_ptr->command_wrk == USE_INVEN)
 		{
+			int nmode = olist_mode;
+
+			/* Show the quiver counts in certain cases, like the 'i' command */
+			if (mode & SHOW_QUIVER)
+				nmode |= OLIST_QUIVER;
+
 			/* Redraw if needed */
-			if (show_list) show_inven(olist_mode);
+			if (show_list)
+				show_inven(nmode);
 
 			/* Begin the prompt */
 			strnfmt(out_val, sizeof(out_val), "Inven:");
@@ -808,21 +858,18 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 			if (!show_list)
 			{
 				my_strcat(out_val, " * to see,", sizeof(out_val));
-				button_add("[*]", '*');
 			}
 
 			/* Indicate legality of "toggle" */
 			if (use_equip)
 			{
 				my_strcat(out_val, " / for Equip,", sizeof(out_val));
-				button_add("[/]", '/');
 			}
 
 			/* Indicate legality of the "floor" */
 			if (allow_floor)
 			{
 				my_strcat(out_val, " - for floor,", sizeof(out_val));
-				button_add("[-]", '-');
 			}
 		}
 
@@ -850,21 +897,18 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 			if (!show_list)
 			{
 				my_strcat(out_val, " * to see,", sizeof(out_val));
-				button_add("[*]", '*');
 			}
 
 			/* Indicate legality of "toggle" */
 			if (use_inven)
 			{
 				my_strcat(out_val, " / for Inven,", sizeof(out_val));
-				button_add("[/]", '/');
 			}
 
 			/* Indicate legality of the "floor" */
 			if (allow_floor)
 			{
 				my_strcat(out_val, " - for floor,", sizeof(out_val));
-				button_add("[!]", '!');
 			}
 		}
 
@@ -891,43 +935,173 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 			if (!show_list)
 			{
 				my_strcat(out_val, " * to see,", sizeof(out_val));
-				button_add("[*]", '*');
 			}
 
 			/* Append */
 			if (use_inven)
 			{
 				my_strcat(out_val, " / for Inven,", sizeof(out_val));
-				button_add("[/]", '/');
 			}
 
 			/* Append */
 			else if (use_equip)
 			{
 				my_strcat(out_val, " / for Equip,", sizeof(out_val));
-				button_add("[/]", '/');
 			}
 		}
 
-		redraw_stuff(p_ptr);
-
 		/* Finish the prompt */
 		my_strcat(out_val, " ESC", sizeof(out_val));
 
-		/* Build the prompt */
-		strnfmt(tmp_val, sizeof(tmp_val), "(%s) %s", out_val, pmt);
-
-		/* Show the prompt */
-		prt(tmp_val, 0, 0);
+		/* if we have a prompt header, show the part that we just built */
+		if (pmt) {
+			/* Build the prompt */
+			strnfmt(tmp_val, sizeof(tmp_val), "(%s) %s", out_val, pmt);
 
+			/* Show the prompt */
+			prt(tmp_val, 0, 0);
+		}
 
 		/* Get a key */
-		which = inkey();
+		//which = inkey();
+		press = inkey_m();
 
 		/* Parse it */
-		switch (which.code)
+		if (press.type == EVT_MOUSE) {
+			if (press.mouse.button == 2) {
+				done = TRUE;
+			} else
+			if (press.mouse.button == 1) {
+				k = -1;
+				if (p_ptr->command_wrk == USE_INVEN) {
+					if (press.mouse.y == 0) {
+						if (use_equip) {
+							p_ptr->command_wrk = USE_EQUIP;
+						} else
+						if (allow_floor) {
+							p_ptr->command_wrk = USE_FLOOR;
+						}
+					} else
+					if ((press.mouse.y <= i2-i1+1) ){
+					//&& (press.mouse.x > Term->wid - 1 - max_len - ex_width)) {
+						//k = label_to_inven(index_to_label(i1+press.mouse.y-1));
+						/* get the item index, allowing for skipped indices */
+						for (j = i1; j <= i2; j++) {
+							if (get_item_okay(j)) {
+								if (press.mouse.y == 1) {
+									k = j;
+									break;
+								}
+								press.mouse.y--;
+							}
+						}
+					}
+				} else
+				if (p_ptr->command_wrk == USE_EQUIP) {
+					if (press.mouse.y == 0) {
+						if (allow_floor) {
+							p_ptr->command_wrk = USE_FLOOR;
+						} else
+						if (use_inven) {
+							p_ptr->command_wrk = USE_INVEN;
+						}
+					} else
+					if (press.mouse.y <= e2-e1+1) {
+						if (olist_mode & OLIST_SEMPTY) {
+							/* If we are showing empties, just set the object (empty objects will just keep the loop going) */
+							k = label_to_equip(index_to_label(e1+press.mouse.y-1));
+						}
+						else {
+							/* get the item index, allowing for skipped indices */
+							for (j = e1; j <= e2; j++) {
+								/* skip the quiver slot which is a blank line in the list */
+								if (j == 36) {
+									press.mouse.y--;
+								} else
+									if (get_item_okay(j)) {
+										if (press.mouse.y == 1) {
+											k = j;
+											break;
+										}
+										press.mouse.y--;
+									}
+							}
+						}
+					}
+				} else
+				if (p_ptr->command_wrk == USE_FLOOR) {
+					if (press.mouse.y == 0) {
+						if (use_inven) {
+							p_ptr->command_wrk = USE_INVEN;
+						} else
+						if (use_equip) {
+							p_ptr->command_wrk = USE_EQUIP;
+						}
+					} else
+					if ((press.mouse.y <= floor_num) && (press.mouse.y >= 1)) {
+						/* Special index */
+						k = 0 - floor_list[press.mouse.y-1];
+						/* get the item index, allowing for skipped indices */
+						for (j = f1; j <= f2; j++) {
+							if (get_item_okay(0 - floor_list[j])) {
+								if (press.mouse.y == 1) {
+									k = 0 - floor_list[j];
+									break;
+								}
+								press.mouse.y--;
+							}
+						}
+						/* check the bounds the item number */
+						if (k < 0) {
+							/* Allow player to "refuse" certain actions */
+							if (!get_item_allow(k, cmdkey, cmd, is_harmless))
+							{
+								done = TRUE;
+							}
+
+							/* Accept that choice */
+							(*cp) = k;
+							item = TRUE;
+							done = TRUE;
+						} else {
+							/* set k to a value that will be invalid below */
+							k = -1;
+						}
+					}
+				}
+				if (k >= 0) {
+					/* Validate the item */
+					if (!get_item_okay(k)) {
+						bell("Illegal object choice (normal)!");
+					}
+
+					/* Allow player to "refuse" certain actions */
+					if (!get_item_allow(k, cmdkey, cmd, is_harmless)) {
+						done = TRUE;
+					}
+
+					/* Accept that choice */
+					(*cp) = k;
+					item = TRUE;
+					done = TRUE;
+				} else
+				if (press.mouse.y == 0) {
+					/* Hack -- Fix screen */
+					if (show_list) {
+						/* Load screen */
+						screen_load();
+
+						/* Save screen */
+						screen_save();
+					}
+				}
+			}
+		} else
+		//switch (which.code)
+		switch (press.key.code)
 		{
 			case ESCAPE:
+			case ' ':
 			{
 				done = TRUE;
 				break;
@@ -988,7 +1162,7 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 						k = 0 - floor_list[0];
 
 						/* Allow player to "refuse" certain actions */
-						if (!get_item_allow(k, cmdkey, is_harmless))
+						if (!get_item_allow(k, cmdkey, cmd, is_harmless))
 						{
 							done = TRUE;
 							break;
@@ -1026,7 +1200,7 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 					if (!get_item_okay(k)) continue;
 
 					/* Allow player to "refuse" certain actions */
-					if (!get_item_allow(k, cmdkey, is_harmless)) continue;
+					if (!get_item_allow(k, cmdkey, cmd, is_harmless)) continue;
 
 					/* Accept that choice */
 					(*cp) = k;
@@ -1045,7 +1219,8 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 			case '7': case '8': case '9':
 			{
 				/* Look up the tag */
-				if (!get_tag(&k, which.code, cmd, quiver_tags))
+				//if (!get_tag(&k, which.code, cmd, quiver_tags))
+				if (!get_tag(&k, press.key.code, cmd, quiver_tags))
 				{
 					bell("Illegal object choice (tag)!");
 					break;
@@ -1066,7 +1241,7 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 				}
 
 				/* Allow player to "refuse" certain actions */
-				if (!get_item_allow(k, cmdkey, is_harmless))
+				if (!get_item_allow(k, cmdkey, cmd, is_harmless))
 				{
 					done = TRUE;
 					break;
@@ -1079,8 +1254,7 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 				break;
 			}
 
-			case '\n':
-			case '\r':
+			case KC_ENTER:
 			{
 				/* Choose "default" inventory item */
 				if (p_ptr->command_wrk == USE_INVEN)
@@ -1130,7 +1304,7 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 				}
 
 				/* Allow player to "refuse" certain actions */
-				if (!get_item_allow(k, cmdkey, is_harmless))
+				if (!get_item_allow(k, cmdkey, cmd, is_harmless))
 				{
 					done = TRUE;
 					break;
@@ -1148,15 +1322,18 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 				bool verify;
 
 				/* Note verify */
-				verify = (isupper((unsigned char)which.code) ? TRUE : FALSE);
+				//verify = (isupper((unsigned char)which.code) ? TRUE : FALSE);
+				verify = (isupper((unsigned char)press.key.code) ? TRUE : FALSE);
 
 				/* Lowercase */
-				which.code = tolower((unsigned char)which.code);
+				//which.code = tolower((unsigned char)which.code);
+				press.key.code = tolower((unsigned char)press.key.code);
 
 				/* Convert letter to inventory index */
 				if (p_ptr->command_wrk == USE_INVEN)
 				{
-					k = label_to_inven(which.code);
+					//k = label_to_inven(which.code);
+					k = label_to_inven(press.key.code);
 
 					if (k < 0)
 					{
@@ -1168,7 +1345,8 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 				/* Convert letter to equipment index */
 				else if (p_ptr->command_wrk == USE_EQUIP)
 				{
-					k = label_to_equip(which.code);
+					//k = label_to_equip(which.code);
+					k = label_to_equip(press.key.code);
 
 					if (k < 0)
 					{
@@ -1180,7 +1358,8 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 				/* Convert letter to floor index */
 				else
 				{
-					k = (islower((unsigned char)which.code) ? A2I((unsigned char)which.code) : -1);
+					//k = (islower((unsigned char)which.code) ? A2I((unsigned char)which.code) : -1);
+					k = (islower((unsigned char)press.key.code) ? A2I((unsigned char)press.key.code) : -1);
 
 					if (k < 0 || k >= floor_num)
 					{
@@ -1207,7 +1386,7 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 				}
 
 				/* Allow player to "refuse" certain actions */
-				if (!get_item_allow(k, cmdkey, is_harmless))
+				if (!get_item_allow(k, cmdkey, cmd, is_harmless))
 				{
 					done = TRUE;
 					break;
@@ -1233,14 +1412,6 @@ bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode)
 		show_list = FALSE;
 	}
 
-
-	/* Kill buttons */
-	button_kill('*');
-	button_kill('/');
-	button_kill('-');
-	button_kill('!');
-	redraw_stuff(p_ptr);
- 
 	/* Forget the item_tester_tval restriction */
 	item_tester_tval = 0;
 
