@@ -8,10 +8,37 @@
 #include "z-bitflag.h"
 #include "game-cmd.h"
 #include "cave.h"
+#include "object/obj-flag.h"
 
 struct player;
 
-/*** Constants ***/
+
+/*** Game constants ***/
+
+/*
+ * Refueling constants
+ */
+#define FUEL_TORCH                5000  /* Maximum amount of fuel in a torch */
+#define FUEL_LAMP                15000  /* Maximum amount of fuel in a lantern */
+#define DEFAULT_TORCH       FUEL_TORCH  /* Default amount of fuel in a torch */
+#define DEFAULT_LAMP   (FUEL_LAMP / 2)  /* Default amount of fuel in a lantern */
+
+/* A "stack" of items is limited to 40 items (hard-coded). */
+#define MAX_STACK_SIZE 41
+
+/* An item's pval (for charges, amount of gold, etc) is limited to s16b */
+#define MAX_PVAL  32767
+
+/*
+ * Maximum number of objects allowed in a single dungeon grid.
+ *
+ * The main-screen has a minimum size of 24 rows, so we can always
+ * display 23 objects + 1 header line.
+ */
+#define MAX_FLOOR_STACK			23
+
+
+/*** API constants ***/
 
 /* Object origin kinds */
 
@@ -74,6 +101,7 @@ enum {
 #define IDENT_FIRED     0x0800  /* Has been used as a missile */
 #define IDENT_NOTART    0x1000  /* Item is known not to be an artifact */
 #define IDENT_FAKE      0x2000  /* Item is a fake, for displaying knowledge */
+#define IDENT_SENSED_THIS_TURN 0x4000 /* Item has had a chance to be sensed on this turn (see sense_inventory()) */
 
 /* Whether to learn egos and flavors with less than complete information */
 #define EASY_LEARN 1
@@ -84,8 +112,7 @@ enum {
 /**
  * Modes for object_desc().
  */
-typedef enum
-{
+enum {
 	ODESC_BASE   = 0x00,   /*!< Only describe the base name */
 	ODESC_COMBAT = 0x01,   /*!< Also show combat bonuses */
 	ODESC_EXTRA  = 0x02,   /*!< Show charges/inscriptions/pvals */
@@ -97,31 +124,32 @@ typedef enum
 	ODESC_PLURAL = 0x08,   /*!< Always pluralise */
 	ODESC_SINGULAR    = 0x10,    /*!< Always singular */
 	ODESC_SPOIL  = 0x20,    /*!< Display regardless of player knowledge */
-	ODESC_PREFIX = 0x40   /* */
-} odesc_detail_t;
+	ODESC_PREFIX = 0x40,   /* */
+
+	ODESC_CAPITAL = 0x80,	/*!< Capitalise object name */
+	ODESC_TERSE = 0x100  	/*!< Make terse names */
+};
 
 
 /**
  * Modes for item lists in "show_inven()"  "show_equip()" and "show_floor()"
  */
-typedef enum
-{
+typedef enum {
 	OLIST_NONE   = 0x00,   /* No options */
    	OLIST_WINDOW = 0x01,   /* Display list in a sub-term (left-align) */
    	OLIST_QUIVER = 0x02,   /* Display quiver lines */
    	OLIST_GOLD   = 0x04,   /* Include gold in the list */
 	OLIST_WEIGHT = 0x08,   /* Show item weight */
 	OLIST_PRICE  = 0x10,   /* Show item price */
-	OLIST_FAIL   = 0x20    /* Show device failure */
-
+	OLIST_FAIL   = 0x20,    /* Show device failure */
+	OLIST_SEMPTY = 0x40
 } olist_detail_t;
 
 
 /**
  * Modes for object_info()
  */
-typedef enum
-{
+typedef enum {
 	OINFO_NONE   = 0x00, /* No options */
 	OINFO_TERSE  = 0x01, /* Keep descriptions brief, e.g. for dumps */
 	OINFO_SUBJ   = 0x02, /* Describe object from the character's POV */
@@ -163,6 +191,28 @@ typedef enum
 	INSCRIP_MAX                  /*!< Maximum number of pseudo-ID markers */
 } obj_pseudo_t;
 
+/*
+ * Chest check types
+ */
+enum chest_query {
+	CHEST_ANY,
+	CHEST_OPENABLE,
+	CHEST_TRAPPED
+};
+
+/*
+ * Bit flags for get_item() function
+ */
+#define USE_EQUIP     0x0001	/* Allow equip items */
+#define USE_INVEN     0x0002	/* Allow inven items */
+#define USE_FLOOR     0x0004	/* Allow floor items */
+#define IS_HARMLESS   0x0008	/* Ignore generic warning inscriptions */
+#define SHOW_PRICES   0x0010	/* Show item prices in item lists */
+#define SHOW_FAIL     0x0020 	/* Show device failure in item lists */
+#define SHOW_QUIVER   0x0040	/* Show quiver summary when looking at inventory */
+#define SHOW_EMPTY    0x0080	/* Show empty slots in equipment display */
+#define QUIVER_TAGS   0x0100	/* 0-9 are quiver slots when selecting */
+
 
 /*
  * Some constants used in randart generation and power calculation
@@ -185,6 +235,13 @@ typedef enum
 
 #define sign(x) ((x) > 0 ? 1 : ((x) < 0 ? -1 : 0))
 
+/* Values for struct object->marked */
+enum {
+	MARK_UNAWARE = 0,
+	MARK_AWARE = 1,
+	MARK_SEEN = 2
+};
+
 
 /*** Macros ***/
 
@@ -242,6 +299,24 @@ typedef enum
 
 /*** Structures ***/
 
+/*
+ * And here's the structure for the "fixed" spell information
+ */
+typedef struct spell {
+	struct spell *next;
+	unsigned int sidx;
+	char *name;
+	char *text;
+
+	byte realm;			/* 0 = mage; 1 = priest */
+	byte tval;			/* Item type for book this spell is in */
+	byte sval;			/* Item sub-type for book (= book number) */
+	byte snum;			/* Position of spell within book */
+
+	byte spell_index;	/* Index into player_magic array */
+} spell_type;
+
+
 /**
  * Information about object types, like rods, wands, etc.
  */
@@ -294,9 +369,9 @@ typedef struct object_kind
 	bitflag pval_flags[MAX_PVALS][OF_SIZE];	/**< pval flags */
 
 	byte d_attr;       /**< Default object attribute */
-	char d_char;       /**< Default object character */
+	wchar_t d_char;       /**< Default object character */
 
-	byte alloc_prob;   /**< Allocation: commonness */
+	int alloc_prob;   /**< Allocation: commonness */
 	byte alloc_min;    /**< Highest normal dungeon level */
 	byte alloc_max;    /**< Lowest normal dungeon level */
 	byte level;        /**< Level (difficulty of activation) */
@@ -314,7 +389,7 @@ typedef struct object_kind
 	/** Game-dependent **/
 
 	byte x_attr;   /**< Desired object attribute (set by user/pref file) */
-	char x_char;   /**< Desired object character (set by user/pref file) */
+	wchar_t x_char;   /**< Desired object character (set by user/pref file) */
 
 	/** Also saved in savefile **/
 
@@ -369,8 +444,8 @@ typedef struct artifact
 	bitflag pval_flags[MAX_PVALS][OF_SIZE];	/**< pval flags */
 
 	byte level;   /** Difficulty level for activation */
-	byte rarity;  /** Unused */
-	byte alloc_prob; /** Chance of being generated (i.e. rarity) */
+
+	int alloc_prob; /** Chance of being generated (i.e. rarity) */
 	byte alloc_min;  /** Minimum depth (can appear earlier) */
 	byte alloc_max;  /** Maximum depth (will NEVER appear deeper) */
 
@@ -382,10 +457,15 @@ typedef struct artifact
 	char *effect_msg;
 
 	random_value time;  /**< Recharge time (if appropriate) */
-
 } artifact_type;
 
 
+
+/*
+ * Number of tval/min-sval/max-sval slots per ego item
+ */
+#define EGO_TVALS_MAX 3
+
 /*
  * Information about "ego-items".
  */
@@ -406,7 +486,7 @@ typedef struct ego_item
 	byte level;		/* Minimum level */
 	byte rarity;		/* Object rarity */
 	byte rating;		/* Level rating boost */
-	byte alloc_prob; 	/** Chance of being generated (i.e. rarity) */
+	int alloc_prob; 	/** Chance of being generated (i.e. rarity) */
 	byte alloc_min;  	/** Minimum depth (can appear earlier) */
 	byte alloc_max;  	/** Maximum depth (will NEVER appear deeper) */
 
@@ -492,7 +572,7 @@ typedef struct object
 
 	byte number;		/* Number of items */
 	byte marked;		/* Object is marked */
-	bool ignore;		/* Object is ignored */
+	byte ignore;		/* Object is ignored */
 
 	s16b next_o_idx;	/* Next object in stack (if any) */
 	s16b held_m_idx;	/* Monster holding us (if any) */
@@ -505,7 +585,8 @@ typedef struct object
 	quark_t note; /* Inscription index */
 } object_type;
 
-typedef struct flavor {
+struct flavor
+{
 	char *text;
 	struct flavor *next;
 	unsigned int fidx;
@@ -514,15 +595,25 @@ typedef struct flavor {
 	byte sval;      /* Associated object sub-type */
 
 	byte d_attr;    /* Default flavor attribute */
-	char d_char;    /* Default flavor character */
+	wchar_t d_char;    /* Default flavor character */
 
 	byte x_attr;    /* Desired flavor attribute */
-	char x_char;    /* Desired flavor character */
-} flavor_type;
+	wchar_t x_char;    /* Desired flavor character */
+};
 
 
 /*** Functions ***/
 
+/* chest.c */
+byte chest_trap_type(const object_type *o_ptr);
+bool is_trapped_chest(const object_type *o_ptr);
+bool is_locked_chest(const object_type *o_ptr);
+void unlock_chest(object_type *o_ptr);
+s16b chest_check(int y, int x, enum chest_query check_type);
+int count_chests(int *y, int *x, enum chest_query check_type);
+bool do_cmd_open_chest(int y, int x, s16b o_idx);
+bool do_cmd_disarm_chest(int y, int x, s16b o_idx);
+
 /* identify.c */
 extern s32b object_last_wield;
 
@@ -559,6 +650,7 @@ void object_notice_on_defend(struct player *p);
 void object_notice_on_wield(object_type *o_ptr);
 void object_notice_on_firing(object_type *o_ptr);
 void wieldeds_notice_flag(struct player *p, int flag);
+void wieldeds_notice_to_hit_on_attack(void);
 void wieldeds_notice_on_attack(void);
 void object_repair_knowledge(object_type *o_ptr);
 bool object_FA_would_be_obvious(const object_type *o_ptr);
@@ -572,7 +664,8 @@ void object_know_all_flags(object_type *o_ptr);
 /* obj-desc.c */
 void object_base_name(char *buf, size_t max, int tval, bool plural);
 void object_kind_name(char *buf, size_t max, const object_kind *kind, bool easy_know);
-size_t object_desc(char *buf, size_t max, const object_type *o_ptr, odesc_detail_t mode);
+size_t obj_desc_name_format(char *buf, size_t max, size_t end, const char *fmt, const char *modstr, bool pluralise);
+size_t object_desc(char *buf, size_t max, const object_type *o_ptr, int mode);
 
 /* obj-info.c */
 textblock *object_info(const object_type *o_ptr, oinfo_detail_t mode);
@@ -583,21 +676,22 @@ void object_info_chardump(ang_file *f, const object_type *o_ptr, int indent, int
 /* obj-make.c */
 void free_obj_alloc(void);
 bool init_obj_alloc(void);
-object_kind *get_obj_num(int level, bool good);
+object_kind *get_obj_num(int level, bool good, int tval);
 void object_prep(object_type *o_ptr, struct object_kind *kind, int lev, aspect rand_aspect);
-s16b apply_magic(object_type *o_ptr, int lev, bool okay, bool good, bool great);
-bool make_object(struct cave *c, object_type *j_ptr, int lev, bool good, bool great, s32b *value);
+s16b apply_magic(object_type *o_ptr, int lev, bool okay, bool good, bool great, bool extra_roll);
+bool make_object(struct cave *c, object_type *j_ptr, int lev, bool good, bool great, bool extra_roll, s32b *value, int tval);
 void make_gold(object_type *j_ptr, int lev, int coin_type);
 void copy_artifact_data(object_type *o_ptr, const artifact_type *a_ptr);
 void ego_apply_magic(object_type *o_ptr, int level);
 void ego_min_pvals(object_type *o_ptr);
 
 /* obj-ui.c */
-void show_inven(olist_detail_t mode);
-void show_equip(olist_detail_t mode);
-void show_floor(const int *floor_list, int floor_num, olist_detail_t mode);
+void show_inven(int mode);
+void show_equip(int mode);
+void show_floor(const int *floor_list, int floor_num, int mode);
 bool verify_item(const char *prompt, int item);
 bool get_item(int *cp, const char *pmt, const char *str, cmd_code cmd, int mode);
+bool get_item_allow(int item, unsigned char ch, cmd_code cmd, bool is_harmless);
 
 /* obj-util.c */
 struct object_kind *objkind_get(int tval, int sval);
@@ -690,15 +784,23 @@ bool obj_can_takeoff(const object_type *o_ptr);
 bool obj_can_wear(const object_type *o_ptr);
 bool obj_can_fire(const object_type *o_ptr);
 bool obj_has_inscrip(const object_type *o_ptr);
+bool obj_is_useable(const object_type *o_ptr);
+bool obj_is_used_aimed(const object_type *o_ptr);
+bool obj_is_used_unaimed(const object_type *o_ptr);
 u16b object_effect(const object_type *o_ptr);
 object_type *object_from_item_idx(int item);
+int inventory_index_matching_object(const object_type *o_ptr);
 bool obj_needs_aim(object_type *o_ptr);
+bool obj_can_fail(const struct object *o);
 bool get_item_okay(int item);
 int scan_items(int *item_list, size_t item_list_max, int mode);
 bool item_is_available(int item, bool (*tester)(const object_type *), int mode);
 extern void display_itemlist(void);
 extern void display_object_idx_recall(s16b o_idx);
-extern void display_object_kind_recall(s16b k_idx);
+extern void display_object_kind_recall(struct object_kind *kind);
+void display_object_recall_interactive(object_type *o_ptr);
+bool is_unknown(const object_type *o_ptr);
+int compare_items(const object_type *o1, const object_type *o2);
 
 bool pack_is_full(void);
 bool pack_is_overfull(void);
