@@ -34,8 +34,6 @@
 
 #define MIN_NAME_LEN 5
 #define MAX_NAME_LEN 9
-#define S_WORD 26
-#define E_WORD S_WORD
 
 /*
  * Inhibiting factors for large bonus values
@@ -163,22 +161,6 @@
 /* Total of abilities */
 #define ART_IDX_TOTAL 87
 
-/* Tallies of different ability types */
-/* ToDo: use N_ELEMENTS for these */
-#define ART_IDX_BOW_COUNT 4
-#define ART_IDX_WEAPON_COUNT 3
-#define ART_IDX_NONWEAPON_COUNT 8
-#define ART_IDX_MELEE_COUNT 9
-#define ART_IDX_ALLARMOR_COUNT 1
-#define ART_IDX_BOOT_COUNT 4
-#define ART_IDX_GLOVE_COUNT 3
-#define ART_IDX_HELM_COUNT 6
-#define ART_IDX_SHIELD_COUNT 2
-#define ART_IDX_CLOAK_COUNT 2
-#define ART_IDX_ARMOR_COUNT 7
-#define ART_IDX_GEN_COUNT 31
-#define ART_IDX_HIGH_RESIST_COUNT 13
-
 /* Arrays of indices by item type, used in frequency generation */
 static s16b art_idx_bow[] =
 	{ART_IDX_BOW_SHOTS, ART_IDX_BOW_MIGHT, ART_IDX_BOW_BRAND, ART_IDX_BOW_SLAY};
@@ -229,7 +211,7 @@ static s16b art_idx_high_resist[] =
 
 /* Initialize the data structures for learned probabilities */
 static s16b artprobs[ART_IDX_TOTAL];
-s16b *baseprobs;
+static s16b *baseprobs;
 static s16b art_bow_total = 0;
 static s16b art_melee_total = 0;
 static s16b art_boot_total = 0;
@@ -246,7 +228,7 @@ static s16b art_total = 0;
  * allocation of memory
  */
 static s16b art_freq[ART_IDX_TOTAL];  	/* artifact attributes */
-s16b *base_freq; 			/* base items */
+static s16b *base_freq; 			/* base items */
 
 /*
  * Mean start and increment values for to_hit, to_dam and AC.  Update these
@@ -295,7 +277,8 @@ char *artifact_gen_name(struct artifact *a, const char ***words) {
 	char buf[BUFLEN];
 	char word[MAX_NAME_LEN + 1];
 	randname_make(RANDNAME_TOLKIEN, MIN_NAME_LEN, MAX_NAME_LEN, word, sizeof(word), words);
-	word[0] = toupper((unsigned char)word[0]);
+	my_strcap(word);
+
 	if (one_in_(3))
 		strnfmt(buf, sizeof(buf), "'%s'", word);
 	else
@@ -340,6 +323,8 @@ static errr init_names(void)
 				strlen(a->name) + 8);
 		}
 
+		string_free(a->text);
+		string_free(a->name);
 		a->text = string_make(desc);
 		a->name = artifact_gen_name(a, name_sections);
 	}
@@ -449,6 +434,8 @@ static void store_base_power (void)
 	art_total = art_melee_total + art_bow_total + art_armor_total +
 	            art_shield_total + art_cloak_total + art_headgear_total +
 	            art_glove_total + art_boot_total + art_other_total;
+
+    FREE(fake_power);
 }
 
 
@@ -479,8 +466,7 @@ static object_kind *choose_item(int a_idx)
 	 * the whole special/flavour issue was sorted out (see ticket #1014)
 	 * Note that Carlammas and Barahir have the same sval as Grond/Morgoth
 	 */
-	while (tval == 0 || tval == TV_SKELETON || tval == TV_BOTTLE ||
-		tval == TV_JUNK || tval == TV_SPIKE || tval == TV_CHEST ||
+	while (tval == 0 || tval == TV_CHEST ||
 		tval == TV_SHOT || tval == TV_ARROW || tval == TV_BOLT ||
 		tval == TV_STAFF || tval == TV_WAND || tval == TV_ROD ||
 		tval == TV_SCROLL || tval == TV_POTION || tval == TV_FLASK ||
@@ -502,7 +488,6 @@ static object_kind *choose_item(int a_idx)
 	k_ptr = lookup_kind(tval, sval);
 	a_ptr->tval = k_ptr->tval;
 	a_ptr->sval = k_ptr->sval;
-	a_ptr->pval[DEFAULT_PVAL] = randcalc(k_ptr->pval[DEFAULT_PVAL], 0, MINIMISE);
 	a_ptr->to_h = randcalc(k_ptr->to_h, 0, MINIMISE);
 	a_ptr->to_d = randcalc(k_ptr->to_d, 0, MINIMISE);
 	a_ptr->to_a = randcalc(k_ptr->to_a, 0, MINIMISE);
@@ -511,8 +496,10 @@ static object_kind *choose_item(int a_idx)
 	a_ptr->ds = k_ptr->ds;
 	a_ptr->weight = k_ptr->weight;
 	of_copy(a_ptr->flags, k_ptr->flags);
-	for (i = 0; i < MAX_PVALS; i++)
+	for (i = 0; i < MAX_PVALS; i++) {
 		of_copy(a_ptr->pval_flags[i], k_ptr->pval_flags[i]);
+		a_ptr->pval[i] = randcalc(k_ptr->pval[i], 0, MINIMISE);
+	}
 	a_ptr->num_pvals = k_ptr->num_pvals;
 	a_ptr->effect = 0;
 
@@ -621,7 +608,6 @@ static void remove_contradictory(artifact_type *a_ptr)
 		if (of_has(a_ptr->flags, OF_WIS)) of_off(a_ptr->flags, OF_SUST_WIS);
 		if (of_has(a_ptr->flags, OF_DEX)) of_off(a_ptr->flags, OF_SUST_DEX);
 		if (of_has(a_ptr->flags, OF_CON)) of_off(a_ptr->flags, OF_SUST_CON);
-		if (of_has(a_ptr->flags, OF_CHR)) of_off(a_ptr->flags, OF_SUST_CHR);
 		of_off(a_ptr->flags, OF_BLOWS);
 	}
 
@@ -700,7 +686,8 @@ static void adjust_freqs(void)
  */
 static void parse_frequencies(void)
 {
-	int i, j;
+	size_t i;
+	int j;
 	const artifact_type *a_ptr;
 	object_kind *k_ptr;
 	s32b m, temp, temp2;
@@ -1105,7 +1092,7 @@ static void parse_frequencies(void)
 		 */
 
 		if (flags_test(a_ptr->flags, OF_SIZE, OF_STR, OF_INT, OF_WIS,
-		                     OF_DEX, OF_CON, OF_CHR, FLAG_END))
+		                     OF_DEX, OF_CON, FLAG_END))
 		{
 			/* Stat bonus case.  Add up the number of individual
 			   bonuses */
@@ -1115,7 +1102,6 @@ static void parse_frequencies(void)
 			if (of_has(a_ptr->flags, OF_WIS)) temp++;
 			if (of_has(a_ptr->flags, OF_DEX)) temp++;
 			if (of_has(a_ptr->flags, OF_CON)) temp++;
-			if (of_has(a_ptr->flags, OF_CHR)) temp++;
 
 			/* Handle a few special cases separately. */
 			if((a_ptr->tval == TV_HELM || a_ptr->tval == TV_CROWN) &&
@@ -1174,7 +1160,7 @@ static void parse_frequencies(void)
 
 		if (flags_test(a_ptr->flags, OF_SIZE, OF_SUST_STR, OF_SUST_INT,
 		                     OF_SUST_WIS, OF_SUST_DEX, OF_SUST_CON,
-		                     OF_SUST_CHR, FLAG_END))
+		                     FLAG_END))
 		{
 			/* Now do sustains, in a similar manner */
 			temp = 0;
@@ -1183,7 +1169,6 @@ static void parse_frequencies(void)
 			if (of_has(a_ptr->flags, OF_SUST_WIS)) temp++;
 			if (of_has(a_ptr->flags, OF_SUST_DEX)) temp++;
 			if (of_has(a_ptr->flags, OF_SUST_CON)) temp++;
-			if (of_has(a_ptr->flags, OF_SUST_CHR)) temp++;
 			file_putf(log_file, "Adding %d for stat sustains.\n", temp);
 
 			(artprobs[ART_IDX_GEN_SUST]) += temp;
@@ -1342,7 +1327,7 @@ static void parse_frequencies(void)
 		if (of_has(a_ptr->flags, OF_LIGHT))
 		{
 			/* Handle permanent light */
-			file_putf(log_file, "Adding 1 for permanent light - general.\n");
+			file_putf(log_file, "Adding 1 for light radius - general.\n");
 
 			(artprobs[ART_IDX_GEN_LIGHT])++;
 		}
@@ -1640,14 +1625,14 @@ static void parse_frequencies(void)
 	 */
 
 	/* Bow-only abilities */
-	for (i = 0; i < ART_IDX_BOW_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_bow); i++)
 	{
 		artprobs[art_idx_bow[i]] = (artprobs[art_idx_bow[i]] * art_total)
 			/ art_bow_total;
 	}
 
 	/* All weapon abilities */
-	for (i = 0; i < ART_IDX_WEAPON_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_weapon); i++)
 	{
 		artprobs[art_idx_weapon[i]] = (artprobs[art_idx_weapon[i]] *
 			art_total) / (art_bow_total + art_melee_total);
@@ -1655,14 +1640,14 @@ static void parse_frequencies(void)
 
 	/* Corresponding non-weapon abilities */
 	temp = art_total - art_melee_total - art_bow_total;
-	for (i = 0; i < ART_IDX_NONWEAPON_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_nonweapon); i++)
 	{
 		artprobs[art_idx_nonweapon[i]] = (artprobs[art_idx_nonweapon[i]] *
 			art_total) / temp;
 	}
 
 	/* All melee weapon abilities */
-	for (i = 0; i < ART_IDX_MELEE_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_melee); i++)
 	{
 		artprobs[art_idx_melee[i]] = (artprobs[art_idx_melee[i]] *
 			art_total) / art_melee_total;
@@ -1671,49 +1656,49 @@ static void parse_frequencies(void)
 	/* All general armor abilities */
 	temp = art_armor_total + art_boot_total + art_shield_total +
 		art_headgear_total + art_cloak_total + art_glove_total;
-	for (i = 0; i < ART_IDX_ALLARMOR_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_allarmor); i++)
 	{
 		artprobs[art_idx_allarmor[i]] = (artprobs[art_idx_allarmor[i]] *
 			art_total) / temp;
 	}
 
 	/* Boots */
-	for (i = 0; i < ART_IDX_BOOT_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_boot); i++)
 	{
 		artprobs[art_idx_boot[i]] = (artprobs[art_idx_boot[i]] *
 			art_total) / art_boot_total;
 	}
 
 	/* Gloves */
-	for (i = 0; i < ART_IDX_GLOVE_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_glove); i++)
 	{
 		artprobs[art_idx_glove[i]] = (artprobs[art_idx_glove[i]] *
 			art_total) / art_glove_total;
 	}
 
 	/* Headgear */
-	for (i = 0; i < ART_IDX_HELM_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_headgear); i++)
 	{
 		artprobs[art_idx_headgear[i]] = (artprobs[art_idx_headgear[i]] *
 			art_total) / art_headgear_total;
 	}
 
 	/* Shields */
-	for (i = 0; i < ART_IDX_SHIELD_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_shield); i++)
 	{
 		artprobs[art_idx_shield[i]] = (artprobs[art_idx_shield[i]] *
 			art_total) / art_shield_total;
 	}
 
 	/* Cloaks */
-	for (i = 0; i < ART_IDX_CLOAK_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_cloak); i++)
 	{
 		artprobs[art_idx_cloak[i]] = (artprobs[art_idx_cloak[i]] *
 			art_total) / art_cloak_total;
 	}
 
 	/* Body armor */
-	for (i = 0; i < ART_IDX_ARMOR_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_armor); i++)
 	{
 		artprobs[art_idx_armor[i]] = (artprobs[art_idx_armor[i]] *
 			art_total) / art_armor_total;
@@ -1835,19 +1820,18 @@ static void add_stat(artifact_type *a_ptr)
 
 	/* Hack: break out if all stats are raised to avoid an infinite loop */
 	if (flags_test_all(a_ptr->flags, OF_SIZE, OF_STR, OF_INT, OF_WIS,
-	                         OF_DEX, OF_CON, OF_CHR, FLAG_END))
+	                         OF_DEX, OF_CON, FLAG_END))
 			return;
 
 	/* Make sure we add one that hasn't been added yet */
 	while (!success)
 	{
-		r = randint0(6);
+		r = randint0(5);
 		if (r == 0) success = add_fixed_pval_flag(a_ptr, OF_STR);
 		else if (r == 1) success = add_fixed_pval_flag(a_ptr, OF_INT);
 		else if (r == 2) success = add_fixed_pval_flag(a_ptr, OF_WIS);
 		else if (r == 3) success = add_fixed_pval_flag(a_ptr, OF_DEX);
 		else if (r == 4) success = add_fixed_pval_flag(a_ptr, OF_CON);
-		else if (r == 5) success = add_fixed_pval_flag(a_ptr, OF_CHR);
 	}
 }
 
@@ -1858,18 +1842,17 @@ static void add_sustain(artifact_type *a_ptr)
 
 	/* Hack: break out if all stats are sustained to avoid an infinite loop */
 	if (flags_test_all(a_ptr->flags, OF_SIZE, OF_SUST_STR, OF_SUST_INT,
-	    OF_SUST_WIS, OF_SUST_DEX, OF_SUST_CON, OF_SUST_CHR, FLAG_END))
+	    OF_SUST_WIS, OF_SUST_DEX, OF_SUST_CON, FLAG_END))
 			return;
 
 	while (!success)
 	{
-		r = randint0(6);
+		r = randint0(5);
 		if (r == 0) success = add_flag(a_ptr, OF_SUST_STR);
 		else if (r == 1) success = add_flag(a_ptr, OF_SUST_INT);
 		else if (r == 2) success = add_flag(a_ptr, OF_SUST_WIS);
 		else if (r == 3) success = add_flag(a_ptr, OF_SUST_DEX);
 		else if (r == 4) success = add_flag(a_ptr, OF_SUST_CON);
-		else if (r == 5) success = add_flag(a_ptr, OF_SUST_CHR);
 	}
 }
 
@@ -1896,12 +1879,13 @@ static void add_low_resist(artifact_type *a_ptr)
 static void add_high_resist(artifact_type *a_ptr)
 {
 	/* Add a high resist, according to the generated frequency distribution. */
-	int r, i, temp;
+	size_t i;
+	int r, temp;
 	int count = 0;
 	bool success = FALSE;
 
 	temp = 0;
-	for (i = 0; i < ART_IDX_HIGH_RESIST_COUNT; i++)
+	for (i = 0; i < N_ELEMENTS(art_idx_high_resist); i++)
 	{
 		temp += artprobs[art_idx_high_resist[i]];
 	}
@@ -1916,7 +1900,7 @@ static void add_high_resist(artifact_type *a_ptr)
 
 		temp = artprobs[art_idx_high_resist[0]];
 		i = 0;
-		while (r > temp && i < ART_IDX_HIGH_RESIST_COUNT)
+		while (r > temp && i < N_ELEMENTS(art_idx_high_resist))
 		{
 			temp += artprobs[art_idx_high_resist[i]];
 			i++;
@@ -1993,7 +1977,6 @@ static void add_to_hit(artifact_type *a_ptr, int fixed, int random)
 		}
 	}
 	a_ptr->to_h += (s16b)(fixed + randint0(random));
-	if (a_ptr->to_h > 0) of_on(a_ptr->flags, OF_SHOW_MODS);
 	file_putf(log_file, "Adding ability: extra to_h (now %+d)\n", a_ptr->to_h);
 }
 
@@ -2017,7 +2000,6 @@ static void add_to_dam(artifact_type *a_ptr, int fixed, int random)
 		}
 	}
 	a_ptr->to_d += (s16b)(fixed + randint0(random));
-	if (a_ptr->to_d > 0) of_on(a_ptr->flags, OF_SHOW_MODS);
 	file_putf(log_file, "Adding ability: extra to_dam (now %+d)\n", a_ptr->to_d);
 }
 
@@ -2143,7 +2125,8 @@ static void add_activation(artifact_type *a_ptr, s32b target_power)
  */
 static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 {
-	int i,j;
+	int i;
+	size_t j;
 	s16b f_temp[ART_IDX_TOTAL];
 
 	/* First, set everything to zero */
@@ -2157,7 +2140,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	/* Bow abilities */
 	if (a_ptr->tval == TV_BOW)
 	{
-		for (j = 0; j < ART_IDX_BOW_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_bow);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_bow[j]] = artprobs[art_idx_bow[j]];
 		}
@@ -2167,7 +2151,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 		a_ptr->tval == TV_HAFTED || a_ptr->tval == TV_POLEARM ||
 		a_ptr->tval == TV_SWORD)
 	{
-		for (j = 0; j < ART_IDX_WEAPON_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_weapon);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_weapon[j]] = artprobs[art_idx_weapon[j]];
 		}
@@ -2175,7 +2160,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	/* General non-weapon abilities */
 	else
 	{
-		for (j = 0; j < ART_IDX_NONWEAPON_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_nonweapon);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_nonweapon[j]] = artprobs[art_idx_nonweapon[j]];
 		}
@@ -2184,7 +2170,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	if (a_ptr->tval == TV_DIGGING || a_ptr->tval == TV_HAFTED ||
 		a_ptr->tval == TV_POLEARM || a_ptr->tval == TV_SWORD)
 	{
-		for (j = 0; j < ART_IDX_MELEE_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_melee);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_melee[j]] = artprobs[art_idx_melee[j]];
 		}
@@ -2195,8 +2182,9 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 		a_ptr->tval == TV_SHIELD || a_ptr->tval == TV_CLOAK ||
 		a_ptr->tval == TV_SOFT_ARMOR || a_ptr->tval == TV_HARD_ARMOR ||
 		a_ptr->tval == TV_DRAG_ARMOR)
-		{
-		for (j = 0; j < ART_IDX_ALLARMOR_COUNT; j++)
+	{
+		size_t n = N_ELEMENTS(art_idx_allarmor);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_allarmor[j]] = artprobs[art_idx_allarmor[j]];
 		}
@@ -2204,7 +2192,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	/* Boot abilities */
 	if (a_ptr->tval == TV_BOOTS)
 	{
-		for (j = 0; j < ART_IDX_BOOT_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_boot);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_boot[j]] = artprobs[art_idx_boot[j]];
 		}
@@ -2212,7 +2201,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	/* Glove abilities */
 	if (a_ptr->tval == TV_GLOVES)
 	{
-		for (j = 0; j < ART_IDX_GLOVE_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_glove);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_glove[j]] = artprobs[art_idx_glove[j]];
 		}
@@ -2220,7 +2210,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	/* Headgear abilities */
 	if (a_ptr->tval == TV_HELM || a_ptr->tval == TV_CROWN)
 	{
-		for (j = 0; j < ART_IDX_HELM_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_headgear);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_headgear[j]] = artprobs[art_idx_headgear[j]];
 		}
@@ -2228,7 +2219,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	/* Shield abilities */
 	if (a_ptr->tval == TV_SHIELD)
 	{
-		for (j = 0; j < ART_IDX_SHIELD_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_shield);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_shield[j]] = artprobs[art_idx_shield[j]];
 		}
@@ -2236,7 +2228,8 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	/* Cloak abilities */
 	if (a_ptr->tval == TV_CLOAK)
 	{
-		for (j = 0; j < ART_IDX_CLOAK_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_cloak);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_cloak[j]] = artprobs[art_idx_cloak[j]];
 		}
@@ -2245,13 +2238,14 @@ static void build_freq_table(artifact_type *a_ptr, s16b *freq)
 	if (a_ptr->tval == TV_SOFT_ARMOR || a_ptr->tval == TV_HARD_ARMOR ||
 		a_ptr->tval == TV_DRAG_ARMOR)
 	{
-		for (j = 0; j < ART_IDX_ARMOR_COUNT; j++)
+		size_t n = N_ELEMENTS(art_idx_armor);
+		for (j = 0; j < n; j++)
 		{
 			f_temp[art_idx_armor[j]] = artprobs[art_idx_armor[j]];
 		}
 	}
 	/* General abilities - no constraint */
-	for (j = 0; j < ART_IDX_GEN_COUNT; j++)
+	for (j = 0; j < N_ELEMENTS(art_idx_gen); j++)
 	{
 		f_temp[art_idx_gen[j]] = artprobs[art_idx_gen[j]];
 	}
@@ -2488,8 +2482,16 @@ static void add_ability_aux(artifact_type *a_ptr, int r, s32b target_power)
 			add_immunity(a_ptr);
 			break;
 
-		case ART_IDX_GEN_LIGHT:
-			add_flag(a_ptr, OF_LIGHT);
+		case ART_IDX_GEN_LIGHT: {
+				if (a_ptr->tval != TV_LIGHT &&
+						!of_is_empty(a_ptr->pval_flags[DEFAULT_PVAL])) {
+					of_on(a_ptr->flags, OF_LIGHT);
+					of_on(a_ptr->pval_flags[DEFAULT_PVAL + 1], OF_LIGHT);
+					a_ptr->pval[DEFAULT_PVAL + 1] = 1;
+					recalc_num_pvals(a_ptr);
+				} else
+					break;
+			}
 			break;
 
 		case ART_IDX_GEN_SDIG:
@@ -2544,6 +2546,10 @@ static void add_ability_aux(artifact_type *a_ptr, int r, s32b target_power)
 			add_flag(a_ptr, OF_RES_DISEN);
 			break;
 
+		case ART_IDX_GEN_PSTUN:
+			add_flag(a_ptr, OF_RES_STUN);
+			break;
+
 		case ART_IDX_GEN_ACTIV:
 			if (!a_ptr->effect) add_activation(a_ptr, target_power);
 			break;
@@ -2826,8 +2832,16 @@ static void scramble_artifact(int a_idx)
 		}
 
 		/* Clear the activations for rings and amulets but not lights */
-		if (a_ptr->tval != TV_LIGHT) a_ptr->effect = 0;
-
+		if (a_ptr->tval != TV_LIGHT)
+			a_ptr->effect = 0;
+		/* Restore lights */
+		else {
+			of_on(a_ptr->flags, OF_LIGHT);
+			of_on(a_ptr->flags, OF_NO_FUEL);
+			of_on(a_ptr->pval_flags[DEFAULT_PVAL], OF_LIGHT);
+			a_ptr->pval[DEFAULT_PVAL] = 3;
+			a_ptr->num_pvals = 1;
+		}
 		/* Artifacts ignore everything */
 		create_mask(f, FALSE, OFT_IGNORE, OFT_MAX);
 		of_union(a_ptr->flags, f);
@@ -2981,16 +2995,9 @@ static void scramble_artifact(int a_idx)
 	file_putf(log_file, "New depths are min %d, max %d\n", a_ptr->alloc_min, a_ptr->alloc_max);
 	file_putf(log_file, "Power-based alloc_prob is %d\n", a_ptr->alloc_prob);
 
-	/* Restore some flags */
-	if (a_ptr->tval == TV_LIGHT) of_on(a_ptr->flags, OF_NO_FUEL);
-	if (a_idx < ART_MIN_NORMAL) of_on(a_ptr->flags, OF_INSTA_ART);
-
-	/*
-	 * Add OF_HIDE_TYPE to all artifacts with nonzero pval because we're
-	 * too lazy to find out which ones need it and which ones don't.
-	 */
-	if (a_ptr->pval[DEFAULT_PVAL])
-		of_on(a_ptr->flags, OF_HIDE_TYPE);
+	/* This will go */
+	if (a_idx < ART_MIN_NORMAL)
+		of_on(a_ptr->flags, OF_INSTA_ART);
 
 	/* Success */
 	file_putf(log_file, ">>>>>>>>>>>>>>>>>>>>>>>>>> ARTIFACT COMPLETED <<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n");
@@ -3069,7 +3076,8 @@ static bool artifacts_acceptable(void)
 				gloves > 0 ? " gloves" : "",
 				boots > 0 ? " boots" : "");
 
-			file_putf(log_file, "Restarting generation process: not enough%s", types);
+			file_putf(log_file, "Restarting generation process: not enough%s",
+				types);
 		}
 		return FALSE;
 	}
