@@ -26,10 +26,6 @@
  */
 
 #define TV_NULL		 0
-#define TV_SKELETON      1	/* Skeletons ('s') */
-#define TV_BOTTLE	 2	/* Empty bottles ('!') */
-#define TV_JUNK          3	/* Sticks, Pottery, etc ('~') */
-#define TV_SPIKE         5	/* Spikes ('~') */
 #define TV_CHEST         7	/* Chests ('~') */
 #define TV_SHOT		16	/* Ammo for slings */
 #define TV_ARROW        17	/* Ammo for bows */
@@ -348,7 +344,7 @@ enum sval_gold /* tval 100 */
 /*
  * Special "sval" limit -- first "normal" food
  */
-#define SV_FOOD_MIN_SHROOM   5
+#define SV_FOOD_MIN_SHROOM   6
 
 /*
  * Special "sval" limit -- first "large" chest
