@@ -6,5 +6,7 @@
 #include "z-type.h"
 
 extern int pathfind_direction_to(struct loc from, struct loc to);
+extern bool findpath(int y, int x);
+extern void run_step(int dir);
 
 #endif /* !PATHFIND_H */
