@@ -28,55 +28,10 @@
 #include "spells.h"
 #include "squelch.h"
 
-/*
- * Stat Table (CHR) -- payment percentages
- */
-const byte adj_chr_gold[STAT_RANGE] =
-{
-	143	/* 3 */,
-	137	/* 4 */,
-	134	/* 5 */,
-	132	/* 6 */,
-	129	/* 7 */,
-	127	/* 8 */,
-	123	/* 9 */,
-	122	/* 10 */,
-	121	/* 11 */,
-	118	/* 12 */,
-	116	/* 13 */,
-	113	/* 14 */,
-	113	/* 15 */,
-	112	/* 16 */,
-	111	/* 17 */,
-	110	/* 18/00-18/09 */,
-	108	/* 18/10-18/19 */,
-	107	/* 18/20-18/29 */,
-	106	/* 18/30-18/39 */,
-	105	/* 18/40-18/49 */,
-	104	/* 18/50-18/59 */,
-	103	/* 18/60-18/69 */,
-	102	/* 18/70-18/79 */,
-	101	/* 18/80-18/89 */,
-	100	/* 18/90-18/99 */,
-	99	/* 18/100-18/109 */,
-	97	/* 18/110-18/119 */,
-	96	/* 18/120-18/129 */,
-	95	/* 18/130-18/139 */,
-	94	/* 18/140-18/149 */,
-	93	/* 18/150-18/159 */,
-	92	/* 18/160-18/169 */,
-	91	/* 18/170-18/179 */,
-	90	/* 18/180-18/189 */,
-	90	/* 18/190-18/199 */,
-	90	/* 18/200-18/209 */,
-	90	/* 18/210-18/219 */,
-	90	/* 18/220+ */
-};
-
 /*
  * Stat Table (INT) -- Magic devices
  */
-static const byte adj_int_dev[STAT_RANGE] =
+const byte adj_int_dev[STAT_RANGE] =
 {
 	0	/* 3 */,
 	0	/* 4 */,
@@ -121,7 +76,7 @@ static const byte adj_int_dev[STAT_RANGE] =
 /*
  * Stat Table (WIS) -- Saving throw
  */
-static const byte adj_wis_sav[STAT_RANGE] =
+const byte adj_wis_sav[STAT_RANGE] =
 {
 	0	/* 3 */,
 	0	/* 4 */,
@@ -167,7 +122,7 @@ static const byte adj_wis_sav[STAT_RANGE] =
 /*
  * Stat Table (DEX) -- disarming
  */
-static const byte adj_dex_dis[STAT_RANGE] =
+const byte adj_dex_dis[STAT_RANGE] =
 {
 	0	/* 3 */,
 	0	/* 4 */,
@@ -213,7 +168,7 @@ static const byte adj_dex_dis[STAT_RANGE] =
 /*
  * Stat Table (INT) -- disarming
  */
-static const byte adj_int_dis[STAT_RANGE] =
+const byte adj_int_dis[STAT_RANGE] =
 {
 	0	/* 3 */,
 	0	/* 4 */,
@@ -258,7 +213,7 @@ static const byte adj_int_dis[STAT_RANGE] =
 /*
  * Stat Table (DEX) -- bonus to ac (plus 128)
  */
-static const byte adj_dex_ta[STAT_RANGE] =
+const byte adj_dex_ta[STAT_RANGE] =
 {
 	128 + -4	/* 3 */,
 	128 + -3	/* 4 */,
@@ -303,7 +258,7 @@ static const byte adj_dex_ta[STAT_RANGE] =
 /*
  * Stat Table (STR) -- bonus to dam (plus 128)
  */
-static const byte adj_str_td[STAT_RANGE] =
+const byte adj_str_td[STAT_RANGE] =
 {
 	128 + -2	/* 3 */,
 	128 + -2	/* 4 */,
@@ -349,7 +304,7 @@ static const byte adj_str_td[STAT_RANGE] =
 /*
  * Stat Table (DEX) -- bonus to hit (plus 128)
  */
-static const byte adj_dex_th[STAT_RANGE] =
+const byte adj_dex_th[STAT_RANGE] =
 {
 	128 + -3	/* 3 */,
 	128 + -2	/* 4 */,
@@ -395,7 +350,7 @@ static const byte adj_dex_th[STAT_RANGE] =
 /*
  * Stat Table (STR) -- bonus to hit (plus 128)
  */
-static const byte adj_str_th[STAT_RANGE] =
+const byte adj_str_th[STAT_RANGE] =
 {
 	128 + -3	/* 3 */,
 	128 + -2	/* 4 */,
@@ -441,7 +396,7 @@ static const byte adj_str_th[STAT_RANGE] =
 /*
  * Stat Table (STR) -- weight limit in deca-pounds
  */
-static const byte adj_str_wgt[STAT_RANGE] =
+const byte adj_str_wgt[STAT_RANGE] =
 {
 	5	/* 3 */,
 	6	/* 4 */,
@@ -533,7 +488,7 @@ const byte adj_str_hold[STAT_RANGE] =
 /*
  * Stat Table (STR) -- digging value
  */
-static const byte adj_str_dig[STAT_RANGE] =
+const byte adj_str_dig[STAT_RANGE] =
 {
 	0	/* 3 */,
 	0	/* 4 */,
@@ -625,7 +580,7 @@ const byte adj_str_blow[STAT_RANGE] =
 /*
  * Stat Table (DEX) -- index into the "blow" table
  */
-static const byte adj_dex_blow[STAT_RANGE] =
+const byte adj_dex_blow[STAT_RANGE] =
 {
 	0	/* 3 */,
 	0	/* 4 */,
@@ -763,7 +718,7 @@ const byte adj_con_fix[STAT_RANGE] =
 /*
  * Stat Table (CON) -- extra 1/100th hitpoints per level
  */
-static const int adj_con_mhp[STAT_RANGE] =
+const int adj_con_mhp[STAT_RANGE] =
 {
 	-250	/* 3 */,
 	-150	/* 4 */,
@@ -805,7 +760,7 @@ static const int adj_con_mhp[STAT_RANGE] =
 	1250	/* 18/220+ */
 };
 
-static const int adj_mag_study[STAT_RANGE] =
+const int adj_mag_study[STAT_RANGE] =
 {
 	  0	/* 3 */,
 	  0	/* 4 */,
@@ -850,7 +805,7 @@ static const int adj_mag_study[STAT_RANGE] =
 /*
  * Stat Table (INT/WIS) -- extra 1/100 mana-points per level
  */
-static const int adj_mag_mana[STAT_RANGE] =
+const int adj_mag_mana[STAT_RANGE] =
 {
 	  0	/* 3 */,
 	 10	/* 4 */,
@@ -920,7 +875,7 @@ static const int adj_mag_mana[STAT_RANGE] =
  * The player gets blows/round equal to 100/this number, up to a maximum of
  * "num" blows/round, plus any "bonus" blows/round.
  */
-static const byte blows_table[12][12] =
+const byte blows_table[12][12] =
 {
 	/* P */
    /* D:   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,   10,  11+ */
@@ -1359,18 +1314,21 @@ static void calc_torch(void)
 	int i;
 
 	s16b old_light = p_ptr->cur_light;
-	bool burn_light = TRUE;
-
 	s16b new_light = 0;
-	int extra_light = 0;
 
 	/* Ascertain lightness if in the town */
-	if (!p_ptr->depth && ((turn % (10L * TOWN_DAWN)) < ((10L * TOWN_DAWN) / 2)))
-		burn_light = FALSE;
+	if (!p_ptr->depth && ((turn % (10L * TOWN_DAWN)) < ((10L * TOWN_DAWN) / 2))) {
+		new_light = 0;
+		if (old_light != new_light) {
+			/* Update the visuals */
+			p_ptr->cur_light = new_light;
+			p_ptr->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
+		}
+		return;
+	}
 
 	/* Examine all wielded objects, use the brightest */
-	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)
-	{
+	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)	{
 		bitflag f[OF_SIZE];
 
 		int amt = 0;
@@ -1382,55 +1340,30 @@ static void calc_torch(void)
 		/* Extract the flags */
 		object_flags(o_ptr, f);
 
+		/* Light radius is now a pval */
+		if (of_has(f, OF_LIGHT))
+			amt = o_ptr->pval[which_pval(o_ptr, OF_LIGHT)];
+
 		/* Cursed objects emit no light */
 		if (of_has(f, OF_LIGHT_CURSE))
 			amt = 0;
 
 		/* Examine actual lights */
-		else if (o_ptr->tval == TV_LIGHT)
-		{
-			int flag_inc = of_has(f, OF_LIGHT) ? 1 : 0;
-
-			/* Artifact lights provide permanent bright light */
-			if (o_ptr->artifact)
-				amt = 3 + flag_inc;
-
-			/* Non-artifact lights and those without fuel provide no light */
-			else if (!burn_light || o_ptr->timeout == 0)
-				amt = 0;
-
-			/* All lit lights provide at least radius 2 light */
-			else
-			{
-				amt = 2 + flag_inc;
-
-				/* Torches below half fuel provide less light */
-				if (o_ptr->sval == SV_LIGHT_TORCH && o_ptr->timeout < (FUEL_TORCH / 4))
-				    amt--;
-			}
-		}
-
-		else
-		{
-			/* LIGHT flag on an non-cursed non-lights always increases radius */
-			if (of_has(f, OF_LIGHT)) extra_light++;
-		}
+		if (o_ptr->tval == TV_LIGHT && !of_has(o_ptr->flags, OF_NO_FUEL) &&
+				o_ptr->timeout == 0)
+			/* Lights without fuel provide no light */
+			amt = 0;
 
 		/* Alter p_ptr->cur_light if reasonable */
-		if (new_light < amt)
-		    new_light = amt;
+	    new_light += amt;
 	}
 
-	/* Add bonus from LIGHT flags */
-	new_light += extra_light;
-
 	/* Limit light */
 	new_light = MIN(new_light, 5);
 	new_light = MAX(new_light, 0);
 
 	/* Notice changes in the "light radius" */
-	if (old_light != new_light)
-	{
+	if (old_light != new_light) {
 		/* Update the visuals */
 		p_ptr->cur_light = new_light;
 		p_ptr->update |= (PU_UPDATE_VIEW | PU_MONSTERS);
@@ -1597,8 +1530,6 @@ void calc_bonuses(object_type inventory[], player_state *state, bool id_only)
 			o_ptr->pval[which_pval(o_ptr, OF_DEX)];
 		if (of_has(f, OF_CON)) state->stat_add[A_CON] +=
 			o_ptr->pval[which_pval(o_ptr, OF_CON)];
-		if (of_has(f, OF_CHR)) state->stat_add[A_CHR] +=
-			o_ptr->pval[which_pval(o_ptr, OF_CHR)];
 
 		/* Affect stealth */
 		if (of_has(f, OF_STEALTH)) state->skills[SKILL_STEALTH] +=
@@ -1689,12 +1620,8 @@ void calc_bonuses(object_type inventory[], player_state *state, bool id_only)
 		/* Extract modifier */
 		add = state->stat_add[i];
 
-		/* Maximize mode */
-		if (OPT(birth_maximize))
-		{
-			/* Modify the stats for race/class */
-			add += (p_ptr->race->r_adj[i] + p_ptr->class->c_adj[i]);
-		}
+		/* Modify the stats for race/class */
+		add += (p_ptr->race->r_adj[i] + p_ptr->class->c_adj[i]);
 
 		/* Extract the new "stat_top" value for the stat */
 		top = modify_stat_value(p_ptr->stat_max[i], add);
@@ -1820,7 +1747,7 @@ void calc_bonuses(object_type inventory[], player_state *state, bool id_only)
 		p_ptr->timed[TMD_AFRAID] = p_ptr->timed[TMD_TERROR];
 
 	if (p_ptr->timed[TMD_TERROR])
-		state->speed += 5;
+		state->speed += 10;
 
 	/* Fear can come from item flags too */
 	if (check_state(p_ptr, OF_AFRAID, p_ptr->state.flags))
@@ -1864,9 +1791,6 @@ void calc_bonuses(object_type inventory[], player_state *state, bool id_only)
 	/* Apply "encumbrance" from weight */
 	if (j > i / 2) state->speed -= ((j - (i / 2)) / (i / 10));
 
-	/* Bloating slows the player down (a little) */
-	if (p_ptr->food >= PY_FOOD_MAX) state->speed -= 10;
-
 	/* Searching slows the player down */
 	if (p_ptr->searching) state->speed -= 10;
 
@@ -1891,9 +1815,6 @@ void calc_bonuses(object_type inventory[], player_state *state, bool id_only)
 
 	/*** Modify skills ***/
 
-	/* Affect Skill -- stealth (bonus one) */
-	state->skills[SKILL_STEALTH] += 1;
-
 	/* Affect Skill -- disarming (DEX and INT) */
 	state->skills[SKILL_DISARM] += adj_dex_dis[state->stat_ind[A_DEX]];
 	state->skills[SKILL_DISARM] += adj_int_dis[state->stat_ind[A_INT]];
@@ -2319,13 +2240,13 @@ void update_stuff(struct player *p)
 	if (p->update & (PU_FORGET_VIEW))
 	{
 		p->update &= ~(PU_FORGET_VIEW);
-		forget_view();
+		forget_view(cave);
 	}
 
 	if (p->update & (PU_UPDATE_VIEW))
 	{
 		p->update &= ~(PU_UPDATE_VIEW);
-		update_view();
+		update_view(cave, p);
 	}
 
 
@@ -2394,7 +2315,6 @@ static const struct flag_event_trigger redraw_events[] =
 	{ PR_STATUS,  EVENT_STATUS },
 	{ PR_STUDY,   EVENT_STUDYSTATUS },
 	{ PR_DTRAP,   EVENT_DETECTIONSTATUS },
-	{ PR_BUTTONS, EVENT_MOUSEBUTTONS },
 
 	{ PR_INVEN,   EVENT_INVENTORY },
 	{ PR_EQUIP,   EVENT_EQUIPMENT },
