@@ -111,14 +111,14 @@ bool warding_glyph(void)
 	int py = p_ptr->py;
 	int px = p_ptr->px;
 
-	if (cave->feat[py][px] != FEAT_FLOOR)
+	if (!cave_canward(cave, py, px))
 	{
 		msg("There is no clear floor on which to cast the spell.");
 		return FALSE;
 	}
 
 	/* Create a glyph */
-	cave_set_feat(cave, py, px, FEAT_GLYPH);
+	cave_add_ward(cave, py, px);
 	return TRUE;
 }
 
@@ -201,10 +201,6 @@ bool do_dec_stat(int stat, bool perma)
 			if (check_state(p_ptr, OF_SUST_CON, p_ptr->state.flags)) sust = TRUE;
 			wieldeds_notice_flag(p_ptr, OF_SUST_CON);
 			break;
-		case A_CHR:
-			if (check_state(p_ptr, OF_SUST_CHR, p_ptr->state.flags)) sust = TRUE;
-			wieldeds_notice_flag(p_ptr, OF_SUST_CHR);
-			break;
 	}
 
 	/* Sustain */
@@ -310,10 +306,10 @@ void identify_pack(void)
 		if (object_is_known(o_ptr)) continue;
 
 		/* Identify it */
-		do_ident_item(i, o_ptr);
+		do_ident_item(o_ptr);
 
-		/* repeat with same slot */
-		i--;
+		/* Reset loop in case the pack reordered. */
+		i = 0;
 	}
 }
 
@@ -417,13 +413,28 @@ bool restore_level(void)
 /*
  * Set word of recall as appropriate
  */
-void set_recall(void)
+bool set_recall(void)
 {
-	/* Ironman */
-	if (OPT(birth_ironman) && !p_ptr->total_winner)
+	/* No recall */
+	if (OPT(birth_no_recall) && !p_ptr->total_winner)
 	{
 		msg("Nothing happens.");
-		return;
+		return FALSE;
+	}
+    
+	/* No recall from quest levels with force_descend */
+	if (OPT(birth_force_descend) && (is_quest(p_ptr->depth))) {
+		msg("Nothing happens.");
+		return TRUE;
+	}
+
+    /* Warn the player if they're descending to an unrecallable level */
+	if (OPT(birth_force_descend) && !(p_ptr->depth) &&
+			(is_quest(p_ptr->max_depth + 1))) {
+		if (!get_check("Are you sure you want to descend? ")) {
+			msg("You prevent the recall from taking place.");
+			return FALSE;
+		}
 	}
 
 	/* Activate recall */
@@ -432,11 +443,8 @@ void set_recall(void)
 		/* Reset recall depth */
 		if ((p_ptr->depth > 0) && (p_ptr->depth != p_ptr->max_depth))
 		{
-			/*
-			 * ToDo: Add a new player_type field "recall_depth"
-			 * ToDo: Poll: Always reset recall depth?
-			 */
-			 if (get_check("Reset recall depth? "))
+			/* ToDo: Add a new player_type field "recall_depth" */
+			if (get_check("Reset recall depth? "))
 				p_ptr->max_depth = p_ptr->depth;
 		}
 
@@ -447,13 +455,18 @@ void set_recall(void)
 	/* Deactivate recall */
 	else
 	{
+		if (!get_check("Word of Recall is already active.  Do you want to cancel it? "))
+			return FALSE;
+
 		p_ptr->word_recall = 0;
 		msg("A tension leaves the air around you...");
 	}
 
 	/* Redraw status line */
-	p_ptr->redraw = PR_STATUS;
+	p_ptr->redraw |= PR_STATUS;
 	handle_stuff(p_ptr);
+
+	return TRUE;
 }
 
 
@@ -488,8 +501,8 @@ void map_area(void)
 	/* Drag the co-ordinates into the dungeon */
 	if (y1 < 0) y1 = 0;
 	if (x1 < 0) x1 = 0;
-	if (y2 > DUNGEON_HGT - 1) y2 = DUNGEON_HGT - 1;
-	if (x2 > DUNGEON_WID - 1) x2 = DUNGEON_WID - 1;
+	if (y2 > cave->height - 1) y2 = cave->height - 1;
+	if (x2 > cave->width - 1) x2 = cave->width - 1;
 
 	/* Scan the dungeon */
 	for (y = y1; y < y2; y++)
@@ -497,12 +510,12 @@ void map_area(void)
 		for (x = x1; x < x2; x++)
 		{
 			/* All non-walls are "checked" */
-			if (cave->feat[y][x] < FEAT_SECRET)
+			if (!cave_seemslikewall(cave, y, x))
 			{
-				if (!in_bounds_fully(y, x)) continue;
+				if (!cave_in_bounds_fully(cave, y, x)) continue;
 
 				/* Memorize normal features */
-				if (cave->feat[y][x] > FEAT_INVIS)
+				if (cave_isinteresting(cave, y, x))
 				{
 					/* Memorize the object */
 					cave->info[y][x] |= (CAVE_MARK);
@@ -516,7 +529,7 @@ void map_area(void)
 					int xx = x + ddx_ddd[i];
 
 					/* Memorize walls (etc) */
-					if (cave->feat[yy][xx] >= FEAT_SECRET)
+					if (cave_seemslikewall(cave, yy, xx))
 					{
 						/* Memorize the walls */
 						cave->info[yy][xx] |= (CAVE_MARK);
@@ -540,6 +553,8 @@ bool detect_traps(bool aware)
 
 	bool detect = FALSE;
 
+	object_type *o_ptr;
+
 	(void)aware;
 
 	/* Pick an area to map */
@@ -557,10 +572,10 @@ bool detect_traps(bool aware)
 	{
 		for (x = x1; x < x2; x++)
 		{
-			if (!in_bounds_fully(y, x)) continue;
+			if (!cave_in_bounds_fully(cave, y, x)) continue;
 
 			/* Detect invisible traps */
-			if (cave->feat[y][x] == FEAT_INVIS)
+			if (cave_issecrettrap(cave, y, x))
 			{
 				/* Pick a trap */
 				pick_trap(y, x);
@@ -576,6 +591,26 @@ bool detect_traps(bool aware)
 				detect = TRUE;
 			}
 
+			/* Scan all objects in the grid to look for traps on chests */
+			for (o_ptr = get_first_object(y, x); o_ptr; o_ptr = get_next_object(o_ptr))
+			{
+				/* Skip anything not a trapped chest */
+				if (!is_trapped_chest(o_ptr)) continue;
+
+				/* Identify once */
+				if (!object_is_known(o_ptr))
+				{
+					/* Know the trap */
+					object_notice_everything(o_ptr);
+
+					/* Notice it */
+					disturb(p_ptr, 0, 0);
+
+					/* We found something to detect */
+					detect = TRUE;
+				}
+			}
+
 			/* Mark as trap-detected */
 			cave->info2[y][x] |= CAVE2_DTRAP;
 		}
@@ -586,7 +621,14 @@ bool detect_traps(bool aware)
 	{
 		for (x = x1 - 1; x < x2 + 1; x++)
 		{
-			if (!in_bounds_fully(y, x)) continue;
+			if (!cave_in_bounds_fully(cave, y, x)) continue;
+
+			/* see if this grid is on the edge */
+			if (dtrap_edge(y, x)) {
+				cave->info2[y][x] |= CAVE2_DEDGE;
+			} else {
+				cave->info2[y][x] &= ~CAVE2_DEDGE;
+			}
 
 			/* Redraw */
 			cave_light_spot(cave, y, x);
@@ -637,17 +679,14 @@ bool detect_doorstairs(bool aware)
 	{
 		for (x = x1; x < x2; x++)
 		{
-			if (!in_bounds_fully(y, x)) continue;
+			if (!cave_in_bounds_fully(cave, y, x)) continue;
 
 			/* Detect secret doors */
-			if (cave->feat[y][x] == FEAT_SECRET)
+			if (cave_issecretdoor(cave, y, x))
 				place_closed_door(cave, y, x);
 
 			/* Detect doors */
-			if (((cave->feat[y][x] >= FEAT_DOOR_HEAD) &&
-			     (cave->feat[y][x] <= FEAT_DOOR_TAIL)) ||
-			    ((cave->feat[y][x] == FEAT_OPEN) ||
-			     (cave->feat[y][x] == FEAT_BROKEN)))
+			if (cave_isdoor(cave, y, x))
 			{
 				/* Hack -- Memorize */
 				cave->info[y][x] |= (CAVE_MARK);
@@ -660,8 +699,7 @@ bool detect_doorstairs(bool aware)
 			}
 
 			/* Detect stairs */
-			if ((cave->feat[y][x] == FEAT_LESS) ||
-			    (cave->feat[y][x] == FEAT_MORE))
+			if (cave_isstairs(cave, y, x))
 			{
 				/* Hack -- Memorize */
 				cave->info[y][x] |= (CAVE_MARK);
@@ -690,7 +728,7 @@ bool detect_doorstairs(bool aware)
 /*
  * Detect all treasure around the player.
  */
-bool detect_treasure(bool aware)
+bool detect_treasure(bool aware, bool full)
 {
 	int i;
 	int y, x;
@@ -711,24 +749,14 @@ bool detect_treasure(bool aware)
 
 
 	/* Scan the dungeon */
-	for (y = y1; y < y2; y++)
-	{
-		for (x = x1; x < x2; x++)
-		{
-			if (!in_bounds_fully(y, x)) continue;
+	for (y = y1; y < y2; y++) {
+		for (x = x1; x < x2; x++) {
+			if (!cave_in_bounds_fully(cave, y, x)) continue;
 
-			/* Notice embedded gold */
-			if ((cave->feat[y][x] == FEAT_MAGMA_H) ||
-			    (cave->feat[y][x] == FEAT_QUARTZ_H))
-			{
-				/* Expose the gold */
-				cave->feat[y][x] += 0x02;
-			}
+			cave_show_vein(cave, y, x);
 
 			/* Magma/Quartz + Known Gold */
-			if ((cave->feat[y][x] == FEAT_MAGMA_K) ||
-			    (cave->feat[y][x] == FEAT_QUARTZ_K))
-			{
+			if (cave_hasgoldvein(cave, y, x)) {
 				/* Hack -- Memorize */
 				cave->info[y][x] |= (CAVE_MARK);
 
@@ -742,8 +770,7 @@ bool detect_treasure(bool aware)
 	}
 
 	/* Scan objects */
-	for (i = 1; i < o_max; i++)
-	{
+	for (i = 1; i < o_max; i++)	{
 		object_type *o_ptr = object_byid(i);
 
 		/* Skip dead objects */
@@ -759,14 +786,15 @@ bool detect_treasure(bool aware)
 		/* Only detect nearby objects */
 		if (x < x1 || y < y1 || x > x2 || y > y2) continue;
 
-		/* Hack -- memorize it */
-		o_ptr->marked = TRUE;
+		/* Memorize it */
+		if (o_ptr->marked < MARK_SEEN)
+			o_ptr->marked = full ? MARK_SEEN : MARK_AWARE;
 
 		/* Redraw */
 		cave_light_spot(cave, y, x);
 
 		/* Detect */
-		if (!squelch_item_ok(o_ptr))
+		if (!squelch_item_ok(o_ptr) || !full)
 			objects = TRUE;
 	}
 
@@ -809,19 +837,13 @@ bool detect_close_buried_treasure(void)
 	{
 		for (x = x1; x < x2; x++)
 		{
-			if (!in_bounds_fully(y, x)) continue;
+			if (!cave_in_bounds_fully(cave, y, x)) continue;
 
 			/* Notice embedded gold */
-			if ((cave->feat[y][x] == FEAT_MAGMA_H) ||
-			    (cave->feat[y][x] == FEAT_QUARTZ_H))
-			{
-				/* Expose the gold */
-				cave->feat[y][x] += 0x02;
-			}
+			cave_show_vein(cave, y, x);
 
 			/* Magma/Quartz + Known Gold */
-			if ((cave->feat[y][x] == FEAT_MAGMA_K) ||
-			    (cave->feat[y][x] == FEAT_QUARTZ_K))
+			if (cave_hasgoldvein(cave, y, x))
 			{
 				/* Hack -- Memorize */
 				cave->info[y][x] |= (CAVE_MARK);
@@ -838,84 +860,6 @@ bool detect_close_buried_treasure(void)
 	return (gold_buried);
 }
 
-/*
- * Detect "magic" objects around the player.
- *
- * This will light up all spaces with "magic" items, including artifacts,
- * ego-items, potions, scrolls, books, rods, wands, staves, amulets, rings,
- * and "enchanted" items of the "good" variety.
- *
- * It can probably be argued that this function is now too powerful.
- */
-bool detect_objects_magic(bool aware)
-{
-	int i, y, x, tv;
-	int x1, x2, y1, y2;
-
-	bool detect = FALSE;
-
-
-	/* Pick an area to map */
-	y1 = p_ptr->py - DETECT_DIST_Y;
-	y2 = p_ptr->py + DETECT_DIST_Y;
-	x1 = p_ptr->px - DETECT_DIST_X;
-	x2 = p_ptr->px + DETECT_DIST_X;
-
-	if (y1 < 0) y1 = 0;
-	if (x1 < 0) x1 = 0;
-
-
-	/* Scan all objects */
-	for (i = 1; i < o_max; i++)
-	{
-		object_type *o_ptr = object_byid(i);
-
-		/* Skip dead objects */
-		if (!o_ptr->kind) continue;
-
-		/* Skip held objects */
-		if (o_ptr->held_m_idx) continue;
-
-		/* Location */
-		y = o_ptr->iy;
-		x = o_ptr->ix;
-
-
-		/* Only detect nearby objects */
-		if (x < x1 || y < y1 || x > x2 || y > y2) continue;
-
-		/* Examine the tval */
-		tv = o_ptr->tval;
-
-		/* Artifacts, misc magic items, or enchanted wearables */
-		if (o_ptr->artifact || o_ptr->ego ||
-		    (tv == TV_AMULET) || (tv == TV_RING) ||
-		    (tv == TV_STAFF) || (tv == TV_WAND) || (tv == TV_ROD) ||
-		    (tv == TV_SCROLL) || (tv == TV_POTION) ||
-		    (tv == TV_MAGIC_BOOK) || (tv == TV_PRAYER_BOOK) ||
-		    ((o_ptr->to_a > 0) || (o_ptr->to_h + o_ptr->to_d > 0)))
-		{
-			/* Memorize the item */
-			o_ptr->marked = TRUE;
-
-			/* Redraw */
-			cave_light_spot(cave, y, x);
-
-			/* Detect */
-			if (!squelch_item_ok(o_ptr))
-				detect = TRUE;
-		}
-	}
-
-	if (detect)
-		msg("You sense the presence of magic objects!");
-	else if (aware && !detect)
-		msg("You sense no magic objects.");
-
-	return detect;
-}
-
-
 /*
  * Detect "normal" monsters around the player.
  */
@@ -942,10 +886,9 @@ bool detect_monsters_normal(bool aware)
 	for (i = 1; i < cave_monster_max(cave); i++)
 	{
 		monster_type *m_ptr = cave_monster(cave, i);
-		monster_race *r_ptr = &r_info[m_ptr->r_idx];
-
+		
 		/* Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
 
 		/* Location */
 		y = m_ptr->fy;
@@ -955,13 +898,13 @@ bool detect_monsters_normal(bool aware)
 		if (x < x1 || y < y1 || x > x2 || y > y2) continue;
 
 		/* Detect all non-invisible, obvious monsters */
-		if (!rf_has(r_ptr->flags, RF_INVISIBLE) && !m_ptr->unaware)
+		if (!rf_has(m_ptr->race->flags, RF_INVISIBLE) && !m_ptr->unaware)
 		{
 			/* Hack -- Detect the monster */
 			m_ptr->mflag |= (MFLAG_MARK | MFLAG_SHOW);
 
 			/* Update the monster */
-			update_mon(i, FALSE);
+			update_mon(m_ptr, FALSE);
 
 			/* Detect */
 			flag = TRUE;
@@ -1002,11 +945,12 @@ bool detect_monsters_invis(bool aware)
 	for (i = 1; i < cave_monster_max(cave); i++)
 	{
 		monster_type *m_ptr = cave_monster(cave, i);
-		monster_race *r_ptr = &r_info[m_ptr->r_idx];
-		monster_lore *l_ptr = &l_list[m_ptr->r_idx];
+		monster_lore *l_ptr;
 
 		/* Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
+		
+		l_ptr = get_lore(m_ptr->race);
 
 		/* Location */
 		y = m_ptr->fy;
@@ -1016,13 +960,13 @@ bool detect_monsters_invis(bool aware)
 		if (x < x1 || y < y1 || x > x2 || y > y2) continue;
 
 		/* Detect invisible monsters */
-		if (rf_has(r_ptr->flags, RF_INVISIBLE))
+		if (rf_has(m_ptr->race->flags, RF_INVISIBLE))
 		{
 			/* Take note that they are invisible */
 			rf_on(l_ptr->flags, RF_INVISIBLE);
 
 			/* Update monster recall window */
-			if (p_ptr->monster_race_idx == m_ptr->r_idx)
+			if (p_ptr->monster_race == m_ptr->race)
 			{
 				/* Redraw stuff */
 				p_ptr->redraw |= (PR_MONSTER);
@@ -1032,7 +976,7 @@ bool detect_monsters_invis(bool aware)
 			m_ptr->mflag |= (MFLAG_MARK | MFLAG_SHOW);
 
 			/* Update the monster */
-			update_mon(i, FALSE);
+			update_mon(m_ptr, FALSE);
 
 			/* Detect */
 			flag = TRUE;
@@ -1073,12 +1017,13 @@ bool detect_monsters_evil(bool aware)
 	for (i = 1; i < cave_monster_max(cave); i++)
 	{
 		monster_type *m_ptr = cave_monster(cave, i);
-		monster_race *r_ptr = &r_info[m_ptr->r_idx];
-		monster_lore *l_ptr = &l_list[m_ptr->r_idx];
+		monster_lore *l_ptr;
 
 		/* Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
 
+		l_ptr = get_lore(m_ptr->race);
+		
 		/* Location */
 		y = m_ptr->fy;
 		x = m_ptr->fx;
@@ -1087,13 +1032,13 @@ bool detect_monsters_evil(bool aware)
 		if (x < x1 || y < y1 || x > x2 || y > y2) continue;
 
 		/* Detect evil monsters */
-		if (rf_has(r_ptr->flags, RF_EVIL))
+		if (rf_has(m_ptr->race->flags, RF_EVIL))
 		{
 			/* Take note that they are evil */
 			rf_on(l_ptr->flags, RF_EVIL);
 
 			/* Update monster recall window */
-			if (p_ptr->monster_race_idx == m_ptr->r_idx)
+			if (p_ptr->monster_race == m_ptr->race)
 			{
 				/* Redraw stuff */
 				p_ptr->redraw |= (PR_MONSTER);
@@ -1103,7 +1048,7 @@ bool detect_monsters_evil(bool aware)
 			m_ptr->mflag |= (MFLAG_MARK | MFLAG_SHOW);
 
 			/* Update the monster */
-			update_mon(i, FALSE);
+			update_mon(m_ptr, FALSE);
 
 			/* Detect */
 			flag = TRUE;
@@ -1118,8 +1063,39 @@ bool detect_monsters_evil(bool aware)
 	return flag;
 }
 
+/* 
+ * Detect all monsters on the level (used for *enlightenment* only)
+ */
+bool detect_monsters_entire_level(void)
+{
+	int i;
+	bool detect = FALSE;
+	
+	for (i = 1; i < cave_monster_max(cave); i++)
+	{
+		monster_type *m_ptr = cave_monster(cave, i);
+	
+		/* Skip dead monsters */
+		if (!m_ptr->race) continue;
 
+		/* Detect the monster */
+		m_ptr->mflag |= (MFLAG_MARK | MFLAG_SHOW);
 
+		/* Update the monster */
+		update_mon(m_ptr, FALSE);
+		
+		detect = TRUE;
+	}
+	
+	if (detect)
+		msg("An image of all nearby life-forms appears in your mind");
+	else
+		/* Let's see who the first person is to ever see this message! */
+		msg("The level is devoid of life");
+		
+	return detect;
+}
+	
 /*
  * Detect everything
  */
@@ -1130,7 +1106,7 @@ bool detect_all(bool aware)
 	/* Detect everything */
 	if (detect_traps(aware)) detect = TRUE;
 	if (detect_doorstairs(aware)) detect = TRUE;
-	if (detect_treasure(aware)) detect = TRUE;
+	if (detect_treasure(aware, FALSE)) detect = TRUE;
 	if (detect_monsters_invis(aware)) detect = TRUE;
 	if (detect_monsters_normal(aware)) detect = TRUE;
 
@@ -1149,7 +1125,7 @@ void stair_creation(void)
 	int px = p_ptr->px;
 
 	/* Only allow stairs to be created on empty floor */
-	if (cave->feat[py][px] != FEAT_FLOOR)
+	if (!cave_isfloor(cave, py, px))
 	{
 		msg("There is no empty floor here.");
 		return;
@@ -1157,24 +1133,8 @@ void stair_creation(void)
 
 	/* Push objects off the grid */
 	if (cave->o_idx[py][px]) push_object(py, px);
-	
-	/* Create a staircase */
-	if (!p_ptr->depth)
-	{
-		cave_set_feat(cave, py, px, FEAT_MORE);
-	}
-	else if (is_quest(p_ptr->depth) || (p_ptr->depth >= MAX_DEPTH-1))
-	{
-		cave_set_feat(cave, py, px, FEAT_LESS);
-	}
-	else if (randint0(100) < 50)
-	{
-		cave_set_feat(cave, py, px, FEAT_MORE);
-	}
-	else
-	{
-		cave_set_feat(cave, py, px, FEAT_LESS);
-	}
+
+	cave_add_stairs(cave, py, px, p_ptr->depth);
 }
 
 
@@ -1237,22 +1197,6 @@ static bool item_tester_hook_armour(const object_type *o_ptr)
  */
 static bool item_tester_unknown(const object_type *o_ptr)
 {
-	/* A hack for a hack - Disable this for the 3.1.2 release */
-	if (FALSE && object_is_not_known_consistently(o_ptr))
-	{
-		/*
-		 * This next hack is pretty terrible, but people playing
-		 * the nightlies will really appreciate not having to reidentify
-		 * every time a new IDENT_ flag is added.  It should be
-		 * removed when the codebase is stable.
-		 */
-		object_type *i_ptr = (object_type *) o_ptr;
-		if (!object_check_for_ident(i_ptr))
-			return TRUE;
-		else
-			return FALSE;
-	}
-
 	return object_is_known(o_ptr) ? FALSE : TRUE;
 }
 
@@ -1530,14 +1474,38 @@ bool ident_spell(void)
 
 
 	/* Identify the object */
-	do_ident_item(item, o_ptr);
+	do_ident_item(o_ptr);
 
 
 	/* Something happened */
 	return (TRUE);
 }
 
+/**
+ * Return TRUE if there are any objects available to identify (whether on floor or in inventory/equip.
+ */
+bool spell_identify_unknown_available(void)
+{
+	int floor_list[MAX_FLOOR_STACK];
+	int floor_num;
+	int i;
+	bool unidentified_inventory = FALSE;
 
+	item_tester_hook = item_tester_unknown;
+	floor_num = scan_floor(floor_list, N_ELEMENTS(floor_list), p_ptr->py, p_ptr->px, 0x0B);
+
+	for (i = 0; i < ALL_INVEN_TOTAL; i++) {
+		if (get_item_okay(i)) {
+			unidentified_inventory = TRUE;
+			break;
+		}
+	}
+
+	/* Reset the item tester to avoid bizarre behaviour of UI-only commands */
+	item_tester_hook = NULL;
+
+	return unidentified_inventory || floor_num > 0;
+}
 
 /*
  * Hook for "get_item()".  Determine if something is rechargable.
@@ -1672,7 +1640,7 @@ bool project_los(int typ, int dam, bool obvious)
 		monster_type *m_ptr = cave_monster(cave, i);
 
 		/* Paranoia -- Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
 
 		/* Location */
 		y = m_ptr->fy;
@@ -1772,12 +1740,11 @@ bool dispel_monsters(int dam)
 /*
  * Wake up all monsters, and speed up "los" monsters.
  */
-void aggravate_monsters(int who)
+void aggravate_monsters(struct monster *who)
 {
 	int i;
 
 	bool sleep = FALSE;
-	bool speed = FALSE;
 
 	/* Aggravate everyone nearby */
 	for (i = 1; i < cave_monster_max(cave); i++)
@@ -1785,31 +1752,28 @@ void aggravate_monsters(int who)
 		monster_type *m_ptr = cave_monster(cave, i);
 
 		/* Paranoia -- Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
 
 		/* Skip aggravating monster (or player) */
-		if (i == who) continue;
+		if (m_ptr == who) continue;
 
 		/* Wake up nearby sleeping monsters */
-		if (m_ptr->cdis < MAX_SIGHT * 2)
-		{
-			/* Wake up */
-			if (m_ptr->m_timed[MON_TMD_SLEEP])
-			{
-				/* Wake up */
-				mon_clear_timed(i, MON_TMD_SLEEP, MON_TMD_FLG_NOMESSAGE, FALSE);
-				sleep = TRUE;
-			}
+		if ((m_ptr->cdis < MAX_SIGHT * 2) && m_ptr->m_timed[MON_TMD_SLEEP]) {
+			mon_clear_timed(m_ptr, MON_TMD_SLEEP, MON_TMD_FLG_NOMESSAGE, FALSE);
+			sleep = TRUE;
 		}
 
 		/* Speed up monsters in line of sight */
-		if (player_has_los_bold(m_ptr->fy, m_ptr->fx))
-			mon_inc_timed(i, MON_TMD_FAST, 25, MON_TMD_FLG_NOTIFY, FALSE);
+		if (player_has_los_bold(m_ptr->fy, m_ptr->fx)) {
+			mon_inc_timed(m_ptr, MON_TMD_FAST, 25, MON_TMD_FLG_NOTIFY, FALSE);
+			if (is_mimicking(m_ptr)) {
+				become_aware(m_ptr);
+			}
+		}
 	}
 
 	/* Messages */
-	if (speed) msg("You feel a sudden stirring nearby!");
-	else if (sleep) msg("You hear a sudden stirring in the distance!");
+	if (sleep) msg("You hear a sudden stirring in the distance!");
 }
 
 
@@ -1831,16 +1795,15 @@ bool banishment(void)
 	for (i = 1; i < cave_monster_max(cave); i++)
 	{
 		monster_type *m_ptr = cave_monster(cave, i);
-		monster_race *r_ptr = &r_info[m_ptr->r_idx];
-
+		
 		/* Paranoia -- Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
 
 		/* Hack -- Skip Unique Monsters */
-		if (rf_has(r_ptr->flags, RF_UNIQUE)) continue;
+		if (rf_has(m_ptr->race->flags, RF_UNIQUE)) continue;
 
 		/* Skip "wrong" monsters */
-		if (r_ptr->d_char != (char)typ.code) continue;
+		if (!char_matches_key(m_ptr->race->d_char, typ.code)) continue;
 
 		/* Delete the monster */
 		delete_monster_idx(i);
@@ -1875,13 +1838,12 @@ bool mass_banishment(void)
 	for (i = 1; i < cave_monster_max(cave); i++)
 	{
 		monster_type *m_ptr = cave_monster(cave, i);
-		monster_race *r_ptr = &r_info[m_ptr->r_idx];
 
 		/* Paranoia -- Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
 
 		/* Hack -- Skip unique monsters */
-		if (rf_has(r_ptr->flags, RF_UNIQUE)) continue;
+		if (rf_has(m_ptr->race->flags, RF_UNIQUE)) continue;
 
 		/* Skip distant monsters */
 		if (m_ptr->cdis > MAX_SIGHT) continue;
@@ -1923,7 +1885,7 @@ bool probing(void)
 		monster_type *m_ptr = cave_monster(cave, i);
 
 		/* Paranoia -- Skip dead monsters */
-		if (!m_ptr->r_idx) continue;
+		if (!m_ptr->race) continue;
 
 		/* Require line of sight */
 		if (!player_has_los_bold(m_ptr->fy, m_ptr->fx)) continue;
@@ -1937,13 +1899,14 @@ bool probing(void)
 			if (!probe) msg("Probing...");
 
 			/* Get "the monster" or "something" */
-			monster_desc(m_name, sizeof(m_name), m_ptr, MDESC_IND1);
+			monster_desc(m_name, sizeof(m_name), m_ptr,
+					MDESC_IND_HID | MDESC_CAPITAL);
 
 			/* Describe the monster */
-			msg("%^s has %d hit points.", m_name, m_ptr->hp);
+			msg("%s has %d hit points.", m_name, m_ptr->hp);
 
 			/* Learn all of the non-spell, non-treasure flags */
-			lore_do_probe(i);
+			lore_do_probe(m_ptr);
 
 			/* Probe worked */
 			probe = TRUE;
@@ -1972,7 +1935,7 @@ bool probing(void)
  */
 void destroy_area(int y1, int x1, int r, bool full)
 {
-	int y, x, k, t;
+	int y, x, k;
 
 	bool flag = FALSE;
 
@@ -1993,8 +1956,8 @@ void destroy_area(int y1, int x1, int r, bool full)
 		for (x = (x1 - r); x <= (x1 + r); x++)
 		{
 			/* Skip illegal grids */
-			if (!in_bounds_fully(y, x)) continue;
-
+			if (!cave_in_bounds_fully(cave, y, x)) continue;
+			
 			/* Extract the distance */
 			k = distance(y1, x1, y, x);
 
@@ -2002,10 +1965,10 @@ void destroy_area(int y1, int x1, int r, bool full)
 			if (k > r) continue;
 
 			/* Lose room and vault */
-			cave->info[y][x] &= ~(CAVE_ROOM | CAVE_ICKY);
+			cave->info[y][x] &= ~(CAVE_ROOM | CAVE_VAULT);
 
-			/* Lose light and knowledge */
-			cave->info[y][x] &= ~(CAVE_GLOW | CAVE_MARK);
+			/* Lose light */
+			cave->info[y][x] &= ~(CAVE_GLOW);
 			
 			cave_light_spot(cave, y, x);
 
@@ -2024,41 +1987,19 @@ void destroy_area(int y1, int x1, int r, bool full)
 
 			/* Delete the monster (if any) */
 			delete_monster(y, x);
+			
+			/* Don't remove stairs */
+			if (cave_isstairs(cave, y, x)) continue;	
+			
+			/* Lose knowledge (keeping knowledge of stairs) */
+			cave->info[y][x] &= ~(CAVE_MARK);
 
 			/* Destroy any grid that isn't a permament wall */
 			if (!cave_isperm(cave, y, x))
 			{
-				int feat = FEAT_FLOOR;
-
 				/* Delete objects */
 				delete_object(y, x);
-
-				/* Wall (or floor) type */
-				t = randint0(200);
-
-				/* Granite */
-				if (t < 20)
-				{
-					/* Create granite wall */
-					feat = FEAT_WALL_EXTRA;
-				}
-
-				/* Quartz */
-				else if (t < 70)
-				{
-					/* Create quartz vein */
-					feat = FEAT_QUARTZ;
-				}
-
-				/* Magma */
-				else if (t < 100)
-				{
-					/* Create magma vein */
-					feat = FEAT_MAGMA;
-				}
-
-				/* Change the feature */
-				cave_set_feat(cave, y, x, feat);
+				cave_destroy(cave, y, x);
 			}
 		}
 	}
@@ -2111,7 +2052,7 @@ void earthquake(int cy, int cx, int r)
 	int py = p_ptr->py;
 	int px = p_ptr->px;
 
-	int i, t, y, x, yy, xx, dy, dx;
+	int i, y, x, yy, xx, dy, dx;
 
 	int damage = 0;
 
@@ -2150,13 +2091,13 @@ void earthquake(int cy, int cx, int r)
 			xx = cx + dx;
 
 			/* Skip illegal grids */
-			if (!in_bounds_fully(yy, xx)) continue;
+			if (!cave_in_bounds_fully(cave, yy, xx)) continue;
 
 			/* Skip distant grids */
 			if (distance(cy, cx, yy, xx) > r) continue;
 
 			/* Lose room and vault */
-			cave->info[yy][xx] &= ~(CAVE_ROOM | CAVE_ICKY);
+			cave->info[yy][xx] &= ~(CAVE_ROOM | CAVE_VAULT);
 
 			/* Lose light and knowledge */
 			cave->info[yy][xx] &= ~(CAVE_GLOW | CAVE_MARK);
@@ -2186,7 +2127,7 @@ void earthquake(int cy, int cx, int r)
 			x = px + ddx_ddd[i];
 
 			/* Skip non-empty grids */
-			if (!cave_empty_bold(y, x)) continue;
+			if (!cave_isempty(cave, y, x)) continue;
 
 			/* Important -- Skip "quake" grids */
 			if (map[16+y-cy][16+x-cx]) continue;
@@ -2279,11 +2220,10 @@ void earthquake(int cy, int cx, int r)
 			/* Process monsters */
 			if (cave->m_idx[yy][xx] > 0)
 			{
-				monster_type *m_ptr = cave_monster(cave, cave->m_idx[yy][xx]);
-				monster_race *r_ptr = &r_info[m_ptr->r_idx];
-
+				monster_type *m_ptr = cave_monster_at(cave, yy, xx);
+				
 				/* Most monsters cannot co-exist with rock */
-				if (!flags_test(r_ptr->flags, RF_SIZE, RF_KILL_WALL, RF_PASS_WALL, FLAG_END))
+				if (!flags_test(m_ptr->race->flags, RF_SIZE, RF_KILL_WALL, RF_PASS_WALL, FLAG_END))
 				{
 					char m_name[80];
 
@@ -2291,7 +2231,7 @@ void earthquake(int cy, int cx, int r)
 					sn = 0;
 
 					/* Monster can move to escape the wall */
-					if (!rf_has(r_ptr->flags, RF_NEVER_MOVE))
+					if (!rf_has(m_ptr->race->flags, RF_NEVER_MOVE))
 					{
 						/* Look for safety */
 						for (i = 0; i < 8; i++)
@@ -2301,10 +2241,11 @@ void earthquake(int cy, int cx, int r)
 							x = xx + ddx_ddd[i];
 
 							/* Skip non-empty grids */
-							if (!cave_empty_bold(y, x)) continue;
+							if (!cave_isempty(cave, y, x)) continue;
 
 							/* Hack -- no safety on glyph of warding */
-							if (cave->feat[y][x] == FEAT_GLYPH) continue;
+							if (cave_iswarded(cave, y, x))
+								continue;
 
 							/* Important -- Skip "quake" grids */
 							if (map[16+y-cy][16+x-cx]) continue;
@@ -2319,21 +2260,21 @@ void earthquake(int cy, int cx, int r)
 					}
 
 					/* Describe the monster */
-					monster_desc(m_name, sizeof(m_name), m_ptr, 0);
+					monster_desc(m_name, sizeof(m_name), m_ptr, MDESC_STANDARD);
 
 					/* Scream in pain */
-					msg("%^s wails out in pain!", m_name);
+					msg("%s wails out in pain!", m_name);
 
 					/* Take damage from the quake */
 					damage = (sn ? damroll(4, 8) : (m_ptr->hp + 1));
 
 					/* Monster is certainly awake */
-					mon_clear_timed(cave->m_idx[yy][xx], MON_TMD_SLEEP,
-						MON_TMD_FLG_NOMESSAGE, FALSE);
+					mon_clear_timed(m_ptr, MON_TMD_SLEEP,
+							MON_TMD_FLG_NOMESSAGE, FALSE);
 
 					/* If the quake finished the monster off, show message */
 					if (m_ptr->hp < damage && m_ptr->hp >= 0)
-						msg("%^s is embedded in the rock!", m_name);
+						msg("%s is embedded in the rock!", m_name);
 
 					/* Apply damage directly */
 					m_ptr->hp -= damage;
@@ -2380,7 +2321,7 @@ void earthquake(int cy, int cx, int r)
 			xx = cx + dx;
 
 			/* ignore invalid grids */
-			if (!in_bounds_fully(yy, xx)) continue;
+			if (!cave_in_bounds_fully(cave, yy, xx)) continue;
 
 			/* Note unaffected grids for light changes, etc. */
 			if (!map[16+yy-cy][16+xx-cx])
@@ -2391,39 +2332,8 @@ void earthquake(int cy, int cx, int r)
 			/* Destroy location (if valid) */
 			else if (cave_valid_bold(yy, xx))
 			{
-				int feat = FEAT_FLOOR;
-
-				bool floor = cave_floor_bold(yy, xx);
-
-				/* Delete objects */
 				delete_object(yy, xx);
-
-				/* Wall (or floor) type */
-				t = (floor ? randint0(100) : 200);
-
-				/* Granite */
-				if (t < 20)
-				{
-					/* Create granite wall */
-					feat = FEAT_WALL_EXTRA;
-				}
-
-				/* Quartz */
-				else if (t < 70)
-				{
-					/* Create quartz vein */
-					feat = FEAT_QUARTZ;
-				}
-
-				/* Magma */
-				else if (t < 100)
-				{
-					/* Create magma vein */
-					feat = FEAT_MAGMA;
-				}
-
-				/* Change the feature */
-				cave_set_feat(cave, yy, xx, feat);
+				cave_earthquake(cave, yy, xx);
 			}
 		}
 	}
@@ -2465,9 +2375,6 @@ static void cave_light(struct point_set *ps)
 		int y = ps->pts[i].y;
 		int x = ps->pts[i].x;
 
-		/* No longer in the array */
-		cave->info[y][x] &= ~(CAVE_TEMP);
-
 		/* Perma-Light */
 		cave->info[y][x] |= (CAVE_GLOW);
 	}
@@ -2492,20 +2399,19 @@ static void cave_light(struct point_set *ps)
 		{
 			int chance = 25;
 
-			monster_type *m_ptr = cave_monster(cave, cave->m_idx[y][x]);
-			monster_race *r_ptr = &r_info[m_ptr->r_idx];
+			monster_type *m_ptr = cave_monster_at(cave, y, x);
 
 			/* Stupid monsters rarely wake up */
-			if (rf_has(r_ptr->flags, RF_STUPID)) chance = 10;
+			if (rf_has(m_ptr->race->flags, RF_STUPID)) chance = 10;
 
 			/* Smart monsters always wake up */
-			if (rf_has(r_ptr->flags, RF_SMART)) chance = 100;
+			if (rf_has(m_ptr->race->flags, RF_SMART)) chance = 100;
 
 			/* Sometimes monsters wake up */
 			if (m_ptr->m_timed[MON_TMD_SLEEP] && (randint0(100) < chance))
 			{
 				/* Wake up! */
-				mon_clear_timed(cave->m_idx[y][x], MON_TMD_SLEEP,
+				mon_clear_timed(m_ptr, MON_TMD_SLEEP,
 					MON_TMD_FLG_NOTIFY, FALSE);
 
 			}
@@ -2532,18 +2438,12 @@ static void cave_unlight(struct point_set *ps)
 		int y = ps->pts[i].y;
 		int x = ps->pts[i].x;
 
-		/* No longer in the array */
-		cave->info[y][x] &= ~(CAVE_TEMP);
-
 		/* Darken the grid */
 		cave->info[y][x] &= ~(CAVE_GLOW);
 
 		/* Hack -- Forget "boring" grids */
-		if (cave->feat[y][x] <= FEAT_INVIS)
-		{
-			/* Forget the grid */
+		if (!cave_isinteresting(cave, y, x))
 			cave->info[y][x] &= ~(CAVE_MARK);
-		}
 	}
 
 	/* Fully update the visuals */
@@ -2568,14 +2468,11 @@ static void cave_unlight(struct point_set *ps)
  */
 static void cave_room_aux(struct point_set *seen, int y, int x)
 {
-	/* Avoid infinite recursion */
-	if (cave->info[y][x] & (CAVE_TEMP)) return;
+	if (point_set_contains(seen, y, x))
+		return;
 
-	/* Do not "leave" the current room */
-	if (!(cave->info[y][x] & (CAVE_ROOM))) return;
-
-	/* Mark the grid as "seen" */
-	cave->info[y][x] |= (CAVE_TEMP);
+	if (!cave_isroom(cave, y, x))
+		return;
 
 	/* Add it to the "seen" set */
 	add_to_point_set(seen, y, x);
@@ -2601,7 +2498,7 @@ static void light_room(int y1, int x1, bool light)
 		x = ps->pts[i].x, y = ps->pts[i].y;
 
 		/* Walls get lit, but stop light */
-		if (!cave_floor_bold(y, x)) continue;
+		if (!cave_ispassable(cave, y, x)) continue;
 
 		/* Spread adjacent */
 		cave_room_aux(ps, y + 1, x);
@@ -3251,7 +3148,6 @@ void ring_of_power(int dir)
 			player_stat_dec(p_ptr, A_WIS, TRUE);
 			player_stat_dec(p_ptr, A_DEX, TRUE);
 			player_stat_dec(p_ptr, A_CON, TRUE);
-			player_stat_dec(p_ptr, A_CHR, TRUE);
 
 			/* Lose some experience (permanently) */
 			player_exp_lose(p_ptr, p_ptr->exp / 4, TRUE);
@@ -3296,24 +3192,20 @@ void ring_of_power(int dir)
 
 /*
  * Identify an item.
- *
- * `item` is used to print the slot occupied by an object in equip/inven.
- * Any negative value assigned to "item" can be used for specifying an object
- * on the floor.
  */
-void do_ident_item(int item, object_type *o_ptr)
+void do_ident_item(object_type *o_ptr)
 {
 	char o_name[80];
 
 	u32b msg_type = 0;
-	int i;
+	int i, index;
 	bool bad = TRUE;
+	object_type *original = ZNEW(object_type);
 
-	/* Identify it */
+    /* Identify and apply autoinscriptions. We use o_ptr here since it points to the inventory
+     * slot that the real object is in. o_ptr does NOT point to the object data itself. */
 	object_flavor_aware(o_ptr);
 	object_notice_everything(o_ptr);
-
-	/* Apply an autoinscription, if necessary */
 	apply_autoinscription(o_ptr);
 
 	/* Set squelch flag */
@@ -3322,45 +3214,59 @@ void do_ident_item(int item, object_type *o_ptr)
 	/* Recalculate bonuses */
 	p_ptr->update |= (PU_BONUS);
 
-	/* Combine / Reorder the pack (later) */
-	p_ptr->notice |= (PN_COMBINE | PN_REORDER | PN_SORT_QUIVER);
-
 	/* Window stuff */
 	p_ptr->redraw |= (PR_INVEN | PR_EQUIP);
 
+    /* Create a copy of the object for later reference, since o_ptr will remain the same after
+     * combinging and reordering. */
+    object_copy(original, o_ptr);
+
+    /* Force inventory cleanup so that we can display updated slot information in the message. */
+	combine_pack();
+	reorder_pack();
+	sort_quiver();
+
 	/* Description */
-	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_PREFIX | ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), original, ODESC_PREFIX | ODESC_FULL);
 
 	/* Determine the message type. */
 	/* CC: we need to think more carefully about how we define "bad" with
 	 * multiple pvals - currently using "all nonzero pvals < 0" */
-	for (i = 0; i < o_ptr->num_pvals; i++)
-		if (o_ptr->pval[i] > 0)
+	for (i = 0; i < original->num_pvals; i++)
+		if (original->pval[i] > 0)
 			bad = FALSE;
 
 	if (bad)
 		msg_type = MSG_IDENT_BAD;
-	else if (o_ptr->artifact)
+	else if (original->artifact)
 		msg_type = MSG_IDENT_ART;
-	else if (o_ptr->ego)
+	else if (original->ego)
 		msg_type = MSG_IDENT_EGO;
 	else
 		msg_type = MSG_GENERIC;
 
 	/* Log artifacts to the history list. */
-	if (o_ptr->artifact)
-		history_add_artifact(o_ptr->artifact, TRUE, TRUE);
+	if (original->artifact)
+		history_add_artifact(original->artifact, TRUE, TRUE);
+
+	/* Get the index of the inventory slot that our real original object is in. */
+	index = inventory_index_matching_object(original);
+	FREE(original);
 
 	/* Describe */
-	if (item >= INVEN_WIELD)
+	if (index >= INVEN_WIELD)
 	{
-		msgt(msg_type, "%^s: %s (%c).",
-			  describe_use(item), o_name, index_to_label(item));
+		/* Format and capitalise */
+		char *msg = format("%s: %s (%c).",
+			  describe_use(index), o_name, index_to_label(index));
+		my_strcap(msg);
+
+		msgt(msg_type, msg);
 	}
-	else if (item >= 0)
+	else if (index >= 0)
 	{
 		msgt(msg_type, "In your pack: %s (%c).",
-			  o_name, index_to_label(item));
+			  o_name, index_to_label(index));
 	}
 	else
 	{
