@@ -6,7 +6,7 @@
  */
 typedef struct
 {
-	int enum_val;
+	unsigned int enum_val;
 	const char *name;
 } quality_name_struct;
 
@@ -17,12 +17,14 @@ typedef enum
 {
 	TYPE_WEAPON_POINTY,
 	TYPE_WEAPON_BLUNT,
+	TYPE_WEAPON_GREAT,
 	TYPE_SHOOTER,
 	TYPE_MISSILE_SLING,
 	TYPE_MISSILE_BOW,
 	TYPE_MISSILE_XBOW,
 	TYPE_ARMOR_ROBE,
 	TYPE_ARMOR_BODY,
+	TYPE_ARMOR_DRAGON,
 	TYPE_ARMOR_CLOAK,
 	TYPE_ARMOR_ELVEN_CLOAK,
 	TYPE_ARMOR_SHIELD,
@@ -76,21 +78,19 @@ int remove_autoinscription(s16b kind);
 int add_autoinscription(s16b kind, const char *inscription);
 void autoinscribe_ground(void);
 void autoinscribe_pack(void);
-bool squelch_tval(int tval);
 void kind_squelch_clear(object_kind *k_ptr);
 bool kind_is_squelched_aware(const object_kind *k_ptr);
 bool kind_is_squelched_unaware(const object_kind *k_ptr);
 void kind_squelch_when_aware(object_kind *k_ptr);
 void kind_squelch_when_unaware(object_kind *k_ptr);
 bool squelch_item_ok(const object_type *o_ptr);
-bool squelch_hide_item(object_type *o_ptr);
+bool object_is_squelched(const object_type *o_ptr);
 void squelch_drop(void);
-void do_cmd_options_item(const char *title, int row);
-bool squelch_interactive(const object_type *o_ptr);
 
 byte squelch_level_of(const object_type *o_ptr);
 squelch_type_t squelch_type_of(const object_type *o_ptr);
 void object_squelch_flavor_of(const object_type *o_ptr);
+const char *squelch_name_for_type(squelch_type_t type);
 
 extern byte squelch_level[];
 extern const size_t squelch_size;
