@@ -5,11 +5,12 @@
 #include "object/object.h"
 #include "parser.h"
 
+extern bool store_in_store;
+
+
 #define STORE_INVEN_MAX		24    /* Max number of discrete objs in inven */
 #define STORE_TURNS		1000  /* Number of turns between turnovers */
 #define STORE_SHUFFLE		25    /* 1/Chance (per day) of an owner changing */
-#define STORE_MIN_KEEP  6       /* Min slots to "always" keep full (>0) */
-#define STORE_MAX_KEEP  18      /* Max slots to "always" keep full (<STORE_INVEN_MAX) */
 
 /* List of store indices */
 enum
@@ -38,14 +39,25 @@ struct store {
 	struct owner *owners;
 	struct owner *owner;
 	unsigned int sidx;
+	const char *name;
 
 	byte stock_num;			/* Stock -- Number of entries */
 	s16b stock_size;		/* Stock -- Total Size of Array */
 	object_type *stock;		/* Stock -- Actual stock items */
 
-	unsigned int table_num;     /* Table -- Number of entries */
-	unsigned int table_size;    /* Table -- Total Size of Array */
-	object_kind **table;        /* Table -- Legal item kinds */
+	/* Always stock these items */
+	size_t always_size;
+	size_t always_num;
+	object_kind **always_table;
+
+	/* Select a number of these items to stock */
+	size_t normal_size;
+	size_t normal_num;
+	object_kind **normal_table;
+
+	int turnover;
+	int normal_stock_min;
+	int normal_stock_max;
 };
 
 void store_init(void);
@@ -57,9 +69,8 @@ s32b price_item(const object_type *o_ptr, bool store_buying, int qty);
 
 extern struct owner *store_ownerbyidx(struct store *s, unsigned int idx);
 
-#ifdef TEST
+struct parser *init_parse_stores(void);
 extern struct parser *store_parser_new(void);
 extern struct parser *store_owner_parser_new(struct store *stores);
-#endif
 
 #endif /* INCLUDED_STORE_H */
