@@ -8,37 +8,29 @@
 
 #include "unit-test.h"
 #include "unit-test-data.h"
-#include "test-utils.h"
 #include "monster/mon-util.h"
 
-int setup_tests(void **state) {
-	read_edit_files();
-	*state = 0;
-	return 0;
-}
-
-int teardown_tests(void *state) {
-	mem_free(state);
-	return 0;
-}
+NOSETUP
+NOTEARDOWN
 
 /* Regression test for #1409 */
 int test_match_monster_bases(void *state) {
 	struct monster_base *base;
+	rb_info = &test_rb_angel;
 
 	/* Scruffy little dog */
-	base = (&r_info[3])->base;
+	base = test_r_littledog.base;
 	require(match_monster_bases(base, "canine", NULL));
 	require(match_monster_bases(base, "zephyr hound", "canine", NULL));
 	require(!match_monster_bases(base, "angel", NULL));
 	require(!match_monster_bases(base, "lich", "vampire", "wraith", NULL));
 
-	/* Morgoth */
-	base = (&r_info[547])->base;
+	/* Human */
+	base = test_r_human.base;
 	require(!match_monster_bases(base, "canine", NULL));
 	require(!match_monster_bases(base, "lich", "vampire", "wraith", NULL));
-	require(match_monster_bases(base, "person", "Morgoth", NULL));
-	require(match_monster_bases(base, "Morgoth", NULL));
+	require(match_monster_bases(base, "person", "townsfolk", NULL));
+	require(match_monster_bases(base, "townsfolk", NULL));
 
 	ok;
 }
