@@ -3,7 +3,7 @@
 #include "unit-test.h"
 #include "unit-test-data.h"
 #include "init.h"
-#include "types.h"
+
 
 int setup_tests(void **state) {
 	*state = init_parse_f();
@@ -35,7 +35,7 @@ int test_g0(void *state) {
 	eq(r, PARSE_ERROR_NONE);
 	f = parser_priv(state);
 	require(f);
-	eq(f->d_char, ':');
+	eq(f->d_char, L':');
 	eq(f->d_attr, TERM_RED);
 	ok;
 }
@@ -81,7 +81,8 @@ int test_x0(void *state) {
 	f = parser_priv(state);
 	require(f);
 	eq(f->locked, 3);
-	eq(f->jammed, 5);
+	/* eq(f->jammed, 5); 
+	 * as of 85b1ff6, the jammed field is unused */
 	eq(f->shopnum, 9);
 	eq(f->dig, 2);
 	ok;
