@@ -3,7 +3,7 @@
 #include "unit-test.h"
 
 #include "init.h"
-#include "player/types.h"
+#include "player/player.h"
 
 int setup_tests(void **state) {
 	*state = init_parse_h();
@@ -28,7 +28,6 @@ int test_n0(void *state) {
 	eq(c->idx, 1);
 	ptreq(e->next, NULL);
 	ptreq(e->roll, 5);
-	ptreq(e->bonus, 2);
 	eq(e->isucc, 3);
 	ok;
 }
