@@ -5,7 +5,7 @@
 #include "init.h"
 #include "monster/constants.h"
 #include "monster/monster.h"
-#include "types.h"
+
 #include "externs.h"
 
 int setup_tests(void **state) {
@@ -34,7 +34,7 @@ int test_t0(void *state) {
 	enum parser_error r;
 	struct monster_race *mr;
 
-	rb_info = &test_rb_info;
+	rb_info = &test_rb_townsfolk;
 	r = parser_parse(state, "T:townsfolk");
 	eq(r, PARSE_ERROR_NONE);
 	mr = parser_priv(state);
