@@ -15,7 +15,7 @@
 
 #include "angband.h"
 #include "object/tvalsval.h"
-#include "player/types.h"
+#include "player/player.h"
 
 static struct player_sex TEST_DATA test_sex = {
 	.title = "Test Sex",
@@ -29,6 +29,26 @@ static struct object_base TEST_DATA sword_base = {
 	.break_perc = 50,
 };
 
+static struct object_base TEST_DATA light_base = {
+	.name = "Test Light~",
+	.tval = TV_LIGHT,
+	.next = NULL,
+	.break_perc = 50,
+};
+
+static struct object_base TEST_DATA flask_base = {
+	.name = "Test Flask~",
+	.tval = TV_FLASK,
+	.next = NULL,
+	.break_perc = 100,
+};
+
+static struct object_base TEST_DATA rod_base = {
+	.name = "Test Rod~",
+	.tval = TV_ROD,
+	.next = NULL,
+};
+
 static struct artifact TEST_DATA test_artifact_sword = {
 	.name = "Test Artifact",
 	.text = "A test artifact.",
@@ -100,7 +120,7 @@ static struct object_kind TEST_DATA test_longsword = {
 	.cost = 20,
 
 	.d_attr = 0,
-	.d_char = '|',
+	.d_char = L'|',
 
 	.alloc_prob = 20,
 	.alloc_min = 1,
@@ -115,6 +135,7 @@ static struct object_kind TEST_DATA test_longsword = {
 static struct object_kind TEST_DATA test_torch = {
 	.name = "Test Torch",
 	.text = "A test torch [1].",
+	.base = &light_base,
 	.kidx = 1,
 	.tval = TV_LIGHT,
 	.sval = SV_LIGHT_TORCH,
@@ -165,7 +186,7 @@ static struct object_kind TEST_DATA test_torch = {
 	.cost = 1,
 
 	.d_attr = 0,
-	.d_char = '~',
+	.d_char = L'~',
 
 	.alloc_prob = 10,
 	.alloc_min = 1,
@@ -177,6 +198,204 @@ static struct object_kind TEST_DATA test_torch = {
 	.flavor = NULL,
 };
 
+static struct object_kind TEST_DATA test_lantern = {
+	.name = "Test Lantern",
+	.text = "A test lantern.",
+	.base = &light_base,
+	.kidx = 1,
+	.tval = TV_LIGHT,
+	.sval = SV_LIGHT_LANTERN,
+	.pval = {
+			{
+				.base = 5000,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+			{
+				.base = 0,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+			{
+				.base = 0,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+	},
+
+	.to_h = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+	.to_d = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+	.to_a = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+
+	.dd = 1,
+	.ds = 1,
+	.weight = 10,
+
+	.cost = 1,
+
+	.d_attr = 0,
+	.d_char = L'~',
+
+	.alloc_prob = 10,
+	.alloc_min = 1,
+	.alloc_max = 10,
+	.level = 0,
+
+	.effect = 0,
+	.gen_mult_prob = 0,
+	.flavor = NULL,
+};
+
+static struct object_kind TEST_DATA test_flask = {
+	.name = "Test Flask",
+	.text = "A test flask.",
+	.base = &flask_base,
+	.kidx = 1,
+	.tval = TV_FLASK,
+	.sval = 0,
+	.pval = {
+			{
+				.base = 7500,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+			{
+				.base = 0,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+			{
+				.base = 0,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+	},
+
+	.to_h = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+	.to_d = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+	.to_a = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+
+	.dd = 1,
+	.ds = 4,
+	.weight = 20,
+
+	.cost = 3,
+
+	.d_attr = 11,
+	.d_char = L'!',
+
+	.alloc_prob = 50,
+	.alloc_min = 1,
+	.alloc_max = 100,
+	.level = 1,
+
+	.effect = 0,
+	.gen_mult_prob = 0,
+	.flavor = NULL,
+};
+
+static struct object_kind TEST_DATA test_rod_treasure_location = {
+	.name = "Test Rod of Treasure Location",
+	.text = "A test rod of treasure location.",
+	.base = &rod_base,
+	.kidx = 1,
+	.tval = TV_ROD,
+	.sval = 1,
+	.pval = {
+			{
+				.base = 0,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+			{
+				.base = 0,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+			{
+				.base = 0,
+				.dice = 0,
+				.sides = 0,
+				.m_bonus = 0,
+			},
+	},
+
+	.to_h = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+	.to_d = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+	.to_a = {
+			.base = 0,
+			.dice = 0,
+			.sides = 0,
+			.m_bonus = 0,
+	},
+
+	.dd = 0,
+	.ds = 0,
+	.weight = 15,
+
+	.cost = 1000,
+
+	.d_attr = 0,
+	.d_char = L'-',
+
+	.alloc_prob = 30,
+	.alloc_min = 8,
+	.alloc_max = 75,
+	.level = 5,
+
+	.effect = 0,
+	.gen_mult_prob = 0,
+	.flavor = NULL,
+};
+
 static struct object_kind TEST_DATA test_gold = {
 	.name = "Test Gold",
 	.text = "Test gold [2].",
@@ -230,7 +449,7 @@ static struct object_kind TEST_DATA test_gold = {
 	.cost = 0,
 
 	.d_attr = 0,
-	.d_char = '$',
+	.d_char = L'$',
 
 	.alloc_prob = 0,
 	.alloc_min = 0,
@@ -250,7 +469,6 @@ static struct player_race TEST_DATA test_race = {
 		[A_CON] = +3,
 		[A_INT] = -1,
 		[A_WIS] = -2,
-		[A_CHR] = +0,
 	},
 	.r_skills = {
 		[SKILL_DISARM] = 0,
@@ -323,7 +541,6 @@ static struct player_class TEST_DATA test_class = {
 		[A_CON] = -1,
 		[A_INT] = -2,
 		[A_WIS] = +3,
-		[A_CHR] = +0,
 	},
 
 	.c_skills = {
@@ -370,7 +587,7 @@ static struct player_class TEST_DATA test_class = {
 	.start_items = &start_longsword,
 };
 
-static struct monster_base TEST_DATA test_rb_info = {
+static struct monster_base TEST_DATA test_rb_townsfolk = {
 	.next = NULL,
 	.name = "townsfolk",
 	.text = "Townsfolk",
@@ -381,6 +598,26 @@ static struct monster_base TEST_DATA test_rb_info = {
 	
 };
 
+static struct monster_base TEST_DATA test_rb_canine = {
+	.next = &test_rb_townsfolk,
+	.name = "canine",
+	.text = "Canine",
+	.flags = "\0\0\0\0\0\0\0\0\0\0\0\0",
+	.spell_flags = "\0\0\0\0\0\0\0\0\0\0\0\0",
+	.d_char = 100,
+	.pain = NULL,
+};
+
+static struct monster_base TEST_DATA test_rb_angel = {
+	.next = &test_rb_canine,
+	.name = "angel",
+	.text = "Angel",
+	.flags = "\0\0\0\0\0\0\0\0\0\0\0\0",
+	.spell_flags = "\0\0\0\0\0\0\0\0\0\0\0\0",
+	.d_char = 65,
+	.pain = NULL,
+};
+
 #define _NOBLOW { .method = RBM_NONE, .effect = RBE_NONE, .d_dice = 0, .d_side = 0 }
 
 static struct monster_race TEST_DATA test_r_human = {
@@ -389,7 +626,54 @@ static struct monster_race TEST_DATA test_r_human = {
 	.name = "Human",
 	.text = "A random test human",
 
-	.base = &test_rb_info,
+	.base = &test_rb_townsfolk,
+
+	.avg_hp = 10,
+	.ac = 12,
+	.sleep = 0,
+	.aaf = 20,
+	.speed = 110,
+	.mexp = 50,
+	.power = 1,
+	.scaled_power = 1,
+	.highest_threat = 5,
+	.freq_innate = 0,
+	.freq_spell = 0,
+
+	.blow = {
+		{
+			.method = RBM_HIT,
+			.effect = RBE_HURT,
+			.d_dice = 3,
+			.d_side = 1,
+		},
+		_NOBLOW,
+		_NOBLOW,
+		_NOBLOW,
+	},
+
+	.level = 1,
+	.rarity = 1,
+
+	.d_attr = 0,
+	.d_char = 't',
+
+	.x_attr = 0,
+	.x_char = 't',
+
+	.max_num = 100,
+	.cur_num = 0,
+
+	.drops = NULL,
+};
+
+static struct monster_race TEST_DATA test_r_littledog = {
+	.next = NULL,
+	.ridx = 0,
+	.name = "Scruffy little dog",
+	.text = "A random test dog",
+
+	.base = &test_rb_canine,
 
 	.avg_hp = 10,
 	.ac = 12,
@@ -459,7 +743,6 @@ static struct player TEST_DATA test_player = {
 	.age = 12,
 	.ht = 40,
 	.wt = 80,
-	.sc = 100,
 	.au = 500,
 	.max_depth = 10,
 	.depth = 6,
@@ -477,7 +760,6 @@ static struct player TEST_DATA test_player = {
 		[A_CON] = 14,
 		[A_WIS] = 10,
 		[A_INT] = 8,
-		[A_CHR] = 12,
 	},
 	.stat_cur = {
 		[A_STR] = 14,
@@ -485,7 +767,6 @@ static struct player TEST_DATA test_player = {
 		[A_CON] = 14,
 		[A_WIS] = 10,
 		[A_INT] = 8,
-		[A_CHR] = 8,
 	},
 	.word_recall = 0,
 	.energy = 100,
