@@ -17,7 +17,6 @@
  */
 
 #include "angband.h"
-#include "button.h"
 #include "cmds.h"
 #include "files.h"
 #include "game-cmd.h"
@@ -25,6 +24,7 @@
 #include "object/tvalsval.h"
 #include "ui-birth.h"
 #include "ui-menu.h"
+#include "ui-options.h"
 
 
 /*
@@ -98,14 +98,7 @@ static enum birth_stage get_quickstart_command(void)
 	/* Prompt for it */
 	prt("New character based on previous one:", 0, 0);
 	prt(prompt, Term->hgt - 1, Term->wid / 2 - strlen(prompt) / 2);
-	
-	/* Buttons */
-	button_kill_all();
-	button_add("[Y]", 'y');
-	button_add("[N]", 'n');
-	button_add("[C]", 'c');
-	redraw_stuff(p_ptr);
-	
+
 	do
 	{
 		/* Get a key */
@@ -131,10 +124,6 @@ static enum birth_stage get_quickstart_command(void)
 			next = BIRTH_COMPLETE;
 		}
 	} while (next == BIRTH_QUICKSTART);
-	
-	/* Buttons */
-	button_kill_all();
-	redraw_stuff(p_ptr);
 
 	/* Clear prompt */
 	clear_from(23);
@@ -204,8 +193,14 @@ static void birthmenu_display(menu_type *menu, int oid, bool cursor,
    only defining the display and handler parts). */
 static const menu_iter birth_iter = { NULL, NULL, birthmenu_display, NULL, NULL };
 
-static void skill_help(s16b skills[], int mhp, int exp, int infra)
+static void skill_help(const s16b r_skills[], const s16b c_skills[], int mhp, int exp, int infra)
 {
+	s16b skills[SKILL_MAX];
+	unsigned i;
+
+	for (i = 0; i < SKILL_MAX ; ++i)
+		skills[i] = (r_skills ? r_skills[i] : 0 ) + (c_skills ? c_skills[i] : 0);
+
 	text_out_e("Hit/Shoot/Throw: %+d/%+d/%+d\n", skills[SKILL_TO_HIT_MELEE], skills[SKILL_TO_HIT_BOW], skills[SKILL_TO_HIT_THROW]);
 	text_out_e("Hit die: %2d   XP mod: %d%%\n", mhp, exp);
 	text_out_e("Disarm: %+3d   Devices: %+3d\n", skills[SKILL_DISARM], skills[SKILL_DEVICE]);
@@ -278,17 +273,22 @@ static void race_help(int i, void *db, const region *l)
 
 	for (j = 0; j < len; j++)
 	{  
-		const char *name1 = stat_names_reduced[j];
-		const char *name2 = stat_names_reduced[j + len];
+		const char *name = stat_names_reduced[j];
+		int adj = r->r_adj[j];
 
-		int adj1 = r->r_adj[j];
-		int adj2 = r->r_adj[j + len];
+		text_out_e("%s%+3d", name, adj);
 
-		text_out_e("%s%+3d  %s%+3d\n", name1, adj1, name2, adj2);
+		if (j*2 + 1 < A_MAX) {
+			name = stat_names_reduced[j + len];
+			adj = r->r_adj[j + len];
+			text_out_e("  %s%+3d", name, adj);
+		}
+
+		text_out("\n");
 	}
 	
 	text_out_e("\n");
-	skill_help(r->r_skills, r->r_mhp, r->r_exp, r->infra);
+	skill_help(r->r_skills, NULL, r->r_mhp, r->r_exp, r->infra);
 	text_out_e("\n");
 
 	for (k = 0; k < OF_MAX; k++)
@@ -322,6 +322,7 @@ static void class_help(int i, void *db, const region *l)
 	int j;
 	size_t k;
 	struct player_class *c = player_id2class(i);
+	const struct player_race *r = p_ptr->race;
 	int len = (A_MAX + 1) / 2;
 
 	int n_flags = 0;
@@ -338,18 +339,23 @@ static void class_help(int i, void *db, const region *l)
 
 	for (j = 0; j < len; j++)
 	{  
-		const char *name1 = stat_names_reduced[j];
-		const char *name2 = stat_names_reduced[j + len];
+		const char *name = stat_names_reduced[j];
+		int adj = c->c_adj[j] + r->r_adj[j];
 
-		int adj1 = c->c_adj[j] + p_ptr->race->r_adj[j];
-		int adj2 = c->c_adj[j + len] + p_ptr->race->r_adj[j + len];
+		text_out_e("%s%+3d", name, adj);
 
-		text_out_e("%s%+3d  %s%+3d\n", name1, adj1, name2, adj2);
+		if (j*2 + 1 < A_MAX) {
+			name = stat_names_reduced[j + len];
+			adj = c->c_adj[j + len] + r->r_adj[j + len];
+			text_out_e("  %s%+3d", name, adj);
+		}
+
+		text_out("\n");
 	}
 
 	text_out_e("\n");
 	
-	skill_help(c->c_skills, c->c_mhp, c->c_exp, -1);
+	skill_help(r->r_skills, c->c_skills, r->r_mhp + c->c_mhp, r->r_exp + c->c_exp, -1);
 
 	if (c->spell_book == TV_MAGIC_BOOK) {
 		text_out_e("\nLearns arcane magic");
@@ -433,7 +439,7 @@ static void setup_menus(void)
 	mdata = sex_menu.menu_data;
 	for (i = 0; i < MAX_SEXES; i++)
 		mdata->items[i] = sex_info[i].title;
-	mdata->hint = "Your 'sex' does not have any significant gameplay effects.";
+	mdata->hint = "Sex does not have any significant gameplay effects.";
 
 	n = 0;
 	for (r = races; r; r = r->next) n++;
@@ -444,7 +450,7 @@ static void setup_menus(void)
 
 	for (i = 0, r = races; r; r = r->next, i++)
 		mdata->items[r->ridx] = r->name;
-	mdata->hint = "Your 'race' determines various intrinsic factors and bonuses.";
+	mdata->hint = "Race affects stats and skills, and may confer resistances and abilities.";
 
 	n = 0;
 	for (c = classes; c; c = c->next) n++;
@@ -455,14 +461,14 @@ static void setup_menus(void)
 
 	for (i = 0, c = classes; c; c = c->next, i++)
 		mdata->items[c->cidx] = c->name;
-	mdata->hint = "Your 'class' determines various intrinsic abilities and bonuses";
+	mdata->hint = "Class affects stats, skills, and other character traits.";
 		
 	/* Roller menu straightforward again */
 	init_birth_menu(&roller_menu, MAX_BIRTH_ROLLERS, 0, &roller_region, FALSE, NULL);
 	mdata = roller_menu.menu_data;
 	for (i = 0; i < MAX_BIRTH_ROLLERS; i++)
 		mdata->items[i] = roller_choices[i];
-	mdata->hint = "Your choice of character generation.  Point-based is recommended.";
+	mdata->hint = "Choose how to generate your intrinsic stats. Point-based is recommended.";
 }
 
 /* Cleans up our stored menu info when we've finished with it. */
@@ -502,12 +508,12 @@ static void clear_question(void)
 
 
 #define BIRTH_MENU_HELPTEXT \
-	"{lightblue}Please select your character from the menu below:{/}\n\n" \
-	"Use the {lightgreen}movement keys{/} to scroll the menu, " \
-	"{lightgreen}Enter{/} to select the current menu item, '{lightgreen}*{/}' " \
-	"for a random menu item, '{lightgreen}ESC{/}' to step back through the " \
-	"birth process, '{lightgreen}={/}' for the birth options, '{lightgreen}?{/} " \
-	"for help, or '{lightgreen}Ctrl-X{/}' to quit."
+	"{light blue}Please select your character traits from the menus below:{/}\n\n" \
+	"Use the {light green}movement keys{/} to scroll the menu, " \
+	"{light green}Enter{/} to select the current menu item, '{light green}*{/}' " \
+	"for a random menu item, '{light green}ESC{/}' to step back through the " \
+	"birth process, '{light green}={/}' for the birth options, '{light green}?{/}' " \
+	"for help, or '{light green}Ctrl-X{/}' to quit."
 
 /* Show the birth instructions on an otherwise blank screen */	
 static void print_menu_instructions(void)
@@ -627,8 +633,6 @@ static enum birth_stage menu_question(enum birth_stage current, menu_type *curre
 /* ------------------------------------------------------------------------
  * The rolling bit of the roller.
  * ------------------------------------------------------------------------ */
-#define ROLLERCOL 42
-
 static enum birth_stage roller_command(bool first_call)
 {
 	char prompt[80] = "";
@@ -647,18 +651,10 @@ static enum birth_stage roller_command(bool first_call)
 	if (first_call)
 		prev_roll = FALSE;
 
-	/* Add buttons */
-	button_add("[ESC]", ESCAPE);
-	button_add("[Enter]", '\r');
-	button_add("[r]", 'r');
-	if (prev_roll) button_add("[p]", 'p');
-	clear_from(Term->hgt - 2);
-	redraw_stuff(p_ptr);
-
 	/* Prepare a prompt (must squeeze everything in) */
 	strnfcat(prompt, sizeof (prompt), &promptlen, "['r' to reroll");
 	if (prev_roll) 
-		strnfcat(prompt, sizeof(prompt), &promptlen, ", 'p' for prev");
+		strnfcat(prompt, sizeof(prompt), &promptlen, ", 'p' for previous roll");
 	strnfcat(prompt, sizeof (prompt), &promptlen, " or 'Enter' to accept]");
 
 	/* Prompt for it */
@@ -669,14 +665,11 @@ static enum birth_stage roller_command(bool first_call)
 
 	if (ch.code == ESCAPE) 
 	{
-		button_kill('r');
-		button_kill('p');
-
 		next = BIRTH_BACK;
 	}
 
 	/* 'Enter' accepts the roll */
-	if ((ch.code == '\r') || (ch.code == '\n')) 
+	if (ch.code == KC_ENTER)
 	{
 		next = BIRTH_NAME_CHOICE;
 	}
@@ -713,13 +706,6 @@ static enum birth_stage roller_command(bool first_call)
 		bell("Illegal roller command!");
 	}
 
-	/* Kill buttons */
-	button_kill(ESCAPE);
-	button_kill('\r');
-	button_kill('r');
-	button_kill('p');
-	redraw_stuff(p_ptr);
-
 	return next;
 }
 
@@ -831,7 +817,7 @@ static enum birth_stage point_based_command(void)
 	}
 	
 	/* Done */
-	else if ((ch.code == '\r') || (ch.code == '\n')) 
+	else if (ch.code == KC_ENTER)
 	{
 		next = BIRTH_NAME_CHOICE;
 	}
@@ -899,14 +885,7 @@ static enum birth_stage get_confirm_command(void)
 
 	/* Prompt for it */
 	prt(prompt, Term->hgt - 1, Term->wid / 2 - strlen(prompt) / 2);
-	
-	/* Buttons */
-	button_kill_all();
-	button_add("[Continue]", 'q');
-	button_add("[ESC]", ESCAPE);
-	button_add("[S]", 'S');
-	redraw_stuff(p_ptr);
-	
+
 	/* Get a key */
 	ke = inkey();
 	
@@ -929,10 +908,6 @@ static enum birth_stage get_confirm_command(void)
 		cmd_insert(CMD_ACCEPT_CHARACTER);
 		next = BIRTH_COMPLETE;
 	}
-	
-	/* Buttons */
-	button_kill_all();
-	redraw_stuff(p_ptr);
 
 	/* Clear prompt */
 	clear_from(23);
