@@ -23,11 +23,14 @@
 #include "history.h"
 #include "monster/mon-lore.h"
 #include "monster/monster.h"
+#include "monster/mon-util.h"
 #include "object/tvalsval.h"
 #include "squelch.h"
 #include "store.h"
 #include "ui.h"
 #include "ui-menu.h"
+#include "ui-options.h"
+#include "grafmode.h"
 
 /* Flag value for missing array entry */
 #define MISSING -17
@@ -62,9 +65,7 @@ static const grouper object_text_order[] =
 	{TV_DRAG_ARMOR,		"Dragon Scale Mail" },
 	{TV_HARD_ARMOR,		"Hard Armor"	},
 	{TV_SOFT_ARMOR,		"Soft Armor"	},
-	{TV_SPIKE,			"Spike"			},
 	{TV_DIGGING,		"Digger"		},
-	{TV_JUNK,			"Junk"			},
 	{0,					NULL			}
 };
 
@@ -94,7 +95,7 @@ typedef struct
 
 	/* Required only for objects with modifiable display attributes */
 	/* Unknown 'flavors' return flavor attributes */
-	char *(*xchar)(int oid);     /* Get character attr for OID (by address) */
+	wchar_t *(*xchar)(int oid);     /* Get character attr for OID (by address) */
 	byte *(*xattr)(int oid);     /* Get color attr for OID (by address) */
 
 	const char *(*xtra_prompt)(int oid);  /* Returns optional extra prompt */
@@ -134,63 +135,63 @@ static int *obj_group_order = NULL;
  */
 static struct
 {
-	const char *chars;
+	const wchar_t *chars;
 	const char *name;
 } monster_group[] =
 {
-	{ (const char *)-1,   "Uniques" },
-	{ "A",        "Angels" },
-	{ "a",        "Ants" },
-	{ "b",        "Bats" },
-	{ "B",        "Birds" },
-	{ "C",        "Canines" },
-	{ "c",        "Centipedes" },
-	{ "uU",       "Demons" },
-	{ "dD",       "Dragons" },
-	{ "vE",       "Elementals/Vortices" },
-	{ "e",        "Eyes/Beholders" },
-	{ "f",        "Felines" },
-	{ "G",        "Ghosts" },
-	{ "OP",       "Giants/Ogres" },
-	{ "g",        "Golems" },
-	{ "H",        "Harpies/Hybrids" },
-	{ "h",        "Hominids (Elves, Dwarves)" },
-	{ "M",        "Hydras" },
-	{ "i",        "Icky Things" },
-	{ "lFI",      "Insects" },
-	{ "j",        "Jellies" },
-	{ "K",        "Killer Beetles" },
-	{ "k",        "Kobolds" },
-	{ "L",        "Lichs" },
-	{ "tp",		  "Men" },
-	{ ".$!?=~_",  "Mimics" },
-	{ "m",        "Molds" },
-	{ ",",        "Mushroom Patches" },
-	{ "n",        "Nagas" },
-	{ "o",        "Orcs" },
-	{ "q",        "Quadrupeds" },
-	{ "Q",        "Quylthulgs" },
-	{ "R",        "Reptiles/Amphibians" },
-	{ "r",        "Rodents" },
-	{ "S",        "Scorpions/Spiders" },
-	{ "s",        "Skeletons/Drujs" },
-	{ "J",        "Snakes" },
-	{ "T",        "Trolls" },
-	{ "V",        "Vampires" },
-	{ "W",        "Wights/Wraiths" },
-	{ "w",        "Worms/Worm Masses" },
-	{ "X",        "Xorns/Xarens" },
-	{ "y",        "Yeeks" },
-	{ "Y",        "Yeti" },
-	{ "Z",        "Zephyr Hounds" },
-	{ "z",        "Zombies" },
+	{ (const wchar_t *)-1,   "Uniques" },
+	{ L"A",        "Ainur" },
+	{ L"a",        "Ants" },
+	{ L"b",        "Bats" },
+	{ L"B",        "Birds" },
+	{ L"C",        "Canines" },
+	{ L"c",        "Centipedes" },
+	{ L"uU",       "Demons" },
+	{ L"dD",       "Dragons" },
+	{ L"vE",       "Elementals/Vortices" },
+	{ L"e",        "Eyes/Beholders" },
+	{ L"f",        "Felines" },
+	{ L"G",        "Ghosts" },
+	{ L"OP",       "Giants/Ogres" },
+	{ L"g",        "Golems" },
+	{ L"H",        "Harpies/Hybrids" },
+	{ L"h",        "Hominids (Elves, Dwarves)" },
+	{ L"M",        "Hydras" },
+	{ L"i",        "Icky Things" },
+	{ L"lFI",      "Insects" },
+	{ L"j",        "Jellies" },
+	{ L"K",        "Killer Beetles" },
+	{ L"k",        "Kobolds" },
+	{ L"L",        "Lichs" },
+	{ L"tp",       "Men" },
+	{ L".$!?=~_",  "Mimics" },
+	{ L"m",        "Molds" },
+	{ L",",        "Mushroom Patches" },
+	{ L"n",        "Nagas" },
+	{ L"o",        "Orcs" },
+	{ L"q",        "Quadrupeds" },
+	{ L"Q",        "Quylthulgs" },
+	{ L"R",        "Reptiles/Amphibians" },
+	{ L"r",        "Rodents" },
+	{ L"S",        "Scorpions/Spiders" },
+	{ L"s",        "Skeletons/Drujs" },
+	{ L"J",        "Snakes" },
+	{ L"T",        "Trolls" },
+	{ L"V",        "Vampires" },
+	{ L"W",        "Wights/Wraiths" },
+	{ L"w",        "Worms/Worm Masses" },
+	{ L"X",        "Xorns/Xarens" },
+	{ L"y",        "Yeeks" },
+	{ L"Y",        "Yeti" },
+	{ L"Z",        "Zephyr Hounds" },
+	{ L"z",        "Zombies" },
 	{ NULL,       NULL }
 };
 
 /*
  * Description of each feature group.
  */
-const char *feature_group_text[] =
+static const char *feature_group_text[] =
 {
 	"Floors",
 	"Traps",
@@ -207,17 +208,24 @@ const char *feature_group_text[] =
 
 
 /* Useful method declarations */
-static void display_visual_list(int col, int row, int height, int width,
+static void display_tiles(int col, int row, int height, int width,
 				byte attr_top, byte char_left);
 
-static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
-				int height, int width,
-				byte *attr_top_ptr, byte *char_left_ptr,
-				byte *cur_attr_ptr, byte *cur_char_ptr,
-				int col, int row, int *delay);
+static bool tile_picker_command(ui_event ke, bool *tile_picker_ptr,
+				int height, int width, byte *attr_top_ptr,
+				byte *char_left_ptr, byte *cur_attr_ptr,
+				byte *cur_char_ptr, int col, int row,
+				int *delay);
 
-static void place_visual_list_cursor(int col, int row, byte a,
-				byte c, byte attr_top, byte char_left);
+static void place_tile_cursor(int col, int row, byte a, byte c,
+				     byte attr_top, byte char_left);
+
+static void display_glyphs(int col, int row, int height, int width, byte a, 
+			   wchar_t c);
+
+static bool glyph_command(ui_event ke, bool *glyph_picker_ptr,
+			  int height, int width, byte *cur_attr_ptr,
+			  wchar_t *cur_char_ptr, int col, int row);
 
 /*
  * Clipboard variables for copy&paste in visual mode
@@ -234,13 +242,13 @@ static int feat_order(int feat)
 
 	switch (f_ptr->d_char)
 	{
-		case '.': 				return 0;
-		case '^': 				return 1;
-		case '\'': case '+': 	return 2;
-		case '<': case '>':		return 3;
-		case '#':				return 4;
-		case '*': case '%' :	return 5;
-		case ';': case ':' :	return 6;
+		case L'.': 				return 0;
+		case L'^': 				return 1;
+		case L'\'': case L'+': 	return 2;
+		case L'<': case L'>':		return 3;
+		case L'#':				return 4;
+		case L'*': case L'%' :	return 5;
+		case L';': case L':' :	return 6;
 
 		default:
 		{
@@ -251,7 +259,7 @@ static int feat_order(int feat)
 
 
 /* Emit a 'graphical' symbol and a padding character if appropriate */
-extern int big_pad(int col, int row, byte a, byte c)
+extern int big_pad(int col, int row, byte a, wchar_t c)
 {
 	Term_putch(col, row, a, c);
 
@@ -305,7 +313,7 @@ static void display_group_member(menu_type *menu, int oid,
 	/* Do visual mode */
 	if (o_funcs->is_visual && o_funcs->xattr)
 	{
-		byte c = *o_funcs->xchar(oid);
+		wchar_t c = *o_funcs->xchar(oid);
 		byte a = *o_funcs->xattr(oid);
 
 		c_put_str(attr, format("%d/%d", a, c), row, 60);
@@ -350,7 +358,9 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 	region object_region = { MISSING, 6, 0, -2 };
 
 	/* display state variables */
-	bool visual_list = FALSE;
+	bool tiles = (current_graphics_mode != NULL);
+	bool tile_picker = FALSE;
+	bool glyph_picker = FALSE;
 	byte attr_top = 0;
 	byte char_left = 0;
 
@@ -388,6 +398,9 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 	/* Disable the roguelike commands for the duration */
 	OPT(rogue_like_commands) = FALSE;
 
+	/* Determine if using tiles or not */
+	if (tiles) tiles = (current_graphics_mode->grafID != 0);
+
 	if (g_funcs.gcomp)
 		sort(obj_list, o_count, sizeof(*obj_list), g_funcs.gcomp);
 
@@ -435,10 +448,12 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 	menu_init(&group_menu, MN_SKIN_SCROLL, menu_find_iter(MN_ITER_STRINGS));
 	menu_setpriv(&group_menu, grp_cnt, g_names);
 	menu_layout(&group_menu, &group_region);
+	group_menu.flags |= MN_DBL_TAP;
 
 	menu_init(&object_menu, MN_SKIN_SCROLL, &object_iter);
 	menu_setpriv(&object_menu, 0, &o_funcs);
 	menu_layout(&object_menu, &object_region);
+	object_menu.flags |= MN_DBL_TAP;
 
 	o_funcs.is_visual = FALSE;
 
@@ -489,7 +504,7 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 		}
 
 		/* HACK ... */
-		if (!visual_list)
+		if (!(tile_picker || glyph_picker)) 
 		{
 			/* ... The object menu may be browsing the entire group... */
 			o_funcs.is_visual = FALSE;
@@ -513,7 +528,8 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 			const char *xtra = o_funcs.xtra_prompt ? o_funcs.xtra_prompt(oid) : "";
 			const char *pvs = "";
 
-			if (visual_list) pvs = ", ENTER to accept";
+			if (tile_picker) pvs = ", ENTER to accept";
+			else if (glyph_picker) pvs = ", 'i' to insert, ENTER to accept";
 			else if (o_funcs.xattr) pvs = ", 'v' for visuals";
 
 			prt(format("<dir>%s%s%s, ESC", pvs, pedit, xtra), hgt - 1, 0);
@@ -527,7 +543,7 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 			panel = 1 - panel;
 		}
 
-		if (g_funcs.summary && !visual_list)
+		if (g_funcs.summary && !tile_picker && !glyph_picker)
 		{
 			g_funcs.summary(g_cur, obj_list, g_o_count, g_offset[g_cur],
 			                object_menu.active.row + object_menu.active.page_rows,
@@ -539,13 +555,24 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 
 		handle_stuff(p_ptr);
 
-		if (visual_list)
+		if (tile_picker) 
 		{
 		        bigcurs = TRUE;
-			display_visual_list(g_name_len + 3, 7, browser_rows-1,
-			                             wid - (g_name_len + 3), attr_top, char_left);
-			place_visual_list_cursor(g_name_len + 3, 7, *o_funcs.xattr(oid), 
-										*o_funcs.xchar(oid), attr_top, char_left);
+			display_tiles(g_name_len + 3, 7, browser_rows - 1,
+				      wid - (g_name_len + 3), attr_top, 
+				      char_left);
+			place_tile_cursor(g_name_len + 3, 7, 
+					  *o_funcs.xattr(oid),
+					  (byte) *o_funcs.xchar(oid), 
+					  attr_top, char_left);
+		}
+
+		if (glyph_picker) 
+		{
+		        display_glyphs(g_name_len + 3, 7, browser_rows - 1,
+				       wid - (g_name_len + 3), 
+				       *o_funcs.xattr(oid),
+				       *o_funcs.xchar(oid));
 		}
 
 		if (delay)
@@ -560,7 +587,7 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 		}
 
 		ke = inkey_ex();
-		if (!visual_list)
+		if (!tile_picker && !glyph_picker)
 		{
 			ui_event ke0 = EVENT_EMPTY;
 
@@ -574,13 +601,25 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 		}
 
 		/* XXX Do visual mode command if needed */
-		if (o_funcs.xattr && o_funcs.xchar)
+		if (o_funcs.xattr && o_funcs.xchar) 
 		{
-			if (visual_mode_command(ke, &visual_list, browser_rows - 1,
-					wid - (g_name_len + 3), &attr_top, &char_left,
-					o_funcs.xattr(oid), (byte *)o_funcs.xchar(oid),
-					g_name_len + 3, 7, &delay))
-				continue;
+		        if (tiles)
+			{
+			        if (tile_picker_command(ke, &tile_picker, 
+				    browser_rows - 1, wid - (g_name_len + 3),
+				    &attr_top, &char_left, o_funcs.xattr(oid),
+				    (byte *) o_funcs.xchar(oid), 
+				    g_name_len + 3, 7, &delay))
+				  continue;
+			}
+			else 
+			{
+			        if (glyph_command(ke, &glyph_picker, 
+				    browser_rows - 1, wid - (g_name_len + 3), 
+				    o_funcs.xattr(oid), o_funcs.xchar(oid), 
+				    g_name_len + 3, 7))
+				  continue;
+			}
 		}
 
 		switch (ke.type)
@@ -598,7 +637,7 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 			case EVT_MOUSE:
 			{
 				/* Change active panels */
-				if (region_inside(&inactive_menu->boundary, &ke))
+				if (region_inside(&inactive_menu->active, &ke))
 				{
 					swap(active_menu, inactive_menu);
 					swap(active_cursor, inactive_cursor);
@@ -664,9 +703,9 @@ static void display_knowledge(const char *title, int *obj_list, int o_count,
 }
 
 /*
- * Display visuals.
+ * Display tiles.
  */
-static void display_visual_list(int col, int row, int height, int width, byte attr_top, byte char_left)
+static void display_tiles(int col, int row, int height, int width, byte attr_top, byte char_left)
 {
 	int i, j;
 
@@ -703,9 +742,9 @@ static void display_visual_list(int col, int row, int height, int width, byte at
 
 
 /*
- * Place the cursor at the collect position for visual mode
+ * Place the cursor at the correct position for tile picking
  */
-static void place_visual_list_cursor(int col, int row, byte a, byte c, byte attr_top, byte char_left)
+static void place_tile_cursor(int col, int row, byte a, byte c, byte attr_top, byte char_left)
 {
 	int i = a - attr_top;
 	int j = c - char_left;
@@ -719,9 +758,9 @@ static void place_visual_list_cursor(int col, int row, byte a, byte c, byte attr
 
 
 /*
- * Remove the visual list and clear the screen 
+ * Remove the tile display and clear the screen 
  */
-static void remove_visual_list(int col, int row, bool *visual_list_ptr, int width, int height)
+static void remove_tiles(int col, int row, bool *picker_ptr, int width, int height)
 {
 	int i;
 
@@ -729,7 +768,7 @@ static void remove_visual_list(int col, int row, bool *visual_list_ptr, int widt
 	bigcurs = FALSE;
 
 	/* Cancel visual list */
-	*visual_list_ptr = FALSE;
+	*picker_ptr = FALSE;
 
 	/* Clear the display lines */
 	for (i = 0; i < height; i++)
@@ -738,13 +777,13 @@ static void remove_visual_list(int col, int row, bool *visual_list_ptr, int widt
 }
 
 /*
- *  Do visual mode command -- Change symbols
+ *  Do tile picker command -- Change tiles
  */
-static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
-				int height, int width,
-				byte *attr_top_ptr, byte *char_left_ptr,
-				byte *cur_attr_ptr, byte *cur_char_ptr,
-				int col, int row, int *delay)
+static bool tile_picker_command(ui_event ke, bool *tile_picker_ptr,
+				int height, int width, byte *attr_top_ptr,
+				byte *char_left_ptr, byte *cur_attr_ptr,
+				byte *cur_char_ptr, int col, int row,
+				int *delay)
 {
 	static byte attr_old = 0;
 	static char char_old = 0;
@@ -759,7 +798,7 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 
 
 	/* Get mouse movement */
-	if (ke.type == EVT_MOUSE)
+	if (*tile_picker_ptr &&  (ke.type == EVT_MOUSE))
 	{
 		int eff_width = actual_width(width);
 		int eff_height = actual_height(height);
@@ -770,7 +809,7 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 		int mx = logical_width(ke.mouse.x - col);
 
 		if ((my >= 0) && (my < eff_height) && (mx >= 0) && (mx < eff_width)
-			&& ((ke.mouse.button) || (a != *attr_top_ptr + my)
+			&& ((ke.mouse.button == 1) || (a != *attr_top_ptr + my)
 				|| (c != *char_left_ptr + mx)))
 		{
 			/* Set the visual */
@@ -792,17 +831,17 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 
 			/* Accept change */
 			if (ke.mouse.button)
-			  remove_visual_list(col, row, visual_list_ptr, width, height);
+			  remove_tiles(col, row, tile_picker_ptr, width, height);
 
 			return TRUE;
 		}
 
 		/* Cancel change */
-		else if (ke.mouse.button)
+		else if (ke.mouse.button == 2)
 		{
 			*cur_attr_ptr = attr_old;
 			*cur_char_ptr = char_old;
-			remove_visual_list(col, row, visual_list_ptr, width, height);
+			remove_tiles(col, row, tile_picker_ptr, width, height);
 
 			return TRUE;
 		}
@@ -821,12 +860,12 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 	{
 		case ESCAPE:
 		{
-			if (*visual_list_ptr)
+			if (*tile_picker_ptr)
 			{
 				/* Cancel change */
 				*cur_attr_ptr = attr_old;
 				*cur_char_ptr = char_old;
-				remove_visual_list(col, row, visual_list_ptr, width, height);
+				remove_tiles(col, row, tile_picker_ptr, width, height);
 
 				return TRUE;
 			}
@@ -834,13 +873,12 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 			break;
 		}
 
-		case '\n':
-		case '\r':
+		case KC_ENTER:
 		{
-			if (*visual_list_ptr)
+			if (*tile_picker_ptr)
 			{
 				/* Accept change */
-			  remove_visual_list(col, row, visual_list_ptr, width, height);
+			  remove_tiles(col, row, tile_picker_ptr, width, height);
 				return TRUE;
 			}
 
@@ -850,9 +888,14 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 		case 'V':
 		case 'v':
 		{
-			if (!*visual_list_ptr)
+		        /* No visual mode without graphics, for now - NRM */
+		       if (current_graphics_mode != NULL)
+			       if (current_graphics_mode->grafID == 0)
+				       break;
+
+			if (!*tile_picker_ptr)
 			{
-				*visual_list_ptr = TRUE;
+				*tile_picker_ptr = TRUE;
 				bigcurs = TRUE;
 
 				*attr_top_ptr = (byte)MAX(0, (int)*cur_attr_ptr - frame_top);
@@ -866,7 +909,7 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 				/* Cancel change */
 				*cur_attr_ptr = attr_old;
 				*cur_char_ptr = char_old;
-				remove_visual_list(col, row, visual_list_ptr, width, height);
+				remove_tiles(col, row, tile_picker_ptr, width, height);
 			}
 
 			return TRUE;
@@ -875,7 +918,7 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 		case 'C':
 		case 'c':
 		{
-			/* Set the visual */
+			/* Set the tile */
 			attr_idx = *cur_attr_ptr;
 			char_idx = *cur_char_ptr;
 
@@ -908,7 +951,7 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 			byte a = *cur_attr_ptr;
 			byte c = *cur_char_ptr;
 
-			if (!*visual_list_ptr)
+			if (!*tile_picker_ptr)
 				break;
 
 			bigcurs = TRUE;
@@ -922,7 +965,7 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 			a += ddy[d];
 			c += ddx[d];
 
-			/* Set the visual */
+			/* Set the tile */
 			*cur_attr_ptr = a;
 			*cur_char_ptr = c;
 
@@ -951,7 +994,178 @@ static bool visual_mode_command(ui_event ke, bool *visual_list_ptr,
 		}
 	}
 
-	/* Visual mode command is not used */
+	/* Tile picker command is not used */
+	return FALSE;
+}
+
+
+/*
+ * Display glyph and colours
+ */
+static void display_glyphs(int col, int row, int height, int width, byte a, 
+			   wchar_t c)
+{
+        int i;
+	int x, y;
+
+	/* Clear the display lines */
+	for (i = 0; i < height; i++)
+	        Term_erase(col, row + i, width);
+
+	/* Prompt */
+	prt("Choose colour:", row + height/2, col);
+	Term_locate(&x, &y);
+	for (i = 0; i < MAX_COLORS; i++) big_pad(x + i, y, i, c);
+	
+	/* Place the cursor */
+	Term_gotoxy(x + a, y);
+}
+
+/*
+ *  Do glyph picker command -- Change glyphs
+ */
+static bool glyph_command(ui_event ke, bool *glyph_picker_ptr,
+			  int height, int width, byte *cur_attr_ptr,
+			  wchar_t *cur_char_ptr, int col, int row)
+{
+        static byte attr_old = 0;
+	static char char_old = 0;
+	
+	/* Get mouse movement */
+	if (*glyph_picker_ptr && (ke.type == EVT_MOUSE))
+	{
+	        byte a = *cur_attr_ptr;
+
+		int mx = logical_width(ke.mouse.x - col);
+		
+		if (ke.mouse.y != row + height/2) return FALSE;
+		
+		if ((mx >= 0) && (mx < MAX_COLORS) && (ke.mouse.button == 1))
+		{
+		        /* Set the visual */
+		        *cur_attr_ptr = a = mx - 14;
+
+			/* Accept change */
+			remove_tiles(col, row, glyph_picker_ptr, width, height);
+			
+			return TRUE;
+		}
+
+		else
+		{
+		        return FALSE;
+		}
+	}
+
+	if (ke.type != EVT_KBRD)
+	        return FALSE;
+
+
+	switch (ke.key.code)
+	{
+	        case ESCAPE:
+		{
+		        if (*glyph_picker_ptr)
+			{
+			        /* Cancel change */
+			        *cur_attr_ptr = attr_old;
+				*cur_char_ptr = char_old;
+				remove_tiles(col, row, glyph_picker_ptr, width, height);
+				
+				return TRUE;
+			}
+
+			break;
+		}
+
+	    case KC_ENTER:
+	    {
+		    if (*glyph_picker_ptr)
+		    {
+			    /* Accept change */
+			    remove_tiles(col, row, glyph_picker_ptr, width, height);
+			    return TRUE;
+		    }
+		    
+		    break;
+	    }
+
+	    case 'V':
+	    case 'v':
+	    {
+		    if (!*glyph_picker_ptr)
+		    {
+			    *glyph_picker_ptr = TRUE;
+
+			    attr_old = *cur_attr_ptr;
+			    char_old = *cur_char_ptr;
+		    }
+		    else
+		    {
+			    /* Cancel change */
+			    *cur_attr_ptr = attr_old;
+			    *cur_char_ptr = char_old;
+			    remove_tiles(col, row, glyph_picker_ptr, width, height);
+		    }
+
+		    return TRUE;
+	    }
+
+	    case 'i':
+	    case 'I':
+	    {
+		    if (*glyph_picker_ptr)
+		    {
+			    char code_point[6];
+			    bool res = FALSE;
+	
+			    /* Ask the user for a code point */
+			    Term_gotoxy(col, row + height/2 + 2);
+			    res = get_string("(up to 5 hex digits):", code_point, 5);
+	
+			    /* Process input */
+			    if (res)
+			    {
+				    unsigned long int point = strtoul(code_point, (char **)NULL, 16);
+				    *cur_char_ptr = (wchar_t) point;
+				    return TRUE;
+			    }
+		    }
+		    
+		    break;
+		    
+		    
+	    }
+	    
+	    default:
+	    {
+		    int d = target_dir(ke.key);
+		    byte a = *cur_attr_ptr;
+		    
+		    if (!*glyph_picker_ptr)
+			break;
+
+		    /* Horizontal only */
+		    if (ddy[d] != 0) break;
+		    
+		    /* Horizontal movement */
+		    if (ddx[d] != 0) {
+			a += ddx[d] + BASIC_COLORS;
+			a = a % BASIC_COLORS;
+			*cur_attr_ptr = a;
+		    }
+    
+	
+		    /* We need to always eat the input even if it is clipped,
+		     * otherwise it will be interpreted as a change object
+		     * selection command with messy results.
+		     */
+		    return TRUE;
+	    }
+	}
+
+
+	/* Glyph picker command is not used */
 	return FALSE;
 }
 
@@ -978,7 +1192,7 @@ static void display_monster(int col, int row, bool cursor, int oid)
 	/* Choose colors */
 	byte attr = curs_attrs[CURS_KNOWN][(int)cursor];
 	byte a = r_ptr->x_attr;
-	byte c = r_ptr->x_char;
+	wchar_t c = r_ptr->x_char;
 
 	if ((tile_height != 1) && (a & 0x80)) {
 		a = r_ptr->d_attr;
@@ -1021,8 +1235,8 @@ static int m_cmp_race(const void *a, const void *b)
 	{
 		/* UNIQUE group is ordered by level & name only */
 		/* Others by order they appear in the group symbols */
-		return strchr(monster_group[gid].chars, r_a->d_char)
-			- strchr(monster_group[gid].chars, r_b->d_char);
+		return wcschr(monster_group[gid].chars, r_a->d_char)
+			- wcschr(monster_group[gid].chars, r_b->d_char);
 	}
 	c = r_a->level - r_b->level;
 	if (c) return c;
@@ -1030,32 +1244,31 @@ static int m_cmp_race(const void *a, const void *b)
 	return strcmp(r_a->name, r_b->name);
 }
 
-static char *m_xchar(int oid) { return &r_info[default_join[oid].oid].x_char; }
+static wchar_t *m_xchar(int oid) { return &r_info[default_join[oid].oid].x_char; }
 static byte *m_xattr(int oid) { return &r_info[default_join[oid].oid].x_attr; }
 static const char *race_name(int gid) { return monster_group[gid].name; }
 
 static void mon_lore(int oid)
 {
+	int r_idx;
+	monster_race *r_ptr;
+	const monster_lore *l_ptr;
+	textblock *tb;
+
+	r_idx = default_join[oid].oid;
+
+	assert(r_idx);
+	r_ptr = &r_info[r_idx];
+	l_ptr = get_lore(r_ptr);
+
 	/* Update the monster recall window */
-	monster_race_track(default_join[oid].oid);
+	monster_race_track(r_ptr);
 	handle_stuff(p_ptr);
 
-	/* Save the screen */
-	screen_save();
-
-	/* Describe */
-	text_out_hook = text_out_to_screen;
-
-	/* Recall monster */
-	roff_top(default_join[oid].oid);
-	Term_gotoxy(0, 2);
-	describe_monster(default_join[oid].oid, FALSE);
-
-	text_out_c(TERM_L_BLUE, "\n[Press any key to continue]\n");
-	(void)anykey();
-
-	/* Load the screen */
-	screen_load();
+	tb = textblock_new();
+	lore_description(tb, r_ptr, l_ptr, FALSE);
+	textui_textblock_show(tb, SCREEN_REGION, NULL);
+	textblock_free(tb);
 }
 
 static void mon_summary(int gid, const int *object_list, int n, int top, int row, int col)
@@ -1103,8 +1316,8 @@ static int count_known_monsters(void)
 
 		for (j = 1; j < N_ELEMENTS(monster_group) - 1; j++)
 		{
-			const char *pat = monster_group[j].chars;
-			if (strchr(pat, r_ptr->d_char)) m_count++;
+			const wchar_t *pat = monster_group[j].chars;
+			if (wcschr(pat, r_ptr->d_char)) m_count++;
 		}
 	}
 
@@ -1136,8 +1349,8 @@ static void do_cmd_knowledge_monsters(const char *name, int row)
 
 		for (j = 1; j < N_ELEMENTS(monster_group) - 1; j++)
 		{
-			const char *pat = monster_group[j].chars;
-			if (strchr(pat, r_ptr->d_char)) m_count++;
+			const wchar_t *pat = monster_group[j].chars;
+			if (wcschr(pat, r_ptr->d_char)) m_count++;
 		}
 	}
 
@@ -1153,10 +1366,10 @@ static void do_cmd_knowledge_monsters(const char *name, int row)
 
 		for (j = 0; j < N_ELEMENTS(monster_group)-1; j++)
 		{
-			const char *pat = monster_group[j].chars;
+			const wchar_t *pat = monster_group[j].chars;
 			if (j == 0 && !rf_has(r_ptr->flags, RF_UNIQUE))
 				continue;
-			else if (j > 0 && !strchr(pat, r_ptr->d_char))
+			else if (j > 0 && !wcschr(pat, r_ptr->d_char))
 				continue;
 
 			monsters[m_count] = m_count;
@@ -1199,10 +1412,11 @@ static void display_artifact(int col, int row, bool cursor, int oid)
 
 static object_type *find_artifact(struct artifact *artifact)
 {
-	int i, j;
+	int i;
+	struct store *s;
 
 	/* Look for the artifact, either in inventory, store or the object list */
-	for (i = 0; i < z_info->o_max; i++)
+	for (i = 1; i < z_info->o_max; i++)
 	{
 		if (object_byid(i)->artifact == artifact)
 			return object_byid(i);
@@ -1214,14 +1428,10 @@ static object_type *find_artifact(struct artifact *artifact)
 			return &p_ptr->inventory[i];
 	}
 
-	for (j = 1; j < (FEAT_SHOP_TAIL - FEAT_SHOP_HEAD + 1); j++)
-	{
-		for (i = 0; i < stores[j].stock_size; i++)
-		{
-			if (stores[j].stock[i].artifact == artifact)
-				return &stores[j].stock[i];
-		}
-	}
+	for (s = stores; s; s = s->next)
+		for (i = 0; i < s->stock_size; i++)
+			if (s->stock[i].artifact == artifact)
+				return &s->stock[i];
 
 	return NULL;
 }
@@ -1259,9 +1469,10 @@ static void desc_art_fake(int a_idx)
 	handle_stuff(p_ptr);
 
 	tb = object_info(o_ptr, OINFO_NONE);
-	object_desc(header, sizeof(header), o_ptr, ODESC_PREFIX | ODESC_FULL);
+	object_desc(header, sizeof(header), o_ptr,
+			ODESC_PREFIX | ODESC_FULL | ODESC_CAPITAL);
 
-	textui_textblock_show(tb, area, format("%^s", header));
+	textui_textblock_show(tb, area, header);
 	textblock_free(tb);
 
 #if 0
@@ -1499,13 +1710,13 @@ static void display_object(int col, int row, bool cursor, int oid)
 	bool use_flavour = (kind->flavor) && !(aware && kind->tval == TV_SCROLL);
 
 	byte a = use_flavour ? kind->flavor->x_attr : kind->x_attr;
-	byte c = use_flavour ? kind->flavor->x_char : kind->x_char;
+	wchar_t c = use_flavour ? kind->flavor->x_char : kind->x_char;
 
 	/* Display known artifacts differently */
 	if (of_has(kind->flags, OF_INSTA_ART) && artifact_is_known(get_artifact_from_kind(kind)))
 		get_artifact_display_name(o_name, sizeof(o_name), get_artifact_from_kind(kind));
 	else
- 		object_kind_name(o_name, sizeof(o_name), kind, OPT(cheat_know));
+ 		object_kind_name(o_name, sizeof(o_name), kind, OPT(cheat_xtra));
 
 	/* If the type is "tried", display that */
 	if (kind->tried && !aware)
@@ -1551,7 +1762,7 @@ static void desc_obj_fake(int k_idx)
 	}
 
 	/* Update the object recall window */
-	track_object_kind(k_idx);
+	track_object_kind(kind);
 	handle_stuff(p_ptr);
 
 	/* Wipe the object */
@@ -1570,9 +1781,10 @@ static void desc_obj_fake(int k_idx)
 	handle_stuff(p_ptr);
 
 	tb = object_info(o_ptr, OINFO_NONE);
-	object_desc(header, sizeof(header), o_ptr, ODESC_PREFIX | ODESC_FULL);
+	object_desc(header, sizeof(header), o_ptr,
+			ODESC_PREFIX | ODESC_FULL | ODESC_CAPITAL);
 
-	textui_textblock_show(tb, area, format("%^s", header));
+	textui_textblock_show(tb, area, header);
 	textblock_free(tb);
 }
 
@@ -1616,7 +1828,7 @@ static int o_cmp_tval(const void *a, const void *b)
 
 static int obj2gid(int oid) { return obj_group_order[k_info[oid].tval]; }
 
-static char *o_xchar(int oid)
+static wchar_t *o_xchar(int oid)
 {
 	object_kind *kind = objkind_byid(oid);
 
@@ -1777,13 +1989,15 @@ static void display_feature(int col, int row, bool cursor, int oid )
 
 	if (tile_height == 1) {
 		/* Display symbols */
-		col = 66;
+		col = 65;
 		col += big_pad(col, row, f_ptr->x_attr[FEAT_LIGHTING_DARK],
 				f_ptr->x_char[FEAT_LIGHTING_DARK]);
 		col += big_pad(col, row, f_ptr->x_attr[FEAT_LIGHTING_LIT],
 				f_ptr->x_char[FEAT_LIGHTING_LIT]);
-		col += big_pad(col, row, f_ptr->x_attr[FEAT_LIGHTING_BRIGHT],
-				f_ptr->x_char[FEAT_LIGHTING_BRIGHT]);
+		col += big_pad(col, row, f_ptr->x_attr[FEAT_LIGHTING_TORCH],
+				f_ptr->x_char[FEAT_LIGHTING_TORCH]);
+		col += big_pad(col, row, f_ptr->x_attr[FEAT_LIGHTING_LOS],
+				f_ptr->x_char[FEAT_LIGHTING_LOS]);
 	}
 }
 
@@ -1802,11 +2016,11 @@ static int f_cmp_fkind(const void *a, const void *b)
 }
 
 static const char *fkind_name(int gid) { return feature_group_text[gid]; }
-/* Disgusting hack to allow 3 in 1 editting of terrain visuals */
+/* Disgusting hack to allow 4 in 1 editing of terrain visuals */
 static enum grid_light_level f_uik_lighting = FEAT_LIGHTING_LIT;
 /* XXX needs *better* retooling for multi-light terrain */
 static byte *f_xattr(int oid) { return &f_info[oid].x_attr[f_uik_lighting]; }
-static char *f_xchar(int oid) { return &f_info[oid].x_char[f_uik_lighting]; }
+static wchar_t *f_xchar(int oid) { return &f_info[oid].x_char[f_uik_lighting]; }
 static void feat_lore(int oid) { (void)oid; /* noop */ }
 static const char *feat_prompt(int oid)
 {
@@ -1822,13 +2036,15 @@ static void f_xtra_act(struct keypress ch, int oid)
 	/* XXX must be a better way to cycle this */
 	if (ch.code == 'l') {
 		switch (f_uik_lighting) {
-				case FEAT_LIGHTING_LIT:  f_uik_lighting = FEAT_LIGHTING_BRIGHT; break;
-				case FEAT_LIGHTING_BRIGHT:  f_uik_lighting = FEAT_LIGHTING_DARK; break;
+				case FEAT_LIGHTING_LIT:  f_uik_lighting = FEAT_LIGHTING_TORCH; break;
+                case FEAT_LIGHTING_TORCH: f_uik_lighting = FEAT_LIGHTING_LOS; break;
+				case FEAT_LIGHTING_LOS:  f_uik_lighting = FEAT_LIGHTING_DARK; break;
 				default:	f_uik_lighting = FEAT_LIGHTING_LIT; break;
 		}		
 	} else if (ch.code == 'L') {
 		switch (f_uik_lighting) {
-				case FEAT_LIGHTING_DARK:  f_uik_lighting = FEAT_LIGHTING_BRIGHT; break;
+				case FEAT_LIGHTING_DARK:  f_uik_lighting = FEAT_LIGHTING_LOS; break;
+                case FEAT_LIGHTING_LOS: f_uik_lighting = FEAT_LIGHTING_TORCH; break;
 				case FEAT_LIGHTING_LIT:  f_uik_lighting = FEAT_LIGHTING_DARK; break;
 				default:	f_uik_lighting = FEAT_LIGHTING_LIT; break;
 		}
