@@ -124,7 +124,7 @@ static void display_string(menu_type *m, int oid, bool cursor,
 }
 
 /* Virtual function table for displaying arrays of strings */
-const menu_iter menu_iter_strings =
+static const menu_iter menu_iter_strings =
 { 
 	NULL,              /* get_tag() */
 	NULL,              /* valid_row() */
@@ -133,9 +133,6 @@ const menu_iter menu_iter_strings =
 	NULL
 };
 
-
-
-
 /* ================== SKINS ============== */
 
 
@@ -185,7 +182,7 @@ static void display_scrolling(menu_type *menu, int cursor, int *top, region *loc
 	}
 
 	if (menu->cursor >= 0)
-		Term_gotoxy(col, row + cursor - *top);
+		Term_gotoxy(col + menu->cursor_x_offset, row + cursor - *top);
 }
 
 static char scroll_get_tag(menu_type *menu, int pos)
@@ -230,13 +227,19 @@ static const menu_skin menu_skin_scroll =
 
 /*** Multi-column menus ***/
 
-/* Find the position of a cursor given a screen address */
 static int columns_get_cursor(int row, int col, int n, int top, region *loc)
 {
-	int rows_per_page = loc->page_rows;
-	int colw = loc->width / (n + rows_per_page - 1) / rows_per_page;
-	int cursor = row + rows_per_page * (col - loc->col) / colw;
+	int w, h, cursor;
+        int rows_per_page = loc->page_rows;
+        int cols = (n + rows_per_page - 1) / rows_per_page;
+	int colw = 23;
 
+	Term_get_size(&w, &h);
+
+	if ((colw * cols) > (w - col))
+		colw = (w - col) / cols;
+
+	cursor = (row - loc->row) + rows_per_page * ((col - loc->col) / colw);
 	if (cursor < 0) cursor = 0;	/* assert: This should never happen */
 	if (cursor >= n) cursor = n - 1;
 
@@ -273,7 +276,7 @@ static void display_columns(menu_type *menu, int cursor, int *top, region *loc)
 	}
 
 	if (menu->cursor >= 0)
-		Term_gotoxy(col + (cursor / rows_per_page) * colw,
+		Term_gotoxy(col + (cursor / rows_per_page) * colw + menu->cursor_x_offset,
 				row + (cursor % rows_per_page) - *top);
 }
 
@@ -398,6 +401,24 @@ static int get_cursor_key(menu_type *menu, int top, struct keypress key)
 	return -1;
 }
 
+static menu_row_style_t menu_row_style_for_validity(menu_row_validity_t row_valid)
+{
+	menu_row_style_t style;
+
+	switch (row_valid) {
+		case MN_ROW_INVALID:
+		case MN_ROW_HIDDEN:
+			style = MN_ROW_STYLE_DISABLED;
+			break;
+		case MN_ROW_VALID:
+		default:
+			style = MN_ROW_STYLE_ENABLED;
+			break;
+	}
+
+	return style;
+}
+
 /* Modal display of menu */
 static void display_menu_row(menu_type *menu, int pos, int top,
                              bool cursor, int row, int col, int width)
@@ -405,11 +426,15 @@ static void display_menu_row(menu_type *menu, int pos, int top,
 	int flags = menu->flags;
 	char sel = 0;
 	int oid = pos;
+	menu_row_validity_t row_valid = MN_ROW_VALID;
 
 	if (menu->filter_list)
 		oid = menu->filter_list[oid];
 
-	if (menu->row_funcs->valid_row && menu->row_funcs->valid_row(menu, oid) == 2)
+	if (menu->row_funcs->valid_row)
+		row_valid = menu->row_funcs->valid_row(menu, oid);
+
+	if (row_valid == MN_ROW_HIDDEN)
 		return;
 
 	if (!(flags & MN_NO_TAGS))
@@ -424,8 +449,8 @@ static void display_menu_row(menu_type *menu, int pos, int top,
 
 	if (sel)
 	{
-		/* TODO: CHECK FOR VALID */
-		byte color = curs_attrs[CURS_KNOWN][0 != (cursor)];
+		menu_row_style_t style = menu_row_style_for_validity(row_valid);
+		byte color = curs_attrs[style][0 != (cursor)];
 		Term_putstr(col, row, 3, color, format("%c) ", sel));
 		col += 3;
 		width -= 3;
@@ -479,14 +504,15 @@ bool menu_handle_mouse(menu_type *menu, const ui_event *in,
 {
 	int new_cursor;
 
-	if (!region_inside(&menu->active, in))
-	{
+	if (in->mouse.button == 2) {
+		out->type = EVT_ESCAPE;
+	} else
+	if (!region_inside(&menu->active, in)) {
 		/* A click to the left of the active region is 'back' */
 		if (!region_inside(&menu->active, in) &&
 				in->mouse.x < menu->active.col)
 			out->type = EVT_ESCAPE;
-	}
-	else
+	} else
 	{
 		int count = menu->filter_list ? menu->filter_count : menu->count;
 
@@ -582,7 +608,7 @@ bool menu_handle_keypress(menu_type *menu, const ui_event *in,
 		}
 	}
 
-	else if (in->key.code == '\n' || in->key.code == '\r')
+	else if (in->key.code == KC_ENTER)
 		out->type = EVT_SELECT;
 
 	/* Try directional movement */
@@ -645,6 +671,9 @@ ui_event menu_select(menu_type *menu, int notify, bool popup)
 
 		/* Handle mouse & keyboard commands */
 		if (in.type == EVT_MOUSE) {
+			if (!no_act && menu_handle_action(menu, &in)) {
+				continue;
+			}
 			menu_handle_mouse(menu, &in, &out);
 		} else if (in.type == EVT_KBRD) {
 			if (!no_act && menu->cmd_keys &&
@@ -818,6 +847,7 @@ void menu_init(menu_type *menu, skin_id skin_id, const menu_iter *iter)
 	menu->row_funcs = iter;
 	menu->skin = skin;
 	menu->cursor = 0;
+	menu->cursor_x_offset = 0;
 }
 
 menu_type *menu_new(skin_id skin_id, const menu_iter *iter)
@@ -834,21 +864,51 @@ menu_type *menu_new_action(menu_action *acts, size_t n)
 	return m;
 }
 
+void menu_free(menu_type *m)
+{
+	mem_free(m);
+}
+
+void menu_set_cursor_x_offset(menu_type *m, int offset)
+{
+	/* This value is used in the menu skin's display_list() function. */
+	m->cursor_x_offset = offset;
+}
 
 /*** Dynamic menu handling ***/
 
 struct menu_entry {
 	char *text;
 	int value;
+	menu_row_validity_t valid;
 
 	struct menu_entry *next;
 };
 
+static int dynamic_valid(menu_type *m, int oid)
+{
+	struct menu_entry *entry;
+
+	for (entry = menu_priv(m); oid; oid--) {
+		entry = entry->next;
+		assert(entry);
+	}
+
+	return entry->valid;
+}
+
 static void dynamic_display(menu_type *m, int oid, bool cursor,
 		int row, int col, int width)
 {
 	struct menu_entry *entry;
-	byte color = curs_attrs[CURS_KNOWN][0 != cursor];
+	byte color = curs_attrs[MN_ROW_STYLE_ENABLED][0 != cursor];
+
+	/* Hack? While row_funcs is private, we need to be consistent with what the menu will do. */
+	if (m->row_funcs->valid_row) {
+		menu_row_validity_t row_valid = m->row_funcs->valid_row(m, oid);
+		menu_row_style_t style = menu_row_style_for_validity(row_valid);
+		color = curs_attrs[style][0 != cursor];
+	}
 
 	for (entry = menu_priv(m); oid; oid--) {
 		entry = entry->next;
@@ -860,7 +920,7 @@ static void dynamic_display(menu_type *m, int oid, bool cursor,
 
 static const menu_iter dynamic_iter = {
 	NULL,	/* tag */
-	NULL,	/* valid */
+	dynamic_valid,
 	dynamic_display,
 	NULL,	/* handler */
 	NULL	/* resize */
@@ -873,7 +933,7 @@ menu_type *menu_dynamic_new(void)
 	return m;
 }
 
-void menu_dynamic_add(menu_type *m, const char *text, int value)
+void menu_dynamic_add_valid(menu_type *m, const char *text, int value, menu_row_validity_t valid)
 {
 	struct menu_entry *head = menu_priv(m);
 	struct menu_entry *new = mem_zalloc(sizeof *new);
@@ -882,6 +942,7 @@ void menu_dynamic_add(menu_type *m, const char *text, int value)
 
 	new->text = string_make(text);
 	new->value = value;
+	new->valid = valid;
 
 	if (head) {
 		struct menu_entry *tail = head;
@@ -899,6 +960,24 @@ void menu_dynamic_add(menu_type *m, const char *text, int value)
 	}
 }
 
+void menu_dynamic_add(menu_type *m, const char *text, int value)
+{
+	menu_dynamic_add_valid(m, text, value, MN_ROW_VALID);
+}
+
+void menu_dynamic_add_label_valid(menu_type *m, const char *text, const char label, int value, char *label_list, menu_row_validity_t valid)
+{
+	if(label && m->selections && (m->selections == label_list)) {
+		label_list[m->count] = label;
+	}
+	menu_dynamic_add_valid(m,text,value, valid);
+}
+
+void menu_dynamic_add_label(menu_type *m, const char *text, const char label, int value, char *label_list)
+{
+	menu_dynamic_add_label_valid(m, text, label, value, label_list, MN_ROW_VALID);
+}
+
 size_t menu_dynamic_longest_entry(menu_type *m)
 {
 	size_t biggest = 0;
@@ -935,7 +1014,7 @@ int menu_dynamic_select(menu_type *m)
 void menu_dynamic_free(menu_type *m)
 {
 	struct menu_entry *entry = menu_priv(m);
-	if (entry) {
+	while (entry) {
 		struct menu_entry *next = entry->next;
 		string_free(entry->text);
 		mem_free(entry);
