@@ -18,6 +18,7 @@
 #include "angband.h"
 
 #include "cave.h"
+#include "cmds.h"
 #include "object/tvalsval.h"
 #include "game-cmd.h"
 #include "spells.h"
@@ -38,6 +39,7 @@ struct spell_menu_data {
 
 	bool browse;
 	bool (*is_valid)(int spell);
+	bool show_description;
 
 	int selected_spell;
 };
@@ -113,8 +115,13 @@ static bool spell_menu_handler(menu_type *m, const ui_event *e, int oid)
 		d->selected_spell = d->spells[oid];
 		return d->browse ? TRUE : FALSE;
 	}
+	else if (e->type == EVT_KBRD) {
+		if (e->key.code == '?') {
+			d->show_description = !d->show_description;
+		}
+	}
 
-	return TRUE;
+	return FALSE;
 }
 
 /**
@@ -125,18 +132,20 @@ static void spell_menu_browser(int oid, void *data, const region *loc)
 	struct spell_menu_data *d = data;
 	int spell = d->spells[oid];
 
-	/* Redirect output to the screen */
-	text_out_hook = text_out_to_screen;
-	text_out_wrap = 0;
-	text_out_indent = loc->col - 1;
-	text_out_pad = 1;
+	if (d->show_description) {
+		/* Redirect output to the screen */
+		text_out_hook = text_out_to_screen;
+		text_out_wrap = 0;
+		text_out_indent = loc->col - 1;
+		text_out_pad = 1;
 
-	Term_gotoxy(loc->col, loc->row + loc->page_rows);
-	text_out("\n%s\n", s_info[(p_ptr->class->spell_book == TV_MAGIC_BOOK) ? spell : spell + PY_MAX_SPELLS].text);
+		Term_gotoxy(loc->col, loc->row + loc->page_rows);
+		text_out("\n%s\n", s_info[(p_ptr->class->spell_book == TV_MAGIC_BOOK) ? spell : spell + PY_MAX_SPELLS].text);
 
-	/* XXX */
-	text_out_pad = 0;
-	text_out_indent = 0;
+		/* XXX */
+		text_out_pad = 0;
+		text_out_indent = 0;
+	}
 }
 
 static const menu_iter spell_menu_iter = {
@@ -169,6 +178,7 @@ static menu_type *spell_menu_new(const object_type *o_ptr,
 	d->is_valid = is_valid;
 	d->selected_spell = -1;
 	d->browse = FALSE;
+	d->show_description = FALSE;
 
 	menu_setpriv(m, d->n_spells, d);
 
@@ -177,6 +187,7 @@ static menu_type *spell_menu_new(const object_type *o_ptr,
 	m->flags = MN_CASELESS_TAGS;
 	m->selections = lower_case;
 	m->browse_hook = spell_menu_browser;
+	m->cmd_keys = "?";
 
 	/* set size */
 	loc.page_rows = d->n_spells + 1;
@@ -199,14 +210,17 @@ static void spell_menu_destroy(menu_type *m)
 static int spell_menu_select(menu_type *m, const char *noun, const char *verb)
 {
 	struct spell_menu_data *d = menu_priv(m);
+	char buf[80];
 
 	screen_save();
-
 	region_erase_bordered(&m->active);
-	prt(format("%^s which %s? ", verb, noun), 0, 0);
+
+	/* Format, capitalise and display */
+	strnfmt(buf, sizeof buf, "%s which %s? ('?' to toggle description)", verb, noun);
+	my_strcap(buf);
+	prt(buf, 0, 0);
 
 	menu_select(m, 0, TRUE);
-
 	screen_load();
 
 	return d->selected_spell;
@@ -222,7 +236,7 @@ static void spell_menu_browse(menu_type *m, const char *noun)
 	screen_save();
 
 	region_erase_bordered(&m->active);
-	prt(format("Browsing %ss.  Press Escape to exit.", noun), 0, 0);
+	prt(format("Browsing %ss. ('?' to toggle description)", noun), 0, 0);
 
 	d->browse = TRUE;
 	menu_select(m, 0, TRUE);
@@ -236,7 +250,7 @@ static void spell_menu_browse(menu_type *m, const char *noun)
  *
  * Returns the spell selected, or -1.
  */
-static int get_spell(const object_type *o_ptr, const char *verb,
+int get_spell(const object_type *o_ptr, const char *verb,
 		bool (*spell_test)(int spell))
 {
 	menu_type *m;
@@ -300,7 +314,7 @@ void textui_obj_study(void)
 
 	item_tester_hook = obj_can_study;
 	if (!get_item(&item, "Study which book? ",
-			"You have no books that you can read.",
+			"You cannot learn any new spells from the books you have.",
 			CMD_STUDY_BOOK, (USE_INVEN | USE_FLOOR)))
 		return;
 
@@ -346,3 +360,29 @@ void textui_obj_cast(void)
 		cmd_set_arg_choice(cmd_get_top(), 0, spell);
 	}
 }
+/* same as above but returns the spell used. two functions to avoid some
+ * compiler warnings initializing commands in cmd0.c */
+int textui_obj_cast_ret(void)
+{
+	int item;
+	int spell;
+
+	const char *verb = ((p_ptr->class->spell_book == TV_MAGIC_BOOK) ? "cast" : "recite");
+
+	item_tester_hook = obj_can_cast_from;
+	if (!get_item(&item, "Cast from which book? ",
+			"You have no books that you can read.",
+			CMD_CAST, (USE_INVEN | USE_FLOOR)))
+		return -1;
+
+	/* Track the object kind */
+	track_object(item);
+
+	/* Ask for a spell */
+	spell = get_spell(object_from_item_idx(item), verb, spell_okay_to_cast);
+	if (spell >= 0) {
+		cmd_insert(CMD_CAST);
+		cmd_set_arg_choice(cmd_get_top(), 0, spell);
+	}
+  return spell;
+}
