@@ -17,177 +17,15 @@
  */
 
 #include "angband.h"
-#include "button.h"
 #include "cmds.h"
+#include "target.h"
+#include "files.h"
 #include "game-event.h"
 #include "randname.h"
 
 
-/*
- * Find the start of a possible Roman numerals suffix by going back from the
- * end of the string to a space, then checking that all the remaining chars
- * are valid Roman numerals.
- * 
- * Return the start position, or NULL if there isn't a valid suffix. 
- */
-char *find_roman_suffix_start(const char *buf)
-{
-	const char *start = strrchr(buf, ' ');
-	const char *p;
-	
-	if (start)
-	{
-		start++;
-		p = start;
-		while (*p)
-		{
-			if (*p != 'I' && *p != 'V' && *p != 'X' && *p != 'L' &&
-			    *p != 'C' && *p != 'D' && *p != 'M')
-			{
-				start = NULL;
-				break;
-			}
-			++p;			    
-		}
-	}
-	return (char *)start;
-}
 
-/*----- Roman numeral functions  ------*/
-
-/*
- * Converts an arabic numeral (int) to a roman numeral (char *).
- *
- * An arabic numeral is accepted in parameter `n`, and the corresponding
- * upper-case roman numeral is placed in the parameter `roman`.  The
- * length of the buffer must be passed in the `bufsize` parameter.  When
- * there is insufficient room in the buffer, or a roman numeral does not
- * exist (e.g. non-positive integers) a value of 0 is returned and the
- * `roman` buffer will be the empty string.  On success, a value of 1 is
- * returned and the zero-terminated roman numeral is placed in the
- * parameter `roman`.
- */
-int int_to_roman(int n, char *roman, size_t bufsize)
-{
-	/* Roman symbols */
-	char roman_symbol_labels[13][3] =
-		{"M", "CM", "D", "CD", "C", "XC", "L", "XL", "X", "IX",
-		 "V", "IV", "I"};
-	int  roman_symbol_values[13] =
-		{1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1};
-
-	/* Clear the roman numeral buffer */
-	roman[0] = '\0';
-
-	/* Roman numerals have no zero or negative numbers */
-	if (n < 1)
-		return 0;
-
-	/* Build the roman numeral in the buffer */
-	while (n > 0)
-	{
-		int i = 0;
-
-		/* Find the largest possible roman symbol */
-		while (n < roman_symbol_values[i])
-			i++;
-
-		/* No room in buffer, so abort */
-		if (strlen(roman) + strlen(roman_symbol_labels[i]) + 1
-			> bufsize)
-			break;
-
-		/* Add the roman symbol to the buffer */
-		my_strcat(roman, roman_symbol_labels[i], bufsize);
-
-		/* Decrease the value of the arabic numeral */
-		n -= roman_symbol_values[i];
-	}
-
-	/* Ran out of space and aborted */
-	if (n > 0)
-	{
-		/* Clean up and return */
-		roman[0] = '\0';
-
-		return 0;
-	}
-
-	return 1;
-}
-
-
-/*
- * Converts a roman numeral (char *) to an arabic numeral (int).
- *
- * The null-terminated roman numeral is accepted in the `roman`
- * parameter and the corresponding integer arabic numeral is returned.
- * Only upper-case values are considered. When the `roman` parameter
- * is empty or does not resemble a roman numeral, a value of -1 is
- * returned.
- *
- * XXX This function will parse certain non-sense strings as roman
- *     numerals, such as IVXCCCVIII
- */
-int roman_to_int(const char *roman)
-{
-	size_t i;
-	int n = 0;
-	char *p;
-
-	char roman_token_chr1[] = "MDCLXVI";
-	const char *roman_token_chr2[] = {0, 0, "DM", 0, "LC", 0, "VX"};
-
-	int roman_token_vals[7][3] = {{1000},
-	                              {500},
-	                              {100, 400, 900},
-	                              {50},
-	                              {10, 40, 90},
-	                              {5},
-	                              {1, 4, 9}};
-
-	if (strlen(roman) == 0)
-		return -1;
-
-	/* Check each character for a roman token, and look ahead to the
-	   character after this one to check for subtraction */
-	for (i = 0; i < strlen(roman); i++)
-	{
-		char c1, c2;
-		int c1i, c2i;
-
-		/* Get the first and second chars of the next roman token */
-		c1 = roman[i];
-		c2 = roman[i + 1];
-
-		/* Find the index for the first character */
-		p = strchr(roman_token_chr1, c1);
-		if (p)
-		{
-			c1i = p - roman_token_chr1;
-		} else {
-			return -1;
-		}
-
-		/* Find the index for the second character */
-		c2i = 0;
-		if (roman_token_chr2[c1i] && c2)
-		{
-			p = strchr(roman_token_chr2[c1i], c2);
-			if (p)
-			{
-				c2i = (p - roman_token_chr2[c1i]) + 1;
-				/* Two-digit token, so skip a char on the next pass */
-				i++;
-			}
-		}
-
-		/* Increase the arabic numeral */
-		n += roman_token_vals[c1i][c2i];
-	}
-
-	return n;
-}
+static bool inkey_xtra;
 
 /*
  * Flush all pending input.
@@ -255,6 +93,11 @@ static ui_event inkey_aux(int scan_cutoff)
  */
 struct keypress *inkey_next = NULL;
 
+/**
+ * See if more propmts will be skipped while in a keymap.
+ */
+static bool keymap_auto_more;
+
 
 #ifdef ALLOW_BORG
 
@@ -322,7 +165,7 @@ ui_event inkey_ex(void)
 	}
 
 	/* Hack -- Use the "inkey_next" pointer */
-	if (inkey_next && inkey_next->code)
+	while (inkey_next && inkey_next->code)
 	{
 		/* Get next character, and advance */
 		ke.key = *inkey_next++;
@@ -331,25 +174,52 @@ ui_event inkey_ex(void)
 		inkey_flag = FALSE;
 		inkey_scan = 0;
 
+		/* peek at the key, and see if we want to skip more prompts */
+		if (ke.key.code == '(') {
+			keymap_auto_more = TRUE;
+			/* since we are not returning this char, make sure the next key below works well */
+			if (!inkey_next || !inkey_next->code) {
+				ke.type = EVT_NONE;
+				break;
+			}
+			continue;
+		} else
+		if (ke.key.code == ')') {
+			keymap_auto_more = FALSE;
+			/* since we are not returning this char, make sure the next key below works well */
+			if (!inkey_next || !inkey_next->code) {
+				ke.type = EVT_NONE;
+				break;
+			}
+			continue;
+		}
+
 		/* Accept result */
 		return (ke);
 	}
 
+	/* make sure that the flag to skip more prompts is off */
+	keymap_auto_more = FALSE;
+
 	/* Forget pointer */
 	inkey_next = NULL;
 
 #ifdef ALLOW_BORG
 
 	/* Mega-Hack -- Use the special hook */
-	if (inkey_hack && ((ke.key = (*inkey_hack)(inkey_xtra)) != 0))
+	if (inkey_hack)
 	{
-		/* Cancel the various "global parameters" */
-		inkey_flag = FALSE;
-		inkey_scan = 0;
-		ke.type = EVT_KBRD;
+		ke.key = (*inkey_hack)(inkey_xtra);
+		if (ke.key.type != EVT_NONE)
+		{
+			/* Cancel the various "global parameters" */
+			inkey_flag = FALSE;
+			inkey_scan = 0;
+			ke.type = EVT_KBRD;
 
-		/* Accept result */
-		return (ke);
+			/* Accept result */
+			return (ke);
+		}
 	}
 
 #endif /* ALLOW_BORG */
@@ -359,7 +229,7 @@ ui_event inkey_ex(void)
 	(void)Term_get_cursor(&cursor_state);
 
 	/* Show the cursor if waiting, except sometimes in "command" mode */
-	if (!inkey_scan && (!inkey_flag || character_icky))
+	if (!inkey_scan && (!inkey_flag || character_icky || (OPT(show_target) && target_sighted())))
 		(void)Term_set_cursor(TRUE);
 
 
@@ -402,26 +272,9 @@ ui_event inkey_ex(void)
 		/* Get a key (see above) */
 		ke = inkey_aux(inkey_scan);
 
-		/* Handle mouse buttons */
-		if ((ke.type == EVT_MOUSE) && (OPT(mouse_buttons)))
-		{
-			/* Check to see if we've hit a button */
-			/* Assuming text buttons here for now - this would have to
-			 * change for GUI buttons */
-			char key = button_get_key(ke.mouse.x, ke.mouse.y);
-
-			if (key)
-			{
-				/* Rewrite the event */
-				/* XXXmacro button implementation needs updating */
-				ke.type = EVT_BUTTON;
-				ke.key.code = key;
-				ke.key.mods = 0;
-
-				/* Done */
-				break;
-			}
-		}
+		if(inkey_scan && ke.type == EVT_NONE)
+			/* The keypress timed out. We need to stop here. */
+			break;
 
 		/* Treat back-quote as escape */
 		if (ke.key.code == '`')
@@ -466,19 +319,68 @@ struct keypress inkey(void)
 	ui_event ke = EVENT_EMPTY;
 
 	/* Only accept a keypress */
-	while (ke.type != EVT_ESCAPE && ke.type != EVT_KBRD)
-		ke = inkey_ex();
+	/*while (ke.type != EVT_ESCAPE && ke.type != EVT_KBRD)
+		ke = inkey_ex();*/
 
-	/* Paranoia */
+	/* Paranoia */ /*
 	if (ke.type == EVT_ESCAPE) {
 		ke.type = EVT_KBRD;
 		ke.key.code = ESCAPE;
 		ke.key.mods = 0;
+  }
+  */
+	while (ke.type != EVT_ESCAPE && ke.type != EVT_KBRD
+			&& ke.type != EVT_MOUSE  && ke.type != EVT_BUTTON)
+		ke = inkey_ex();
+
+	/* make the event a keypress */
+	if (ke.type == EVT_ESCAPE) {
+		ke.type = EVT_KBRD;
+		ke.key.code = ESCAPE;
+		ke.key.mods = 0;
+	} else
+	if (ke.type == EVT_MOUSE) {
+		if (ke.mouse.button == 1) {
+			ke.type = EVT_KBRD;
+			ke.key.code = '\n';
+			ke.key.mods = 0;
+		} else {
+			ke.type = EVT_KBRD;
+			ke.key.code = ESCAPE;
+			ke.key.mods = 0;
+		}
+	} else
+	if (ke.type == EVT_BUTTON) {
+		ke.type = EVT_KBRD;
 	}
 
 	return ke.key;
 }
 
+/*
+ * Get a "keypress" or a "mousepress" from the user.
+ * on return the event must be either a key press or a mouse press
+ */
+ui_event inkey_m(void)
+{
+	ui_event ke = EVENT_EMPTY;
+
+	/* Only accept a keypress */
+	while (ke.type != EVT_ESCAPE && ke.type != EVT_KBRD
+			&& ke.type != EVT_MOUSE  && ke.type != EVT_BUTTON)
+		ke = inkey_ex();
+	if (ke.type == EVT_ESCAPE) {
+		ke.type = EVT_KBRD;
+		ke.key.code = ESCAPE;
+		ke.key.mods = 0;
+	} else
+	if (ke.type == EVT_BUTTON) {
+		ke.type = EVT_KBRD;
+	}
+
+  return ke;
+}
+
 
 
 /*
@@ -528,7 +430,7 @@ static void msg_flush(int x)
 	/* Pause for response */
 	Term_putstr(x, 0, -1, a, "-more-");
 
-	if (!OPT(auto_more))
+	if ((!OPT(auto_more)) && !keymap_auto_more)
 		anykey();
 
 	/* Clear the line */
@@ -728,92 +630,28 @@ void message_flush(void)
 
 
 
-/*
- * Save the screen, and increase the "icky" depth.
- *
- * This function must match exactly one call to "screen_load()".
- */
-void screen_save(void)
-{
-	/* Hack -- Flush messages */
-	message_flush();
-
-	/* Save the screen (if legal) */
-	Term_save();
-
-	/* Increase "icky" depth */
-	character_icky++;
-}
-
+/*** text_out() ***/
 
 /*
- * Load the screen, and decrease the "icky" depth.
- *
- * This function must match exactly one call to "screen_save()".
+ * Function hook to output (colored) text to the screen or to a file.
  */
-void screen_load(void)
-{
-	/* Hack -- Flush messages */
-	message_flush();
-
-	/* Load the screen (if legal) */
-	Term_load();
-
-	/* Decrease "icky" depth */
-	character_icky--;
-
-	/* Mega hack -redraw big graphics - sorry NRM */
-	if (character_icky == 0 && (tile_width > 1 || tile_height > 1))
-		Term_redraw();
-}
-
+void (*text_out_hook)(byte a, const char *str);
 
 /*
- * Display a string on the screen using an attribute.
- *
- * At the given location, using the given attribute, if allowed,
- * add the given string.  Do not clear the line.
+ * Hack -- Where to wrap the text when using text_out().  Use the default
+ * value (for example the screen width) when 'text_out_wrap' is 0.
  */
-void c_put_str(byte attr, const char *str, int row, int col)
-{
-	/* Position cursor, Dump the attr/text */
-	Term_putstr(col, row, -1, attr, str);
-}
-
+int text_out_wrap = 0;
 
 /*
- * As above, but in "white"
+ * Hack -- Indentation for the text when using text_out().
  */
-void put_str(const char *str, int row, int col)
-{
-	/* Spawn */
-	Term_putstr(col, row, -1, TERM_WHITE, str);
-}
-
-
+int text_out_indent = 0;
 
 /*
- * Display a string on the screen using an attribute, and clear
- * to the end of the line.
+ * Hack -- Padding after wrapping
  */
-void c_prt(byte attr, const char *str, int row, int col)
-{
-	/* Clear line, position cursor */
-	Term_erase(col, row, 255);
-
-	/* Dump the attr/text */
-	Term_addstr(-1, attr, str);
-}
-
-
-/*
- * As above, but in "white"
- */
-void prt(const char *str, int row, int col)
-{
-	/* Spawn */
-	c_prt(TERM_WHITE, str, row, col);
-}
+int text_out_pad = 0;
 
 
 /*
@@ -838,11 +676,8 @@ void text_out_to_screen(byte a, const char *str)
 
 	int wrap;
 
-	const char *s;
-	char buf[1024];
-
-	/* We use either ascii or system-specific encoding */
-	int encoding = (OPT(xchars_to_file)) ? SYSTEM_SPECIFIC : ASCII;
+	const wchar_t *s;
+	wchar_t buf[1024];
 
 	/* Obtain the size */
 	(void)Term_get_size(&wid, &h);
@@ -851,10 +686,7 @@ void text_out_to_screen(byte a, const char *str)
 	(void)Term_locate(&x, &y);
 
 	/* Copy to a rewriteable string */
-	my_strcpy(buf, str, 1024);
-	
-	/* Translate it to 7-bit ASCII or system-specific format */
-	xstr_trans(buf, encoding);
+	Term_mbstowcs(buf, str, 1024);
 	
 	/* Use special wrapping boundary? */
 	if ((text_out_wrap > 0) && (text_out_wrap < wid))
@@ -865,10 +697,10 @@ void text_out_to_screen(byte a, const char *str)
 	/* Process the string */
 	for (s = buf; *s; s++)
 	{
-		char ch;
+		wchar_t ch;
 
 		/* Force wrap */
-		if (*s == '\n')
+		if (*s == L'\n')
 		{
 			/* Wrap */
 			x = text_out_indent;
@@ -884,15 +716,15 @@ void text_out_to_screen(byte a, const char *str)
 		}
 
 		/* Clean up the char */
-		ch = (my_isprint((unsigned char)*s) ? *s : ' ');
+		ch = (iswprint(*s) ? *s : L' ');
 
 		/* Wrap words as needed */
-		if ((x >= wrap - 1) && (ch != ' '))
+		if ((x >= wrap - 1) && (ch != L' '))
 		{
 			int i, n = 0;
 
-			byte av[256];
-			char cv[256];
+			int av[256];
+			wchar_t cv[256];
 
 			/* Wrap word */
 			if (x < wrap)
@@ -904,7 +736,7 @@ void text_out_to_screen(byte a, const char *str)
 					Term_what(i, y, &av[i], &cv[i]);
 
 					/* Break on space */
-					if (cv[i] == ' ') break;
+					if (cv[i] == L' ') break;
 
 					/* Track current word */
 					n = i;
@@ -947,6 +779,12 @@ void text_out_to_screen(byte a, const char *str)
 }
 
 
+/*
+ * Hack - the destination file for text_out_to_file.
+ */
+ang_file *text_out_file = NULL;
+
+
 /*
  * Write text to the given file and apply line-wrapping.
  *
@@ -971,29 +809,28 @@ void text_out_to_file(byte a, const char *str)
 	/* Wrap width */
 	int wrap = (text_out_wrap ? text_out_wrap : 75);
 
-	/* We use either ascii or system-specific encoding */
- 	int encoding = OPT(xchars_to_file) ? SYSTEM_SPECIFIC : ASCII;
-
 	/* Unused parameter */
 	(void)a;
 
 	/* Copy to a rewriteable string */
  	my_strcpy(buf, str, 1024);
 
- 	/* Translate it to 7-bit ASCII or system-specific format */
- 	xstr_trans(buf, encoding);
-
 	/* Current location within "buf" */
  	s = buf;
 
 	/* Process the string */
 	while (*s)
 	{
-		char ch;
 		int n = 0;
 		int len = wrap - pos;
 		int l_space = -1;
 
+		/* In case we are already past the wrap point (which can happen with
+		 * punctuation at the end of the line), make sure we don't overrun.
+		 */
+		if (len < 0)
+			len = 0;
+
 		/* If we are at the start of the line... */
 		if (pos == 0)
 		{
@@ -1051,17 +888,8 @@ void text_out_to_file(byte a, const char *str)
 		}
 
 		/* Write that line to file */
-		for (n = 0; n < len; n++)
-		{
-			/* Ensure the character is printable */
-			ch = (my_isprint((unsigned char) s[n]) ? s[n] : ' ');
-
-			/* Write out the character */
-			file_writec(text_out_file, ch);
-
-			/* Increment */
-			pos++;
-		}
+		file_write(text_out_file, s, len);
+		pos += len;
 
 		/* Move 's' past the stuff we've written */
 		s += len;
@@ -1161,7 +989,7 @@ static bool next_section(const char *source, size_t init, const char **text, siz
 	{
 		const char *s = next + 1;
 
-		while (*s && isalpha((unsigned char) *s)) s++;
+		while (*s && (isalpha((unsigned char) *s) || isspace((unsigned char) *s))) s++;
 
 		/* Woo!  valid opening tag thing */
 		if (*s == '}')
@@ -1257,10 +1085,10 @@ void text_out_e(const char *fmt, ...)
 
 		if (tag)
 		{
-			char tagbuffer[11];
+			char tagbuffer[16];
 
-			/* Colour names are less than 11 characters long. */
-			assert(taglen < 11);
+			/* Colour names are less than 16 characters long. */
+			assert(taglen < 16);
 
 			memcpy(tagbuffer, tag, taglen);
 			tagbuffer[taglen] = '\0';
@@ -1311,15 +1139,12 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, st
 		{
 			*curs = 0;
 			return TRUE;
-			break;
 		}
 		
-		case '\n':
-		case '\r':
+		case KC_ENTER:
 		{
 			*curs = *len;
 			return TRUE;
-			break;
 		}
 		
 		case ARROW_LEFT:
@@ -1336,8 +1161,8 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, st
 			break;
 		}
 		
-		case 0x7F:
-		case '\010':
+               case KC_BACKSPACE:
+               case KC_DELETE:
 		{
 			/* If this is the first time round, backspace means "delete all" */
 			if (firsttime)
@@ -1350,13 +1175,21 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, st
 			}
 
 			/* Refuse to backspace into oblivion */
-			if (*curs == 0) break;
+			if((keypress.code == KC_BACKSPACE && *curs == 0) ||
+			   (keypress.code == KC_DELETE && *curs >= *len))
+				break;
 
 			/* Move the string from k to nul along to the left by 1 */
-			memmove(&buf[*curs - 1], &buf[*curs], *len - *curs);
+			if(keypress.code == KC_BACKSPACE)
+				memmove(&buf[*curs - 1], &buf[*curs],
+					*len - *curs);
+			else
+				memmove(&buf[*curs], &buf[*curs+1],
+					*len - *curs -1);
 
 			/* Decrement */
-			(*curs)--;
+			if(keypress.code == KC_BACKSPACE)
+				(*curs)--;
 			(*len)--;
 
 			/* Terminate */
@@ -1369,7 +1202,7 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, st
 		{
 			bool atnull = (buf[*curs] == 0);
 
-			if (!my_isprint((unsigned char)keypress.code))
+			if (!isprint(keypress.code))
 			{
 				bell("Illegal edit key!");
 				break;
@@ -1436,7 +1269,7 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, st
  * 'askfor_aux_keypress' (the default handler if you supply NULL for
  * 'keypress_h') for an example.
  */
-bool askfor_aux(char *buf, size_t len, bool keypress_h(char *, size_t, size_t *, size_t *, struct keypress, bool))
+bool askfor_aux(char *buf, size_t len, bool (*keypress_h)(char *, size_t, size_t *, size_t *, struct keypress, bool))
 {
 	int y, x;
 
@@ -1513,7 +1346,7 @@ static bool get_name_keypress(char *buf, size_t buflen, size_t *curs, size_t *le
 		case '*':
 		{
 			*len = randname_make(RANDNAME_TOLKIEN, 4, 8, buf, buflen, name_sections);
-			buf[0] = toupper((unsigned char) buf[0]);
+			my_strcap(buf);
 			*curs = 0;
 			result = FALSE;
 			break;
@@ -1588,9 +1421,6 @@ bool get_string(const char *prompt, char *buf, size_t len)
 	/* Ask the user for a string */
 	res = askfor_aux(buf, len, NULL);
 
-	/* Translate it to 8-bit (Latin-1) */
- 	xstr_trans(buf, LATIN1);
-
 	/* Clear prompt */
 	prt("", 0, 0);
 
@@ -1602,29 +1432,15 @@ bool get_string(const char *prompt, char *buf, size_t len)
 
 /*
  * Request a "quantity" from the user
- *
- * Allow "p_ptr->command_arg" to specify a quantity
  */
 s16b get_quantity(const char *prompt, int max)
 {
 	int amt = 1;
 
-
-	/* Use "command_arg" */
-	if (p_ptr->command_arg)
-	{
-		/* Extract a number */
-		amt = p_ptr->command_arg;
-
-		/* Clear "command_arg" */
-		p_ptr->command_arg = 0;
-	}
-
 	/* Prompt if needed */
-	else if ((max != 1))
+	if (max != 1)
 	{
 		char tmp[80];
-
 		char buf[80];
 
 		/* Build a prompt if needed */
@@ -1670,43 +1486,29 @@ s16b get_quantity(const char *prompt, int max)
  */
 bool get_check(const char *prompt)
 {
-	struct keypress ke;
+	//struct keypress ke;
+	ui_event ke;
 
 	char buf[80];
 
-	bool repeat = FALSE;
-  
 	/* Paranoia XXX XXX XXX */
 	message_flush();
 
 	/* Hack -- Build a "useful" prompt */
 	strnfmt(buf, 78, "%.70s[y/n] ", prompt);
 
-	/* Hack - kill the repeat button */
-	if (button_kill('n')) repeat = TRUE;
-	
-	/* Make some buttons */
-	button_add("[y]", 'y');
-	button_add("[n]", 'n');
-	redraw_stuff(p_ptr);
-  
 	/* Prompt for it */
 	prt(buf, 0, 0);
-	ke = inkey();
+	ke = inkey_m();
 
-	/* Kill the buttons */
-	button_kill('y');
-	button_kill('n');
-
-	/* Hack - restore the repeat button */
-	if (repeat) button_add("[Rpt]", 'n');
-	redraw_stuff(p_ptr);
-  
 	/* Erase the prompt */
 	prt("", 0, 0);
 
 	/* Normal negation */
-	if ((ke.code != 'Y') && (ke.code != 'y')) return (FALSE);
+	if (ke.type == EVT_MOUSE) {
+		if ((ke.mouse.button != 1) && (ke.mouse.y != 0)) return (FALSE);
+	} else
+	if ((ke.key.code != 'Y') && (ke.key.code != 'y')) return (FALSE);
 
 	/* Success */
 	return (TRUE);
@@ -1724,28 +1526,15 @@ bool get_check(const char *prompt)
  */
 char get_char(const char *prompt, const char *options, size_t len, char fallback)
 {
-	size_t i;
 	struct keypress key;
-	char button[4], buf[80];
-	bool repeat = FALSE;
-  
+	char buf[80];
+
 	/* Paranoia XXX XXX XXX */
 	message_flush();
 
 	/* Hack -- Build a "useful" prompt */
 	strnfmt(buf, 78, "%.70s[%s] ", prompt, options);
 
-	/* Hack - kill the repeat button */
-	if (button_kill('n')) repeat = TRUE;
-	
-	/* Make some buttons */
-	for (i = 0; i < len; i++)
-	{
-		strnfmt(button, 4, "[%c]", options[i]);
-		button_add(button, options[i]);
-	}
-	redraw_stuff(p_ptr);
-  
 	/* Prompt for it */
 	prt(buf, 0, 0);
 
@@ -1759,13 +1548,6 @@ char get_char(const char *prompt, const char *options, size_t len, char fallback
 	if (!strchr(options, (char)key.code))
 		key.code = fallback;
 
-	/* Kill the buttons */
-	for (i = 0; i < len; i++) button_kill(options[i]);
-
-	/* Hack - restore the repeat button */
-	if (repeat) button_add("[Rpt]", 'n');
-	redraw_stuff(p_ptr);
-  
 	/* Erase the prompt */
 	prt("", 0, 0);
 
@@ -1792,14 +1574,11 @@ static bool get_file_text(const char *suggested_name, char *path, size_t len)
 	path_build(path, len, ANGBAND_DIR_USER, buf);
 
 	/* Check if it already exists */
-	if (file_exists(buf))
-	{
-		char buf2[160];
-		strnfmt(buf2, sizeof(buf2), "Replace existing file %s?", buf);
+	if (file_exists(path) && !get_check("Replace existing file? "))
+		return FALSE;
 
-		if (get_check(buf2) == FALSE)
-			return FALSE;
-	}
+	/* Tell the user where it's saved to. */
+	msg("Saving as %s.", path);
 
 	return TRUE;
 }
@@ -1846,7 +1625,7 @@ bool get_com_ex(const char *prompt, ui_event *command)
 	prt(prompt, 0, 0);
 
 	/* Get a key */
-	ke = inkey_ex();
+	ke = inkey_m();
 
 	/* Clear the prompt */
 	prt("", 0, 0);
@@ -1855,9 +1634,9 @@ bool get_com_ex(const char *prompt, ui_event *command)
 	*command = ke;
 
 	/* Done */
-	if (ke.type == EVT_KBRD && ke.key.code == ESCAPE)
-		return FALSE;
-	return TRUE;
+	if ((ke.type == EVT_KBRD && ke.key.code != ESCAPE) || (ke.type == EVT_MOUSE))
+	  return TRUE;
+	return FALSE;
 }
 
 
@@ -1954,6 +1733,20 @@ const char *attr_to_text(byte a)
 		return ("Icky");
 }
 
+/**
+ * Return whether the given display char matches an entered symbol
+ *
+ * Horrible hack. TODO UTF-8 find some way of entering mb chars
+ */
+bool char_matches_key(wchar_t c, keycode_t key)
+{
+	wchar_t keychar[2];
+	char k[2] = {'\0', '\0'};
+
+	k[0] = (char)key;
+	Term_mbstowcs(keychar, k, 1);
+	return (c == keychar[0]);
+}
 
 #ifdef SUPPORT_GAMMA
 
@@ -2081,7 +1874,7 @@ void build_gamma_table(int gamma)
 		 * Store the value in the table so that the
 		 * floating point pow function isn't needed.
 		 */
-		gamma_table[i] = ((long)(value / 256) * i) / 256;
+		gamma_table[i] = (byte)(((long)(value / 256) * i) / 256);
 	}
 }
 
