@@ -286,6 +286,9 @@ BOOL ReadDIB(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo)
 		}
 	}
 
+	pInfo->ImageWidth = lpbi->biWidth;
+	pInfo->ImageHeight = lpbi->biHeight;
+
 	/* offset to the bits from start of DIB header */
 	offBits = lpbi->biSize + nNumColors * sizeof(RGBQUAD);
 
@@ -336,6 +339,8 @@ void FreeDIB(DIBINIT *dib)
 	dib->hPalette = NULL;
 	dib->hBitmap = NULL;
 	dib->hDIB = NULL;
+	dib->ImageWidth = 0;
+	dib->ImageHeight = 0;
 }
 
 #endif /* USE_WIN || WINDOWS || WIN32 */
