@@ -42,7 +42,7 @@
  * case, the DIBINIT structure pointed to by pInfo is filled with the appropriate
  * handles, and FALSE if something went wrong.
  */
-BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
+BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask, BOOL premultiply) {
 	png_structp png_ptr;
 	png_infop info_ptr;
 	byte header[8];
@@ -62,40 +62,40 @@ BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
 
 	BOOL update = FALSE;
 	
-	// open the file and test it for being a png
+	/* open the file and test it for being a png */
 	FILE *fp = fopen(lpFileName, "rb");
 	if (!fp)
 	{
-		//plog_fmt("Unable to open PNG file.");
+		/*plog_fmt("Unable to open PNG file."); */
 		return (FALSE);
 	}
 
 	fread(header, 1, 8, fp);
 	if (png_sig_cmp(header, 0, 8)) {
-		//plog_fmt("Unable to open PNG file - not a PNG file.");
+		/*plog_fmt("Unable to open PNG file - not a PNG file."); */
 		fclose(fp);
 		return (FALSE);
 	}
 	
-	// Create the png structure
+	/* Create the png structure */
 	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
 	if(!png_ptr)
 	{
-		//plog_fmt("Unable to initialize PNG library");
+		/*plog_fmt("Unable to initialize PNG library"); */
 		fclose(fp);
 		return (FALSE);
 	}
 	
-	// create the info structure
+	/* create the info structure */
 	info_ptr = png_create_info_struct(png_ptr);
 	if (!info_ptr)
 	{
 		png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
-		//plog_fmt("Failed to create PNG info structure.");
+		/*plog_fmt("Failed to create PNG info structure."); */
 		return FALSE;
 	}
 	
-	// setup error handling for init
+	/* setup error handling for init */
 	png_init_io(png_ptr, fp);
 	png_set_sig_bytes(png_ptr, 8);
 	
@@ -138,7 +138,7 @@ BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
 	}
 
 	png_set_bgr(png_ptr);
-	// after these requests, the data should always be RGB or ARGB
+	/* after these requests, the data should always be RGB or ARGB */
 
 	/* initialize row_pointers */
 	row_pointers = (png_bytep*) malloc(sizeof(png_bytep) * height);
@@ -153,7 +153,48 @@ BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
 
 	/* we are done with the file pointer, so close it */
 	fclose(fp);
-	
+
+	/* pre multiply the image colors by the alhpa if thats what we want */
+	if (premultiply && (color_type == PNG_COLOR_TYPE_RGB_ALPHA)) {
+		int x;
+		png_byte r,g,b,a;
+		png_bytep row;
+		/* process the file */
+		for (y = 0; y < height; ++y) {
+			row = row_pointers[y];
+			for (x = 0; x < width; ++x) {
+				a = *(row + x*4 + 3);
+				if (a == 0) {
+					/* for every alpha that is fully transparent, make the
+					 * corresponding color true black */
+					*(row + x*4 + 0) = 0;
+					*(row + x*4 + 1) = 0;
+					*(row + x*4 + 2) = 0;
+				} else
+				if (a != 255) {
+					float rf,gf,bf,af;
+					/* blend the color value based on this value */
+					r = *(row + x*4 + 0);
+					g = *(row + x*4 + 1);
+					b = *(row + x*4 + 2);
+
+					rf = ((float)r) / 255.f;
+					gf = ((float)g) / 255.f;
+					bf = ((float)b) / 255.f;
+					af = ((float)a) / 255.f;
+        
+					r = (png_byte)(rf*af*255.f);
+					g = (png_byte)(gf*af*255.f);
+					b = (png_byte)(bf*af*255.f);
+        
+					*(row + x*4 + 0) = r;
+					*(row + x*4 + 1) = g;
+					*(row + x*4 + 2) = b;
+				}
+			}
+		}
+	}
+  
 	/* create the DIB */
 	bi.bmiHeader.biWidth = (LONG)width;
 	bi.bmiHeader.biHeight = -((LONG)height);
@@ -163,9 +204,9 @@ BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
 	bi.bmiHeader.biBitCount = 24;
 	bi.bmiHeader.biCompression = BI_RGB;
 	bi.bmiHeader.biPlanes = 1;
-	bi.bmiHeader.biSize = 40; // the size of the structure
-	bi.bmiHeader.biXPelsPerMeter = 3424; // just a number I saw when testing this with a sample
-	bi.bmiHeader.biYPelsPerMeter = 3424; // just a number I saw when testing this with a sample
+	bi.bmiHeader.biSize = 40; /* the size of the structure */
+	bi.bmiHeader.biXPelsPerMeter = 3424; /* just a number I saw when testing this with a sample */
+	bi.bmiHeader.biYPelsPerMeter = 3424; /* just a number I saw when testing this with a sample */
 	bi.bmiHeader.biSizeImage = width*height*3;
 	
 	biSrc.bmiHeader.biWidth = (LONG)width;
@@ -175,26 +216,31 @@ BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
 	biSrc.bmiHeader.biClrImportant = 0;
 	biSrc.bmiHeader.biCompression = BI_RGB;
 	biSrc.bmiHeader.biPlanes = 1;
-	biSrc.bmiHeader.biSize = 40; // the size of the structure
-	biSrc.bmiHeader.biXPelsPerMeter = 3424; // just a number I saw when testing this with a sample
-	biSrc.bmiHeader.biYPelsPerMeter = 3424; // just a number I saw when testing this with a sample
+	biSrc.bmiHeader.biSize = 40; /* the size of the structure */
+	biSrc.bmiHeader.biXPelsPerMeter = 3424; /* just a number I saw when testing this with a sample */
+	biSrc.bmiHeader.biYPelsPerMeter = 3424; /* just a number I saw when testing this with a sample */
 	
 	if (color_type == PNG_COLOR_TYPE_RGB_ALPHA) {
 		biSrc.bmiHeader.biBitCount = 32;
 		biSrc.bmiHeader.biSizeImage = width*height*4;
+
+		if (!pMask) {
+			bi.bmiHeader.biBitCount = 32;
+			bi.bmiHeader.biSizeImage = width*height*4;
+		}
 	} else {
 		biSrc.bmiHeader.biBitCount = 24;
 		biSrc.bmiHeader.biSizeImage = width*height*3;
 	}
-		
+
 	hDC = GetDC(hWnd);
 	
 	hPalette = GetStockObject(DEFAULT_PALETTE);
-	// Need to realize palette for converting DIB to bitmap. 
+	/* Need to realize palette for converting DIB to bitmap. */
 	hOldPal = SelectPalette(hDC, hPalette, TRUE);
 	RealizePalette(hDC);
 
-	// copy the data to the DIB
+	/* copy the data to the DIB */
 	hBitmap = CreateDIBitmap(hDC, &(bi.bmiHeader), 0, NULL,
 							 &biSrc, DIB_RGB_COLORS);
 		
@@ -204,7 +250,7 @@ BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
 		{
 			if (SetDIBits(hDC, hBitmap, height-y-1, 1, row_pointers[y], &biSrc, DIB_RGB_COLORS) != 1)
 			{
-				//plog_fmt("Failed to alloc temporary memory for PNG data.");
+				/*plog_fmt("Failed to alloc temporary memory for PNG data."); */
 				DeleteObject(hBitmap);
 				hBitmap = NULL;
 				noerror = FALSE;
@@ -224,6 +270,8 @@ BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
 		pInfo->hBitmap = hBitmap;
 		pInfo->hPalette = hPalette;
 		pInfo->hDIB = 0;
+		pInfo->ImageWidth = width;
+		pInfo->ImageHeight = height;
 	}
 	
 	if (pMask && (color_type == PNG_COLOR_TYPE_RGB_ALPHA))
@@ -287,14 +335,14 @@ BOOL ReadDIB2_PNG(HWND hWnd, LPSTR lpFileName, DIBINIT *pInfo, DIBINIT *pMask) {
 		}
 	}
 	
-	// release the image memory
+	/* release the image memory */
 	for (y = 0; y < height; ++y)
 	{
 		free(row_pointers[y]);
 	}
 	free(row_pointers);
 	
-	// release all the the PNG Structures
+	/* release all the the PNG Structures */
 	if (info_ptr) {
 		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
 		info_ptr = NULL;
