@@ -34,11 +34,11 @@
 #define SPELL_LIGHT_AREA                3
 #define SPELL_FIND_TRAPS_DOORS          4
 #define SPELL_CURE_LIGHT_WOUNDS         5
-#define SPELL_TREASURE_DETECTION        6
-/* #define SPELL_OBJECT_DETECTION          7 */
+#define SPELL_OBJECT_DETECTION        6
+
 #define SPELL_IDENTIFY                  8
 #define SPELL_DETECT_INVISIBLE          9
-#define SPELL_DETECT_ENCHANTMENT        10
+#define SPELL_TREASURE_DETECTION        10
 #define SPELL_STINKING_CLOUD            11
 #define SPELL_LIGHTNING_BOLT            12
 #define SPELL_CONFUSE_MONSTER           13
@@ -109,7 +109,7 @@
 #define PRAYER_CHANT                   11
 #define PRAYER_SANCTUARY               12
 #define PRAYER_SATISFY_HUNGER          13
-#define PRAYER_REMOVE_CURSE            14
+//#define PRAYER_REMOVE_CURSE            14
 #define PRAYER_RESIST_HEAT_COLD        15
 
 /* Chants and Blessings */
@@ -155,7 +155,7 @@
 /* Holy Infusions */
 #define PRAYER_UNBARRING_WAYS          46
 #define PRAYER_RECHARGING              47
-#define PRAYER_DISPEL_CURSE            48
+//#define PRAYER_DISPEL_CURSE            48
 #define PRAYER_ENCHANT_WEAPON          49
 #define PRAYER_ENCHANT_ARMOUR          50
 #define PRAYER_ELEMENTAL_BRAND         51
@@ -505,9 +505,9 @@ static bool cast_mage_spell(int spell, int dir)
 			break;
 		}
 
-		case SPELL_TREASURE_DETECTION:
+		case SPELL_OBJECT_DETECTION:
 		{
-			(void)detect_treasure(TRUE);
+			(void)detect_treasure(TRUE, TRUE);
 			break;
 		}
 
@@ -675,7 +675,6 @@ static bool cast_mage_spell(int spell, int dir)
 		case SPELL_BANISHMENT:
 		{
 			return banishment();
-			break;
 		}
 
 		case SPELL_DOOR_CREATION:
@@ -704,8 +703,7 @@ static bool cast_mage_spell(int spell, int dir)
 
 		case SPELL_WORD_OF_RECALL:
 		{
-			set_recall();
-			break;
+			return set_recall();
 		}
 
 		case SPELL_ACID_BOLT:
@@ -750,9 +748,9 @@ static bool cast_mage_spell(int spell, int dir)
 			break;
 		}
 
-		case SPELL_DETECT_ENCHANTMENT:
+		case SPELL_TREASURE_DETECTION:
 		{
-			(void)detect_objects_magic(TRUE);
+			(void)detect_treasure(TRUE, FALSE);
 			break;
 		}
 
@@ -983,12 +981,14 @@ static bool cast_priest_spell(int spell, int dir)
 			player_set_food(p_ptr, PY_FOOD_MAX - 1);
 			break;
 		}
-
+		
+        /* Remove curse has been removed in 3.4 until curses are redone
 		case PRAYER_REMOVE_CURSE:
 		{
 			remove_curse();
 			break;
 		}
+		*/
 
 		case PRAYER_RESIST_HEAT_COLD:
 		{
@@ -1144,7 +1144,7 @@ static bool cast_priest_spell(int spell, int dir)
 
 		case PRAYER_CLAIRVOYANCE:
 		{
-			wiz_light();
+			wiz_light(FALSE);
 			break;
 		}
 
@@ -1184,7 +1184,6 @@ static bool cast_priest_spell(int spell, int dir)
 			(void)do_res_stat(A_WIS);
 			(void)do_res_stat(A_DEX);
 			(void)do_res_stat(A_CON);
-			(void)do_res_stat(A_CHR);
 			break;
 		}
 
@@ -1235,15 +1234,17 @@ static bool cast_priest_spell(int spell, int dir)
 
 		case PRAYER_RECHARGING:
 		{
-			return recharge(15);
+			return recharge(20 + plev);
 		}
-
+		
+        /* Dispel Curse has been removed in 3.4 until curses are redone 
 		case PRAYER_DISPEL_CURSE:
 		{
 			(void)remove_all_curse();
 			break;
 		}
-
+		*/
+		
 		case PRAYER_ENCHANT_WEAPON:
 		{
 			return enchant_spell(randint0(4) + 1, randint0(4) + 1, 0);
@@ -1286,8 +1287,7 @@ static bool cast_priest_spell(int spell, int dir)
 
 		case PRAYER_WORD_OF_RECALL:
 		{
-			set_recall();
-			break;
+			return set_recall();
 		}
 
 		case PRAYER_ALTER_REALITY:
@@ -1317,3 +1317,8 @@ bool cast_spell(int tval, int index, int dir)
 		return cast_priest_spell(index, dir);
 	}
 }
+
+bool spell_is_identify(int book, int spell)
+{
+	return (book == TV_MAGIC_BOOK && spell == SPELL_IDENTIFY) || (book == TV_PRAYER_BOOK && spell == PRAYER_PERCEPTION);
+}
