@@ -20,14 +20,17 @@
 
 #include "angband.h"
 #include "buildid.h"
-#include "button.h"
 #include "cave.h"
 #include "files.h"
 #include "game-event.h"
 #include "game-cmd.h"
+#include "grafmode.h"
+#include "hint.h"
 #include "monster/mon-lore.h"
+#include "monster/mon-list.h"
 #include "monster/mon-util.h"
 #include "monster/monster.h"
+#include "object/obj-list.h"
 #include "object/tvalsval.h"
 #include "textui.h"
 #include "ui-birth.h"
@@ -37,7 +40,7 @@
  * of the basic player events.  For convenience, these have been grouped 
  * in this list.
  */
-game_event_type player_events[] =
+static game_event_type player_events[] =
 {
 	EVENT_RACE_CLASS,
 	EVENT_PLAYERTITLE,
@@ -56,13 +59,12 @@ game_event_type player_events[] =
 	EVENT_DUNGEONLEVEL,
 };
 
-game_event_type statusline_events[] =
+static game_event_type statusline_events[] =
 {
 	EVENT_STUDYSTATUS,
 	EVENT_STATUS,
 	EVENT_DETECTIONSTATUS,
 	EVENT_STATE,
-	EVENT_MOUSEBUTTONS
 };
 
 
@@ -243,7 +245,7 @@ static void prt_equippy(int row, int col)
 	int i;
 
 	byte a;
-	char c;
+	wchar_t c;
 
 	object_type *o_ptr;
 
@@ -326,24 +328,18 @@ static void prt_sp(int row, int col)
  */
 byte monster_health_attr(void)
 {
-	byte attr = TERM_WHITE;
-	
-	/* Not tracking */
-	if (!p_ptr->health_who)
+	struct monster *mon = p_ptr->health_who;
+	byte attr;
+
+	if (!mon) {
+		/* Not tracking */
 		attr = TERM_DARK;
 
-	/* Tracking an unseen, hallucinatory, or dead monster */
-	else if ((!cave_monster(cave, p_ptr->health_who)->ml) ||
-			(p_ptr->timed[TMD_IMAGE]) ||
-			(cave_monster(cave, p_ptr->health_who)->hp < 0))
-	{
+	} else if (!mon->ml || mon->hp < 0 || p_ptr->timed[TMD_IMAGE]) {
 		/* The monster health is "unknown" */
 		attr = TERM_WHITE;
-	}
-	
-	else
-	{
-		struct monster *mon = cave_monster(cave, p_ptr->health_who);
+
+	} else {
 		int pct;
 
 		/* Default to almost dead */
@@ -393,20 +389,18 @@ byte monster_health_attr(void)
 static void prt_health(int row, int col)
 {
 	byte attr = monster_health_attr();
-	struct monster *mon;
-	
+	struct monster *mon = p_ptr->health_who;
+
 	/* Not tracking */
-	if (!p_ptr->health_who)
+	if (!mon)
 	{
 		/* Erase the health bar */
 		Term_erase(col, row, 12);
 		return;
 	}
 
-	mon = cave_monster(cave, p_ptr->health_who);
-
 	/* Tracking an unseen, hallucinatory, or dead monster */
-	if ((!mon->ml) || /* Unseen */
+	if (!mon->ml || /* Unseen */
 			(p_ptr->timed[TMD_IMAGE]) || /* Hallucination */
 			(mon->hp < 0)) /* Dead (?) */
 	{
@@ -417,15 +411,11 @@ static void prt_health(int row, int col)
 	/* Tracking a visible monster */
 	else
 	{
-		int pct, len;
-
-		monster_type *m_ptr = cave_monster(cave, p_ptr->health_who);
-
 		/* Extract the "percent" of health */
-		pct = 100L * m_ptr->hp / m_ptr->maxhp;
+		int pct = 100L * mon->hp / mon->maxhp;
 
 		/* Convert percent into "health" */
-		len = (pct < 10) ? 1 : (pct < 90) ? (pct / 10 + 1) : 10;
+		int len = (pct < 10) ? 1 : (pct < 90) ? (pct / 10 + 1) : 10;
 
 		/* Default to "unknown" */
 		Term_putstr(col, row, 12, TERM_WHITE, "[----------]");
@@ -502,7 +492,6 @@ static void prt_dex(int row, int col) { prt_stat(A_DEX, row, col); }
 static void prt_wis(int row, int col) { prt_stat(A_WIS, row, col); }
 static void prt_int(int row, int col) { prt_stat(A_INT, row, col); }
 static void prt_con(int row, int col) { prt_stat(A_CON, row, col); }
-static void prt_chr(int row, int col) { prt_stat(A_CHR, row, col); }
 static void prt_race(int row, int col) { prt_field(p_ptr->race->name, row, col); }
 static void prt_class(int row, int col) { prt_field(p_ptr->class->name, row, col); }
 
@@ -529,7 +518,6 @@ static const struct side_handler_t
 	{ prt_wis,      4, EVENT_STATS },
 	{ prt_dex,      3, EVENT_STATS },
 	{ prt_con,      2, EVENT_STATS },
-	{ prt_chr,      1, EVENT_STATS },
 	{ NULL,        15, 0 },
 	{ prt_ac,       7, EVENT_AC },
 	{ prt_hp,       8, EVENT_HP },
@@ -602,7 +590,7 @@ static void hp_colour_change(game_event_type type, game_event_data *data, void *
 	 * using this command when graphics mode is on
 	 * causes the character to be a black square.
 	 */
-	if ((OPT(hp_changes_color)) && (arg_graphics == GRAPHICS_NONE))
+	if ((OPT(hp_changes_color)) && (use_graphics == GRAPHICS_NONE))
 	{
 		cave_light_spot(cave, p_ptr->py, p_ptr->px);
 	}
@@ -656,7 +644,6 @@ static const struct state_info hunger_data[] =
 	{ PY_FOOD_ALERT, S("Hungry"),   TERM_YELLOW },
 	{ PY_FOOD_FULL,  S(""),         TERM_L_GREEN },
 	{ PY_FOOD_MAX,   S("Full"),     TERM_L_GREEN },
-	{ PY_FOOD_UPPER, S("Gorged"),   TERM_GREEN },
 };
 
 /* For the various TMD_* effects */
@@ -752,7 +739,7 @@ static size_t prt_stun(int row, int col)
  */
 static size_t prt_hunger(int row, int col)
 {
-	PRINT_STATE(<, hunger_data, p_ptr->food, row, col);
+	PRINT_STATE(<=, hunger_data, p_ptr->food, row, col);
 	return 0;
 }
 
@@ -773,10 +760,10 @@ static size_t prt_state(int row, int col)
 
 
 	/* Resting */
-	if (p_ptr->resting)
+	if (player_is_resting(p_ptr))
 	{
 		int i;
-		int n = p_ptr->resting;
+		int n = player_resting_count(p_ptr);
 
 		/* Start with "Rest" */
 		my_strcpy(text, "Rest      ", sizeof(text));
@@ -825,19 +812,19 @@ static size_t prt_state(int row, int col)
 		}
 
 		/* Rest until healed */
-		else if (n == -1)
+		else if (n == REST_ALL_POINTS)
 		{
 			text[5] = text[6] = text[7] = text[8] = text[9] = '*';
 		}
 
 		/* Rest until done */
-		else if (n == -2)
+		else if (n == REST_COMPLETE)
 		{
 			text[5] = text[6] = text[7] = text[8] = text[9] = '&';
 		}
 		
 		/* Rest until HP or SP filled */
-		else if (n == -3)
+		else if (n == REST_SOME_POINTS)
 		{
 			text[5] = text[6] = text[7] = text[8] = text[9] = '!';
 		}
@@ -867,7 +854,120 @@ static size_t prt_state(int row, int col)
 	return strlen(text);
 }
 
+static const byte obj_feeling_color[] = 
+{
+	/* Colors used to display each obj feeling 	*/
+	TERM_WHITE,  /* "Looks like any other level." */
+	TERM_L_PURPLE, /* "you sense an item of wondrous power!" */
+	TERM_L_RED, /* "there are superb treasures here." */
+	TERM_ORANGE, /* "there are excellent treasures here." */
+	TERM_YELLOW, /* "there are very good treasures here." */
+	TERM_YELLOW, /* "there are good treasures here." */
+	TERM_L_GREEN, /* "there may be something worthwhile here." */
+	TERM_L_GREEN, /* "there may not be much interesting here." */
+	TERM_L_GREEN, /* "there aren't many treasures here." */
+	TERM_L_BLUE, /* "there are only scraps of junk here." */
+	TERM_L_BLUE  /* "there are naught but cobwebs here. */
+};
 
+static const byte mon_feeling_color[] = 
+{
+	/* Colors used to display each monster feeling */
+	TERM_WHITE, /* "You are still uncertain about this place" */
+	TERM_RED, /* "Omens of death haunt this place" */
+	TERM_ORANGE, /* "This place seems murderous" */
+	TERM_ORANGE, /* "This place seems terribly dangerous" */
+	TERM_YELLOW, /* "You feel anxious about this place" */
+	TERM_YELLOW, /* "You feel nervous about this place" */
+	TERM_GREEN, /* "This place does not seem too risky" */
+	TERM_GREEN, /* "This place seems reasonably safe" */
+	TERM_BLUE, /* "This seems a tame, sheltered place" */
+	TERM_BLUE, /* "This seems a quiet, peaceful place" */
+};
+
+/*
+ * Prints level feelings at status if they are enabled.
+ */
+static size_t prt_level_feeling(int row, int col) 
+{
+	u16b obj_feeling;
+	u16b mon_feeling;
+	char obj_feeling_str[6];
+	char mon_feeling_str[6];
+	int new_col;
+	byte obj_feeling_color_print;
+	
+	/* Don't show feelings for cold-hearted characters */
+	if (OPT(birth_no_feelings)) return 0;
+        
+	/* No useful feeling in town */
+	if (!p_ptr->depth) return 0;
+
+	/* Get feelings */
+	obj_feeling = cave->feeling / 10;
+	mon_feeling = cave->feeling - (10 * obj_feeling);
+	
+	/* 
+	 *   Convert object feeling to a symbol easier to parse
+	 * for a human.
+	 *   0 -> * "Looks like any other level."
+	 *   1 -> $ "you sense an item of wondrous power!" (special feeling)
+	 *   2 to 10 are feelings from 2 meaning superb feeling to 10
+	 * meaning naught but cowebs.
+	 *   It is easier for the player to have poor feelings as a
+	 * low number and superb feelings as a higher one. So for
+	 * display we reverse this numbers and substract 1.
+	 *   Thus (2-10) becomes (1-9 reversed)
+	 *
+	 *   But before that check if the player has explored enough
+	 * to get a feeling. If not display as ?
+	 */
+	if (cave->feeling_squares < FEELING1) {
+		my_strcpy( obj_feeling_str, "?", sizeof(obj_feeling_str) );
+		obj_feeling_color_print = TERM_WHITE;
+	} else {
+		obj_feeling_color_print = obj_feeling_color[obj_feeling];
+		if ( obj_feeling==0 ) {
+			my_strcpy( obj_feeling_str, "*", sizeof(obj_feeling_str) );
+		} else if ( obj_feeling==1 ) {
+			my_strcpy( obj_feeling_str, "$", sizeof(obj_feeling_str) );
+		} else {
+			strnfmt( obj_feeling_str, 5, "%d", 
+					(unsigned int) (11-obj_feeling) );
+		}
+	}
+	
+	/* 
+	 *   Convert monster feeling to a symbol easier to parse
+	 * for a human.
+	 *   0 -> ? . Monster feeling should never be 0, but we check
+	 * it just in case.
+	 *   1 to 9 are feelings from omens of death to quiet, paceful.
+	 * We also reverse this so that what we show is a danger feeling.
+	 */
+	if ( mon_feeling==0 ) {
+		my_strcpy( mon_feeling_str, "?", sizeof(mon_feeling_str) );
+	} else {
+		strnfmt( mon_feeling_str, 5, "%d", 
+				(unsigned int) ( 10-mon_feeling ) );
+	}
+	
+	/* Display it */
+	c_put_str( TERM_WHITE, "LF:", row, col);
+	new_col = col + 3;
+	c_put_str( mon_feeling_color[mon_feeling],
+			mon_feeling_str, row, new_col );
+	new_col += strlen( mon_feeling_str );
+	c_put_str( TERM_WHITE, "-", row, new_col );
+	++new_col;
+	c_put_str( obj_feeling_color_print, obj_feeling_str,
+			row, new_col );
+	new_col += strlen( obj_feeling_str ) + 1;
+	
+	return new_col - col;
+}
+
+                                                                                                                
 /*
  * Prints trap detection status
  */
@@ -879,7 +979,7 @@ static size_t prt_dtrap(int row, int col)
 	if (info & (CAVE2_DTRAP))
 	{
 		/* The player is on the border */
-		if (dtrap_edge(p_ptr->py, p_ptr->px))
+		if (info & (CAVE2_DEDGE))
 			c_put_str(TERM_YELLOW, "DTrap", row, col);
 		else
 			c_put_str(TERM_L_GREEN, "DTrap", row, col);
@@ -891,16 +991,25 @@ static size_t prt_dtrap(int row, int col)
 }
 
 
-
 /*
- * Print whether a character is studying or not.
+ * Print how many spells the player can study.
  */
 static size_t prt_study(int row, int col)
 {
+	char *text;
+	int attr = TERM_WHITE;
+
+	/* Can the player learn new spells? */
 	if (p_ptr->new_spells)
 	{
-		char *text = format("Study (%d)", p_ptr->new_spells);
-		put_str(text, row, col);
+		/* If the player does not carry a book with spells they can study,
+		   the message is displayed in a darker colour */
+		if (!player_book_has_unlearned_spells(p_ptr))
+			attr = TERM_L_DARK;
+
+		/* Print study message */
+		text = format("Study (%d)", p_ptr->new_spells);
+		c_put_str(attr, text, row, col);
 		return strlen(text) + 1;
 	}
 
@@ -908,7 +1017,6 @@ static size_t prt_study(int row, int col)
 }
 
 
-
 /*
  * Print all timed effects.
  */
@@ -936,30 +1044,18 @@ static size_t prt_unignore(int row, int col)
 	if (p_ptr->unignoring) {
 		const char *str = "Unignoring";
 		put_str(str, row, col);
-		return strlen(str);
+		return strlen(str) + 1;
 	}
 
 	return 0;
 }
 
-/*
- * Print mouse buttons
- */
-static size_t prt_buttons(int row, int col)
-{
-	if (OPT(mouse_buttons))
-		return button_print(row, col);
-
-	return 0;
-}
-
-
 /* Useful typedef */
 typedef size_t status_f(int row, int col);
 
-status_f *status_handlers[] =
-{ prt_buttons, prt_unignore, prt_recall, prt_state, prt_cut, prt_stun,
-  prt_hunger, prt_study, prt_tmd, prt_dtrap };
+static status_f *status_handlers[] =
+{ prt_level_feeling, prt_unignore, prt_recall, prt_state, prt_cut, 
+  prt_stun, prt_hunger, prt_study, prt_tmd, prt_dtrap };
 
 
 /*
@@ -1010,8 +1106,8 @@ static void update_maps(game_event_type type, game_event_data *data, void *user)
 	else
 	{
 		grid_data g;
-		byte a, ta;
-		char c, tc;
+		int a, ta;
+		wchar_t c, tc;
 		
 		int ky, kx;
 		int vy, vx;
@@ -1175,7 +1271,8 @@ static void update_itemlist_subwindow(game_event_type type, game_event_data *dat
 	/* Activate */
 	Term_activate(inv_term);
 
-	display_itemlist();
+    clear_from(0);
+    object_list_show_subwindow(Term->hgt, Term->wid);
 	Term_fresh();
 	
 	/* Restore */
@@ -1190,7 +1287,8 @@ static void update_monlist_subwindow(game_event_type type, game_event_data *data
 	/* Activate */
 	Term_activate(inv_term);
 
-	display_monlist();
+	clear_from(0);
+	monster_list_show_subwindow(Term->hgt, Term->wid);
 	Term_fresh();
 	
 	/* Restore */
@@ -1207,8 +1305,8 @@ static void update_monster_subwindow(game_event_type type, game_event_data *data
 	Term_activate(inv_term);
 
 	/* Display monster race info */
-	if (p_ptr->monster_race_idx)
-		display_roff(p_ptr->monster_race_idx);
+	if (p_ptr->monster_race)
+		lore_show_subwindow(p_ptr->monster_race, get_lore(p_ptr->monster_race));
 
 	Term_fresh();
 	
@@ -1227,8 +1325,8 @@ static void update_object_subwindow(game_event_type type, game_event_data *data,
 	
 	if (p_ptr->object_idx != NO_OBJECT)
 		display_object_idx_recall(p_ptr->object_idx);
-	else if(p_ptr->object_kind_idx != NO_OBJECT)
-		display_object_kind_recall(p_ptr->object_kind_idx);
+	else if (p_ptr->object_kind)
+		display_object_kind_recall(p_ptr->object_kind);
 	Term_fresh();
 	
 	/* Restore */
@@ -1262,6 +1360,8 @@ static void update_messages_subwindow(game_event_type type, game_event_data *dat
 
 		if (count == 1)
 			msg = str;
+		else if (count == 0)
+			msg = " ";
 		else
 			msg = format("%s <%dx>", str, count);
 
@@ -1312,6 +1412,17 @@ static void update_minimap_subwindow(game_event_type type,
 
 		flags->needs_redraw = FALSE;
 	}
+	else if (type == EVENT_DUNGEONLEVEL) {
+		/* XXX map_height and map_width need to be kept in sync with display_map() */
+		term *t = angband_term[flags->win_idx];
+		int map_height = t->hgt - 2;
+		int map_width = t->wid - 2;
+
+		/* Clear the entire term if the new map isn't going to fit the entire thing */
+		if (cave->height <= map_height || cave->width <= map_width) {
+			flags->needs_redraw = TRUE;
+		}
+	}
 }
 
 
@@ -1519,6 +1630,8 @@ static void subwindow_flag_changed(int win_idx, u32b flag, bool new_state)
 					       update_minimap_subwindow,
 					       &minimap_data[win_idx]);
 
+			register_or_deregister(EVENT_DUNGEONLEVEL, update_minimap_subwindow, &minimap_data[win_idx]);
+
 			register_or_deregister(EVENT_END,
 					       update_minimap_subwindow,
 					       &minimap_data[win_idx]);
@@ -1670,7 +1783,7 @@ static void show_splashscreen(game_event_type type, game_event_data *data, void
 
 	/* Open the News file */
 	path_build(buf, sizeof(buf), ANGBAND_DIR_FILE, "news.txt");
-	fp = file_open(buf, MODE_READ, -1);
+	fp = file_open(buf, MODE_READ, FTYPE_TEXT);
 
 	text_out_hook = text_out_to_screen;
 
@@ -1721,7 +1834,7 @@ static void see_floor_items(game_event_type type, game_event_data *data, void *u
 	size_t i;
 
 	/* Scan all marked objects in the grid */
-	floor_num = scan_floor(floor_list, N_ELEMENTS(floor_list), py, px, 0x03);
+	floor_num = scan_floor(floor_list, N_ELEMENTS(floor_list), py, px, 0x09);
 	if (floor_num == 0) return;
 
 	for (i = 0; i < floor_num; i++)
@@ -1770,9 +1883,16 @@ static void see_floor_items(game_event_type type, game_event_data *data, void *u
 		/* Restore screen */
 		screen_load();
 	}
-}
 
-extern game_event_handler ui_enter_birthscreen;
+	/* Update the map to display the items that are felt during blindness. */
+	if (blind) {
+		for (i = 0; i < floor_num; i++) {
+			/* Since the messages are detailed, we use MARK_SEEN to match description. */
+			object_type *o_ptr = object_byid(floor_list[i]);
+			o_ptr->marked = MARK_SEEN;
+		}
+	}
+}
 
 /* ------------------------------------------------------------------------
  * Initialising
