@@ -2,7 +2,7 @@
  * File: z-file.c
  * Purpose: Low-level file (and directory) handling
  *
- * Copyright (c) 1997-2007 Ben Harrison, pelpel, Andrew Sidwell, Matthew Jones
+ * Copyright (c) 1997-2007 Ben Harrison, pelpel, Andi Sidwell, Matthew Jones
  *
  * This work is free software; you can redistribute it and/or modify it
  * under the terms of either:
@@ -25,10 +25,6 @@
 # include <direct.h>
 #endif
 
-#ifdef MACH_O_CARBON
-# include <Carbon/Carbon.h>
-#endif
-
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
@@ -65,7 +61,7 @@ int player_egid;
  */
 void safe_setuid_drop(void)
 {
-#ifdef SET_UID
+#ifdef SETGID
 # if defined(HAVE_SETRESGID)
 
 	if (setresgid(-1, getgid(), -1) != 0)
@@ -77,7 +73,7 @@ void safe_setuid_drop(void)
 		quit("setegid(): cannot drop permissions correctly!");
 
 # endif
-#endif /* SET_UID */
+#endif /* SETGID */
 }
 
 
@@ -86,7 +82,7 @@ void safe_setuid_drop(void)
  */
 void safe_setuid_grab(void)
 {
-#ifdef SET_UID
+#ifdef SETGID
 # if defined(HAVE_SETRESGID)
 
 	if (setresgid(-1, player_egid, -1) != 0)
@@ -98,7 +94,7 @@ void safe_setuid_grab(void)
 		quit("setegid(): cannot grab permissions correctly!");
 
 # endif
-#endif /* SET_UID */
+#endif /* SETGID */
 }
 
 
@@ -114,10 +110,9 @@ static void path_parse(char *buf, size_t max, const char *file)
 }
 
 
-
 static void path_process(char *buf, size_t len, size_t *cur_len, const char *path)
 {
-#if defined(SET_UID) || defined(USE_PRIVATE_PATHS)
+#if defined(UNIX)
 
 	/* Home directory on Unixes */
 	if (path[0] == '~')
@@ -142,27 +137,8 @@ static void path_process(char *buf, size_t len, size_t *cur_len, const char *pat
 			username = user;
 		}
 
-#ifndef MACH_O_CARBON
-
 		/* Look up a user (or "current" user) */
 		pw = username[0] ? getpwnam(username) : getpwuid(getuid());
-
-#else /* MACH_O_CARBON */
-
-		{
-			/* On Macs getlogin() can incorrectly return root, so get the username via system frameworks */
-			CFStringRef cfusername = CSCopyUserName(TRUE);
-			CFIndex cfbufferlength = CFStringGetMaximumSizeForEncoding(CFStringGetLength(cfusername), kCFStringEncodingUTF8) + 1;
-			char *macusername = mem_alloc(cfbufferlength);
-			CFStringGetCString(cfusername, macusername, cfbufferlength, kCFStringEncodingUTF8);
-			CFRelease(cfusername);
-
-			/* Look up the user */
-			pw = getpwnam(macusername);
-			mem_free(macusername);
-		}
-#endif /* !MACH_O_CARBON */
-
 		if (!pw) return;
 
 		/* Copy across */
@@ -171,11 +147,9 @@ static void path_process(char *buf, size_t len, size_t *cur_len, const char *pat
 	}
 	else
 
-#endif
+#endif /* defined(UNIX) */
 
-	{
-		strnfcat(buf, len, cur_len, "%s", path);
-	}
+	strnfcat(buf, len, cur_len, "%s", path);
 }
 
 
@@ -208,7 +182,7 @@ size_t path_build(char *buf, size_t len, const char *base, const char *leaf)
 	 *   or there's no base path,
 	 * We use the leafname only.
 	 */
-#if defined(SET_UID) || defined(USE_PRIVATE_PATHS)
+#if defined(UNIX)
 	if ((!base || !base[0]) || prefix(leaf, PATH_SEP) || leaf[0] == '~')
 #else
 	if ((!base || !base[0]) || prefix(leaf, PATH_SEP))
@@ -227,14 +201,34 @@ size_t path_build(char *buf, size_t len, const char *base, const char *leaf)
 	return cur_len;
 }
 
+/*
+ * Return the index of the filename in a path, using PATH_SEPC. If no path
+ * separator is found, return 0.
+ */
+size_t path_filename_index(const char *path)
+{
+	int i;
 
+	if (strlen(path) == 0)
+		return 0;
+
+	for (i = strlen(path) - 1; i >= 0; i--) {
+		if (path[i] == PATH_SEPC)
+			return i + 1;
+	}
+
+	return 0;
+}
 
 /*** File-handling API ***/
 
-/* On Windows, fwrite() and fread() are broken. */
-#if defined(WINDOWS) || defined(SET_UID)
-# define HAVE_WRITE
-# define HAVE_READ
+/* Some defines for compatibility between various build platforms */
+#ifndef S_IRUSR
+#define S_IRUSR S_IREAD
+#endif
+
+#ifndef S_IWUSR
+#define S_IWUSR S_IWRITE
 #endif
 
 /* if the flag O_BINARY is not defined, it is not needed , but we still
@@ -243,6 +237,11 @@ size_t path_build(char *buf, size_t len, const char *base, const char *leaf)
 #define O_BINARY 0
 #endif
 
+/* Avoid a compiler warning when cross compiling for windows */
+#ifdef __STRICT_ANSI__
+FILE *fdopen(int handle, const char *mode);
+#endif
+
 /* Private structure to hold file pointers and useful info. */
 struct ang_file
 {
@@ -368,25 +367,30 @@ ang_file *file_open(const char *fname, file_mode mode, file_type ftype)
 	path_parse(buf, sizeof(buf), fname);
 
 	switch (mode) {
-		case MODE_WRITE: {
+		case MODE_WRITE: { 
 			if (ftype == FTYPE_SAVE) {
 				/* open only if the file does not exist */
 				int fd;
-				fd = open(buf,
-					O_CREAT | O_EXCL | O_WRONLY | O_BINARY, S_IRUSR | S_IWUSR);
+				fd = open(buf, O_CREAT | O_EXCL | O_WRONLY | O_BINARY, S_IRUSR | S_IWUSR);
 				if (fd < 0) {
 					/* there was some error */
 					f->fh = NULL;
 				} else {
 					f->fh = fdopen(fd, "wb");
 				}
-			} else
+			} else {
 				f->fh = fopen(buf, "wb");
-			 break;
-		 }
-		case MODE_READ:   f->fh = fopen(buf, "rb"); break;
-		case MODE_APPEND: f->fh = fopen(buf, "a+"); break;
-		default:          f->fh = fopen(buf, "__");
+			}
+			break;
+		}
+		case MODE_READ:
+			f->fh = fopen(buf, "rb");
+			break;
+		case MODE_APPEND:
+			f->fh = fopen(buf, "a+");
+			break;
+		default:
+			assert(0);
 	}
 
 	if (f->fh == NULL)
@@ -428,7 +432,7 @@ bool file_close(ang_file *f)
  */
 void file_lock(ang_file *f)
 {
-#if defined(HAVE_FCNTL_H) && defined(SET_UID)
+#if defined(HAVE_FCNTL_H) && defined(UNIX)
 	struct flock lock;
 	lock.l_type = (f->mode == MODE_READ ? F_RDLCK : F_WRLCK);
 	lock.l_whence = SEEK_SET;
@@ -436,7 +440,7 @@ void file_lock(ang_file *f)
 	lock.l_len = 0;
 	lock.l_pid = 0;
 	fcntl(fileno(f->fh), F_SETLKW, &lock);
-#endif /* HAVE_FCNTL_H && SET_UID */
+#endif /* HAVE_FCNTL_H && UNIX */
 }
 
 /*
@@ -444,7 +448,7 @@ void file_lock(ang_file *f)
  */
 void file_unlock(ang_file *f)
 {
-#if defined(HAVE_FCNTL_H) && defined(SET_UID)
+#if defined(HAVE_FCNTL_H) && defined(UNIX)
 	struct flock lock;
 	lock.l_type = F_UNLCK;
 	lock.l_whence = SEEK_SET;
@@ -452,7 +456,7 @@ void file_unlock(ang_file *f)
 	lock.l_len = 0;
 	lock.l_pid = 0;
 	fcntl(fileno(f->fh), F_SETLK, &lock);
-#endif /* HAVE_FCNTL_H && SET_UID */
+#endif /* HAVE_FCNTL_H && UNIX */
 }
 
 
@@ -461,9 +465,9 @@ void file_unlock(ang_file *f)
 /*
  * Seek to location 'pos' in file 'f'.
  */
-bool file_seek(ang_file *f, u32b pos)
+bool file_skip(ang_file *f, int bytes)
 {
-	return (fseek(f->fh, pos, SEEK_SET) == 0);
+	return (fseek(f->fh, bytes, SEEK_CUR) == 0);
 }
 
 /*
@@ -525,8 +529,6 @@ bool file_getl(ang_file *f, char *buf, size_t len)
 	byte b;
 	size_t i = 0;
 
-	bool check_encodes = FALSE;
-
 	/* Leave a byte for the terminating 0 */
 	size_t max_len = len - 1;
 
@@ -575,26 +577,9 @@ bool file_getl(ang_file *f, char *buf, size_t len)
 			continue;
 		}
 
-		/* Ignore non-printables */
-		else if (my_isprint((unsigned char)c))
-  		{
-			buf[i++] = c;
-
-			/* Notice possible encode */
- 			if (c == '[') check_encodes = TRUE;
-
-			continue;
-		}
-		else
-		{
-			buf[i++] = '?';
-			continue;
-		}
+		buf[i++] = c;
 	}
 
-	/* Translate encodes if necessary */
- 	if (check_encodes) xstr_trans(buf, LATIN1);
-
 	buf[i] = '\0';
 	return TRUE;
 }
@@ -650,30 +635,6 @@ bool file_vputf(ang_file *f, const char *fmt, va_list vp)
 	return file_put(f, buf);
 }
 
-/*
- * Format and translate a string, then print it out to file.
- */
-bool x_file_putf(ang_file *f, int encoding, const char *fmt, ...)
-{
-	va_list vp;
-
- 	char buf[1024];
-
- 	/* Begin the Varargs Stuff */
- 	va_start(vp, fmt);
-
- 	/* Format the args, save the length */
- 	(void)vstrnfmt(buf, sizeof(buf), fmt, vp);
-
- 	/* End the Varargs Stuff */
- 	va_end(vp);
-
- 	/* Translate*/
- 	xstr_trans(buf, encoding);
-
- 	return file_put(f, buf);
-}
-
 
 bool dir_exists(const char *path)
 {
@@ -826,8 +787,7 @@ void my_dclose(ang_dir *dir)
 	FREE(dir);
 }
 
-#endif /* WINDOWS */
-
+#else /* WINDOWS */
 
 #ifdef HAVE_DIRENT_H
 
@@ -909,4 +869,5 @@ void my_dclose(ang_dir *dir)
 }
 
 #endif /* HAVE_DIRENT_H */
+#endif /* WINDOWS */
 
