@@ -39,7 +39,11 @@ void safe_setuid_drop(void);
  */
 size_t path_build(char *buf, size_t len, const char *base, const char *leaf);
 
-
+/**
+ * Return the index of the filename in a path, using PATH_SEPC. If no path
+ * separator is found, return 0.
+ */
+size_t path_filename_index( const char *path );
 
 /*** File access code ***/
 
@@ -171,20 +175,14 @@ bool file_put(ang_file *f, const char *buf);
 bool file_putf(ang_file *f, const char *fmt, ...);
 bool file_vputf(ang_file *f, const char *fmt, va_list vp);
 
-/**
- * Format and translate a string, then print it out to file.
- */
-bool x_file_putf(ang_file *f, int encoding, const char *fmt, ...);
-
 
 /** Byte-based IO */
 
 /**
- * Seek to position `pos` in the file represented by `f`.
- *
- * Returns TRUE if successful, FALSE otherwise.
+ * Skip 'bytes' bytes.
+ * \returns TRUE if successful, FALSE otherwise.
  */
-bool file_seek(ang_file *f, u32b pos);
+bool file_skip(ang_file *f, int bytes);
 
 /**
  * Reads n bytes from file 'f' into buffer 'buf'.
