@@ -277,7 +277,7 @@ s16b Rand_normal(int mean, int stand) {
 	}
 
 	/* Convert the index into an offset */
-	offset = (long)stand * (long)low / RANDNOR_STD;
+	offset = (s16b)((long)stand * (long)low / RANDNOR_STD);
 
 	/* One half should be negative */
 	if (one_in_(2)) return (mean - offset);
@@ -310,14 +310,12 @@ int damcalc(int num, int sides, aspect dam_aspect) {
 	switch (dam_aspect) {
 		case MAXIMISE:
 		case EXTREMIFY: return num * sides;
-
 		case RANDOMISE: return damroll(num, sides);
-
 		case MINIMISE: return num;
-
 		case AVERAGE: return num * (sides + 1) / 2;
 	}
 
+	assert(0 && "Should never reach here");
 	return 0;
 }
 
@@ -419,14 +417,12 @@ s16b m_bonus_calc(int max, int level, aspect bonus_aspect) {
 	switch (bonus_aspect) {
 		case EXTREMIFY:
 		case MAXIMISE:  return max;
-
 		case RANDOMISE: return m_bonus(max, level);
-
 		case MINIMISE:  return 0;
-
 		case AVERAGE:   return max * level / MAX_DEPTH;
 	}
 
+	assert(0 && "Should never reach here");
 	return 0;
 }
 
@@ -472,11 +468,12 @@ void rand_fix(u32b val) {
 	rand_fixval = val;
 }
 
-/*
+int getpid(void);
+
+/**
  * Another simple RNG that does not use any of the above state
  * (so can be used without disturbing the game's RNG state)
  */
-int getpid(void);
 u32b Rand_simple(u32b m) {
 	static time_t seed;
 	time_t v;
