@@ -18,6 +18,139 @@
 #include "angband.h"
 #include "z-term.h"
 
+/*** Colour constants ***/
+
+/*
+ * Global table of color definitions (mostly zeros)
+ */
+byte angband_color_table[MAX_COLORS][4] =
+{
+	{0x00, 0x00, 0x00, 0x00}, /* 0  TERM_DARK */
+	{0x00, 0xff, 0xff, 0xff}, /* 1  TERM_WHITE */
+	{0x00, 0x80, 0x80, 0x80}, /* 2  TERM_SLATE */
+	{0x00, 0xff, 0x80, 0x00}, /* 3  TERM_ORANGE */
+	{0x00, 0xc0, 0x00, 0x00}, /* 4  TERM_RED */
+	{0x00, 0x00, 0x80, 0x40}, /* 5  TERM_GREEN */
+	{0x00, 0x00, 0x40, 0xff}, /* 6  TERM_BLUE */
+	{0x00, 0x80, 0x40, 0x00}, /* 7  TERM_UMBER */
+	{0x00, 0x60, 0x60, 0x60}, /* 8  TERM_L_DARK */
+	{0x00, 0xc0, 0xc0, 0xc0}, /* 9  TERM_L_WHITE */
+	{0x00, 0xff, 0x00, 0xff}, /* 10 TERM_L_PURPLE */
+	{0x00, 0xff, 0xff, 0x00}, /* 11 TERM_YELLOW */
+	{0x00, 0xff, 0x40, 0x40}, /* 12 TERM_L_RED */
+	{0x00, 0x00, 0xff, 0x00}, /* 13 TERM_L_GREEN */
+	{0x00, 0x00, 0xff, 0xff}, /* 14 TERM_L_BLUE */
+	{0x00, 0xc0, 0x80, 0x40}, /* 15 TERM_L_UMBER */
+	{0x00, 0x90, 0x00, 0x90}, /* 16 TERM_PURPLE */
+	{0x00, 0x90, 0x20, 0xff}, /* 17 TERM_VIOLET */
+	{0x00, 0x00, 0xa0, 0xa0}, /* 18 TERM_TEAL */
+	{0x00, 0x6c, 0x6c, 0x30}, /* 19 TERM_MUD */
+	{0x00, 0xff, 0xff, 0x90}, /* 20 TERM_L_YELLOW */
+	{0x00, 0xff, 0x00, 0xa0}, /* 21 TERM_MAGENTA */
+	{0x00, 0x20, 0xff, 0xdc}, /* 22 TERM_L_TEAL */
+	{0x00, 0xb8, 0xa8, 0xff}, /* 23 TERM_L_VIOLET */
+	{0x00, 0xff, 0x80, 0x80}, /* 24 TERM_L_PINK */
+	{0x00, 0xb4, 0xb4, 0x00}, /* 25 TERM_MUSTARD */
+	{0x00, 0xa0, 0xc0, 0xd0}, /* 26 TERM_BLUE_SLATE */
+	{0x00, 0x00, 0xb0, 0xff}, /* 27 TERM_DEEP_L_BLUE */
+	{0x00, 0x28, 0x28, 0x28}, /* 28 TERM_SHADE */
+};
+
+/*
+ * Global array of color names and translations.
+ */
+color_type color_table[MAX_COLORS] =
+{
+	/* full mono vga blind lighter darker highlight metallic misc */
+	{'d', "Dark", {0, 0, 0, TERM_DARK, TERM_L_DARK, TERM_DARK,
+				   TERM_L_DARK, TERM_L_DARK, TERM_DARK}},
+
+	{'w', "White", {1, 1, 1, TERM_WHITE, TERM_YELLOW, TERM_SLATE,
+					TERM_L_BLUE, TERM_YELLOW, TERM_WHITE}},
+
+	{'s', "Slate", {2, 1, 2, TERM_SLATE, TERM_L_WHITE, TERM_L_DARK,
+					TERM_L_WHITE, TERM_L_WHITE, TERM_SLATE}},
+
+	{'o', "Orange", {3, 1, 3, TERM_L_WHITE, TERM_YELLOW, TERM_SLATE,
+					 TERM_YELLOW, TERM_YELLOW, TERM_ORANGE}},
+
+	{'r', "Red", {4, 1, 4, TERM_SLATE, TERM_L_RED, TERM_SLATE,
+				  TERM_L_RED, TERM_L_RED, TERM_RED}},
+
+	{'g', "Green", {5, 1, 5, TERM_SLATE, TERM_L_GREEN, TERM_SLATE,
+					TERM_L_GREEN, TERM_L_GREEN, TERM_GREEN}},
+
+	{'b', "Blue", {6, 1, 6, TERM_SLATE, TERM_L_BLUE, TERM_SLATE,
+				   TERM_L_BLUE, TERM_L_BLUE, TERM_BLUE}},
+
+	{'u', "Umber", {7, 1, 7, TERM_L_DARK, TERM_L_UMBER, TERM_L_DARK,
+					TERM_L_UMBER, TERM_L_UMBER, TERM_UMBER}},
+
+	{'D', "Light Dark", {8, 1, 8, TERM_L_DARK, TERM_SLATE, TERM_L_DARK,
+						 TERM_SLATE, TERM_SLATE, TERM_L_DARK}},
+
+	{'W', "Light Slate", {9, 1, 9, TERM_L_WHITE, TERM_WHITE, TERM_SLATE,
+						  TERM_WHITE, TERM_WHITE, TERM_SLATE}},
+
+	{'P', "Light Purple", {10, 1, 10, TERM_SLATE, TERM_YELLOW, TERM_SLATE,
+						   TERM_YELLOW, TERM_YELLOW, TERM_L_PURPLE}},
+
+	{'y', "Yellow", {11, 1, 11, TERM_L_WHITE, TERM_L_YELLOW, TERM_L_WHITE,
+					 TERM_WHITE, TERM_WHITE, TERM_YELLOW}},
+
+	{'R', "Light Red", {12, 1, 12, TERM_L_WHITE, TERM_YELLOW, TERM_RED,
+						TERM_YELLOW, TERM_YELLOW, TERM_L_RED}},
+
+	{'G', "Light Green", {13, 1, 13, TERM_L_WHITE, TERM_YELLOW, TERM_GREEN,
+						  TERM_YELLOW, TERM_YELLOW, TERM_L_GREEN}},
+
+	{'B', "Light Blue", {14, 1, 14, TERM_L_WHITE, TERM_YELLOW, TERM_BLUE,
+						 TERM_YELLOW, TERM_YELLOW, TERM_L_BLUE}},
+
+	{'U', "Light Umber", {15, 1, 15, TERM_L_WHITE, TERM_YELLOW, TERM_UMBER,
+						  TERM_YELLOW, TERM_YELLOW, TERM_L_UMBER}},
+
+	/* "new" colors */
+	{'p', "Purple", {16, 1, 10,TERM_SLATE, TERM_L_PURPLE, TERM_SLATE,
+					 TERM_L_PURPLE, TERM_L_PURPLE, TERM_L_PURPLE}},
+
+	{'v', "Violet", {17, 1, 10,TERM_SLATE, TERM_L_PURPLE, TERM_SLATE,
+					 TERM_L_PURPLE, TERM_L_PURPLE, TERM_L_PURPLE}},
+
+	{'t', "Teal", {18, 1, 6, TERM_SLATE, TERM_L_TEAL, TERM_SLATE,
+				   TERM_L_TEAL, TERM_L_TEAL, TERM_L_BLUE}},
+
+	{'m', "Mud", {19, 1, 5, TERM_SLATE, TERM_MUSTARD, TERM_SLATE,
+				  TERM_MUSTARD, TERM_MUSTARD, TERM_UMBER}},
+
+	{'Y', "Light Yellow", {20, 1, 11, TERM_WHITE, TERM_WHITE, TERM_YELLOW,
+						   TERM_WHITE, TERM_WHITE, TERM_L_YELLOW}},
+
+	{'i', "Magenta-Pink", {21, 1, 12, TERM_SLATE, TERM_L_PINK, TERM_RED,
+						   TERM_L_PINK, TERM_L_PINK, TERM_L_PURPLE}},
+
+	{'T', "Light Teal", {22, 1, 14, TERM_L_WHITE, TERM_YELLOW, TERM_TEAL,
+						 TERM_YELLOW, TERM_YELLOW, TERM_L_BLUE}},
+
+	{'V', "Light Violet", {23, 1, 10, TERM_L_WHITE, TERM_YELLOW, TERM_VIOLET,
+						   TERM_YELLOW, TERM_YELLOW, TERM_L_PURPLE}},
+
+	{'I', "Light Pink", {24, 1, 12, TERM_L_WHITE, TERM_YELLOW, TERM_MAGENTA,
+						 TERM_YELLOW, TERM_YELLOW, TERM_L_PURPLE}},
+
+	{'M', "Mustard", {25, 1, 11, TERM_SLATE, TERM_YELLOW, TERM_SLATE,
+					  TERM_YELLOW, TERM_YELLOW, TERM_YELLOW}},
+
+	{'z', "Blue Slate",  {26, 1, 9, TERM_SLATE, TERM_DEEP_L_BLUE, TERM_SLATE,
+						  TERM_DEEP_L_BLUE, TERM_DEEP_L_BLUE, TERM_L_WHITE}},
+
+	{'Z', "Deep Light Blue", {27, 1, 14, TERM_L_WHITE, TERM_L_BLUE, TERM_BLUE_SLATE,
+							  TERM_L_BLUE, TERM_L_BLUE, TERM_L_BLUE}},
+
+	/* Rest to be filled in when the game loads */
+};
+
+
 
 /*
  * This file provides a generic, efficient, terminal window package,
@@ -225,7 +358,7 @@
  * that the contents of "cp" are null-terminated.  This hook is optional,
  * unless the setting of the "always_pict" or "higher_pict" flags make
  * it required.  Note that recently, this hook was changed from taking
- * a byte "a" and a char "c" to taking a length "n", an array of bytes
+ * a int "a" and a char "c" to taking a length "n", an array of ints
  * "ap" and an array of chars "cp".  Old implementations of this hook
  * should now iterate over all "n" attr/char pairs.
  * The two new arrays "tap" and "tcp" can contain the attr/char pairs
@@ -273,6 +406,7 @@ term *Term = NULL;
 /* grumbles */
 int log_i = 0;
 int log_size = 0;
+struct keypress keylog[KEYLOG_SIZE];
 
 
 /*** Local routines ***/
@@ -312,20 +446,20 @@ static errr term_win_init(term_win *s, int w, int h)
 	int y;
 
 	/* Make the window access arrays */
-	s->a = C_ZNEW(h, byte *);
-	s->c = C_ZNEW(h, char *);
+	s->a = C_ZNEW(h, int *);
+	s->c = C_ZNEW(h, wchar_t *);
 
 	/* Make the window content arrays */
-	s->va = C_ZNEW(h * w, byte);
-	s->vc = C_ZNEW(h * w, char);
+	s->va = C_ZNEW(h * w, int);
+	s->vc = C_ZNEW(h * w, wchar_t);
 
 	/* Make the terrain access arrays */
-	s->ta = C_ZNEW(h, byte *);
-	s->tc = C_ZNEW(h, char *);
+	s->ta = C_ZNEW(h, int *);
+	s->tc = C_ZNEW(h, wchar_t *);
 
 	/* Make the terrain content arrays */
-	s->vta = C_ZNEW(h * w, byte);
-	s->vtc = C_ZNEW(h * w, char);
+	s->vta = C_ZNEW(h * w, int);
+	s->vtc = C_ZNEW(h * w, wchar_t);
 
 	/* Prepare the window access arrays */
 	for (y = 0; y < h; y++)
@@ -352,17 +486,17 @@ static errr term_win_copy(term_win *s, term_win *f, int w, int h)
 	/* Copy contents */
 	for (y = 0; y < h; y++)
 	{
-		byte *f_aa = f->a[y];
-		char *f_cc = f->c[y];
+		int *f_aa = f->a[y];
+		wchar_t *f_cc = f->c[y];
 
-		byte *s_aa = s->a[y];
-		char *s_cc = s->c[y];
+		int *s_aa = s->a[y];
+		wchar_t *s_cc = s->c[y];
 
-		byte *f_taa = f->ta[y];
-		char *f_tcc = f->tc[y];
+		int *f_taa = f->ta[y];
+		wchar_t *f_tcc = f->tc[y];
 
-		byte *s_taa = s->ta[y];
-		char *s_tcc = s->tc[y];
+		int *s_taa = s->ta[y];
+		wchar_t *s_tcc = s->tc[y];
 
 		for (x = 0; x < w; x++)
 		{
@@ -401,7 +535,16 @@ errr Term_xtra(int n, int v)
 	return ((*Term->xtra_hook)(n, v));
 }
 
-
+/*
+ * Allow override of the multi-byte to wide char conversion
+ */
+size_t Term_mbstowcs(wchar_t *dest, const char *src, int n)
+{
+	if (Term && Term->mbcs_hook)
+		return (*Term->mbcs_hook)(dest, src, n);
+	else
+		return mbstowcs(dest, src, n);
+}
 
 /*** Fake hooks ***/
 
@@ -433,7 +576,7 @@ static errr Term_wipe_hack(int x, int y, int n)
 /*
  * Hack -- fake hook for "Term_text()" (see above)
  */
-static errr Term_text_hack(int x, int y, int n, byte a, const char *cp)
+static errr Term_text_hack(int x, int y, int n, int a, const wchar_t *cp)
 {
 	/* Compiler silliness */
 	if (x || y || n || a || cp) return (-2);
@@ -446,7 +589,7 @@ static errr Term_text_hack(int x, int y, int n, byte a, const char *cp)
 /*
  * Hack -- fake hook for "Term_pict()" (see above)
  */
-static errr Term_pict_hack(int x, int y, int n, const byte *ap, const char *cp, const byte *tap, const char *tcp)
+static errr Term_pict_hack(int x, int y, int n, const int *ap, const wchar_t *cp, const int *tap, const wchar_t *tcp)
 {
 	/* Compiler silliness */
 	if (x || y || n || ap || cp || tap || tcp) return (-2);
@@ -464,19 +607,19 @@ static errr Term_pict_hack(int x, int y, int n, const byte *ap, const char *cp,
  *
  * Assumes given location and values are valid.
  */
-void Term_queue_char(term *t, int x, int y, byte a, char c, byte ta, char tc)
+void Term_queue_char(term *t, int x, int y, int a, wchar_t c, int ta, wchar_t tc)
 {
-	byte *scr_aa = t->scr->a[y];
-	char *scr_cc = t->scr->c[y];
+	int *scr_aa = t->scr->a[y];
+	wchar_t *scr_cc = t->scr->c[y];
 
-	byte oa = scr_aa[x];
-	char oc = scr_cc[x];
+	int oa = scr_aa[x];
+	wchar_t oc = scr_cc[x];
 
-	byte *scr_taa = t->scr->ta[y];
-	char *scr_tcc = t->scr->tc[y];
+	int *scr_taa = t->scr->ta[y];
+	wchar_t *scr_tcc = t->scr->tc[y];
 
-	byte ota = scr_taa[x];
-	char otc = scr_tcc[x];
+	int ota = scr_taa[x];
+	wchar_t otc = scr_tcc[x];
 
 	/* Don't change is the terrain value is 0 */
 	if (!ta) ta = ota;
@@ -503,7 +646,7 @@ void Term_queue_char(term *t, int x, int y, byte a, char c, byte ta, char tc)
 
 /* Queue a large-sized tile */
 
-void Term_big_queue_char(term *t, int x, int y, byte a, char c, byte a1, char c1)
+void Term_big_queue_char(term *t, int x, int y, int a, wchar_t c, int a1, wchar_t c1)
 {
         int hor, vert;
 
@@ -526,27 +669,33 @@ void Term_big_queue_char(term *t, int x, int y, byte a, char c, byte a1, char c1
 			}
 
 			/* Now vertical */
-			for (vert = 1; vert <= tile_height; vert++)
-			{
+			for (vert = 1; vert <= tile_height; vert++){
+			
+				/* Leave space on bottom for status */
+				if (y + vert + 1 < t-> hgt) {
 			        /* Queue dummy character */
 			        if (a & 0x80)
 				        Term_queue_char(t, x + hor, y + vert, 255, -1, 0, 0);
-				else
+					else
 				        Term_queue_char(t, x + hor, y + vert, TERM_WHITE, ' ', a1, c1);
+				}
 			}
 		}
 	}
 	else
 	{
 	        /* Only vertical */
-	        for (vert = 1; vert <= tile_height; vert++)
-		{
-		        /* Queue dummy character */
-		        if (a & 0x80)
-			        Term_queue_char(t, x, y + vert, 255, -1, 0, 0);
-			else
-			        Term_queue_char(t, x, y + vert, TERM_WHITE, ' ', a1, c1);
-		}
+	        for (vert = 1; vert <= tile_height; vert++) {	
+			
+				/* Leave space on bottom for status */
+				if (y + vert + 1 < t->hgt) {
+					/* Queue dummy character */
+					if (a & 0x80)
+						Term_queue_char(t, x, y + vert, 255, -1, 0, 0);
+					else
+						Term_queue_char(t, x, y + vert, TERM_WHITE, ' ', a1, c1);
+				}
+			}
 	}
 }
 
@@ -558,31 +707,31 @@ void Term_big_queue_char(term *t, int x, int y, byte a, char c, byte a1, char c1
  * a valid location, so the first "n" characters of "s" can all be added
  * starting at (x,y) without causing any illegal operations.
  */
-void Term_queue_chars(int x, int y, int n, byte a, const char *s)
+void Term_queue_chars(int x, int y, int n, int a, const wchar_t *s)
 {
 	int x1 = -1, x2 = -1;
 
-	byte *scr_aa = Term->scr->a[y];
-	char *scr_cc = Term->scr->c[y];
+	int *scr_aa = Term->scr->a[y];
+	wchar_t *scr_cc = Term->scr->c[y];
 
-	byte *scr_taa = Term->scr->ta[y];
-	char *scr_tcc = Term->scr->tc[y];
+	int *scr_taa = Term->scr->ta[y];
+	wchar_t *scr_tcc = Term->scr->tc[y];
 
 	/* Queue the attr/chars */
 	for ( ; n; x++, s++, n--)
 	{
-		byte oa = scr_aa[x];
-		char oc = scr_cc[x];
+		int oa = scr_aa[x];
+		wchar_t oc = scr_cc[x];
 
-		byte ota = scr_taa[x];
-		char otc = scr_tcc[x];
+		int ota = scr_taa[x];
+		wchar_t otc = scr_tcc[x];
 
 		/* Hack -- Ignore non-changes */
 		if ((oa == a) && (oc == *s) && (ota == 0) && (otc == 0)) continue;
 
 		/* Save the "literal" information */
 		scr_aa[x] = a;
-		scr_cc[x] = xchar_trans(*s);
+		scr_cc[x] = *s;
 
 		scr_taa[x] = 0;
 		scr_tcc[x] = 0;
@@ -619,23 +768,23 @@ static void Term_fresh_row_pict(int y, int x1, int x2)
 {
 	int x;
 
-	byte *old_aa = Term->old->a[y];
-	char *old_cc = Term->old->c[y];
+	int *old_aa = Term->old->a[y];
+	wchar_t *old_cc = Term->old->c[y];
 
-	byte *scr_aa = Term->scr->a[y];
-	char *scr_cc = Term->scr->c[y];
+	int *scr_aa = Term->scr->a[y];
+	wchar_t *scr_cc = Term->scr->c[y];
 
-	byte *old_taa = Term->old->ta[y];
-	char *old_tcc = Term->old->tc[y];
+	int *old_taa = Term->old->ta[y];
+	wchar_t *old_tcc = Term->old->tc[y];
 
-	byte *scr_taa = Term->scr->ta[y];
-	char *scr_tcc = Term->scr->tc[y];
+	int *scr_taa = Term->scr->ta[y];
+	wchar_t *scr_tcc = Term->scr->tc[y];
 
-	byte ota;
-	char otc;
+	int ota;
+	wchar_t otc;
 
-	byte nta;
-	char ntc;
+	int nta;
+	wchar_t ntc;
 
 	/* Pending length */
 	int fn = 0;
@@ -643,11 +792,11 @@ static void Term_fresh_row_pict(int y, int x1, int x2)
 	/* Pending start */
 	int fx = 0;
 
-	byte oa;
-	char oc;
+	int oa;
+	wchar_t oc;
 
-	byte na;
-	char nc;
+	int na;
+	wchar_t nc;
 
 	/* Scan "modified" columns */
 	for (x = x1; x <= x2; x++)
@@ -716,20 +865,20 @@ static void Term_fresh_row_both(int y, int x1, int x2)
 {
 	int x;
 
-	byte *old_aa = Term->old->a[y];
-	char *old_cc = Term->old->c[y];
-	byte *scr_aa = Term->scr->a[y];
-	char *scr_cc = Term->scr->c[y];
+	int *old_aa = Term->old->a[y];
+	wchar_t *old_cc = Term->old->c[y];
+	int *scr_aa = Term->scr->a[y];
+	wchar_t *scr_cc = Term->scr->c[y];
 
-	byte *old_taa = Term->old->ta[y];
-	char *old_tcc = Term->old->tc[y];
-	byte *scr_taa = Term->scr->ta[y];
-	char *scr_tcc = Term->scr->tc[y];
+	int *old_taa = Term->old->ta[y];
+	wchar_t *old_tcc = Term->old->tc[y];
+	int *scr_taa = Term->scr->ta[y];
+	wchar_t *scr_tcc = Term->scr->tc[y];
 
-	byte ota;
-	char otc;
-	byte nta;
-	char ntc;
+	int ota;
+	wchar_t otc;
+	int nta;
+	wchar_t ntc;
 
 	/* The "always_text" flag */
 	int always_text = Term->always_text;
@@ -741,13 +890,13 @@ static void Term_fresh_row_both(int y, int x1, int x2)
 	int fx = 0;
 
 	/* Pending attr */
-	byte fa = Term->attr_blank;
+	int fa = Term->attr_blank;
 
-	byte oa;
-	char oc;
+	int oa;
+	wchar_t oc;
 
-	byte na;
-	char nc;
+	int na;
+	wchar_t nc;
 
 	/* Scan "modified" columns */
 	for (x = x1; x <= x2; x++)
@@ -799,7 +948,7 @@ static void Term_fresh_row_both(int y, int x1, int x2)
 		old_tcc[x] = ntc;
 
 		/* Handle high-bit attr/chars */
-		if ((na & 0x80) && (nc & 0x80))
+		if ((na & 0x80))
 		{
 			/* Flush */
 			if (fn)
@@ -821,7 +970,7 @@ static void Term_fresh_row_both(int y, int x1, int x2)
 			}
 
 			/* 2nd byte of bigtile */
-			if ((na == 255) && (nc == (char) -1)) continue;
+			if (na == 255) continue;
 
 			/* Hack -- Draw the special attr/char pair */
 			(void)((*Term->pict_hook)(x, y, 1, &na, &nc, &nta, &ntc));
@@ -887,11 +1036,11 @@ static void Term_fresh_row_text(int y, int x1, int x2)
 {
 	int x;
 
-	byte *old_aa = Term->old->a[y];
-	char *old_cc = Term->old->c[y];
+	int *old_aa = Term->old->a[y];
+	wchar_t *old_cc = Term->old->c[y];
 
-	byte *scr_aa = Term->scr->a[y];
-	char *scr_cc = Term->scr->c[y];
+	int *scr_aa = Term->scr->a[y];
+	wchar_t *scr_cc = Term->scr->c[y];
 
 	/* The "always_text" flag */
 	int always_text = Term->always_text;
@@ -903,13 +1052,13 @@ static void Term_fresh_row_text(int y, int x1, int x2)
 	int fx = 0;
 
 	/* Pending attr */
-	byte fa = Term->attr_blank;
+	int fa = Term->attr_blank;
 
-	byte oa;
-	char oc;
+	int oa;
+	wchar_t oc;
 
-	byte na;
-	char nc;
+	int na;
+	wchar_t nc;
 
 
 	/* Scan "modified" columns */
@@ -1000,6 +1149,30 @@ static void Term_fresh_row_text(int y, int x1, int x2)
 	}
 }
 
+/*
+ * Mark a spot as needing refresh (see "Term_fresh")
+ */
+errr Term_mark(int x, int y)
+{
+	int *old_aa = Term->old->a[y];
+	wchar_t *old_cc = Term->old->c[y];
+	int *old_taa = Term->old->ta[y];
+	wchar_t *old_tcc = Term->old->tc[y];
+
+	/*
+	 * using 0x80 as the blank attribute and an impossible value for
+	 * the blank char is ok since this function is only called by tile
+	 * functions, but ideally there should be a test to use the blank text
+	 * attr/char pair
+	 */
+	old_aa[x] = 0x80; 
+	old_cc[x] = 0;
+	old_taa[x] = 0x80;
+	old_tcc[x] = 0;
+
+	return (0);
+}
+
 byte tile_width = 1;            /* Tile width in units of font width */
 byte tile_height = 1;           /* Tile height in units of font height */
 
@@ -1162,8 +1335,8 @@ errr Term_fresh(void)
 	/* Handle "total erase" */
 	if (Term->total_erase)
 	{
-		byte na = Term->attr_blank;
-		char nc = Term->char_blank;
+		int na = Term->attr_blank;
+		wchar_t nc = Term->char_blank;
 
 		/* Physically erase the entire window */
 		Term_xtra(TERM_XTRA_CLEAR, 0);
@@ -1175,10 +1348,10 @@ errr Term_fresh(void)
 		/* Wipe each row */
 		for (y = 0; y < h; y++)
 		{
-			byte *aa = old->a[y];
-			char *cc = old->c[y];
-			byte *taa = old->ta[y];
-			char *tcc = old->tc[y];
+			int *aa = old->a[y];
+			wchar_t *cc = old->c[y];
+			int *taa = old->ta[y];
+			wchar_t *tcc = old->tc[y];
 
 			/* Wipe each column */
 			for (x = 0; x < w; x++)
@@ -1217,34 +1390,34 @@ errr Term_fresh(void)
 			int tx = old->cx;
 			int ty = old->cy;
 
-			byte *old_aa = old->a[ty];
-			char *old_cc = old->c[ty];
+			int *scr_aa = scr->a[ty];
+			wchar_t *scr_cc = scr->c[ty];
 
-			byte oa = old_aa[tx];
-			char oc = old_cc[tx];
+			int sa = scr_aa[tx];
+			wchar_t sc = scr_cc[tx];
 
-			byte *old_taa = old->ta[ty];
-			char *old_tcc = old->tc[ty];
+			int *scr_taa = scr->ta[ty];
+			wchar_t *scr_tcc = scr->tc[ty];
 
-			byte ota = old_taa[tx];
-			char otc = old_tcc[tx];
+			int sta = scr_taa[tx];
+			wchar_t stc = scr_tcc[tx];
 
 			/* Hack -- use "Term_pict()" always */
 			if (Term->always_pict)
 			{
-				(void)((*Term->pict_hook)(tx, ty, 1, &oa, &oc, &ota, &otc));
+				(void)((*Term->pict_hook)(tx, ty, 1, &sa, &sc, &sta, &stc));
 			}
 
 			/* Hack -- use "Term_pict()" sometimes */
-			else if (Term->higher_pict && (oa & 0x80) && (oc & 0x80))
+			else if (Term->higher_pict && (sa & 0x80))
 			{
-				(void)((*Term->pict_hook)(tx, ty, 1, &oa, &oc, &ota, &otc));
+				(void)((*Term->pict_hook)(tx, ty, 1, &sa, &sc, &sta, &stc));
 			}
 
 			/* Hack -- restore the actual character */
-			else if (oa || Term->always_text)
+			else if (sa || Term->always_text)
 			{
-				(void)((*Term->text_hook)(tx, ty, 1, oa, &oc));
+				(void)((*Term->text_hook)(tx, ty, 1, sa, &sc));
 			}
 
 			/* Hack -- erase the grid */
@@ -1454,7 +1627,7 @@ errr Term_gotoxy(int x, int y)
  * Do not change the cursor position
  * No visual changes until "Term_fresh()".
  */
-errr Term_draw(int x, int y, byte a, char c)
+errr Term_draw(int x, int y, int a, wchar_t c)
 {
 	int w = Term->wid;
 	int h = Term->hgt;
@@ -1483,14 +1656,14 @@ errr Term_draw(int x, int y, byte a, char c)
  *
  * We queue the given attr/char for display at the current
  * cursor location, and advance the cursor to the right,
- * marking it as unuable and returning "1" if it leaves
+ * marking it as unusable and returning "1" if it leaves
  * the screen, and otherwise returning "0".
  *
  * So when this function, or the following one, return a
  * positive value, future calls to either function will
  * return negative ones.
  */
-errr Term_addch(byte a, char c)
+errr Term_addch(int a, wchar_t c)
 {
 	int w = Term->wid;
 
@@ -1536,7 +1709,7 @@ errr Term_addch(byte a, char c)
  * positive value, future calls to either function will
  * return negative ones.
  */
-errr Term_addstr(int n, byte a, const char *buf)
+errr Term_addstr(int n, int a, const char *buf)
 {
 	int k;
 
@@ -1544,13 +1717,10 @@ errr Term_addstr(int n, byte a, const char *buf)
 
 	errr res = 0;
 
-	char s[1024];
+	wchar_t s[1024];
 
 	/* Copy to a rewriteable string */
- 	my_strcpy(s, buf, 1024);
-
- 	/* Translate it to 7-bit ASCII or system-specific format */
- 	xstr_trans(s, LATIN1);
+ 	Term_mbstowcs(s, buf, 1024);
 
 	/* Handle "unusable" cursor */
 	if (Term->scr->cu) return (-1);
@@ -1581,7 +1751,7 @@ errr Term_addstr(int n, byte a, const char *buf)
 /*
  * Move to a location and, using an attr, add a char
  */
-errr Term_putch(int x, int y, byte a, char c)
+errr Term_putch(int x, int y, int a, wchar_t c)
 {
 	errr res;
 
@@ -1599,7 +1769,7 @@ errr Term_putch(int x, int y, byte a, char c)
 /*
  * Move to a location and, using an attr, add a big tile
  */
-void Term_big_putch(int x, int y, byte a, char c)
+void Term_big_putch(int x, int y, int a, wchar_t c)
 {
 	int hor, vert;
 
@@ -1650,7 +1820,7 @@ void Term_big_putch(int x, int y, byte a, char c)
 /*
  * Move to a location and, using an attr, add a string
  */
-errr Term_putstr(int x, int y, int n, byte a, const char *s)
+errr Term_putstr(int x, int y, int n, int a, const char *s)
 {
 	errr res;
 
@@ -1682,14 +1852,14 @@ errr Term_erase(int x, int y, int n)
 	int x1 = -1;
 	int x2 = -1;
 
-	byte na = Term->attr_blank;
-	char nc = Term->char_blank;
+	int na = Term->attr_blank;
+	wchar_t nc = Term->char_blank;
 
-	byte *scr_aa;
-	char *scr_cc;
+	int *scr_aa;
+	wchar_t *scr_cc;
 
-	byte *scr_taa;
-	char *scr_tcc;
+	int *scr_taa;
+	wchar_t *scr_tcc;
 
 	/* Place cursor */
 	if (Term_gotoxy(x, y)) return (-1);
@@ -1704,17 +1874,11 @@ errr Term_erase(int x, int y, int n)
 	scr_taa = Term->scr->ta[y];
 	scr_tcc = Term->scr->tc[y];
 
-	if ((n > 0) && (scr_cc[x] == (char) -1) && (scr_aa[x] == 255))
-	{
-		x--;
-		n++;
-	}
-
 	/* Scan every column */
 	for (i = 0; i < n; i++, x++)
 	{
-		byte oa = scr_aa[x];
-		char oc = scr_cc[x];
+		int oa = scr_aa[x];
+		wchar_t oc = scr_cc[x];
 
 		/* Hack -- Ignore "non-changes" */
 		if ((oa == na) && (oc == nc)) continue;
@@ -1762,8 +1926,8 @@ errr Term_clear(void)
 	int w = Term->wid;
 	int h = Term->hgt;
 
-	byte na = Term->attr_blank;
-	char nc = Term->char_blank;
+	int na = Term->attr_blank;
+	wchar_t nc = Term->char_blank;
 
 	/* Cursor usable */
 	Term->scr->cu = 0;
@@ -1774,10 +1938,10 @@ errr Term_clear(void)
 	/* Wipe each row */
 	for (y = 0; y < h; y++)
 	{
-		byte *scr_aa = Term->scr->a[y];
-		char *scr_cc = Term->scr->c[y];
-		byte *scr_taa = Term->scr->ta[y];
-		char *scr_tcc = Term->scr->tc[y];
+		int *scr_aa = Term->scr->a[y];
+		wchar_t *scr_cc = Term->scr->c[y];
+		int *scr_taa = Term->scr->ta[y];
+		wchar_t *scr_tcc = Term->scr->tc[y];
 
 		/* Wipe each column */
 		for (x = 0; x < w; x++)
@@ -1832,7 +1996,7 @@ errr Term_redraw_section(int x1, int y1, int x2, int y2)
 {
 	int i, j;
 
-	char *c_ptr;
+	wchar_t *c_ptr;
 
 	/* Bounds checking */
 	if (y2 >= Term->hgt) y2 = Term->hgt - 1;
@@ -1924,7 +2088,7 @@ errr Term_locate(int *x, int *y)
  * Note that this refers to what will be on the window after the
  * next call to "Term_fresh()".  It may or may not already be there.
  */
-errr Term_what(int x, int y, byte *a, char *c)
+errr Term_what(int x, int y, int *a, wchar_t *c)
 {
 	int w = Term->wid;
 	int h = Term->hgt;
@@ -1985,6 +2149,24 @@ errr Term_keypress(keycode_t k, byte mods)
 	/* Hack -- Refuse to enqueue non-keys */
 	if (!k) return (-1);
 
+	if(!Term->complex_input) {
+		switch (k)
+		{
+			case '\r':
+			case '\n':
+			  	k = KC_ENTER;
+			  	break;
+			case 8:
+			  	k = KC_BACKSPACE;
+			  	break;
+			case 9:
+			  	k = KC_TAB;
+			  	break;
+			case 27:
+			  	k = ESCAPE;
+			  	break;
+		}
+	}
 	/* Store the char, advance the queue */
 	Term->key_queue[Term->key_head].type = EVT_KBRD;
 	Term->key_queue[Term->key_head].key.code = k;
@@ -2004,28 +2186,36 @@ errr Term_keypress(keycode_t k, byte mods)
 /*
  * Add a mouse event to the "queue"
  */
-errr Term_mousepress(int x, int y, char button)
+errr Term_mousepress(int x, int y, char button)/*, byte mods);*/
 {
-  /* Store the char, advance the queue */
-  Term->key_queue[Term->key_head].type = EVT_MOUSE;
-  Term->key_queue[Term->key_head].mouse.x = x;
-  Term->key_queue[Term->key_head].mouse.y = y;
-  Term->key_queue[Term->key_head].mouse.button = button;
-  Term->key_head++;
-  
-  /* Circular queue, handle wrap */
-  if (Term->key_head == Term->key_size) Term->key_head = 0;
-  
-  /* Success (unless overflow) */
-  if (Term->key_head != Term->key_tail) return (0);
+	/* Store the char, advance the queue */
+	Term->key_queue[Term->key_head].type = EVT_MOUSE;
+	Term->key_queue[Term->key_head].mouse.x = x;
+	Term->key_queue[Term->key_head].mouse.y = y;
+	/* XXX for now I encode the mods into the button number, so I would
+	 * not have to worry about the other platforms, when all platforms set
+	 * mods, this code should be replaced with :
+	 * Term->key_queue[Term->key_head].mouse.button = button;
+	 * Term->key_queue[Term->key_head].mouse.mods = mods;
+	 */
+	Term->key_queue[Term->key_head].mouse.button = (button & 0x0F);
+	Term->key_queue[Term->key_head].mouse.mods = ((button & 0xF0)>>4);
+
+	Term->key_head++;
+
+	/* Circular queue, handle wrap */
+	if (Term->key_head == Term->key_size) Term->key_head = 0;
+
+	/* Success (unless overflow) */
+	if (Term->key_head != Term->key_tail) return (0);
   
 #if 0
-  /* Hack -- Forget the oldest key */
-  if (++Term->key_tail == Term->key_size) Term->key_tail = 0;
+	/* Hack -- Forget the oldest key */
+	if (++Term->key_tail == Term->key_size) Term->key_tail = 0;
 #endif
   
-  /* Problem */
-  return (1);
+	/* Problem */
+	return (1);
 }
 
 
@@ -2233,8 +2423,8 @@ errr Term_resize(int w, int h)
 
 	int wid, hgt;
 
-	byte *hold_x1;
-	byte *hold_x2;
+	int *hold_x1;
+	int *hold_x2;
 
 	term_win *hold_old;
 	term_win *hold_scr;
@@ -2278,8 +2468,8 @@ errr Term_resize(int w, int h)
 	hold_tmp = Term->tmp;
 
 	/* Create new scanners */
-	Term->x1 = C_ZNEW(h, byte);
-	Term->x2 = C_ZNEW(h, byte);
+	Term->x1 = C_ZNEW(h, int);
+	Term->x2 = C_ZNEW(h, int);
 
 	/* Create new window */
 	Term->old = ZNEW(term_win);
@@ -2540,8 +2730,8 @@ errr term_init(term *t, int w, int h, int k)
 	t->hgt = h;
 
 	/* Allocate change arrays */
-	t->x1 = C_ZNEW(h, byte);
-	t->x2 = C_ZNEW(h, byte);
+	t->x1 = C_ZNEW(h, int);
+	t->x2 = C_ZNEW(h, int);
 
 
 	/* Allocate "displayed" */
@@ -2576,7 +2766,7 @@ errr term_init(term *t, int w, int h, int k)
 
 	/* Default "blank" */
 	t->attr_blank = 0;
-	t->char_blank = ' ';
+	t->char_blank = L' ';
 
 	/* No saves yet */
 	t->saved = 0;
