@@ -39,19 +39,19 @@ typedef struct term_win term_win;
 struct term_win
 {
 	bool cu, cv;
-	byte cx, cy;
+	int cx, cy;
 
-	byte **a;
-	char **c;
+	int **a;
+	wchar_t **c;
 
-	byte *va;
-	char *vc;
+	int *va;
+	wchar_t *vc;
 
-	byte **ta;
-	char **tc;
+	int **ta;
+	wchar_t **tc;
 
-	byte *vta;
-	char *vtc;
+	int *vta;
+	wchar_t *vtc;
 
 	term_win *next;
 };
@@ -116,6 +116,8 @@ struct term_win
  *	- Value "char_blank"
  *	  Use this "char" value for "blank" grids
  *
+ *	- Flag "complex_input"
+ *	  Distinguish between Enter/^m/^j, Tab/^i, etc.
  *
  *	- Ignore this pointer
  *
@@ -180,8 +182,10 @@ struct term
 	bool never_bored;
 	bool never_frosh;
 
-	byte attr_blank;
-	char char_blank;
+	int attr_blank;
+	wchar_t char_blank;
+
+	bool complex_input;
 
 	ui_event *key_queue;
 
@@ -190,18 +194,18 @@ struct term
 	u16b key_xtra;
 	u16b key_size;
 
-	byte wid;
-	byte hgt;
+	int wid;
+	int hgt;
 
-	byte y1;
-	byte y2;
+	int y1;
+	int y2;
 
-	byte *x1;
-	byte *x2;
+	int *x1;
+	int *x2;
 
 	/* Offsets used by the map subwindows */
-	byte offset_x;
-	byte offset_y;
+	int offset_x;
+	int offset_y;
 
 	term_win *old;
 	term_win *scr;
@@ -223,11 +227,14 @@ struct term
 
 	errr (*wipe_hook)(int x, int y, int n);
 
-	errr (*text_hook)(int x, int y, int n, byte a, const char *s);
+	errr (*text_hook)(int x, int y, int n, int a, const wchar_t *s);
 
-	errr (*pict_hook)(int x, int y, int n, const byte *ap, const char *cp, const byte *tap, const char *tcp);
+	errr (*pict_hook)(int x, int y, int n, const int *ap, const wchar_t *cp, const int *tap, const wchar_t *tcp);
+
+	size_t (*mbcs_hook)(wchar_t *dest, const char *src, int n);
+
+	void (*view_map_hook)(term *t);
 
-	byte (*xchar_hook)(byte c);
 };
 
 
@@ -267,8 +274,7 @@ struct term
 #define TERM_XTRA_DELAY 13    /* Delay some milliseconds (optional) */
 
 
-/*** Color constants ***/
-
+/*** Colors ***/
 
 /*
  * Angband "attributes" (with symbols, and base (R,G,B) codes)
@@ -305,6 +311,7 @@ struct term
 #define TERM_MUSTARD     25    /* M */
 #define TERM_BLUE_SLATE  26    /* z */
 #define TERM_DEEP_L_BLUE 27    /* Z */
+#define TERM_SHADE       28    /* for shaded backgrounds */
 
 /* The following allow color 'translations' to support environments with a limited color depth
  * as well as translate colours to alternates for e.g. menu highlighting. */
@@ -325,7 +332,25 @@ struct term
  * Maximum number of colours, and number of "basic" Angband colours
  */ 
 #define MAX_COLORS        256
-#define BASIC_COLORS    28
+#define BASIC_COLORS    29
+#define BG_BLACK 0	/* The set number for the black-background glyphs */
+#define BG_SAME  1	/* The set number for the same-background glyphs */
+#define BG_DARK  2	/* The set number for the dark-background glyphs */
+#define BG_MAX   3	/* The max number of backgrounds */
+
+/*
+ * A game color.
+ */
+typedef struct color_type color_type;
+struct color_type
+{
+	char index_char;            /* Character index:  'r' = red, etc. */
+	char name[32];              /* Color name */
+	byte color_translate[MAX_ATTR];       /* Index for various in-game translations */
+};
+
+extern byte angband_color_table[MAX_COLORS][4];
+extern color_type color_table[MAX_COLORS];
 
 
 
@@ -333,7 +358,7 @@ struct term
 #define KEYLOG_SIZE 8
 extern int log_i;
 extern int log_size;
-struct keypress keylog[KEYLOG_SIZE];
+extern struct keypress keylog[KEYLOG_SIZE];
 
 
 /**** Available Variables ****/
@@ -347,29 +372,31 @@ extern bool smlcurs;
 /**** Available Functions ****/
 
 extern errr Term_xtra(int n, int v);
+extern size_t Term_mbstowcs(wchar_t *dest, const char *src, int n);
 
-extern void Term_queue_char(term *t, int x, int y, byte a, char c, byte ta, char tc);
-extern void Term_big_queue_char(term *t, int x, int y, byte a, char c, byte a1, char c1);
-extern void Term_queue_chars(int x, int y, int n, byte a, const char *s);
+extern void Term_queue_char(term *t, int x, int y, int a, wchar_t c, int ta, wchar_t tc);
+extern void Term_big_queue_char(term *t, int x, int y, int a, wchar_t c, int a1, wchar_t c1);
+extern void Term_queue_chars(int x, int y, int n, int a, const wchar_t *s);
 
 extern errr Term_fresh(void);
 extern errr Term_set_cursor(bool v);
 extern errr Term_gotoxy(int x, int y);
-extern errr Term_draw(int x, int y, byte a, char c);
-extern errr Term_addch(byte a, char c);
-extern errr Term_addstr(int n, byte a, const char *s);
-extern errr Term_putch(int x, int y, byte a, char c);
-extern void Term_big_putch(int x, int y, byte a, char c);
-extern errr Term_putstr(int x, int y, int n, byte a, const char *s);
+extern errr Term_draw(int x, int y, int a, wchar_t c);
+extern errr Term_addch(int a, wchar_t c);
+extern errr Term_addstr(int n, int a, const char *s);
+extern errr Term_putch(int x, int y, int a, wchar_t c);
+extern void Term_big_putch(int x, int y, int a, wchar_t c);
+extern errr Term_putstr(int x, int y, int n, int a, const char *s);
 extern errr Term_erase(int x, int y, int n);
 extern errr Term_clear(void);
 extern errr Term_redraw(void);
 extern errr Term_redraw_section(int x1, int y1, int x2, int y2);
+extern errr Term_mark(int x, int y);
 
 extern errr Term_get_cursor(bool *v);
 extern errr Term_get_size(int *w, int *h);
 extern errr Term_locate(int *x, int *y);
-extern errr Term_what(int x, int y, byte *a, char *c);
+extern errr Term_what(int x, int y, int *a, wchar_t *c);
 
 extern errr Term_flush(void);
 extern errr Term_mousepress(int x, int y, char button);
