@@ -25,6 +25,49 @@
  */
 char *argv0 = NULL;
 
+
+/*
+ * Count the number of characters in a UTF-8 encoded string
+ *
+ * Taken from http://canonical.org/~kragen/strlen-utf8.html
+ */
+size_t utf8_strlen(char *s)
+{
+	size_t i = 0, j = 0;
+	while (s[i]) {
+		if ((s[i] & 0xc0) != 0x80) j++;
+		i++;
+	}
+	return j;
+}
+
+/*
+ * Clip a null-terminated UTF-8 string 's' to 'n' unicode characters.
+ * e.g. utf8_clipto("example", 4) will clip after 'm', resulting in 'exam'.
+ */
+void utf8_clipto(char *s, size_t n)
+{
+	size_t i = 0, j = 0;
+	bool terminate_next = FALSE;
+
+	if (n == 0) {
+		s[i] = 0;
+		return;
+	}
+
+	while (s[i]) {
+		if ((s[i] & 0xc0) != 0x80) {
+			j++;
+			if (terminate_next)
+				break;
+			if (j == n)
+				terminate_next = TRUE;
+		}
+		i++;
+	}
+	s[i] = 0;
+}
+
 /*
  * Case insensitive comparison between two strings
  */
@@ -177,6 +220,15 @@ size_t my_strcat(char *buf, const char *src, size_t bufsize)
 	}
 }
 
+/**
+ * Capitalise the first letter of string 'str'.
+ */
+void my_strcap(char *buf)
+{
+	if (buf && buf[0])
+		buf[0] = toupper((unsigned char) buf[0]);
+}
+
 
 /*
  * Determine if string "a" is equal to string "b"
@@ -242,6 +294,55 @@ bool prefix_i(const char *s, const char *t)
 	return (TRUE);
 }
 
+/*
+ * rewrite string s in-place "skipping" every occurrence of character c
+ */
+void strskip(char *s, const char c){
+	char *in=s;
+	char *out=s;
+	while(*in){
+		if(*in!=c){
+			*out=*in;
+			out++;
+		}
+		in++;
+	}
+	*out=0;
+}
+
+/*
+ * rewrite string s in-place removing escape character c
+ * note that pairs of c will leave one instance of c in out
+ */
+void strescape(char *s, const char c){
+	char *in=s;
+	char *out=s;
+	bool escapenext = FALSE;
+	while(*in){
+		if(*in!=c || escapenext){
+			*out=*in;
+			out++;
+			escapenext = FALSE;
+		} else if (*in == c) {
+			escapenext = TRUE;
+		}
+		in++;
+	}
+	*out=0;
+}
+
+/*
+ * returns TRUE if string only contains spaces
+ */
+bool contains_only_spaces(const char* s){
+	char spaces[]=" \t";
+	while(*s){
+		if(strchr(spaces,*s)!=NULL)
+			return FALSE;
+		s++;
+	}
+	return TRUE;
+}
 
 /*
  * Redefinable "plog" action
@@ -279,13 +380,13 @@ void quit(const char *str)
 	if (quit_aux) (*quit_aux)(str);
 
 	/* Success */
-	if (!str) (void)(exit(EXIT_SUCCESS));
+	if (!str) exit(EXIT_SUCCESS);
 
 	/* Send the string to plog() */
 	plog(str);
 
 	/* Failure */
-	(void)(exit(EXIT_FAILURE));
+	exit(EXIT_FAILURE);
 }
 
 /* Arithmetic mean of the first 'size' entries of the array 'nums' */
